{
================================================================================
*
*       Package:      MDK - Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 7/29/08 10:20a $
*       Site Name:    Hines OIFO
*       Date Created: 4/29/07
*       Developers:   petit           
*       Description:  Contains the application project file
*       Notes:        Updated by
*                     andriyevskiy              
*
================================================================================
*
}
unit oDBController_Renal;

interface

uses
  Dialogs,
  Classes,
  Sysutils,
  TRPCB,
  oDBController
  , Controls
  ;

type
  TRenalVitalSign = (vtTemp, vtPulse, vtRespiration, vtBloodPressure, vtPain,
    vtWeight);

  TDBController_Renal = class(TDBController)
  private
    function IENS(aIEN: int64): string; overload;
    function IENS(aIEN: string): string; overload;
  public
    ResultInfo: string;

    function GetActiveRenalPatients(var aPatientList: string): boolean;

    function GetAllStudiesByPatient(aPatientID: int64; var aStudyList: string): boolean;

    function GetStudy(aStudyID: integer;
      var aStudyInfo, aStudySummary, aStudyFlowsheet, aMedLog,
      aNoteList, anEventLog: string): boolean;
    function SetStudy(aStudyID: integer;
      aStudyInfo, aStudySummary, aStudyFlowsheet, aMedLog,
      aNoteList, anEventLog: string): boolean;

    function GetStudyItem(aFile, aNode, anItemID: string; var aValue: string): boolean;
    function SetStudyItem(aFile, aNode, anItemID: string; aValue: string;
      aPatient: string = ''): boolean;

    function GetInstrumentResultList(aStudyID: integer;
      var aInstrumentResultList: string;
      aGetAll: boolean = False): boolean;

    function GetInstrumentResult(aResultID: integer; var aInstrumentResult: string): boolean;
    function GetBillingData(aStudyID: integer;var aBillingData: string): boolean;

    function SetBillingData(aStudyID: integer;
      aICD01, aICD02, aICD03, aICD04, aCPT01: string;
      aSvcConn, aMST, aAOExposure, aIRExposure, aEnvContaminants,
      aHeadNeckCancer, aCombatVet: Integer;
      aProviderList: array of string): boolean;

    function getLockStatus(aStudyID: integer): String;
    function LockStudy(aStudyID: integer): boolean;
    function UnlockStudy(aStudyID: integer): boolean;
    function SubmitStudy(aStudyID: integer;aUserESig: string;aDocumentText: TStringList): boolean;

    function GetPatientAccessPoints(aPatientID: int64;
      var aAccessPoints, aAccessHistory, aInfectionHistory: string): boolean;
    function SetPatientAccessPoints(aPatientID: int64;
      aAccessPoints, aAccessHistory, aInfectionHistory: string): boolean;

    function LockSetting(aSettingName: string): boolean;
    function UnlockSetting(aSettingName: string): boolean;

    function GetUserOption(aOptionName: string;var aOptionValue: string): boolean;
    function SetUserOption(aOptionName, aOptionValue: string): boolean;
    function GetUserOptionByDUZ(aOptionName, aDUZ: string;var aOptionValue: string): boolean;
    function SetUserOptionByDUZ(aOptionName, aDUZ, aOptionValue: string): boolean;
    function DeleteUserOption(aOptionName: string): boolean;
    function GetSystemOption(aOptionName: string;var aOptionValue: string): boolean;
    function SetSystemOption(aOptionName, aOptionValue: string): boolean;
    function DeleteSystemOption(aOptionName: string): boolean;

    function ValidateElectronicSignature(aSignature: string): boolean;
    function LookupProvider(aPartialName: string; var aReturnList: string): boolean;
    function LookupFacility(aPartialName: string; var aReturnList: string): boolean;
    function LookupAdministrator(aPartialName: string;var aReturnList: string): boolean;


    function GetCCOWContext: string;

    {Old style RPC calls}//====================================================
    procedure getPatientInfo(aDFN: string; Results: TStringList);
    procedure getPatientLabInfo(aDFN: string; Results: TStringList);
    procedure getPatientAllergies(aDFN: string;Results: TStringList);
    procedure getPatientDirectives(aDFN: string;Results: TStringList);
    procedure getPatientWarnings(aDFN: string;Results: TStringList);
    procedure getPatientVAccination(aDFN: string;Results: TStringList);

    function SetVitalsMeasurement(
      aDateTimeTaken: TDateTime;
      aPatientDFN: string;
      aVitalsType: TRenalVitalSign;
      aHospitalLocation: string;
      aUserDUZ: string;
      aValue: string): boolean;

    function SubmitNoteNow(
      aStudyID: integer;
      aUserID: string;
      aUserESig: string;
      // The users E-Sig in clear text, it will be encrypted before sending
      aDocumentText: TStringList;
      aTitle: string): boolean;

    function getNoteList(aStudyID: integer): Boolean;
    function getNote(aStudyID: integer; aNote: string): Boolean;

    function getClinicProcedures(aDFN, aClinic: string; var aReturnList: string): boolean;
    function getClinicDiagnosis(aDFN, aClinic: string; var aReturnList: string): boolean;
    function getServiceConnections(aDFN, aClinic: string; var aReturnList: string): boolean;

    function getLexiconData(aTarget, anOption: string; var aReturnList: string):Boolean;
    function getEncounterData(aStudy: string; var aReturnList: string): Boolean;
    function setEncounterData(aStudy: string; aBillingData: TStringList; var
      aReturnList: string): Boolean;

    function getLookupEntries(aFile, aTarget: string): TStringList;

    function getLocation(aStudy: string): string;
    function setLocation(aStudy, aLocation: string): string;
    function validatePCEData(aStudy, aData: string): string;

    function getUserNameByIEN(anIEN: string): string;
    function getUserNameByIEN2(anIEN: string): string;

    function getQualifiersByVitalType(aTypeIEN: string): string;
    function getQualifiersAll: string;
    function getVitalTypesAll: string;

    function getGatewayStatus: string;
    function getGatewayStatusInfo: string;
    function getGatewayRunningStatus: string;
    function getServerCurrentDateTime: string;

    function getRPCResults(aParamList, aDelim: string): string;

    function getStudyStatus(aStudy: string): string;

    function RegisterAccessToSensitiveData(anID, anExeNameAndVersion: string): string;
    function IsDataSensitive(aPatientID: string; var sSecurityWarning: string): Boolean;

    function getSensitiveRecordAccessInfo(aDFN: string): string;
    function getSensitiveRecordBulletinInfo(aDFN: string): string;
    function getOptionListXML(aUserDUZ: string = '.5'): string;
    function setStudyStatus(aStudyID, aStatus: string): string;

    function getPCEInfo(aPatientID, aStudyID: string): string;

    function getUserOptionList(aUser: string): string;
    function getCPTIUTitles(aTarget: string): string;
    function getAppintmentInfo(aPatientID, aStudyID: string): string;
    function getVersionInfo(aGUIVersion: string): string;
    function getSupportedVersion: string;
    function getPhysicianOrders(aPatient,anOrderIEN:String):String;

    function setStudyUseStatus(aStudyID,aStatusID,aStatusBy,aStatusAt,aWS:String): String;
    function getStudyUseStatus(sStudyID: String;var sStatus,sStatusBy,sStatusAt,sWS:String): Boolean;
    function getLockedStudies:String;
    function getUserInfo:String;

    function getXPar(aName:String): String;
    function setXPar(aName, aValue:String): String;
  end;

function NewDBController_Renal(aRPCBroker: TRPCBroker): TDBController_Renal;

var
  VerifyWriteIsComplete: Boolean;

implementation

uses
  oDBConnection
  , Hash // For E-sig encoding before sending over the wire
  , MFunStr
  , DateUtils
  , uMDK_Utils //  , uROR_CommonUtils // Standard M Function for Piece call
  , uMDK_LocalDB
  ;

const
  RENAL_VITAL_SIGN_NAME: array[TRenalVitalSign] of
  string = ('T', 'P', 'R', 'BP', 'PN', 'WT');

  BOOL201: array[boolean] of string = ('0', '1');

  RPC_BILLING_WIDGET =  'MD TMDWIDGET';
  RPC_CLINIC_INFO =     'MD TMDCIDC';
  RPC_CLINIC_WORKLOAD = 'MD TMDCLIN';
  RPC_CurrentTime =     'GMV GET CURRENT TIME';
  RPC_ENCOUNTER =       'MD TMDENCOUNTER';
  RPC_FILE_VITALS =     'GMV ADD VM';
  RPC_GATEWAY =         'MD GATEWAY';
  RPC_GETSET_RENAL =    'MDK GET/SET RENAL DATA';
  RPC_GET_VISTA_DATA =  'MDK GET VISTA DATA';
  RPC_LEXICON =         'MD TMDLEX';
  RPC_MANAGER =         'GMV MANAGER';
  RPC_PHYSICIAN_ORDERS ='MDK GET ORDERS';
  RPC_PT_INQUIRY =      'ORWPT PTINQ';
  RPC_SENSITIVE_RA =    'DG SENSITIVE RECORD ACCESS';
  RPC_SENSITIVE_RB =    'DG SENSITIVE RECORD BULLETIN';
  RPC_TIU_TITLES =      'MD TMDRECORDID';
  RPC_UTILITY =         'MDK UTILITY';
  RPC_VERSION_INFO =    RPC_GETSET_RENAL;

  SYSTEM_ID = '.5';

function NewDBController_Renal(aRPCBroker: TRPCBroker): TDBController_Renal;
begin
  Result := TDBController_Renal.Create(NewDBConnection(aRPCBroker));
end;

////////////////////////////////////////////////////////////////////////////////
{ TDBController_Renal }

function TDBController_Renal.IENS(aIEN: int64): string;
{Private function to take an integer and return an IENS for fileman}
begin
  Result := Format('%d,', [aIEN]);
end;

// vhaishandria 060419 ////////////////////////////////////////////////////start
function TDBController_Renal.IENS(aIEN: string): string;
{Private function to take an integer and return an IENS for fileman}
begin
  Result := Format('%s,', [aIEN]);
end;
// vhaishandria 060419 //////////////////////////////////////////////////////end

function TDBController_Renal.GetActiveRenalPatients(
  var aPatientList: string): boolean;
{ TODO 1 -oPetit -cMandatory : Get Active Renal Patients }
begin
  if not Assigned(DBConnection) then
    Result := getLocal(LocalName(lmActiveStudies, []), aPatientList)
  else
    try
      Result := DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['ACTIVE'], nil);
      aPatientList := DBConnection.ExecResults.Text;
      if SaveRPCResults then
        setLocal(LocalName(lmActiveStudies, []), aPatientList)
    except
      on E: Exception do
        begin
          Result := False;
          aPatientList := Format(XML_STATUS_ERROR, [E.Message]);
        end;
    end;
end;

function TDBController_Renal.GetAllStudiesByPatient(
  aPatientID: int64;
  var aStudyList: string): boolean;
{ TODO 1 -oPetit -cMandatory : Get All Studies By Patient}
begin
  if not Assigned(DBConnection) then
    Result := getLocal(LocalName(lmPatientStudyList, [IntToStr(aPatientID)]),
      aStudyList)
  else
    try
      Result := DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['ALLBYPT',
        IntToStr(aPatientID)], nil);
      aStudyList := DBConnection.ExecResults.Text;
      if SaveRPCResults then
        setLocal(LocalName(lmPatientStudyList, [IntToStr(aPatientID)]),
          aStudyList)
    except
      on E: Exception do
        begin
          Result := False;
          aStudyList := Format(XML_STATUS_ERROR, [E.Message]);
        end;
    end;
end;

function TDBController_Renal.GetCCOWContext: string;
begin
  try
    if DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['CCOW'], nil) then
      Result := DBConnection.ExecResults[0]
    else
      Result := '';
  except
    Result := '';
  end;
end;

function TDBController_Renal.GetInstrumentResult(
  aResultID: integer;
  var aInstrumentResult: string): boolean;
{ TODO 1 -oPetit -cMandatory : Get Instrument Data }
begin
  if not Assigned(DBConnection) then
    Result := getLocal(LocalName(lmInstrumentResults, [IntToStr(aResultID)]),
      aInstrumentResult)
  else
    try
      if DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETRSLT',
        IntToStr(aResultID)], nil) then
        begin
          DBConnection.ExecResults.Delete(0); // peel off the counter flag
          aInstrumentResult := DBConnection.ExecResults.Text;
          Result := True;
          if SaveRPCResults then
            setLocal(LocalName(lmInstrumentResults, [IntToStr(aResultID)]),
              aInstrumentResult)
        end
      else
        Result := False;
    except
      on E: Exception do
        begin
          Result := False;
          aInstrumentResult := Format(XML_STATUS_ERROR, [e.Message]);
        end;
    end;
end;

function TDBController_Renal.GetInstrumentResultList(
  aStudyID: integer;
  var aInstrumentResultList: string;
  aGetAll: boolean = False): boolean;
{ TODO 1 -oPetit -cMandatory : Get Instrument Result List }
begin
  if not Assigned(DBConnection) then
    Result := getLocal(LocalName(lmStudyResultList, [IntToStr(aStudyID)]),
      aInstrumentResultList)
  else
    try
      Result := DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['ALLRSLT',
        IntToStr(aStudyID), BOOL201[aGetAll]], nil);
      aInstrumentResultList := DBConnection.ExecResults.Text;
      if SaveRPCResults then
        setLocal(LocalName(lmStudyResultList, [IntToStr(aStudyID)]),
          aInstrumentResultList)
    except
      on E: Exception do
        begin
          Result := False;
          aInstrumentResultList := Format(XML_STATUS_ERROR, [E.Message]);
        end;
    end;
end;

function TDBController_Renal.GetBillingData(
  aStudyID: integer;
  var aBillingData: string): boolean;
{ TODO 1 -oPetit -cMandatory : Get Billing Data }
begin
  if not Assigned(DBConnection) then
    Result := getLocal(LocalName(lmStudyBilling, [IntToStr(aStudyID)]),
      aBillingData)
  else
    try
      Result := DBConnection.ExecRemoteProc(RPC_BILLING_WIDGET, ['GETBILL',
        IntToStr(aStudyID)], nil);
      aBillingData := DBConnection.ExecResults.Text;
      if SaveRPCResults then
        setLocal(LocalName(lmStudyBilling, [IntToStr(aStudyID)]), aBillingData)
    except
      on E: Exception do
        begin
          Result := False;
          aBillingData := Format(XML_STATUS_ERROR, [E.Message]);
        end;
    end;
end;

function TDBController_Renal.SetBillingData(
  aStudyID: integer;
  aICD01, aICD02, aICD03, aICD04, aCPT01: string;
  aSvcConn, aMST, aAOExposure, aIRExposure, aEnvContaminants, aHeadNeckCancer,
  aCombatVet: Integer {boolean};
  aProviderList: array of string): boolean;
{ TODO 1 -oPetit -cMandatory : Set Study Billing Data }
var
  aBillingData: TStringList;
  aProviderID: integer;
  i: integer;
begin
  aBillingData := TStringList.Create;
  aBillingData.Add(aICD01);
  aBillingData.Add(aICD02);
  aBillingData.Add(aICD03);
  aBillingData.Add(aICD04);
  aBillingData.Add(aCPT01);
  aBillingData.Add(IntToStr(aSvcConn));
  aBillingData.Add(IntToStr(aMST));
  aBillingData.Add(IntToStr(aAOExposure));
  aBillingData.Add(IntToStr(aIRExposure));
  aBillingData.Add(IntToStr(aEnvContaminants));
  aBillingData.Add(IntToStr(aHeadNeckCancer));
  aBillingData.Add(IntToStr(aCombatVet));
  if Length(aProviderList) > 0 then
    for i := Low(aProviderList) to High(aProviderList) do
      begin
        aProviderID := StrToIntDef(Piece(aProviderList[i], '^', 1), -1);
        if aProviderID > 0 then
          aBillingData.Add(Format('`%d', [aProviderID]));
      end;

  if not Assigned(DBConnection) then
    Result := setLocal(LocalName(lmStudyBilling, [IntToStr(aStudyID)]),
      aBillingData.Text)
  else
    begin
      try
        Result := DBConnection.ExecRemoteProc(RPC_BILLING_WIDGET, ['SETBILL',
          IntToStr(aStudyID)], aBillingData);
      except
        Result := False;
      end;
      if SaveRPCResults then
        setLocal(LocalName(lmStudyBilling, [IntToStr(aStudyID)]),
          aBillingData.Text);
    end;
  // vhaishandria 050427
  if aBillingData <> nil then
    aBillingData.Free;
end;

function TDBController_Renal.GetUserOption(
  aOptionName: string;
  var aOptionValue: string): boolean;
{ TODO 1 -oPetit -cMandatory : Get Renal Option }
begin
  if not Assigned(DBConnection) then
    Result := getLocal(LocalName(lmOptions, ['DUZ', aOptionName]), aOptionValue)
  else
    begin
      Result := DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETOPT',
        aOptionName, '@DUZ'], nil);
      aOptionValue := DBConnection.ExecResults.Text;
      if SaveRPCResults then
        setLocal(LocalName(lmOptions, ['DUZ', aOptionName]), aOptionValue)
    end;
end;

function TDBController_Renal.SetUserOption(
  aOptionName, aOptionValue: string): boolean;
{ TODO 1 -oPetit -cMandatory : Set Renal Option}
begin
  if not Assigned(DBConnection) then
    Result := setLocal(LocalName(lmOptions, ['DUZ', aOptionName]), aOptionValue)
  else
    begin
// vhaishandria 2007-07-27 ----------------------------------------------- begin
      Result := LockSetting(aOptionName);
      if not Result then Exit;
// vhaishandria 2007-07-27 ------------------------------------------------- end
    Result := DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETOPT',
      aOptionName, '@DUZ'], aOptionValue);
// vhaishandria 2007-07-27 ----------------------------------------------- begin
      UnLockSetting(aOptionName);
// vhaishandria 2007-07-27 ------------------------------------------------- end
    end;
end;

function TDBController_Renal.GetUserOptionByDUZ(
  aOptionName, aDUZ: string;
  var aOptionValue: string): boolean;
begin
  if not Assigned(DBConnection) then
    Result := getLocal(LocalName(lmOptions, [aDUZ, aOptionName]), aOptionValue)
  else
    begin
      Result := DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETOPT',
        aOptionName, aDUZ], nil);
      aOptionValue := DBConnection.ExecResults.Text;
      if SaveRPCResults then
        setLocal(LocalName(lmOptions, [aDUZ, aOptionName]), aOptionValue)
    end;
end;

function TDBController_Renal.SetUserOptionByDUZ(
  aOptionName, aDUZ, aOptionValue: string): boolean;
begin
  if not Assigned(DBConnection) then
    Result := setLocal(LocalName(lmOptions, [aDUZ, aOptionName]), aOptionValue)
  else
    begin
// vhaishandria 2007-08-29 No locking for user option
// vhaishandria 2007-07-27 ----------------------------------------------- begin
//      Result := LockSetting(aOptionName);
//      if not Result then Exit;
// vhaishandria 2007-07-27 ------------------------------------------------- end

      Result := DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETOPT',
        aOptionName, aDUZ], aOptionValue);
// vhaishandria 2007-08-29 No locking for user option
// vhaishandria 2007-07-27 ----------------------------------------------- begin
//      UnLockSetting(aOptionName);
// vhaishandria 2007-07-27 ------------------------------------------------- end
    end;
end;

function TDBController_Renal.DeleteUserOption(
  aOptionName: string): boolean;
begin
  //  if not Assigned(DBConnection) then
  //    Result :=  deleteLocal('RPC\MDK GETSET RENAL DATA\GETOPT\USER\'+aOptionName)
  //  else
  Result := DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['DELOPT',
    aOptionName,'@DUZ'], nil);
end;

function TDBController_Renal.GetSystemOption(
  aOptionName: string;
  var aOptionValue: string): boolean;
begin
  if not Assigned(DBConnection) then
    //    Result := getLocal(LocalName(lmOptions, ['SYSTEM', aOptionName]), aOptionValue)
    Result := getLocal(LocalName(lmOptions, ['.5', aOptionName]), aOptionValue)
  else
    begin
      Result := DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETOPT',
        aOptionName, '.5'], nil);
      aOptionValue := DBConnection.ExecResults.Text;
      if SaveRPCResults then
        setLocal(LocalName(lmOptions, ['.5', aOptionName]), aOptionValue)
    end;
end;

function TDBController_Renal.SetSystemOption(
  aOptionName, aOptionValue: string): boolean;
begin
  if not Assigned(DBConnection) then
    Result := setLocal(LocalName(lmOptions, ['.5', aOptionName]), aOptionValue)
  else
    begin
// vhaishandria 2007-07-27 ----------------------------------------------- begin
      Result := LockSetting(aOptionName);
      if not Result then Exit;
// vhaishandria 2007-07-27 ------------------------------------------------- end
      Result := DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETOPT',
        aOptionName, '.5'], aOptionValue);
// vhaishandria 2007-07-27 ----------------------------------------------- begin
      UnLockSetting(aOptionName);
// vhaishandria 2007-07-27 ------------------------------------------------- end
      if SaveRPCResults then
        Result := setLocal(LocalName(lmOptions, ['.5', aOptionName]),
          aOptionValue)
    end;
end;

function TDBController_Renal.DeleteSystemOption(
  aOptionName: string): boolean;
begin
  if not Assigned(DBConnection) then
    Result := deleteLocal(LocalName(lmOptions, ['.5', aOptionName]))
  else
    Result := DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['DELOPT',
      aOptionName, '.5'], nil);
end;

function TDBController_Renal.GetPatientAccessPoints(
  aPatientID: int64;
  var aAccessPoints, aAccessHistory, aInfectionHistory: string): boolean;
{ TODO 1 -oPetit -cMandatory : Get Access Points }
var
  sID: string;
begin
  if not Assigned(DBConnection) then
    begin
      sID := IntToStr(aPatientID);
      getLocal(LocalName(lmPatientInfo, [sID, 'AccessPoints.XML']),
        aAccessPoints);
      getLocal(LocalName(lmPatientInfo, [sID, 'AccessHistory.XML']),
        aAccessHistory);
      getLocal(LocalName(lmPatientInfo, [sID, 'AccessInfectionHistory.XML']),
        aInfectionHistory);
      Result := True;
    end
  else
    try
      sID := IENS(aPatientID);
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETXML', '704.201', sID,
        '.1'], nil);
      aAccessPoints := DBConnection.ExecResults.Text;
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETXML', '704.201', sID,
        '.2'], nil);
      aAccessHistory := DBConnection.ExecResults.Text;
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETXML', '704.201', sID,
        '.3'], nil);
      aInfectionHistory := DBConnection.ExecResults.Text;
      Result := True;

      if SaveRPCResults then
        begin
          sID := IntToStr(aPatientID);
          setLocal(LocalName(lmPatientInfo, [sID, 'AccessPoints.XML']),
            aAccessPoints);
          setLocal(LocalName(lmPatientInfo, [sID, 'AccessHistory.XML']),
            aAccessHistory);
          setLocal(LocalName(lmPatientInfo, [sID,
            'AccessInfectionHistory.XML']), aInfectionHistory);
        end;
    except
      on E: Exception do
        Result := false;
    end;
end;

function TDBController_Renal.SetPatientAccessPoints(
  aPatientID: int64;
  aAccessPoints, aAccessHistory, aInfectionHistory: string): boolean;
{ TODO 1 -oPetit -cMandatory : Set Patient Access Points }
var
  sID: string;
begin
  if not Assigned(DBConnection) then
    begin
      sID := IntToStr(aPatientID);
      setLocal(LocalName(lmPatientInfo, [sID, 'AccessPoints.XML']),
        aAccessPoints);
      setLocal(LocalName(lmPatientInfo, [sID, 'AccessHistory.XML']),
        aAccessHistory);
      setLocal(LocalName(lmPatientInfo, [sID, 'AccessInfectionHistory.XML']),
        aInfectionHistory);
      Result := True;
    end
  else
    try
      sID := IENS(aPatientID);
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETXML', '704.201', sID,
        '.1'], aAccessPoints);
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETXML', '704.201', sID,
        '.2'], aAccessHistory);
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETXML', '704.201', sID,
        '.3'], aInfectionHistory);
      Result := True;
    except
      Result := False;
    end;
end;

function TDBController_Renal.GetStudy(
  aStudyID: integer;
  var aStudyInfo, aStudySummary, aStudyFlowsheet, aMedLog,
  aNoteList, anEventLog: string): boolean;
{ TODO 1 -oPetit -cMandatory : Get Study }
var
  sID: string;
begin
  if not Assigned(DBConnection) then
    begin
      sID := IntToStr(aStudyID);
      getLocal(LocalName(lmStudyComponent, [sID, 'Info']), aStudyInfo);
      getLocal(LocalName(lmStudyComponent, [sID, 'Summary']), aStudySummary);
      getLocal(LocalName(lmStudyComponent, [sID, 'Flowsheet']),
        aStudyFlowsheet);
      getLocal(LocalName(lmStudyComponent, [sID, 'MedLog']), aMedLog);
      getLocal(LocalName(lmStudyComponent, [sID, 'Notes']), aNoteList);
      getLocal(LocalName(lmStudyComponent, [sID, 'Events']), anEventLog);

      Result := True;
    end
  else
    try
      sID := IENS(aStudyID);
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETXML', '704.202', sID,
        '.1'], nil);
      aStudyInfo := DBConnection.ExecResults.Text;

      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETXML', '704.202', sID,
        '.2'], nil);
      aStudySummary := DBConnection.ExecResults.Text;

      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETXML', '704.202', sID,
        '.3'], nil);
      aStudyFlowsheet := DBConnection.ExecResults.Text;

      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETXML', '704.202', sID,
        '.4'], nil);
      aMedLog := DBConnection.ExecResults.Text;

      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETXML', '704.202', sID,
        '.5'], nil);
      aNoteList := DBConnection.ExecResults.Text;

      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETXML', '704.202', sID,
        '.6'], nil);
      anEventLog := DBConnection.ExecResults.Text;

      Result := True;

      if SaveRPCResults then
        begin
          sID := IntToStr(aStudyID);
          setLocal(LocalName(lmStudyComponent, [sID, 'Info']), aStudyInfo);
          setLocal(LocalName(lmStudyComponent, [sID, 'Summary']),
            aStudySummary);
          setLocal(LocalName(lmStudyComponent, [sID, 'Flowsheet']),
            aStudyFlowsheet);
          setLocal(LocalName(lmStudyComponent, [sID, 'MedLog']), aMedLog);
          setLocal(LocalName(lmStudyComponent, [sID, 'Notes']), aNoteList);
          setLocal(LocalName(lmStudyComponent, [sID, 'Events']), anEventLog);
        end;
    except
      Result := False;
    end;
end;

function TDBController_Renal.GetStudyItem(
  aFile,
  aNode,
  anItemID: string;
  var aValue: string): boolean;
var
  ComponentName: string;
  { TODO 1 -oPetit -cMandatory : Get Study }
begin
  Result := False;
  aValue := '';
  if (aFile = '704.202') //
  and (aNode <> '.1') // StudyInfo
  and (aNode <> '.2') // StudySummary
  and (aNode <> '.3') // StudyFlowsheet
  and (aNode <> '.4') // StudyMedLog
  and (aNode <> '.5') // StudyNoteList
  and (aNode <> '.6') {// StudyEventLog} then
    Exit;

  if (aFile = '704.201') //
  and (aNode <> '.1') // AP
  and (aNode <> '.2') // APH
  and (aNode <> '.3') {// APIH} then
    Exit;

  if not Assigned(DBConnection) then
    begin
      if (aFile = '704.202') then
        begin
          if (aNode = '.1') then
            ComponentName := 'Info'; // StudyInfo
          if (aNode = '.2') then
            ComponentName := 'Summary'; // StudySummary
          if (aNode = '.3') then
            ComponentName := 'Flowsheet'; // StudyFlowsheet
          if (aNode = '.4') then
            ComponentName := 'MedLog'; // StudyMedLog
          if (aNode = '.5') then
            ComponentName := 'Notes'; // StudyNoteList
          if (aNode = '.6') then
            ComponentName := 'Events'; // StudyEventLog
          Result := getLocal(LocalName(lmStudyComponent, [anItemID,
            ComponentName]), aValue);
        end
      else
        begin
          if (aNode = '.1') then
            ComponentName := 'AccessPoints'; // StudyInfo
          if (aNode = '.2') then
            ComponentName := 'AccessHistory'; // StudySummary
          if (aNode = '.3') then
            ComponentName := 'AccessInfectionHistory'; // StudyFlowsheet
          Result := getLocal(LocalName(lmPatientInfo, [anItemID,
            ComponentName]), aValue);
        end;
    end
  else
    try
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['GETXML', aFile,
        IENS(anItemID), aNode], nil);
      aValue := DBConnection.ExecResults.Text;

      Result := True;
    except
      Result := False;
    end;
end;

function TDBController_Renal.SetStudy(
  aStudyID: integer;
  aStudyInfo, aStudySummary, aStudyFlowsheet, aMedLog, aNoteList, anEventLog:
  string): boolean;
{ TODO 1 -oPetit -cMandatory : Set Study Data }
var
  sID: string;
begin
  if not Assigned(DBConnection) then
    begin
      sID := IntToStr(aStudyID);
      setLocal(LocalName(lmStudyComponent, [sID, 'Info']), aStudyInfo);
      setLocal(LocalName(lmStudyComponent, [sID, 'Summary']), aStudySummary);
      setLocal(LocalName(lmStudyComponent, [sID, 'Flowsheet']),
        aStudyFlowsheet);
      setLocal(LocalName(lmStudyComponent, [sID, 'MedLog']), aMedLog);
      setLocal(LocalName(lmStudyComponent, [sID, 'Notes']), aNoteList);
      setLocal(LocalName(lmStudyComponent, [sID, 'Events']), anEventLog);

      Result := True;
    end
  else
    try
      sID := IENS(aStudyID);
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETXML', '704.202',
        IENS(aStudyID), '.1'], aStudyInfo);
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETXML', '704.202',
        IENS(aStudyID), '.2'], aStudySummary);
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETXML', '704.202',
        IENS(aStudyID), '.3'], aStudyFlowsheet);
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETXML', '704.202',
        IENS(aStudyID), '.4'], aMedLog);
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETXML', '704.202',
        IENS(aStudyID), '.5'], aNoteList);
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETXML', '704.202',
        IENS(aStudyID), '.6'], anEventLog);
      Result := True;
      if SaveRPCResults then
        begin
          sID := IntToStr(aStudyID);
          setLocal(LocalName(lmStudyComponent, [sID, 'Info']), aStudyInfo);
          setLocal(LocalName(lmStudyComponent, [sID, 'Summary']),
            aStudySummary);
          setLocal(LocalName(lmStudyComponent, [sID, 'Flowsheet']),
            aStudyFlowsheet);
          setLocal(LocalName(lmStudyComponent, [sID, 'MedLog']), aMedLog);
          setLocal(LocalName(lmStudyComponent, [sID, 'Notes']), aNoteList);
          setLocal(LocalName(lmStudyComponent, [sID, 'Events']), anEventLog);
        end;
    except
      Result := False;
    end;
end;

function TDBController_Renal.SetStudyItem(
  aFile,
  aNode,
  anItemID: string;
  aValue: string;
  aPatient: string = ''): boolean;
var
  ComponentName: string;

  { TODO 1 -oPetit -cMandatory : Set Study Data }
begin
  Result := False;

  if (aFile <> '704.202') and
    (aFile <> '704.201') then
    Exit;

  if (aFile = '704.202') // 704.202
  and (aNode <> '.1') // StudyInfo
  and (aNode <> '.2') // StudySummary
  and (aNode <> '.3') // StudyFlowsheet
  and (aNode <> '.4') // StudyMedLog
  and (aNode <> '.5') // StudyNoteList
  and (aNode <> '.6') {// StudyEventLog} then
    Exit;

  if (aFile = '704.201') // 704.201
  and (aNode <> '.1') // AP
  and (aNode <> '.2') // APH
  and (aNode <> '.3') {// APIH} then
    Exit;

  if not Assigned(DBConnection) then
    begin
      if (aFile = '704.202') then
        begin
          if (aNode = '.1') then
            ComponentName := 'Info'; // StudyInfo
          if (aNode = '.2') then
            ComponentName := 'Summary'; // StudySummary
          if (aNode = '.3') then
            ComponentName := 'Flowsheet'; // StudyFlowsheet
          if (aNode = '.4') then
            ComponentName := 'MedLog'; // StudyMedLog
          if (aNode = '.5') then
            ComponentName := 'Notes'; // StudyNoteList
          if (aNode = '.6') then
            ComponentName := 'Events'; // StudyEventLog
          Result := setLocal(LocalName(lmStudyComponent, [anItemID,
            ComponentName]), aValue);
        end
      else
        begin
          if (aNode = '.1') then
            ComponentName := 'AccessPoints.XML'; // StudyInfo
          if (aNode = '.2') then
            ComponentName := 'AccessHistory.XML'; // StudySummary
          if (aNode = '.3') then
            ComponentName := 'AccessInfectionHistory.XML'; // StudyFlowsheet
          Result := setLocal(LocalName(lmPatientInfo, [aPatient,
            ComponentName]), aValue);
        end;
    end
  else
    try
      // Note SETXML requires ',' to be added to the study ID
      //
      DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['SETXML', aFile, anItemID,
        aNode], aValue);
      Result := True;
    except
      Result := False;
    end;
end;

function TDBController_Renal.LockStudy(
  aStudyID: integer): boolean;
{ TODO 1 -oPetit -cMandatory : Lock Study }
var
  s : String;
begin
  if not Assigned(DBConnection) then
    begin
//      Result := True
      Result := getLocalLock(IntToStr(aStudyID),s);
      if s <> 'LOCKED' then
          Result := setLocalLock(IntToStr(aStudyID),'LOCKED');
    end
  else if DBConnection.ExecRemoteProc(RPC_UTILITY, ['LOCK', '704.202',
    IENS(aStudyID)], nil) then
    Result := Copy(DBConnection.ExecResults.Text, 1, 2) <> '-1'
  else
    Result := False;
  if Assigned(DBConnection) then
    ResultInfo := piece(DBConnection.ExecResults.Text,'^',2);
end;

function TDBController_Renal.UnlockStudy(
  aStudyID: integer): boolean;
{ TODO 1 -oPetit -cMandatory : Unlock Study }
var
  s: String;
begin
  if not Assigned(DBConnection) then
    begin
//      Result := True
      Result := getLocalLock(IntToStr(aStudyID),s);
      if s = 'LOCKED' then
          Result := setLocalLock(IntToStr(aStudyID),'');
    end
  else if DBConnection.ExecRemoteProc(RPC_UTILITY, ['UNLOCK', '704.202',
    IENS(aStudyID)], nil) then
    Result := Copy(DBConnection.ExecResults.Text, 1, 2) <> '-1'
  else
    Result := False;
  if Assigned(DBConnection) then
    ResultInfo := piece(DBConnection.ExecResults.Text,'^',2);
end;

function FormatInfo(anInfo: string): string;
var
  ss,
    s: string;
begin
  s := anInfo;
  ss := piece(s, '^', 4, 4);
  ss := copy(ss, 4, 2) + '/' + copy(ss, 6, 2) + '/' + copy(ss, 2, 2) + '  ' +
    copy(ss, 9, 2) + ':' + copy(ss, 11, 2) + ':' + copy(ss, 13, 2)
    + #13#10 + '             (' + ss + ')'
    ;
  Result :=
    'Reason:   ' + piece(s, '^', 2, 2) + #13#10 +
    'User DUZ: ' + piece(s, '^', 3, 3) + #13#10 +
    'Since:    ' + ss + #13#10 +
    'Job #:    ' + piece(s, '^', 5, 5);
end;

function TDBController_Renal.LockSetting(
  aSettingName: string): boolean;
var
  s: string;
begin
  if not Assigned(DBConnection) then
    begin
      Result := True;
      ResultInfo := '';
    end
  else
    begin
      if DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['LOCKOPT', aSettingName,
        '1'], nil) then
        begin
          Result := (Copy(DBConnection.ExecResults.Text, 1, 2) <> '-1');
        end
      else
        Result := {False;}
          (Copy(DBConnection.ExecResults.Text, 1, 18) = '-1^No such setting');

      s := DBConnection.ExecResults.Text;
      ResultInfo := FormatInfo(s);
    end;
end;

function TDBController_Renal.UnlockSetting(
  aSettingName: string): boolean;
var
  s: string;
begin
  if not Assigned(DBConnection) then
    begin
      Result := True;
      ResultInfo := '';
    end
  else
    begin
      if DBConnection.ExecRemoteProc(RPC_GETSET_RENAL, ['LOCKOPT', aSettingName,
        '0'], nil) then
        begin
          Result := Copy(DBConnection.ExecResults.Text, 1, 2) <> '-1';
        end
      else
        Result := False;
      s := DBConnection.ExecResults.Text;
      ResultInfo := FormatInfo(s);
    end;
end;

function TDBController_Renal.ValidateElectronicSignature(
  aSignature: string): boolean;
{Encrypt is included in the Hash Unit from VistA Broker}
begin
  if not Assigned(DBConnection) then
    Result := aSignature = 'TRUE'
  else
    begin
      aSignature := Encrypt(aSignature);
      Result := DBConnection.ExecRemoteProc(RPC_BILLING_WIDGET, ['ESIG',
        aSignature], nil);
    end;
end;

////////////////////////////////////////////////////////////////////////////////

function TDBController_Renal.LookupProvider(
  aPartialName: string;
  var aReturnList: string): boolean;
begin
  if not Assigned(DBConnection) then
    begin
      getLocal(LocalName(lmProviders, []), aReturnList);
      Result := True;
    end
  else
    try
      DBConnection.execRemoteProc(RPC_GET_VISTA_DATA, ['GETPROV', aPartialName],
        nil);
      aReturnList := DBConnection.ExecResults.Text;
      Result := True;
      if SaveRPCResults then
        setLocal(LocalName(lmProviders, []), aReturnList);
    except
      Result := False;
    end;
end;

function TDBController_Renal.LookupFacility(
  aPartialName: string;
  var aReturnList: string): boolean;
var
  sl: TStringList;
  i: integer;
begin
  if not Assigned(DBConnection) then
    getLocal(LocalName(lmFacilities, []), aReturnList)
  else
    try
      DBConnection.execRemoteProc(RPC_GETSET_RENAL, ['GETINST', aPartialName],
        nil);
      aReturnList := DBConnection.ExecResults.Text;
      if SaveRPCResults then
        setLocal(LocalName(lmFacilities, []), aReturnList)
    except
    end;

  sl := TStringList.Create;
  if pos('<STATUS>OK</STATUS>', aReturnList) = 0 then
    Result := False
  else
    begin
      Result := True;
      sl.Text := aReturnList;
      aReturnList := #13#10;
      for i := 0 to sl.Count - 1 do
        begin
          if pos('<STATION_NUMBER>', sl[i]) <> 1 then
            continue
          else
            aReturnList := aReturnList +
              piece(piece(sl[i], '<STATION_NUMBER>', 2), '<', 1) + '^' +
              piece(piece(sl[i + 1], '<STATION_NAME>', 2), '<', 1) + #13#10;
        end;
    end;
  sl.Free;
end;

function TDBController_Renal.LookupAdministrator(
  aPartialName: string;
  var aReturnList: string): boolean;
var
  sl: TStringList;
  i: integer;
begin
  if not Assigned(DBConnection) then
    getLocal(LocalName(lmAdministrators, []), aReturnList)
  else
    try
      DBConnection.execRemoteProc(RPC_GETSET_RENAL, ['GETADM', aPartialName],
        nil);
      aReturnList := DBConnection.ExecResults.Text;
      if SaveRPCResults then
        setLocal(LocalName(lmAdministrators, []), aReturnList)
    except
    end;

  sl := TStringList.Create;
  if pos('<STATUS>OK</STATUS>', aReturnList) = 0 then
    Result := False
  else
    begin
      sl.Text := aReturnList;
      aReturnList := #13#10;
      for i := 0 to sl.Count - 1 do
        begin
          if pos('<NUMBER>', sl[i]) <> 1 then
            continue
          else
            aReturnList := aReturnList +
              piece(piece(sl[i], '<NUMBER>', 2), '<', 1) + '^' +
              piece(piece(sl[i + 1], '<NAME>', 2), '<', 1) + #13#10;
        end;
      Result := True;
    end;
  sl.Free;
end;

function TDBController_Renal.SubmitNoteNow(
  aStudyID: integer;
  aUserID: string;
  aUserESig: string; // E-Sig in clear text, it will be encrypted before sending
  aDocumentText: TStringList;
  aTitle: string): boolean;
begin
  if not Assigned(DBConnection) then
    begin
      setLocal(LocalName(lmTIU, [IntToStr(aStudyID),
        FormatDateTime('yyyymmddhhnnss', Now)]), aDocumentText.Text);
      Result := True;
    end
  else
    try
      Result := DBConnection.ExecRemoteProc('MD TMDNOTE',
        ['NEWDOC', IntToStr(aStudyID), '', '', aUserID, Encrypt(aUserESig), aTitle],
        aDocumentText);
      ResultInfo := DBConnection.ExecResults.Text;
    except
      Result := False;
    end;
end;

function TDBController_Renal.SubmitStudy(
  aStudyID: integer;
  aUserESig: string; // E-Sig in clear text, it will be encrypted before sending
  aDocumentText: TStringList): boolean;
var
  s: string;
begin
  if not Assigned(DBConnection) then
    begin
      s := aDocumentText.Text;
      setLocal(LocalName(lmStudyReport, [IntToStr(aStudyID)]), s);
      Result := True;
    end
  else
    try
      Result := DBConnection.ExecRemoteProc( RPC_BILLING_WIDGET,
        ['SUBMIT', IntToStr(aStudyID), Encrypt(aUserESig)],
        aDocumentText);
      ResultInfo := DBConnection.ExecResults.Text;
      if SaveRPCResults then
        setLocal(LocalName(lmStudyReport, [IntToStr(aStudyID)]), s);
    except
      Result := False;
    end;
end;

procedure TDBController_Renal.getPatientInfo(
  aDFN: string;
  Results: TStringList);
var
  i: Integer;
  s: string;
begin
  if not Assigned(DBConnection) then
    begin
      getLocal(localName(lmPatientInfo, [aDFN, 'Inquiry.txt']), s);
      Results.Text := s;
    end
  else
    try
      Results.Clear;
      DBConnection.ExecRemoteProc(RPC_PT_INQUIRY, [aDFN], nil);
      for i := 0 to DBConnection.ExecResults.Count - 1 do
        Results.Add(DBConnection.ExecResults[i]);
      if SaveRPCResults then
        setLocal(localName(lmPatientInfo, [aDFN, 'Inquiry.txt']), Results.Text);
    except
      on E: Exception do
        Results.Text := e.Message;
    end;
end;

procedure TDBController_Renal.getPatientLabInfo(
  aDFN: string;
  Results: TStringList);
var
  i: Integer;
  s: string;
  sDFN: string;
begin
  sDFN := Piece(aDFN, '^', 1);
  if not Assigned(DBConnection) then
    begin
      getLocal(localName(lmPatientInfo, [sDFN, 'Lab.txt']), s);
      Results.Text := s;
    end
  else
    try
      Results.Clear;
      DBConnection.ExecRemoteProc(RPC_GET_VISTA_DATA, ['LAB', aDFN], nil);
      if Pos('-1', DBConnection.ExecResults[0]) = 1 then
        Results.Add({'Error getting patient lab information'#13#10#13#10+}
          Piece(DBConnection.ExecResults[0],'^',2))
      else
        for i := 1 to DBConnection.ExecResults.Count - 1 do
          begin
            s := Piece(DBConnection.ExecResults[i], '^', 2);
            try
              s := copy(s, 5, 2) + '/' + copy(s, 7, 2) + '/' + copy(s, 1, 4);
            except
              s := '';
            end;
            Results.Add(
              Piece(DBConnection.ExecResults[i], '^', 1) + '  -- ' +
              s + ' --  ' +
              Piece(DBConnection.ExecResults[i], '^', 3) + ' ' +
              Piece(DBConnection.ExecResults[i], '^', 4)
              );
          end;
      if SaveRPCResults then
        setLocal(localName(lmPatientInfo, [sDFN, 'Lab.txt']), s);
    except
      on E: Exception do
        Results.Text := e.Message;
    end;

end;

procedure TDBController_Renal.getPatientAllergies(
  aDFN: string;
  Results: TStringList);
var
  i: Integer;
  s: string;
begin
  if not Assigned(DBConnection) then
    begin
      getLocal(localName(lmPatientInfo, [aDFN, 'Allergy.txt']), s);
      Results.Text := s;
      Exit;
    end
  else
    try
      Results.Clear;
      DBConnection.ExecRemoteProc(RPC_GET_VISTA_DATA, ['ALLERGY', aDFN], nil);
      s := Piece(DBConnection.ExecResults[0], '^', 1);
      if s = '-1' then
        Results.Add('No Allergies')
      else
        begin
          if (DBConnection.ExecResults.Count = 2) and (Pos('No',
            DBConnection.ExecResults[1]) <> 1) then
            Results.Add('Allergies:');
          for i := 1 to DBConnection.ExecResults.Count - 1 do
            Results.Add(DBConnection.ExecResults[i]);
        end;
      if SaveRPCResults then
        setLocal(localName(lmPatientInfo, [aDFN, 'Allergy.txt']), s);
    except
      on E: Exception do
        Results.Text := e.Message;
    end;
end;

procedure TDBController_Renal.getPatientDirectives(
  aDFN: string;
  Results: TStringList);
var
  i: Integer;
  s: string;
begin
  if not Assigned(DBConnection) then
    begin
      getLocal(localName(lmPatientInfo, [aDFN, 'Directives.txt']), s);
      Results.Text := s;
    end
  else
    try
      Results.Clear;

      DBConnection.ExecRemoteProc(RPC_GET_VISTA_DATA, ['AD', aDFN], nil);

      s := Piece(DBConnection.ExecResults[0], '^', 1);
      if s = '-1' then
        Results.Add('No Advanced Directive')
          // changed to singular by vhaishandria 050506
      else
        for i := 1 to DBConnection.ExecResults.Count - 1 do
          Results.Add({'Advanced directives: ' + }DBConnection.ExecResults[i]);

      if SaveRPCResults then
        setLocal(localName(lmPatientInfo, [aDFN, 'Directives.txt']), s);
    except
      on E: Exception do
        Results.Text := e.Message;
    end;
end;

procedure TDBController_Renal.getPatientWarnings(
  aDFN: string;
  Results: TStringList);
var
  i: Integer;
  s: string;
begin
  if not Assigned(DBConnection) then
    begin
      getLocal(localName(lmPatientInfo, [aDFN, 'Warnings.txt']), s);
      Results.Text := s;
    end
  else
    try
      Results.Clear;
      DBConnection.ExecRemoteProc(RPC_GET_VISTA_DATA, ['CW', aDFN], nil); //??

      s := Piece(DBConnection.ExecResults[0], '^', 1);
      if s = '-1' then
        Results.Add('No Personal Warnings')
          // changed to singular by vhaishandria 050506
      else
        for i := 1 to DBConnection.ExecResults.Count - 1 do
          begin
            s := DBConnection.ExecResults[i];
            if pos('None', s) > 0 then
              Results.Add(s)
            else
              Results.Add(FMDateTimeStr(Piece(s, '^', 5)));
          end;
      if SaveRPCResults then
        setLocal(localName(lmPatientInfo, [aDFN, 'Warnings.txt']), s);
    except
      on E: Exception do
        Results.Text := e.Message;
    end;
end;

procedure TDBController_Renal.getPatientVAccination(
  aDFN: string;
  Results: TStringList);
var
  i: Integer;
  s: string;
begin
  if not Assigned(DBConnection) then
    begin
      getLocal(localName(lmPatientInfo, [aDFN, 'Vaccination.txt']), s);
      Results.Text := s;
    end
  else
    try
      Results.Clear;
      DBConnection.ExecRemoteProc(RPC_GET_VISTA_DATA, ['SHOTS', aDFN], nil);

      s := Piece(DBConnection.ExecResults[0], '^', 1);
      if s = '-1' then
        Results.Add('No Data Found')
      else
        begin
          if (DBConnection.ExecResults.Count = 2) and (Pos('No',
            DBConnection.ExecResults[1]) <> 1) then
            Results.Add('Vaccination:');
          for i := 1 to DBConnection.ExecResults.Count - 1 do
            Results.Add(DBConnection.ExecResults[i]);
        end;
      if SaveRPCResults then
        setLocal(localName(lmPatientInfo, [aDFN, 'Vaccination.txt']), s);
    except
      on E: Exception do
        Results.Text := e.Message;
    end;
end;

function TDBController_Renal.SetVitalsMeasurement(
  aDateTimeTaken: TDateTime;
  aPatientDFN: string;
  aVitalsType: TRenalVitalSign;
  aHospitalLocation: string;
  aUserDUZ: string;
  aValue: string): boolean;

  function QuickFMDateTime(aDT: TDateTime): string;
  var
    Y, M, D, hh, mm, ss, ms: Word;
  begin
    DecodeDateTime(aDT, Y, M, D, hh, mm, ss, ms);
    Y := Y - 1700; {Fileman specific}
    Result := FloatToStr(
      (Y * 10000) + (M * 100) + D +
      (hh * 0.01) + (mm * 0.0001) + (ss * 0.000001));
  end;

begin
  if not Assigned(DBConnection) then
    begin
      setLocal(localName(lmPatientInfo, [aPatientDFN, 'LatestVitals']), aValue);
      Result := True;
    end
  else
    try
      Result := DBConnection.ExecRemoteProc(
        RPC_GETSET_RENAL, [
        'VITALS',
          QuickFMDateTime(aDateTimeTaken), {P1}
        aPatientDFN, {P2}
        RENAL_VITAL_SIGN_NAME[aVitalsType], {P3}
        aValue, {P4}
        aHospitalLocation, {P5}
        aUserDUZ {+'*'+ QUALIFIER +';' } {P6}
        ],
          nil);
      if SaveRPCResults then
        setLocal(localName(lmPatientInfo, [aPatientDFN, 'LatestVitals']),
          aValue);
    except
      Result := False;
    end;
end;

////////////////////////////////////////////////////////////////////////////////

function TDBController_Renal.getNoteList(aStudyID: integer): Boolean;
var
  s: string;
begin
  if not Assigned(DBConnection) then
    begin
      Result := getLocalList(LocalName(lmTIUList, [aStudyID]), s);
      ResultInfo := s;
    end
  else
    try
      Result := DBConnection.ExecRemoteProc('MD TMDNOTE',
        ['NOTELIST', IntToStr(aStudyID)], '');
      ResultInfo := DBConnection.ExecResults.Text;
    except
      Result := False;
    end;
end;

function TDBController_Renal.getNote(aStudyID: integer; aNote: string): Boolean;
begin
  if not Assigned(DBConnection) then
    Result := getLocal(LocalName(lmTIU, [IntToStr(aStudyID), aNote]), ResultInfo)
  else
    try
      Result := DBConnection.ExecRemoteProc('MD TMDNOTE',
        ['VIEWTIU', IntToStr(aStudyID), aNote], '');
      ResultInfo := DBConnection.ExecResults.Text;
      if SaveRPCResults then
        setLocal(LocalName(lmTIU, [IntToStr(aStudyID), aNote]), ResultInfo);
    except
      Result := False;
    end;
  if SaveRPCResults then
    setLocal(LocalName(lmTIU, [IntToStr(aStudyID), aNote]), ResultInfo);
end;

////////////////////////////////////////////////////////////////////////////////

function TDBController_Renal.getLexiconData(aTarget, anOption: string; var
  aReturnList: string): Boolean;
begin
  if not Assigned(DBConnection) then
    Result := getLocal(LocalName(lmLexicon, [anOption]), aReturnList)
  else
    try
      DBConnection.execRemoteProc(RPC_LEXICON, [aTarget, anOption], nil);
      aReturnList := DBConnection.ExecResults.Text;
      Result := True;
      if SaveRPCResults then
        setLocal(LocalName(lmLexicon, [anOption]), aReturnList);
    except
      Result := False;
    end;
end;

////////////////////////////////////////////////////////////////////////////////

function TDBController_Renal.getClinicProcedures(
  aDFN, aClinic: string;
  var aReturnList: string): boolean;
begin
  if not Assigned(DBConnection) then
    Result := getLocal(LocalName(lmPatientProceduresInfo, [aDFN, aClinic]),
      aReturnList)
  else
    try
      DBConnection.execRemoteProc(RPC_CLINIC_INFO, ['PROC', aDFN, aClinic],
        nil);
      aReturnList := DBConnection.ExecResults.Text;
      Result := True;
      if SaveRPCResults then
        setLocal(LocalName(lmPatientProceduresInfo, [aDFN, aClinic]),
          aReturnList)
    except
      Result := False;
    end;
end;

function TDBController_Renal.getClinicDiagnosis(
  aDFN, aClinic: string;
  var aReturnList: string): boolean;
begin
  if not Assigned(DBConnection) then
    Result := getLocal(LocalName(lmPatientDiagnosisInfo, [aDFN, aClinic]),
      aReturnList)
  else
    try
      DBConnection.execRemoteProc(RPC_CLINIC_INFO, ['DIAG', aDFN, aClinic],
        nil);
      aReturnList := DBConnection.ExecResults.Text;
      Result := True;
      if SaveRPCResults then
        setLocal(LocalName(lmPatientDiagnosisInfo, [aDFN, aClinic]), aReturnList)
    except
      Result := False;
    end;
end;

function TDBController_Renal.getServiceConnections(
  aDFN, aClinic: string;
  var aReturnList: string): boolean;
begin
  if not Assigned(DBConnection) then
    Result := getLocal(LocalName(lmPatientServiceInfo, [aDFN, aClinic]),
      aReturnList)
  else
    try
      DBConnection.execRemoteProc(RPC_CLINIC_INFO, ['SCDISP', aDFN, aClinic],
        nil);
      aReturnList := DBConnection.ExecResults.Text;
      Result := True;
      if SaveRPCResults then
        setLocal(LocalName(lmPatientServiceInfo, [aDFN, aClinic]), aReturnList)
    except
      Result := False;
    end;
end;

function TDBController_Renal.getEncounterData(aStudy: string; var aReturnList:
  string): Boolean;
begin
  if not Assigned(DBConnection) then
    Result := getLocal(LocalName(lmStudyEncounter, [aStudy]), aReturnList)
  else
    try
      DBConnection.execRemoteProc(RPC_ENCOUNTER, [aStudy], nil);
      aReturnList := DBConnection.ExecResults.Text;
      Result := True;
      if SaveRPCResults then
        setLocal(LocalName(lmStudyEncounter, [aStudy]), aReturnList)
    except
      Result := False;
    end;
end;

function TDBController_Renal.setEncounterData(aStudy: string; aBillingData:
  TStringList; var aReturnList: string): Boolean;
begin
  aReturnList := '';
  if not Assigned(DBConnection) then
    Result := setLocal(LocalName(lmStudyEncounter, [aStudy]), aReturnList)
  else
    try
      RESULT := DBConnection.execRemoteProc(RPC_BILLING_WIDGET,
        ['PCE', aStudy, '', '', '', '', ''], aBillingData);
      aReturnList := DBConnection.ExecResults.Text;
      if SaveRPCResults then
        setLocal(LocalName(lmStudyEncounter, [aStudy]), aReturnList)
    except
      Result := False;
    end;
end;

////////////////////////////////////////////////////////////////////////////////

function TDBController_Renal.getLookupEntries(aFile, aTarget: string):
  TStringList;
var
  SL: TStringList;
  s: string;
begin
  SL := TStringList.Create;
  if not Assigned(DBConnection) then
    begin
      if aTarget = '' then
        aTarget := '_';
      getLocal(LOCAL_DIR + aFile + '\' + aTarget, s);
      SL.Text := s;
    end
  else
    begin
      DBConnection.execRemoteProc(RPC_MANAGER, ['LOOKUP', aFile + '^' +
        aTarget], nil);
      SL.Text := DBConnection.ExecResults.Text;
      if SaveRPCResults then
        begin
          if aTarget = '' then
            aTarget := '_';
          setLocal(LOCAL_DIR + aFile + '\' + aTarget, SL.Text);
        end;
    end;
  Result := SL;
end;

////////////////////////////////////////////////////////////////////////////////

function TDBController_Renal.getLocation(aStudy: string): string;
var
  s: string;
begin
  if not Assigned(DBConnection) then
    begin
      getLocal(LocalName(lmStudyLocation, [aStudy]), s);
      Result := s;
    end
  else
    try
      DBConnection.execRemoteProc(RPC_BILLING_WIDGET, ['GETLOC', aStudy], nil);
      Result := DBConnection.ExecResults.Text;
      if SaveRPCResults then
        setLocal(LocalName(lmStudyLocation, [aStudy]), s);
    except
      Result := '-1^RPC ' + RPC_BILLING_WIDGET + ' Failed' + #13#10 + 'Study:' +
        aStudy;
    end;
end;

function TDBController_Renal.setLocation(aStudy, aLocation: string): string;
begin
  if not Assigned(DBConnection) then
    begin
      setLocal(LocalName(lmStudyLocation, [aStudy]), aLocation);
      Result := '';
    end
  else
    try
      DBConnection.execRemoteProc(RPC_BILLING_WIDGET, ['SETLOC', aStudy,
        aLocation], nil);
      Result := DBConnection.ExecResults.Text;
      if SaveRPCResults then
        setLocal(LocalName(lmStudyLocation, [aStudy]), aLocation);
    except
      Result := '-1^RPC ' + RPC_BILLING_WIDGET + ' Failed' + #13#10 +
        'Study:' + aStudy + #13#10 + 'Location:' + aLocation;
    end;
end;

function TDBController_Renal.validatePCEData(aStudy, aData: string): string;
begin
  if not Assigned(DBConnection) then
    begin
      Result := '';
      Exit;
    end
  else
    try
      DBConnection.execRemoteProc(RPC_BILLING_WIDGET, ['CHECK', aStudy, aData],
        nil);
      Result := DBConnection.ExecResults.Text;
    except
      Result := '-1^RPC ' + RPC_BILLING_WIDGET + ' Failed' + #13#10 +
        'Study:' + aStudy + #13#10 + 'Data:' + aData;
    end;
end;

////////////////////////////////////////////////////////////////////////////////

function TDBController_Renal.getUserNameByIEN(anIEN: string): string;
begin
  if not Assigned(DBConnection) then
    Result := getItemLocal(LocalName(lmUserNamesList, []), anIEN)
  else
    try
      DBConnection.execRemoteProc(RPC_BILLING_WIDGET, ['NAME', '', anIEN], nil);
      Result := Piece(DBConnection.ExecResults.Text, '^', 2);
      if Result = '' then
        Result := anIEN;
      if SaveRPCResults then
        setItemLocal(LocalName(lmUserNamesList, []), Result, anIEN);
    except
      Result := '-1^RPC ' + RPC_BILLING_WIDGET + ' Failed' + #13#10 +
        'Option: NAME' + #13#10 + 'IEN:' + anIEN;
    end;
end;

function TDBController_Renal.getUserNameByIEN2(anIEN: string): string;
begin
  Result := '';
  if not Assigned(DBConnection) then
    Result := getItemLocal(LocalName(lmUserNamesList, []), anIEN)
  else
    try
      if DBConnection.execRemoteProc(RPC_MANAGER, ['GETDATA', '200^' + anIEN +
        '^.01'], nil) then
        Result := DBConnection.ExecResults.Text
      else
        Result := '<' + anIEN + '>'; // 061128 vhaishandria

      if SaveRPCResults then
        setItemLocal(LocalName(lmUserNamesList, []), Result, anIEN);
    except
    end;
end;

function TDBController_Renal.getStudyStatus(aStudy: string): string;
begin
  if not Assigned(DBConnection) then
    getLocal(LocalName(lmStudyStatus, [aStudy]), Result)
  else
    try
      DBConnection.execRemoteProc(RPC_BILLING_WIDGET, ['STAT', aStudy], nil);
      Result := DBConnection.ExecResults.Text;
      if SaveRPCResults then
        setLocal(LocalName(lmStudyStatus, [aStudy]), Result)
    except
      Result := '-1^RPC ' + RPC_BILLING_WIDGET + ' Failed' + #13#10 +
        'Option: STAT' + #13#10 + 'Study ID:' + aStudy;
    end;
end;

function TDBController_Renal.getSensitiveRecordAccessInfo(aDFN: string): string;
begin
  Result := '';
  if not Assigned(DBConnection) then
    getLocal(LocalName(lmSensitivAccess, [aDFN]), Result)
  else
    try
      if DBConnection.execRemoteProc(RPC_SENSITIVE_RA, [aDFN], nil) then
        Result := DBConnection.ExecResults.Text;
      if SaveRPCResults then
        setLocal(LocalName(lmSensitivAccess, [aDFN]), Result)
    except
    end;
end;

function TDBController_Renal.getSensitiveRecordBulletinInfo(aDFN: string):
  string;
begin
  Result := '';
  if not Assigned(DBConnection) then
    getLocal(LocalName(lmSensitivBulletin, [aDFN]), Result)
  else
    try
      if DBConnection.execRemoteProc(RPC_SENSITIVE_RB, [aDFN], nil) then
        Result := DBConnection.ExecResults.Text;
      if SaveRPCResults then
        setLocal(LocalName(lmSensitivBulletin, [aDFN]), Result)
    except
    end;
end;

////////////////////////////////////////////////////////////////////////////////

function TDBController_Renal.getQualifiersByVitalType(aTypeIEN: string): string;
begin
  Result := '';
  if not Assigned(DBConnection) then
    getLocal(LocalName(lmVitalsQualifiers, [aTypeIEN]), Result)
  else
    try
      if DBConnection.execRemoteProc(RPC_MANAGER, ['GETCATS', aTypeIEN], nil)
        then
        Result := DBConnection.ExecResults.Text;
      if SaveRPCResults then
        setLocal(LocalName(lmVitalsQualifiers, [aTypeIEN]), Result)
    except
    end;
end;

function TDBController_Renal.getQualifiersAll: string;
begin
  Result := '';
  if not Assigned(DBConnection) then
    getLocal(LocalName(lmVitalsQualifiersAll, []), Result)
  else
    try
      if DBConnection.execRemoteProc(RPC_MANAGER, ['GETLIST', '120.52'], nil)
        then
        Result := DBConnection.ExecResults.Text;
      if SaveRPCResults then
        setLocal(LocalName(lmVitalsQualifiersAll, []), Result)
    except
    end;
end;

function TDBController_Renal.getVitalTypesAll: string;
begin
  Result := '';
  if not Assigned(DBConnection) then
    getLocal(LocalName(lmVitalsTypes, []), Result)
  else
    try
      if DBConnection.execRemoteProc(RPC_MANAGER, ['GETLIST', '120.51'], nil)
        then
        Result := DBConnection.ExecResults.Text;
      if SaveRPCResults then
        setLocal(LocalName(lmVitalsTypes, []), Result)
    except
    end;
end;

function TDBController_Renal.getGatewayStatus: string;
var
  s: string;
begin
  Result := 'Down';
  if not Assigned(DBConnection) then
    Result := 'Gateway Status:' + 'Demo'
  else
    try
      if not DBConnection.execRemoteProc(RPC_GATEWAY, ['STATUS'], nil) then
        Exit;
      s := DBConnection.ExecResults.Text;
      if pos('Shutdown Flag^Yes', s) = 0 then
        Result := 'Up ';
    except
    end;
end;

function TDBController_Renal.getGatewayRunningStatus: string;
var
  s: string;
begin
  Result := 'Down';
  if not Assigned(DBConnection) then
    Result := 'Gateway Status:' + 'Demo Running'
  else
    try
      if not DBConnection.execRemoteProc(RPC_GATEWAY, ['RUNNING'], nil) then
        Exit;
      s := DBConnection.ExecResults.Text;
      if pos('1', s) = 1 then
        Result := 'Up ';
    except
    end;
end;

function TDBController_Renal.getGatewayStatusInfo: string;
var
  ss, c,
    s: string;
  i: Integer;
begin
  Result := 'OFF';
  if not Assigned(DBConnection) then
    Result := 'Gateway Status Info:' + #13#10#13#10 + 'Demo'
  else
    try
      if not DBConnection.execRemoteProc(RPC_GATEWAY, ['STATUS'], nil) then
        Exit;
      s := DBConnection.ExecResults.Text;
      ss := '';
      for i := 1 to Length(s) do
        begin
          c := copy(s, i, 1);
          if c = '^' then
            c := ': ';
          ss := ss + c;
        end;
      Result := ss;
    except
      Result := 'Error processing Gateway status';
    end;
end;

function TDBController_Renal.getServerCurrentDateTime: string;
begin
  Result := '';
  if not Assigned(DBConnection) then
    Exit;

  if DBConnection.execRemoteProc(RPC_GET_VISTA_DATA, ['TIME'], nil) then
    // dummy parameter
    Result := DBConnection.ExecResults.Text
  else
    Result := '';
  if pos(#13#10, Result) > 0 then
    Result := copy(Result, 1, Length(Result) - 2);
end;

function TDBController_Renal.getRPCResults(aParamList, aDelim: string): string;
var
  Params: array of string;
  i: Integer;
  sName,
    s: string;
  sl: TStringList;
begin
  Result := aParamList;
  sl := TStringList.Create;
  i := 1;
  repeat
    s := Piece(aParamList, aDelim, i);
    if s = '' then
      break;
    if pos(#13#10, s) > 0 then
      s := copy(s, 1, pos(#13#10, s) - 1);
    sl.Add(s);
    inc(i);
  until False;
  sName := '';
  if sl.Count > 0 then
    sName := sl[0];
  if sl.Count > 1 then
    begin
      SetLength(Params, sl.Count - 1);
      for i := 1 to sl.Count - 1 do
        begin
          Params[i - 1] := sl[i];
        end;
    end;
  sl.Free;

  if not Assigned(DBConnection) then
    Result := aParamList
  else if sName <> '' then
    begin
      if DBConnection.execRemoteProc(sName, Params, nil) then
        Result := DBConnection.ExecResults.Text
      else
        Result := aParamList;
    end;
end;

function TDBController_Renal.RegisterAccessToSensitiveData(anID,
  anExeNameAndVersion: string): string;
const
  RPC = 'GMV PTSELECT';
  OPTION = 'LOGSEC';
begin
  if not Assigned(DBConnection) then
    begin
      Result := '';
    end
  else
    try
      DBConnection.execRemoteProc(RPC,
        [OPTION, anID, 'RPCCALL^Clinical Procedure ' + anExeNameAndVersion],
        nil);
      Result := DBConnection.ExecResults.Text;
    except
      Result := '-1^RPC ' + RPC + ' Failed' + #13#10 + 'Patient' + anID;
    end;
end;

function TDBController_Renal.IsDataSensitive(aPatientID: string
  ; var sSecurityWarning: string): Boolean;
var
  s: string;
const
  RPC = 'GMV PTSELECT';
  OPTION = 'SELECT';

  SECURITY_HEADER3 = '$$MSGHDR^3';
  SECURITY_HEADER0 = '$$MSGHDR^0';
begin
  Result := False;
  if not Assigned(DBConnection) then
    begin
      s := getItemLocal(LocalName(lmSensitivItem, []), aPatientID);
      sSecurityWarning := 'Security Warning';
      Result := S = 'TRUE';
      Exit;
    end
  else
    try
      if DBConnection.execRemoteProc(RPC, [OPTION, aPatientID], nil) then
        begin
      s := DBConnection.ExecResults.Text;
      Result := (pos(SECURITY_HEADER3, s) <> 0) or (pos(SECURITY_HEADER0, s) <>
        0);
      if Result then
        begin
          sSecurityWarning :=
            copy(s, pos('This record is', s), Length(s));
          sSecurityWarning := piece(sSecurityWarning, '$$', 1);
        end;
        end
      else
        begin
          Result := True;
          sSecurityWarning := 'Failed to retreive the security information';
        end;
    except
    end;
end;

function TDBController_Renal.getOptionListXML(aUserDUZ: string = '.5'): string;
begin
  Result := '';
  if not Assigned(DBConnection) then
    Result := ''
  else
    try
      if DBConnection.execRemoteProc(RPC_GETSET_RENAL, ['ALLOPT', '', aUserDUZ],
        nil) then
        Result := DBConnection.ExecResults.Text;
    except
    end;
end;

function TDBController_Renal.setStudyStatus(aStudyID, aStatus: string): string;
(*
The RPC is MD TMDWIDGET.

D RPC^MDRPCOW(.RESULTS,OPTION,P1,P2)

Results  Return global = 1^Done
OPTION  STATUS
P1  CP Study number
P2- Status number

0:New
1:Submitted
2:Error
3:Complete
4:Ready to Complete
5:Pending Instrument Data
6:Cancelled
*)
begin
  Result := '';
  if not Assigned(DBConnection) then
    Result := ''
  else
    try
      if DBConnection.execRemoteProc(RPC_BILLING_WIDGET, ['STATUS', aStudyID,
        aStatus], nil) then
        Result := DBConnection.ExecResults.Text;
    except
    end;
end;

function TDBController_Renal.getPCEInfo(aPatientID, aStudyID: string): string;
begin
  if not Assigned(DBConnection) then
    Result := 'DB Connection is not assigned'
  else
    try
      (*
      MD TMDCIDC

      OPTION=PCEDISP
      DFN=PATIENT #
      MDSTUD=STUDY #

      >D RPC^MDRPCW(.RESULTS,"PCEDISP",400,3243)

      >D ^%G

      Global ^TMP("MDRPCW",$J -- NOTE: translation in effect
      ^TMP("MDRPCW",540310399,1)=Visit #:
                              2)=Encounter Date/Time: 11/21/06
                              3)=Visit # For Encounter Date:
                              4)=Previous Visit #: 6481 11/20/06
                              5)=Diagnosis: END STAGE RENAL DISEASE Primary
                              6)=CPT: PERITON DIALYSIS PLAN DOCD-1
                              7)=CPT: ESRD RELATED SERVICES, MONTH-7
      *)
      if DBConnection.execRemoteProc(RPC_CLINIC_INFO, ['PCEDISP', aPatientID,
        aStudyID], nil) then
        Result := DBConnection.ExecResults.Text
      else
        Result := 'RPC Failed.' + #13#10 + DBConnection.ExecResults.Text;
    except
    end;
end;

function TDBController_Renal.getUserOptionList(aUser: string): string;
var
  i: integer;
  SL: TStringList;
const
  kw = '<SETTING_NAME>';
begin
  Result := '';
  if not Assigned(DBConnection) then
    Exit;
  try
    if DBConnection.execRemoteProc(RPC_GETSET_RENAL, ['ALLOPT', '', '.5'], nil)
      then
      begin
        SL := TStringList.Create;
        SL.Text := DBConnection.ExecResults.Text;
        i := 0;
        while i < SL.Count do
          begin
            if pos(kw, sl[i]) <> 1 then
              sl.delete(i)
            else
              begin
                sl[i] := piece(piece(SL[i], kw, 2), '<', 1);
                if trim(sl[i]) <> '' then
                  inc(i)
                else
                  sl.delete(i);
              end;
          end;
        Result := SL.Text;
        SL.Free;
      end;
  except
  end;
end;

{
Here is the RPC call for CP Titles: MD TMDRECORDID.
Input: RESULTS  Return global array.

         OPTION: PTRLKUP
         DD: 702.01
         IENS: null
         FLD:.04
         DATA: What user typed in for query.

Sample run below:

D RPC^MDRPCOR(.RESULTS,"PTRLKUP",702.01,"",.04,"CP")

DEV>ZW RESULTS
RESULTS="^TMP(539038553)"
DEV>D ^%G
Global ^TMP($J -- NOTE: translation in effect
^TMP(539038553,0)=10
               1)=8925.1;1256^CP COLONOSCOPY
               2)=8925.1;1264^CP EKG
               3)=8925.1;746^CP ENDOSCOPY
               4)=8925.1;753^CP HEMO FALLS ASSESSMENT
               5)=8925.1;754^CP HEMO PROVIDER NOTE
               6)=8925.1;752^CP HEMODIALYSIS
               7)=8925.1;642^CP MUSE EKG
               8)=8925.1;609^CP PFT
               9)=8925.1;610^CP SBK PFT
              10)=8925.1;1271^CP_DEMO
}

function TDBController_Renal.getCPTIUTitles(aTarget: string): string;
var
  s: string;
  i: integer;
  SL: TStringList;
begin
  Result := '';
  if not Assigned(DBConnection) then
    begin
      getLocal(LocalName(lmTIUTitles, []), Result);
      Exit;
    end;
  try
    if DBConnection.execRemoteProc(RPC_TIU_TITLES, ['PTRLKUP', '702.01', '',
      '.04', aTarget {'CP'}], nil) then
      begin
        SL := TStringList.Create;
        SL.Text := DBConnection.ExecResults.Text;
        i := 0;
        while i < SL.Count do
          begin
            //            s := piece(SL[i],'^',2);
            s := SL[i];
            if trim(s) = '' then
              SL.Delete(i)
            else
              begin
                SL[i] := s;
                inc(i);
              end;
          end;
        Result := SL.Text;
        if SaveRPCResults then
          setLocal(LocalName(lmTIUTitles, []), Result);
        SL.Free;
      end;
  except
  end;
end;

(*
   Here is the new RPC in MD TMDCIDC.  Use the RPC to grey out the billing Update buttons when the Summary tab is opened.

RESULTS = Return result global
OPTION =TIMEMET
DFN = Patient IEN
MDSTUD = Study number

D RPC^MDRPCW(.RESULTS,"TIMEMET",305,3211)

DEV>D ^%G

Global ^TMP("MDRPCW",$J -- NOTE: translation in effect

^TMP("MDRPCW",539116008,0)=1^Appointment/Visit Date/Time have met.

Other Possible results are:

-1^No Visit Date/Time.
-1^No Visit.
0^Appointment/Visit Date/Time not met.

*)

function TDBController_Renal.getAppintmentInfo(aPatientID, aStudyID: string):
  string;
begin
  Result := '';
  if not Assigned(DBConnection) then
    begin
      Result := '1^Appointment/Visit Date/Time have met';
      Exit;
    end;
  try
    if DBConnection.execRemoteProc(RPC_CLINIC_INFO, ['TIMEMET', aPatientID,
      aStudyID], nil) then
      begin
        Result := DBConnection.ExecResults.Text;
        //        if SaveRPCResults then
        //          setLocal(LocalName(lmTIUTitles, []), Result);
      end;
  except
  end;
end;

function TDBController_Renal.getVersionInfo(aGUIVersion: string): string;
begin
  Result := '';
  if not Assigned(DBConnection) then
    begin
      Result := 'YES';
      Exit;
    end;
  try
    if DBConnection.execRemoteProc(RPC_VERSION_INFO, ['GUI', aGUIVersion], nil)
      then
      Result := DBConnection.ExecResults[0];
  except
  end;
end;

function TDBController_Renal.getSupportedVersion: string;
begin
  Result := '<Unknown>';
  if not Assigned(DBConnection) then
    begin
      //?
      Exit;
    end;
  try
    if DBConnection.execRemoteProc(RPC_VERSION_INFO, ['CURRENT', '*'], nil) then
      Result := DBConnection.ExecResults[0];
  except
  end;
end;

function TDBController_Renal.getPhysicianOrders(aPatient,anOrderIEN:String):String;
begin
  Result := '';
  try
    if DBConnection.execRemoteProc(RPC_PHYSICIAN_ORDERS, ['ORDERS', aPatient, anOrderIEN], nil) then
      Result := Result + DBConnection.ExecResults.Text;
  except
  end;
end;

function TDBController_Renal.setStudyUseStatus(aStudyID,aStatusID,aStatusBy,aStatusAt,aWS:String): String;
begin
  if not Assigned(DBConnection) then
    begin
      setLocalStatus(aStudyID,aStatusID,aStatusBy,aStatusAt,aWS);
      Result := 'OK';
      Exit;
    end;
  try
    DBConnection.execRemoteProc('MDK GET/SET RENAL DATA', ['UPDUSER',
      aStudyID,aStatusBy,aStatusAt,aWS,aStatusID], nil);
    Result := DBConnection.ExecResults[0];
  except
  end;

end;

function TDBController_Renal.getStudyUseStatus(sStudyID: String;var sStatus,sStatusBy,sStatusAt,sWS:String): Boolean;
begin
  if not Assigned(DBConnection) then
    begin
      Result := getLocalStatus(sStudyID,sStatus,sStatusBy,sStatusAt,sWS);
      Exit;
    end
  else
    Raise Exception.Create('Undefined getStudyUseStatus');
end;

function TDBController_Renal.getLockStatus(aStudyID: integer): String;
var
  s: String;
begin
  Result := 'Unknown';
  if not Assigned(DBConnection) then
    begin
      if getLocalLock(IntToStr(aStudyID),s) then
        Result := s;
    end
  else
    begin
      if LockStudy(aStudyID) then
        begin
          Result := 'ENABLED';
          UnLockStudy(aStudyID);
        end
      else
        Result := 'IN USE';
    end;
end;

function TDBController_Renal.getLockedStudies:String;
begin
  Result := '';
  if not Assigned(DBConnection) then
    Exit
  else
    try
      if DBConnection.execRemoteProc('MDK UTILITY', ['INUSE'], nil) then
        Result := DBConnection.ExecResults.Text;
    except
    end;
end;

function TDBController_Renal.getUserInfo:String;
begin
  Result := '';
  if not Assigned(DBConnection) then
    Exit
  else
    try
      if DBConnection.execRemoteProc('MDK GET/SET RENAL DATA', ['USERINF'], nil) then
        Result := DBConnection.ExecResults.Text;
    except
    end;
end;

function TDBController_Renal.getXPar(aName:String): String;
begin
  Result := '-1';
  if not Assigned(DBConnection) then
    Exit
  else
    try
      if DBConnection.execRemoteProc('MDK GET VISTA DATA', ['GETLD'], nil) then
        Result := DBConnection.ExecResults.Text;
    except
    end;
end;

function TDBController_Renal.setXPar(aName, aValue:String): String;
begin
  Result := '-1';
  if not Assigned(DBConnection) then
    Exit
  else
    try
      if DBConnection.execRemoteProc('MDK GET VISTA DATA', ['SETLD',aValue], nil) then
        Result := DBConnection.ExecResults.Text;
    except
    end;
end;


initialization
  VerifyWriteIsComplete := False;
end.
