unit dMDK_Data;
{
================================================================================
*
*       Package:      MDK - Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 2/22/08 11:14a $
*       Site Name:    Hines OIFO
*       Date Created: 7/29/05
*       Developers:   andriyevskiy                  
*       Description:  Contains the Application Client Data Set variables
*       Notes:
*
================================================================================
*       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/APP-HEMODIALYSIS/Forms/dMDK_Data.pas $
*
*       $ History: dMDK_Data.pas $
}

interface

uses
  SysUtils, Classes, DB, DBClient, Dialogs, Variants, DBCtrls, Controls,
  Graphics;

{===============================================================================
  dmData providers ClienDataSets and DataSources objects and links CDSs with DSs.
  CDSs structure supposed to be defined dynamically on dmData object creation
  CDSs are not supposed to be saved in the files or retrieve data from files
  CDSs wil be populated by another module
===============================================================================}
type
  TCDSCreate = procedure(aCDS: TClientDataSet);

  TdmData = class(TDataModule)
    dsEvents: TDataSource;
    cdsEvents: TClientDataSet;
    cdsSummary: TClientDataSet;
    dsSummary: TDataSource;
    cdsFlowSheet: TClientDataSet;
    dsFlowSheet: TDataSource;
    cdsAP: TClientDataSet;
    cdsAPHistory: TClientDataSet;
    dsAP: TDataSource;
    dsAPHistory: TDataSource;
    cdsVAInfection: TClientDataSet;
    dsVAInfection: TDataSource;
    cdsInfo: TClientDataSet;
    dsInfo: TDataSource;
    cdsStudyHistory: TClientDataSet;
    dsStudyHistory: TDataSource;
    cdsStudyHistoryPreBPStanding: TStringField;
    cdsStudyHistoryPreBPSeated: TStringField;
    cdsStudyHistoryPostBPStanding: TStringField;
    cdsStudyHistoryPostBPSeated: TStringField;
    cdsMedList: TClientDataSet;
    dsMedList: TDataSource;
    cdsSummaryNotes: TStringField;
    cdsInfoAdvDir: TStringField;
    cdsDict: TClientDataSet;
    dsDict: TDataSource;
    cdsSource: TClientDataSet;
    dsSource: TDataSource;
    cdsTimeline: TClientDataSet;
    dsTimeline: TDataSource;
    cdsStatus: TClientDataSet;
    cdsStatusPage: TStringField;
    cdsStatusStatusID: TIntegerField;
    cdsStatusComments: TStringField;
    dsStatus: TDataSource;
    dsStudyEvents: TDataSource;
    cdsStudyEvents: TClientDataSet;
    dsNotes: TDataSource;
    cdsNotes: TClientDataSet;
    cdsNotesAuthorName: TStringField;
    cdsSubmitHistory: TClientDataSet;
    dsSubmitHistory: TDataSource;
    cdsWorkflow: TClientDataSet;
    dsWorkflow: TDataSource;
    procedure DataModuleCreate(Sender: TObject);
    procedure cdsStudyHistoryCalcFields(DataSet: TDataSet);
    procedure DataModuleDestroy(Sender: TObject);
    procedure cdsInfoCalcFields(DataSet: TDataSet);
    procedure cdsMedListAfterInsert(DataSet: TDataSet);
    procedure cdsSummaryBeforeInsert(DataSet: TDataSet);
    procedure cdsNotesAfterInsert(DataSet: TDataSet);
    procedure cdsNotesAuthorNameGetText(Sender: TField; var Text: string;
      DisplayText: Boolean);
    procedure dsNotesDataChange(Sender: TObject; Field: TField);
    procedure dsStudyHistoryDataChange(Sender: TObject; Field: TField);
    procedure dsWorkflowDataChange(Sender: TObject; Field: TField);
    procedure dsInfoDataChange(Sender: TObject; Field: TField);
    procedure dsMedListDataChange(Sender: TObject; Field: TField);
  private
    { Private declarations }
    fIgnoreDebug: Boolean;
    fSessionID: string;
    fPatientID: string;
    fActor: string;
    fAction: string;
    fResult: string;
    fParams: string;
    sID: string;

    fID: Integer; // internal sequence number
    fStart: TDateTime;

    procedure CreateCDS(aCDS: TClientDataSet; aCDSCreator: TCDSCreate; aLabel:
      string);

    procedure CreateEvents;
    procedure CreateDictionary;
    procedure CreateFlowSheet;
    procedure CreateSummary;
    procedure CreateSummaryHistory;
    procedure CreateMedList;
    procedure CreateInfo;
    procedure CreateVA;
    procedure CreateAssessHistory;
    procedure CreateVAInfection;
    procedure CreateStatus;
    procedure CreateNotes;
    procedure CreateStudyEvents;
    procedure CreateSubmitHistory;

    function getNextID: string;
    function getDuration: TDateTime;

    procedure AddStudyEvent(anID, aKind, aComment, aSession: string;
      dateTime: TDateTime; aDuration: TDateTime = 0);
    procedure EventAdd(anID, aKind, aComment, aSession: string;
      dateTime: TDateTime; aDuration: TDateTime = 0; aLocal: Boolean = False);

  public
    bIgnore: Boolean;

    { Public declarations }
    property PatientID: string read fPatientID write fPatientID;
    property SessionID: string read fSessionID write fSessionID;
    property Actor: string read fActor write fActor;
    property Action: string read fAction write fAction;
    property Results: string read fResult write fResult;
    property Params: string read fParams write fParams;

    property NextID: string read getNextID;

    property Duration: TDateTime read getDuration;
    property Start: TDateTime write fStart;

    procedure RegisterEvent(aKind, aComment, aSession: string; aDuration:
      TDateTime = 0; aLocal: Boolean = False);
    procedure RecreateCDS(aDS: TClientDataSet);

    function getFieldData(aName: string): string;

    function getCDSByName(aName: string): TClientDataSet;

    function getMedicineAdministrated: string;
    function getFlowsheetReport: string;
    function getMedicineReport: string;
    function getScheduleReport: string;
    function getPainReport(PreTreatment: Boolean): string;
    function getFlowNotesReport: string;
    function getCommentsReport(aContextFilter: string): string;
    function getGeneralInfoReport: string;

    procedure ChangeCurrentNoteValidStatus;
    procedure SetCurrentNoteStatus(aStatus: string);
    procedure LockCurrentNote;
    procedure LockNotes;
    procedure DeleteCurrentNote;

    procedure PatchDictionary(aString: string);
    function CheckCDSIDField(aDS: TClientDataSet; aIDField, aIDValue: string; var
      badValue: string): Boolean;
    procedure CreateStudyEventsCDS(aDS: TClientDataSet);

    function ValidDataSetXML(XML: string): Boolean;

    function UpdateFlowsheetIndex(aDescending: Boolean): Boolean;
    function UpdateMedlistIndex(aDescending: Boolean): Boolean;
    procedure SaveErrorsLog;
  end;

var
  dmData: TdmData;

const
  fnEvents_ID = 'ID';
  fnEvents_WS = 'WS';
  fnEvents_Type = 'Kind';
  fnEvents_DateTime = 'DateTime';
  fnEvents_Duration = 'Duration';
  fnEvents_StudyID = 'StudyID';
  fnEvents_PatientID = 'PatientID';
  fnEvents_Comments = 'Comments';
  fnEvents_Action = 'Action';
  fnEvents_Actor = 'Actor';
  fnEvents_Params = 'Params';
  fnEvents_Result = 'Result';

  // Study List
  fnSL_StudyID = 'StudyID';
  fnSL_PatientID = 'PatientID';
  fnSL_Sensitive = 'SENSITIVE';
  fnSL_Status = 'Status';
  fnSL_StatusTime = 'StatusTime';
  fnSL_StatusBy = 'StatusBy';
  fnSL_StatusAt = 'StatusAt';
  fnSL_Comments = 'Comments';

  fnSL_LockStatus = 'LockStatus';
  fnSL_LockedBy = 'LockedStatus';
  fnSL_LastModified = 'LastModified';

  fnSL_CheckedIn = 'CheckedIn';
  fnSL_Name = 'Name';
  fnSL_DOB = 'DOB';
  fnSL_SSN = 'SSN';
  fnSL_Sex = 'Sex';

  fnSL_Source = 'Source';

  fnSL_UserID = 'UserID';
  fnSL_SessionID = 'SessionID';
  fnSL_Workstation = 'Workstation';
  fnSL_RegisteredAt = 'RegisteredAt';
  fnSL_Result = 'Result';
  fnSL_StatusOld = 'StatusOld';

  fnWL_Enabled = 'Enabled';
  fnWL_ItemType = 'ItemType';
  fnWL_ItemID = 'ItemID';
  fnWL_RegisteredBy = 'RegisteredBy';
  fnWL_RegisteredAt = 'RegisteredAt';
  fnWL_Comments = 'Comments';
  fnWL_StatusBy = 'StatusBy';
  fnWL_StatusAt = 'StatusAt';

  WL_USER = 'USER';
  WL_WORKSTATION = 'WORKSTATION';
  WL_STUDY = 'STUDY';
  WL_PATIENT = 'PATIENT';
  WL_STATUS_NEW = 'STATUS_NEW';
  WL_STATUS_OLD = 'STATUS_OLD';

const // Event Types
  etDebug = 'DEBUG';
  etError = 'ERROR';
  etWarning = 'WARNING';
  etInfo = 'INFO';

function ValidDateTime(aValue: string): Boolean;

implementation

uses
  uMDK_Common
  , uMDK_CDSCommon
  , uMDK_VADataSet
  , uMDK_Summary
  , uMDK_Info
  , uMDK_MedList
  , uMDK_Engine
  , uMDK_Patients
  , Windows
  , uMDK_Options
  , uROR_CommonUtils
  , uMDK_Notes
  , uMDK_SubmitHistory
  , uMDK_Dictionary
  , uMDK_Flowsheet, fMDK_Flowsheet, uMDK_Timeline,
  fMDK_Summary, fMDK_Pre, fMDK_Post, fMDK_RxLab, uMDK_Workflow, fMDK_Cover,
  fMDK_Main, fGMV_AboutDlg;

{$R *.dfm}
////////////////////////////////////////////////////////////////////////////////
const
  _CR_ = #13#10;

  flComments = 4000;

function ValidDateTime(aValue: string): Boolean;
var
  dt: TDateTime;
begin
  if (trim(aValue) = '') then
    Result := devOptions.bValue(donIgnoreBlankDateFields)
  else
    try
      dt := StrToDateTime(aValue);
      if dt > 0 then
        Result := True
      else
        Result := False;
    except
      on E: EConvertError do
        Result := False;
    end;
end;

////////////////////////////////////////////////////////////////////////////////

procedure TdmData.DataModuleCreate(Sender: TObject);
begin
  bIgnore := False;
  fID := 0;
  fIgnoreDebug := False;
  fSessionID := '';
  fPatientID := '';

  // Create Data dictionary

  CreateEvents;
  CreateDictionary;
  CreateSummary;
  CreateSummaryHistory;
  CreateMedList;
  CreateFlowSheet;
  CreateInfo;
  CreateVA;
  CreateAssessHistory;
  CreateVAInfection;
  CreateStatus;
  CreateNotes;
  CreateStudyEvents;
  CreateSubmitHistory;

  CreateCDS(cdsTimeline, CreateTimelineCDS, 'Timeline Table');

  setupWorkflowCDS(cdsWorkflow);
end;

////////////////////////////////////////////////////////////////////////////////

function TdmData.getDuration: TDateTime;
begin
  Result := 24 * 3600 * (Now - fStart);
end;

procedure TdmData.CreateStatus;
begin
  cdsStatus.Edit;
  cdsStatus.AppendRecord([pnCover, 0, '']);
  cdsStatus.Edit;
  cdsStatus.AppendRecord([pnRxLab, 0, '']);
  cdsStatus.Edit;
  cdsStatus.AppendRecord([pnPre, 0, '']);
  cdsStatus.Edit;
  cdsStatus.AppendRecord([pnAccess, 0, '']);
  cdsStatus.Edit;
  cdsStatus.AppendRecord([pnFlow, 0, '']);
  cdsStatus.Edit;
  cdsStatus.AppendRecord([pnPost, 0, '']);
  cdsStatus.Edit;
  cdsStatus.AppendRecord([pnSummary, 0, '']);
end;

procedure TdmData.RecreateCDS(aDS: TClientDataSet);
var
  aField: TFieldDef;

  procedure clearFields(aNum: Integer);
  begin
    try
      while aDS.FieldDefs.Count > aNum do
        begin
          aField := aDS.FieldDefs[aNum];
          aDS.FieldDefs.Delete(aNum);
        end;
    except
    end;
  end;

begin
  fStart := Now;

  if aDS = dmData.cdsEvents then
    CreateEvents;

  if aDS = dmData.cdsSummary then
    begin
      ClearFields(1);
      CreateSummary;
    end;

  if aDS = dmData.cdsStudyHistory then
    begin
      ClearFields(4);
      CreateSummaryHistory;
    end;

  if aDS = dmData.cdsSubmitHistory then
    CreateSubmitHistory;

  if aDS = dmData.cdsFlowSheet then
    CreateFlowSheet;

  if aDS = dmData.cdsInfo then
    begin
      ClearFields(1);
      CreateInfo;
    end;

  if aDS = dmData.cdsAP then
    CreateVA;

  if aDS = dmData.cdsAPHistory then
    CreateAssessHistory;

  if aDS = dmData.cdsVAInfection then
    CreateVAInfection;

  Action := 'RECREATE TABLE';
  Params := aDS.Name;
  RegisterEvent(etInfo, 'Table <' + aDS.Name + '> ReCreated', 'Startup',
    getDuration);
end;

///////////////////////////////////////////////////////////////////// FLOW SHEET

procedure TdmData.CreateCDS(aCDS: TClientDataSet; aCDSCreator: TCDSCreate;
  aLabel: string);
begin
  Start := Now;
  try
    aCDSCreator(aCDS);
    aCDS.Open;
    Action := 'Open ' + aLabel;
  except
    RegisterEvent(etError, 'Open ' + aLabel + ' Failed', '', Duration);
  end;
end;

procedure TdmData.CreateFlowSheet;
begin
  CreateCDS(cdsFlowsheet, CreateFlowsheetDS, 'Flowsheet Table');
end;

procedure TdmData.CreateSummary;
begin
  CreateCDS(cdsSummary, CreateSummaryDS, 'Summary Table');
end;

procedure TdmData.CreateSummaryHistory;
begin
  CreateCDS(cdsStudyHistory, CreateSummaryDS, 'Summary History Table');
end;

procedure TdmData.CreateSubmitHistory;
begin
  CreateCDS(cdsSubmitHistory, CreateSubmitHistoryDS, 'Submit Table');
end;

procedure TdmData.CreateMedList;
begin
  Start := Now;
  try
    CreateMedListCDS(cdsMedList, dsSummary);
    cdsMedList.Open;
    Action := 'Open Med List Table';
  except
    RegisterEvent(etError, 'Open Med List Table Failed', 'Startup', Duration);
  end;
  Action := '';
end;

procedure TdmData.CreateInfo;
begin
  CreateCDS(cdsInfo, CreateInfoDS, 'Info Table');
end;

procedure TdmData.CreateNotes;
begin
  Start := Now;
  try
    CreateNoteListCDS(cdsNotes);
    cdsNotes.Open;
    Action := 'Open Notes Table';
  except
    RegisterEvent(etError, 'Open Notes Table Failed', 'Startup', Duration);
  end;
  Action := '';
end;

procedure TdmData.CreateStudyEvents;
begin
  Start := Now;
  try
    Action := 'Create Study Events Table (Log)';
    CreateStudyEventsCDS(cdsStudyEvents);
    cdsStudyEvents.Open;
    Action := 'Open Study Events Table';
  except
    RegisterEvent(etError, 'Open Study Events Table Failed', 'Startup',
      Duration);
  end;
  Action := '';
end;

//////////////////////////////////////////////////////////////// VASCULAR ACCESS

procedure TdmData.CreateVA;
begin
  if not FileExists(cdsAP.FileName) then
    CreateVASiteDS(cdsAP);
  cdsAP.Open;
end;

procedure TdmData.CreateAssessHistory;
begin
  if not FileExists(cdsAPHistory.FileName) then
    CreateVAHistoryDS(cdsAPHistory, dsAP);
  cdsAPHistory.Open;
end;

procedure TdmData.CreateVAInfection;
begin
  if not FileExists(cdsVAInfection.FileName) then
    CreateVAInfectionDS(cdsVAInfection, dsAP);
  cdsVAInfection.Open;
end;

///////////////////////////////////////////////////////////////////////// EVENTS

procedure TdmData.CreateEvents;
begin
  NewStringField(cdsEvents, fnEvents_ID, 'ID', 9, 40);
  NewStringField(cdsEvents, fnEvents_DateTime, 'Time', 12, 22);
  NewStringField(cdsEvents, fnEvents_Type, 'Event Type', 8, 20);
  NewStringField(cdsEvents, fnEvents_StudyID, 'Study #', 6, 8);
  NewStringField(cdsEvents, fnEvents_PatientID, 'Patient DFN', 12, 12);
  NewStringField(cdsEvents, fnEvents_Action, 'Action', 16, 80);
  NewStringField(cdsEvents, fnEvents_Comments, 'Comments', 80, flComments);
  NewFloatField(cdsEvents, fnEvents_Duration, 'Duration', '#0.0###');
  NewStringField(cdsEvents, fnEvents_Actor, 'User', 8, 40);
  NewStringField(cdsEvents, fnEvents_Params, 'Params', 8, 80);
{$IFDEF AANTEST}cdsEvents.FieldList.Fields[cdsEvents.FieldList.Count - 1].Visible := True;
{$ELSE}cdsEvents.FieldList.Fields[cdsEvents.FieldList.Count - 1].Visible := False;
{$ENDIF}

  NewStringField(cdsEvents, fnEvents_Result, 'Result', 8, 20);
{$IFDEF AANTEST}cdsEvents.FieldList.Fields[cdsEvents.FieldList.Count - 1].Visible := True;
{$ELSE}cdsEvents.FieldList.Fields[cdsEvents.FieldList.Count - 1].Visible := False;
{$ENDIF}

  cdsEvents.IndexFieldNames := 'ID';
  cdsEvents.CreateDataSet;

  cdsEvents.Open;
end;

procedure TdmData.CreateStudyEventsCDS(aDS: TClientDataSet);
begin
  NewStringField(aDS, fnEvents_ID, 'ID', 9, 40);
  NewStringField(aDS, fnEvents_WS, 'Workstation', 9, 40);
  NewStringField(aDS, fnEvents_DateTime, 'Time', 12, 22);
  NewStringField(aDS, fnEvents_Type, 'Type', 8, 20);
  NewStringField(aDS, fnEvents_StudyID, 'Study', 6, 8);
  NewStringField(aDS, fnEvents_PatientID, 'Patient', 4, 8);
  NewStringField(aDS, fnEvents_Actor, 'Actor', 8, 40);
  NewStringField(aDS, fnEvents_Action, 'Action', 16, 80);
  NewStringField(aDS, fnEvents_Params, 'Params', 8, 80);
  NewStringField(aDS, fnEvents_Result, 'Result', 8, 20);
  NewFloatField(aDS, fnEvents_Duration, 'Duration', '#0.0###');
  NewStringField(aDS, fnEvents_Comments, 'Comments', 80, flComments);

  aDS.IndexFieldNames := 'ID';
  aDS.CreateDataSet;

  aDS.Open;
end;

procedure TdmData.CreateDictionary;
begin
  prepareDictionaryCDS(cdsDict);
end;

////////////////////////////////////////////////////////////////////////////////

function tdmData.getNextID: string;
var
  s: string;
begin
  try
    if Assigned(Engine) then
      s := FormatDateTime('yymmdd hh:mm:ss.zzz',
        Engine.getCurrentServerDateTime)
    else
      s := FormatDateTime('yymmdd hh:nn:ss.zzz', Now);
  except
    s := FormatDateTime('yymmdd hh:nn:ss.zzz', Now);
  end;

  if s = sID then
    begin
      inc(fID);
      Result := sID + ' ' + Format('%-3.3d', [fID]);
      ;
    end
  else
    begin
      sID := s;
      fID := 0;
      Result := sID;
    end;
end;

procedure TdmData.AddStudyEvent(anID, aKind, aComment, aSession: string;
  dateTime: TDateTime; aDuration: TDateTime = 0);
var
  pBuffer: PChar;
  Buffer: array[0..200] of char;
  iBuffer: DWord;
  s: string;
begin
  if not cdsStudyEvents.Active then
    try
      cdsStudyEvents.Open;
    except
      Exit;
    end;
  if PatientID = '' then
    Exit;

  pBuffer := @Buffer;
  iBuffer := 200;
  try
    GetComputerName(@Buffer, iBuffer);
    s := copy(pBuffer, 1, iBuffer);
  except
  end;

  try
    cdsStudyEvents.Insert;
    cdsStudyEvents.FieldByName(fnEvents_ID).AsString := anID;
    cdsStudyEvents.FieldByName(fnEvents_WS).AsString := s;

    cdsStudyEvents.FieldByName(fnEvents_Type).AsString := aKind;
    cdsStudyEvents.FieldByName(fnEvents_StudyID).AsString := aSession;
    cdsStudyEvents.FieldByName(fnEvents_PatientID).AsString := PatientID;

    cdsStudyEvents.FieldByName(fnEvents_Comments).AsString := aComment;
    cdsStudyEvents.FieldByName(fnEvents_Duration).AsFloat := aDuration;
    cdsStudyEvents.FieldByName(fnEvents_DateTime).AsString :=
      FormatDateTime(gMDK_DateTimeFMT, DateTime);

    cdsStudyEvents.FieldByName(fnEvents_Action).AsString := Action;
    cdsStudyEvents.FieldByName(fnEvents_Actor).AsString := Actor;
    cdsStudyEvents.FieldByName(fnEvents_Result).AsString := Results;
    cdsStudyEvents.FieldByName(fnEvents_Params).AsString := Params;

    cdsStudyEvents.Post;
  except
  end;
end;

procedure TdmData.EventAdd(anID, aKind, aComment, aSession: string;
  dateTime: TDateTime; aDuration: TDateTime = 0; aLocal: Boolean = False);
begin
  try
    cdsEvents.Insert;
    cdsEvents.FieldByName(fnEvents_ID).AsString := anID;
    cdsEvents.FieldByName(fnEvents_Type).AsString := aKind;
    cdsEvents.FieldByName(fnEvents_StudyID).AsString := aSession;
    cdsEvents.FieldByName(fnEvents_PatientID).AsString := PatientID;

    cdsEvents.FieldByName(fnEvents_Comments).AsString := aComment;
    cdsEvents.FieldByName(fnEvents_Duration).AsFloat := aDuration;
    cdsEvents.FieldByName(fnEvents_DateTime).AsDateTime := DateTime;

    cdsEvents.FieldByName(fnEvents_Action).AsString := Action;
    cdsEvents.FieldByName(fnEvents_Actor).AsString := Actor;
    cdsEvents.FieldByName(fnEvents_Result).AsString := Results;
    cdsEvents.FieldByName(fnEvents_Params).AsString := Params;

    cdsEvents.Post;
  except
  end;

  if aLocal then
    AddStudyEvent(anID, aKind, aComment, aSession, dateTime, aDuration);
end;

procedure TdmData.RegisterEvent(aKind, aComment, aSession: string; aDuration:
  TDateTime = 0; aLocal: Boolean = False);
var
  iProcessed: Integer;
  sLabel,
    sID: string;
  b: Boolean;
begin
  if fIgnoreDebug and (aKind = etDebug) then
    Exit;
  iProcessed := 1;
  sLabel := aSession;
  if aSession = 'Startup' then
    begin
      Actor := aSession;
      sLabel := '';
    end;
  repeat
    sID := NextID;

    b := aLocal and devOptions.bValue(donRegisterStudyEvents);

    EventAdd(sID, aKind, copy(aComment, iProcessed, flComments),
      sLabel,
      now, // Engine.getCurrentServerDateTime,
      aDuration,
      b);
    sLabel := cToBeContinued;
    inc(iProcessed, flComments);
  until iProcessed >= Length(aComment);
end;

////////////////////////////////////////////////////////////////////////////////

procedure TdmData.cdsStudyHistoryCalcFields(DataSet: TDataSet);

  function CalculateBP(aSys, aDias: Real): string;
  var
    s, ss: string;
  begin
    Result := '---';
    s := '---';
    ss := '---';
    try
      s := Format('%3.0f', [aSys]);
      while (s <> '') and (pos(' ', s) = 1) do
        s := copy(s, 2, Length(s));
      if s = '' then
        s := '-';
      ss := Format('%3.0f', [aDias]);
      while (ss <> '') and (pos(' ', ss) = 1) do
        ss := copy(ss, 2, Length(ss));

      if ss = '' then
        ss := '-';
      Result := s + '/' + ss;
    except
      Result := '***';
    end;
  end;

begin
  try
    cdsStudyHistoryPreBPSeated.Value :=
      CalculateBP(cdsStudyHistory.FieldValues[fnS_PreBPSysSitting],
      cdsStudyHistory.FieldValues[fnS_PreBPDiasSitting]);
    cdsStudyHistoryPreBPStanding.Value :=
      CalculateBP(cdsStudyHistory.FieldValues[fnS_PreBPSysStanding],
      cdsStudyHistory.FieldValues[fnS_PreBPDiasStanding]);
    cdsStudyHistoryPostBPSeated.Value :=
      CalculateBP(cdsStudyHistory.FieldValues[fnS_PostBPSysSitting],
      cdsStudyHistory.FieldValues[fnS_PostBPDiasSitting]);
    cdsStudyHistoryPostBPStanding.Value :=
      CalculateBP(cdsStudyHistory.FieldValues[fnS_PostBPSysStanding],
      cdsStudyHistory.FieldValues[fnS_PostBPDiasStanding]);
  except
  end;
end;

procedure TdmData.DataModuleDestroy(Sender: TObject);
begin
  cdsSummary.Close;
  cdsInfo.Close;
  cdsFlowSheet.Close;
  cdsStudyHistory.Close;
  cdsAP.Close;
  cdsAPHistory.Close;
  cdsVAInfection.Close;
  cdsMedList.Close;

  try
    if Assigned(cdsEvents) then
      cdsEvents.Close;
  except
  end;

  cdsEvents.Close;
  cdsDict.Close;
  cdsStatus.Close;
  cdsNotes.Close;
  cdsStudyEvents.Close;
  cdsSubmitHistory.CLose;
end;

procedure TdmData.cdsInfoCalcFields(DataSet: TDataSet);
begin
  try
    if cdsInfo.FieldByName(fnI_AD).AsString = 'True' then
      cdsInfoAdvDir.Value := 'Yes'
    else if cdsInfo.FieldByName(fnI_AD).AsString = 'False' then
      cdsInfoAdvDir.Value := 'No'
    else
      cdsInfoAdvDir.Value := '';
  except
    cdsInfoAdvDir.Value := '';
  end;
end;

////////////////////////////////////////////////////////////////////////////////

procedure TdmData.cdsMedListAfterInsert(DataSet: TDataSet);
begin
  DataSet.FieldByName(fnML_ID).AsString := dmData.NextID;
  DataSet.FieldByName(fnML_PatientID).AsString := Engine.CurrentPatient.ID;
  DataSet.FieldByName(fnML_StudyID).AsString := Engine.CurrentPatient.SessionID;
end;

////////////////////////////////////////////////////////////////////////////////

function TdmData.getFieldData(aName: string): string;
var
  aDataType,
    aFormat,
    aTable,
    aColumn: string;

  function getCDSFieldValue(aCDS: TClientDataSet; aField: string): string;
  var
    s: string;
  begin
    try
      if (aDataType = '11') and (aFormat <> '') then // Date and Time
        s := FormatDateTime(aFormat, aCDS.FieldByName(aField).AsDateTime)
      else if (aDataType = '5') then // boolean
        begin
          s := aCDS.FieldByName(aField).AsString;
          if s = 'True' then
            s := 'Yes';
          if s = 'False' then
            s := 'No';

        end
      else
        s := aCDS.FieldByName(aField).AsString;
    except
      s := '';
    end;
    if s = '' then
      s := Preferences.sValue(onBlanksPlaceholder);
    Result := s;
  end;

begin
  if cdsDict.Locate(fnDict_Tag, VarArrayOf([aName]), []) then
    begin
      aTable := cdsDict.FieldByName(fnDict_Table).AsString;
      aColumn := cdsDict.FieldByName(fnDict_Column).AsString;
      aFormat := cdsDict.FieldByName(fnDict_Format).AsString;
      aDataType := cdsDict.FieldByName(fnDict_Type).AsString;
      //      anEditMask := cdsDict.FieldByName(fnDict_EditMask).AsString;
      //      TheUnit := cdsDict.FieldByName(fnDict_Units).AsString;
    end;

  if aTable = 'cdsSummary' then
    Result := getCDSFieldValue(cdsSummary, aColumn)
  else if aTable = 'cdsInfo' then
    Result := getCDSFieldValue(cdsInfo, aColumn)
  else if aTable = 'cdsFlowSheet' then
    Result := getCDSFieldValue(cdsFlowSheet, aColumn)
  else
    Result := '<' + aName + '>';
end;

////////////////////////////////////////////////////////////////////////////////

function TdmData.getCDSByName(aName: string): TClientDataSet;
var
  i: Integer;
begin
  Result := nil;
  if aName = '' then
    Exit;
  Result := nil;
  for i := 0 to ComponentCount - 1 do
    if (Components[i] is TClientDataSet) and
      (TClientDataSet(Components[i]).Name = aName) then
      begin
        Result := TClientDataSet(Components[i]);
        break;
      end;
end;

procedure TdmData.cdsSummaryBeforeInsert(DataSet: TDataSet);
begin
  if not ValidDateTime(DataSet.FieldByName(fnS_rxTxLength).AsString) then
    begin
      ShowMessage('Invalid format' +
        DataSet.FieldByName(fnS_rxTxLength).AsString);
    end;
end;

function TdmData.getFlowsheetReport: string;
var
  iLength: Integer;
  BM: TBookmark;
  zz, z: string;
  ss, s: string;
  DT: TDateTime;
  bb,
    cc: string;

  procedure ProcessRecord;
  var
    ll: string;
  begin
    cc := '|';
    bb := ' ';
    if cdsFlowSheet.FieldByName(fnFS_Status).AsString <> fsValid then
      begin
        ll := ' The next entry was marked as "INVALID" ';
        //        ll := ' The next entry was marked as "'+cdsFlowSheet.FieldByName(fnFS_Status).AsString+'" ';
        while length(ll) < iLength - 6 do
          ll := '.' + ll;
        ss := ss + ll + #13#10;
        zz := zz + ll + #13#10;
      end;

    DT := cdsFlowSheet.FieldByName(fnFS_Time).AsDateTime;
    s := FormatDateTime('hh:nn:ss ', DT);
    if s = '' then
      s := '         ';
    ss := ss + s;
    zz := zz + s;
{$IFDEF FLOWEVENTS}
    z := cdsFlowSheet.FieldByName(fnFS_Event).AsString;
    if z <> '' then
      begin
        ll := CC + Format('%-60s', [cdsFlowSheet.FieldByName(fnFS_Event).AsString
          + ' ' +
            cdsFlowSheet.FieldByName(fnFS_Comments).AsString]) + #13#10;

        ss := ss + ll;

        if Preferences.bValue(onShowFlowsheetEventsCopies) then
          zz := zz + ll
        else
          zz := zz + #13#10;
      end
    else
      begin
{$ENDIF}
        ss := ss + CC + Format('%7.7s',
          [cdsFlowSheet.FieldByName(fnFS_BP).AsString]);

        ss := ss + CC + Format('%3.3s',
          [cdsFlowSheet.FieldByName(fnFS_HR).AsString]);
        ss := ss + CC + Format('%5.5s',
          [cdsFlowSheet.FieldByName(fnFS_BFR).AsString]);
        ss := ss + CC + Format('%5.5s',
          [cdsFlowSheet.FieldByName(fnFS_DFR).AsString]);
        ss := ss + CC + Format('%6.6s',
          [cdsFlowSheet.FieldByName(fnFS_AP).AsString]);
        ss := ss + CC + Format('%6.6s',
          [cdsFlowSheet.FieldByName(fnFS_VP).AsString]);
        ss := ss + CC + Format('%4.4s',
          [cdsFlowSheet.FieldByName(fnFS_TMP).AsString]);
        ss := ss + CC + Format('%6.6s',
          [cdsFlowSheet.FieldByName(fnFS_UFR).AsString]);
        ss := ss + CC + Format('%6.6s',
          [cdsFlowSheet.FieldByName(fnFS_Fluid).AsString]);
        ss := ss + CC + Format('%6.6s',
          [cdsFlowSheet.FieldByName(fnFS_HEP).AsString]);

        zz := zz + CC + Format('%6.6s ',
          [cdsFlowSheet.FieldByName(fnFS_MAP).AsString]);
        zz := zz + CC + Format('%6.6s ',
          [cdsFlowSheet.FieldByName(fnFS_COND).AsString]);
        zz := zz + CC + Format('%6.6s ',
          [cdsFlowSheet.FieldByName(fnFS_BIC).AsString]);
        zz := zz + CC + Format('%6.6s ',
          [cdsFlowSheet.FieldByName(fnFS_DIAL).AsString]);

        zz := zz + CC + Format('%6.6s',
          [cdsFlowSheet.FieldByName(fnFS_IONI).AsString]);
        zz := zz + CC + Format('%6.6s',
          [cdsFlowSheet.FieldByName(fnFS_KTV).AsString]);
        zz := zz + #13#10;
        ss := ss + #13#10;
{$IFDEF FLOWEVENTS}
      end;
{$ENDIF}
  end;

begin
  iLength := devOptions.iValue(donReportLineLength);
  ss := '';
  zz := '';
  Result := '';
  if not cdsFlowsheet.Active then
    begin
      Result := 'Flowsheet is not availabe';
      Exit;
    end;

  BM := cdsFlowSheet.GetBookmark;

  ss := ss +
    '   Time  |   BP  | HR| BFR | DFR |  AP  |  VP  | TMP|  UFR | Fluid| HEP  ' +
    #13#10;
  ss := ss +
    '-------------------------------------------------------------------------' +
    #13#10;

  zz := zz + '   Time  |  MAP  | COND  |  BIC  | DIAL  | IONI  |  Kt/V ' +
    #13#10;
  zz := zz + '---------------------------------------------------------' +
    #13#10;

  if not Preferences.bValue(onReverseFlowsheetOrder) then
    begin
      cdsFlowSheet.First;
      if not cdsFlowsheet.Eof then
        begin
          while not cdsFlowSheet.Eof do
            begin
              ProcessRecord;
              cdsFlowSheet.Next;
            end;
        end;
    end
  else
    begin
      cdsFlowSheet.Last;
      while not cdsFlowSheet.Bof do
        begin
          ProcessRecord;
          cdsFlowSheet.Prior;
        end;
    end;
  cdsFlowSheet.GotoBookmark(BM);
  cdsFlowSheet.FreeBookmark(BM);

  if cdsFlowsheet.RecordCount > 0 then
    Result := ss + #13#10#13#10 + zz
  else
    Result := 'Flowsheet contains no data' + #13#10;
end;

function TdmData.getMedicineAdministrated: string;
var
  BM: TBookmark;
const
  T = char(VK_TAB);
  TT = '   ';

begin
  BM := cdsMedList.GetBookmark;
  Result := '';
  cdsMedList.First;
  if cdsMedList.Eof then
    Result := 'No Medication Administered'
  else
    begin
      while not cdsMedList.Eof do
        begin
          Result := Result + FormatDateTime(gMDK_DateFMT,
            cdsMedList.FieldByName(fnML_Date).AsDateTime) + '  ';
          Result := Result + FormatDateTime(gMDK_TimeFMT,
            cdsMedList.FieldByName(fnML_Time).AsDateTime) + #13#10;
          Result := Result + T + 'Medication: ' +
            cdsMedList.FieldByName(fnML_Medication).AsString + #13#10;
          Result := Result + T + 'Dose:       ' +
            cdsMedList.FieldByName(fnML_Dose).AsString + #13#10;
          Result := Result + T + 'Rout:       ' +
            cdsMedList.FieldByName(fnML_Rout).AsString + #13#10;
          Result := Result + T + 'Given By:   ' +
            cdsMedList.FieldByName(fnML_GivenBy).AsString + #13#10;
          Result := Result + T + 'Reason:     ' +
            cdsMedList.FieldByName(fnML_Reason).AsString + #13#10;
          Result := Result + T + 'Outcome:    ' +
            cdsMedList.FieldByName(fnML_Outcome).AsString + #13#10;

          if (cdsMedList.FieldByName(fnML_Status).AsString <> 'V') and
            (cdsMedList.FieldByName(fnML_Status).AsString <> 'VALID') then
            begin
              Result := Result + T + 'Status:     ' +
                cdsMedList.FieldByName(fnML_Status).AsString + #13#10;
              Result := Result + T + 'Set By:     ' +
                cdsMedList.FieldByName(fnML_StatusBy).AsString + #13#10;
              Result := Result + T + 'Set On:     ' +
                cdsMedList.FieldByName(fnML_StatusDate).AsString + #13#10;
            end;

          Result := Result + #13#10;
          cdsMedList.Next;
        end;
    end;
  cdsMedList.GotoBookmark(BM);
  cdsMedList.FreeBookmark(BM);
end;

function TdmData.getMedicineReport: string;
var
  bFIlter: Boolean;
  BM: TBookmark;
  sItem,
//  sMargin,
  ss: string;
  iLen: Integer;
  sl: TStringList;

  slMedStatuses: TStringList;

  function getMedListStatus(aCode: string): string;
  begin
    Result := slMedStatuses.Values[aCode];
    if Result = '' then
      Result := acode;
  end;

  function getUserName(aCode: string): string;
  begin
    Result := Engine.getUserNameByID(aCode);
    if Result = '' then
      Result := aCode;
  end;

  function formatLines(aTopMargin, aMargin, aValue: string): string;
  var
    i: integer;
    s: string;
  begin
    Result := '';
    sl.Text := aValue;
    for i := 0 to sl.Count - 1 do
      begin
        s := sl[i];
        if i = 0 then
          s := aTopMargin + s
        else
          s := aMargin + s;

        repeat
          Result := Result + copy(s, 1, iLen) + _CR_;
          s := copy(s, iLen + 1, Length(s));
          if s = '' then
            break;
        until false;
      end;
  end;

  function StatusHeader:String;
  var
    sStatus:String;
  begin
    Result := '';
    with cdsMedList do
      begin
        sStatus := FieldByName(fnML_Status).AsString;
        Result := Format('%-12.12s',[sStatus]) +
          Format('%s', [
          'Set By ' + getUserName(FieldByName(fnML_StatusBy).AsString) +
          ' on ' + FieldByName(fnML_StatusDate).AsString]) + _CR_;
        Result := Result+'            Record ID   '+FieldByName(fnML_ID).AsString+_CR_;
        Result := Result+'            Replaces    '+FieldByName(fnML_ParentID).AsString+_CR_;
      end;
  end;

  function ItemMargin:String;
  var
    sStatus:String;
  begin
    Result := '';
    sStatus := cdsMedList.FieldByName(fnML_Status).AsString;
    if sStatus = stML_DELETED then
      Result := '            ';
    if sStatus = stML_REPLACED then
      Result := '            ';
  end;

begin
  ss := '';

  iLen := devOptions.iValue(donReportLineLength);
  Result := ''; //'Flowsheet report';
  if not cdsMedList.Active then
    Exit;
  slMedStatuses := TStringList.Create;
  slMedStatuses.Text := Engine.getSystemOptionByName(onMedicationStatuses);

  sl := TStringList.Create;
  BM := cdsMedList.GetBookmark;
  bFilter := cdsMedList.Filtered;
  cdsMedList.Filtered := False;
  cdsMedList.First;
  if not cdsMedList.EOF then
    begin
      with cdsMedList do
        begin
          while not Eof do
            begin
              sItem := '';
              sItem := sItem + FormatLines('','',StatusHeader);

              sItem := sItem + FormatLines('','',
                FormatDateTime(gMDK_DateFMT,
                cdsMedList.FieldByName(fnML_Date).AsDateTime)+
                Format('  %s',
                [FieldByName(fnML_Medication).AsString + ',  ' +
                FieldByName(fnML_Dose).AsString + ' (' +
                  FieldByName(fnML_Units).AsString + ') ' +
                  FieldByName(fnML_Rout).AsString
                  ]));

              sItem := sItem + FormatLines('','',FormatDateTime('hh:nn:ss    ',
                FieldByName(fnML_Time).AsDateTime)+
                 'given by ' + getUserName(FieldByName(fnML_GivenBy).AsString));

              sItem := sItem + FormatLines('Reason:     ',
                '              ', FieldByName(fnML_Reason).AsString);
              sItem := sItem + FormatLines('Outcome:    ',
                '              ', FieldByName(fnML_Outcome).AsString);
              sItem := sItem + _CR_;

              ss := ss + FormatLines(ItemMargin,ItemMargin,sItem);
              Next;
            end;
        end;
    end;
  sl.Free;
  cdsMedList.Filtered := bFilter;
  cdsMedList.GotoBookmark(BM);
  cdsMedList.FreeBookmark(BM);
  slMedStatuses.Free;
  Result := ss;
end;

function TdmData.getScheduleReport: string;
begin
  Result := '';
  if cdsInfo.FieldByName(fnI_Mon).AsBoolean then
    Result := Result + 'Monday, ';
  if cdsInfo.FieldByName(fnI_Tue).AsBoolean then
    Result := Result + 'Tuesday, ';
  if cdsInfo.FieldByName(fnI_Wed).AsBoolean then
    Result := Result + 'Wednesday, ';
  if cdsInfo.FieldByName(fnI_Thu).AsBoolean then
    Result := Result + 'Thursday, ';
  if cdsInfo.FieldByName(fnI_Fri).AsBoolean then
    Result := Result + 'Friday, ';
  if cdsInfo.FieldByName(fnI_Sat).AsBoolean then
    Result := Result + 'Saturday, ';
  if cdsInfo.FieldByName(fnI_Sun).AsBoolean then
    Result := Result + 'Sunday, ';

  if Result = '' then
    Result := 'No schedule data'
  else
    Result := copy(Result, 1, Length(Result) - 2);
end;

function TdmData.getPainReport(PreTreatment: Boolean): string;
var
  s: string;
  i: Integer;

  function FieldValue(aName: string): string;
  begin
    if cdsSummary.FieldByName(aName).AsString = '' then
      result := Preferences.sValue(onBlanksPlaceholder)
    else
      Result := cdsSummary.FieldByName(aName).AsString;
  end;

begin
  Result := '';
  if PreTreatment then
    begin
      s := 'PRE-TREATMENT PAIN ASSESSMENT' + _CR_;
      if cdsSummary.FieldByName(fnS_PrePain).AsString = '' then
        begin
          s := s + 'Pain evaluation was not performed' + _CR_;
        end
      else
        begin
          i := cdsSummary.FieldByName(fnS_PrePain).AsInteger;
          if i = 0 then
            begin
              s := s + 'No pain (pain level - 0)' + _CR_;
              Result := s;
              Exit;
            end;
          s := s + 'How bad is the pain? ' + FieldValue(fnS_PrePain) + _CR_;

          if Preferences.bValue(onConsiderPainTolerance) then
            begin
              s := s + 'Can patient tolerate pain? ' +
                FieldValue(fnS_PrePainTolerance) + _CR_;
              if Uppercase(FieldValue(fnS_PrePainTolerance)) = 'YES' then
                begin
                  Result := s;
                  Exit;
                end;
            end;

          s := s + 'Where is The Pain Located? ' +
            FieldValue(fnS_PrePainLocation) +
            _CR_;
          s := s + 'Pain Form? ' + FieldValue(fnS_PrePainAcute) + _CR_;
          s := s + 'Pain Treatment? ' + FieldValue(fnS_PrePainTreatment) + _CR_;
          if i < StrToInt(Preferences.sValue(onPainLevel)) then
            begin
              Result := s;
              Exit;
            end;
          S := S + _CR_;
          S := S + 'Detailed Pain Assessment ' + _CR_;
          //      S := S + '1. Location of pain: ' + FieldValue(fnS_PrePainIASP) + _CR_;
          //      S := S + '2. Intensity: ' + FieldValue(fnS_PrePainIntensity) + _CR_;
          S := S + '1. Frequency text: ' + FieldValue(fnS_PrePainFrequencyText)
            +
            _CR_;
          S := S + '2. Onset: ' + FieldValue(fnS_PrePainOnset) + _CR_;
          S := S + '3. Duration of episodes: ' + FieldValue(fnS_PrePainDuration)
            +
            _CR_;
          S := S + '4. Quality of pain: ' + FieldValue(fnS_PrePainQuality) +
            _CR_;

          S := S + 'The patient indicated that:' + _CR_;
          S := S + '1. The impact of the pain was: ' +
            FieldValue(fnS_PrePainImpact)
            + _CR_;
          S := S + '2. The following makes the pain worse: ' +
            FieldValue(fnS_PrePainWorse) + _CR_;
          S := S + '3. The following relieves the pain: ' +
            FieldValue(fnS_PrePainRelieve) + _CR_;
          S := S + '4. The medications that help the pain are: ' +
            FieldValue(fnS_PrePainHelpMed) + _CR_;
          S := S + '5. Other treatments used for pain are: ' +
            FieldValue(fnS_PrePainOtherTreatments) + _CR_;
          S := S + '6. Other comments: ' + FieldValue(fnS_PrePainOtherComments)
            +
            _CR_;
        end;
    end
  else
    begin
      s := 'POST-TREATMENT PAIN ASSESSMENT' + _CR_;
      if cdsSummary.FieldByName(fnS_PostPain).AsString = '' then
        begin
          s := s + 'Pain evaluation was not performed' + _CR_;
        end
      else
        begin
          i := cdsSummary.FieldByName(fnS_PostPain).AsInteger;
          //      s := 'POST-PAIN ASSESSMENT' + _CR_;
          if i = 0 then
            begin
              s := s + 'No pain (pain level - 0)' + _CR_;
              Result := s;
              Exit;
            end;
          s := s + 'How bad is the pain? ' + FieldValue(fnS_PostPain) + _CR_;
          if Preferences.bValue(onConsiderPainTolerance) then
            begin
              s := s + 'Can patient tolerate pain? ' +
                FieldValue(fnS_PostPainTolerance) + _CR_;
              if Uppercase(FieldValue(fnS_PrePainTolerance)) = 'YES' then
                begin
                  Result := s;
                  Exit;
                end;
            end;
          s := s + 'Where is The Pain Located? ' +
            FieldValue(fnS_PostPainLocation)
            + _CR_;
          s := s + 'Pain Form? ' + FieldValue(fnS_PostPainAcute) + _CR_;
          s := s + 'Pain Treatment? ' + FieldValue(fnS_PostPainTreatment) +
            _CR_;
          if i < StrToInt(Preferences.sValue(onPainLevel)) then
            begin
              Result := s;
              Exit;
            end;
          S := S + _CR_;
          S := S + 'Detailed Pain Assessment ' + _CR_;
          //      S := S + '1. Location of pain: ' + FieldValue(fnS_PostPainIASP) + _CR_;
          //      S := S + '2. Intensity: ' + FieldValue(fnS_PostPainIntensity) + _CR_;
          S := S + '1. Frequency text: ' + FieldValue(fnS_PostPainFrequencyText)
            +
            _CR_;
          S := S + '2. Onset: ' + FieldValue(fnS_PostPainOnset) + _CR_;
          S := S + '3. Duration of episodes: ' + FieldValue(fnS_PostPainDuration)
            +
            _CR_;
          S := S + '4. Quality of pain: ' + FieldValue(fnS_PostPainQuality) +
            _CR_;

          S := S + 'The patient indicated that:' + _CR_;
          S := S + '1. The impact of the pain was: ' +
            FieldValue(fnS_PostPainImpact)
            + _CR_;
          S := S + '2. The following makes the pain worse: ' +
            FieldValue(fnS_PostPainWorse) + _CR_;
          S := S + '3. The following relieves the pain: ' +
            FieldValue(fnS_PostPainRelieve) + _CR_;
          S := S + '4. The medications that help the pain are: ' +
            FieldValue(fnS_PostPainHelpMed) + _CR_;
          S := S + '5. Other treatments used for pain are: ' +
            FieldValue(fnS_PostPainOtherTreatments) + _CR_;
          S := S + '6. Other comments: ' + FieldValue(fnS_PostPainOtherComments)
            +
            _CR_;
        end;
    end;
  Result := s;
end;

////////////////////////////////////////////////////////////////////////////////

function TdmData.getCommentsReport(aContextFilter: string): string;
var
  sFilter: string;
  bFiltered: Boolean;
begin
  bFiltered := cdsNotes.Filtered;
  sFilter := cdsNotes.Filter;

  if aContextFilter <> '' then
    begin
      setCDSFilter(cdsNotes, aContextFilter);
      try
        cdsNotes.Filtered := True;
      except
      end;
    end
  else
    Engine.NoteList_CDS.Filtered := False;

  Result := getNotesReport('', cdsNotes);
  if bFiltered then
    begin
      cdsNotes.Filter := sFilter;
      cdsNotes.Filtered := True;
    end;
end;

function TdmData.getFlowNotesReport: string;
begin
  Result := '';
  if not cdsNotes.Active then
    Exit;
  Result := getCommentsReport(getNotesContextFilter(4));
end;

procedure TdmData.cdsNotesAfterInsert(DataSet: TDataSet);
begin
  cdsNotes.FieldByName(fnNL_ID).AsString := IntToStr(cdsNotes.RecordCount);
  cdsNotes.FieldByName(fnNL_Lock).AsString := fsUnlocked;
  cdsNotes.FieldByName(fnNL_StudyID).AsString :=
    Engine.CurrentPatient.SessionID;
  cdsNotes.FieldByName(fnNL_PatientID).AsString := Engine.CurrentPatient.ID;
  cdsNotes.FieldByName(fnNL_AuthorID).AsString := Engine.CurrentOperatorID;
end;

procedure TdmData.cdsNotesAuthorNameGetText(Sender: TField;
  var Text: string; DisplayText: Boolean);
var
  s: string;
  i: Integer;
begin
  try
    s := cdsNotes.FieldByName(fnNL_AuthorID).AsString;
    i := StrToIntDef(Trim(s), -1);
    if (i > 0) and (s <> '') then
      s := Engine.getUserNameByID(s);
    if (s = '') or (s = #$D#$A) then
      s := cdsNotes.FieldByName(fnNL_Tag).AsString;

    Text := s;
  except
  end;
end;

procedure TdmData.dsNotesDataChange(Sender: TObject; Field: TField);
begin
  if Assigned(cdsNotes) and (cdsNotes.Active) and Assigned(frmMain) then
    frmMain.RefreshCurrentWindow;
end;

procedure TdmData.SetCurrentNoteStatus(aStatus: string);
begin
  try
    cdsNotes.Edit;
    cdsNotes.FieldByName(fnNL_Status).AsString := aStatus;
    cdsNotes.FieldByName(fnNL_StatusBy).AsString := Engine.CurrentOperatorID;
    cdsNotes.FieldByName(fnNL_StatusDate).AsDateTime :=
      Engine.getCurrentServerDateTime;
    cdsNotes.Post;
  except
  end;
end;

procedure TdmData.ChangeCurrentNoteValidStatus;
begin
  if (cdsNotes.FieldByName(fnNL_Status).AsString = fsValid) then
    setCurrentNoteStatus(fsInvalid)
  else
    setCurrentNoteStatus(fsValid)
end;

procedure TdmData.LockCurrentNote;
begin
  if cdsNotes.FieldByName(fnNL_Lock).AsString = fsLocked then
    Exit;
  try
    cdsNotes.Edit;
    cdsNotes.FieldByName(fnNL_Lock).AsString := fsLocked;
    cdsNotes.Post;
  except
  end;
end;

procedure TdmData.LockNotes;
var
  BM: TBookmark;
  sFilter: string;
  bFilter: Boolean;
begin
  BM := cdsNotes.GetBookmark;

  sFilter := cdsNotes.Filter;
  bFilter := cdsNotes.Filtered;
  cdsNotes.Filtered := False;

  cdsNotes.DisableControls;
  cdsNotes.First;
  while not cdsNotes.EOF do
    begin
      LockCurrentNote;
      cdsNotes.Next;
    end;
  if Assigned(BM) then
    try
      cdsNotes.GotoBookmark(BM);
      cdsNotes.FreeBookmark(BM);
    except
//      on E: Exception do
//        MessageDLG('Lock Notes:'+#13#10+E.Message,mtWarning,[mbOK],0);
    end;

  cdsNotes.EnableControls;

  if bFilter then
    begin
      cdsNotes.Filter := sFilter;
      cdsNotes.Filtered := True;
    end;
end;

procedure TdmData.DeleteCurrentNote;
begin
  if cdsNotes.FieldByName(fnNL_Lock).AsString = fsLocked then
    Exit;
  try
    cdsNotes.Edit;
    cdsNotes.Delete;
    cdsNotes.Post;
  except
  end;
end;
////////////////////////////////////////////////////////////////////////////////

procedure TdmData.PatchDictionary(aString: string);
var
  sOldDictionaryXML,
    aTable, aColumn,
    sOldValue,
    sCopyValue,
    sValue: string;
  cdsCopy: TClientDataSet;
  fld: TFieldDef;
  fn: string;
  iCase,
    i: Integer;
begin
  cdsDict.MergeChangeLog;
  sOldDictionaryXML := cdsDict.XMLData;

  sValue := aString;
  while pos(#13 + #10, sValue) = 1 do
    sValue := copy(sValue, 3, Length(sValue));
  while pos(#10 + #13, sValue) = 1 do
    sValue := copy(sValue, 3, Length(sValue));
  if (aString = '') or (aString = #13 + #10) or (pos('-1^', sValue) = 1) then
    begin
      ShowMessage('No data to process');
      exit;
    end;
  cdsCopy := TClientDataSet.Create(nil);
  try
    cdsCopy.XMLData := sValue;
    cdsCopy.Open;

    while not cdsCopy.Eof do
      begin
        aTable := cdsCopy.FieldByName(fnDict_Table).AsString;
        aColumn := cdsCopy.FieldByName(fnDict_Column).AsString;
        if not cdsDict.Locate(fnDict_Table + ';' + fnDict_Column,
          VarArrayOf([aTable, aColumn]), []) then
          cdsDict.Insert;

        for i := 0 to cdsDict.FieldCount - 1 do
          begin
            if i >= cdsDict.FieldDefs.Count then
              break;
            fld := cdsDict.FieldDefs[i];
            fn := fld.Name;
            sOldValue := cdsDict.FieldByName(fn).AsString;
            try
              try
                sCopyValue := cdsCopy.FieldByName(fn).AsString;
              except
                on E: Exception do
                  sCopyValue := '';
              end;
              if (sOldValue = '') then
                cdsDict.FieldByName(fn).AsString := sCopyValue
              else if (sOldValue <> sCopyValue) then
                begin
                  iCase := MessageDlg('Do you want to replace the value of the <'
                    + fn + '> column?' +
                    #13#10#13#10 + 'Old Value: "' + sOldValue + '" ' +
                    #13#10 + 'New Value: "' + sCopyValue + '"' +
                    #13#10 + #13#10 +
                    '(To stop updates and restore the original dictionary select "Cancel")',
                    mtInformation,
                    [mbOk, mbNo, mbCancel], 0);

                  case iCase of
                    mrOK: cdsDict.FieldByName(fn).AsString := sCopyValue;
                    mrCancel:
                      begin
                        cdsCopy.Last;
                        try
                          cdsDict.Close;
                          cdsDict.XMLData := sOldDictionaryXML;
                          cdsDict.Open;
                        except
                        end;
                        Break;
                      end;
                  end;
                end;
            except
            end;
          end;
        try
          cdsDict.Post;
        except
        end;

        cdsCopy.Next;
      end;
  except
  end;
  cdsCopy.Free;
end;

function TdmData.CheckCDSIDField(aDS: TClientDataSet; aIDField, aIDValue:
  string; var badValue: string): Boolean;
var
  b: Boolean;
  BM: TBookmark;
begin
  b := True;

  BM := aDS.GetBookmark;

  aDS.First;
  while not aDS.Eof do
    begin
      badValue := aDS.FieldByName(aIDField).AsString;
      b := badValue = aIDValue;
      if not b then
        break;
      aDS.Next;
    end;

  aDS.GotoBookmark(BM);
  aDS.FreeBookmark(BM);
  Result := b;
end;

function TdmData.ValidDataSetXML(XML: string): Boolean;
var
  CDS: TClientDataSet;
begin
  CDS := nil;
  try
    CDS := TClientDataSet.Create(nil);
    try
      CDS.XMLData := XML;
      CDS.Open;
      Result := CDS.Active;
    except
      on E: Exception do
        Result := False;
    end;
  finally
    CDS.Free;
  end;
end;

function TdmData.UpdateFlowsheetIndex(aDescending: Boolean): Boolean;
begin
  Result := False;
  with cdsFlowSheet do
    begin
      try
        if not Active then
          try
            Open;
          except
            on e: Exception do
              Exit;
          end;

        try
          if (IndexName = 'Flowsheet_Index') then
            DeleteIndex('Flowsheet_Index');
        except
          on e: Exception do
            Exit;
        end;

        if Preferences.bValue(onReverseFlowsheetOrder) then
          AddIndex('Flowsheet_Index',fnFS_Date + ';' + fnFS_Time,[ixDescending])
        else
          AddIndex('Flowsheet_Index',fnFS_Date + ';' + fnFS_Time,[]);

//        IndexFieldNames := fnFS_Date + ';' + fnFS_Time;
        IndexDefs.Update;
        IndexName := 'Flowsheet_Index';

      except
        on e: Exception do
          ShowMessage('Flowsheet Index update error:' + #13#10 + e.Message);
      end;
    end;
end;

procedure TdmData.dsStudyHistoryDataChange(Sender: TObject; Field: TField);
begin
  if assigned(frmCover) then
    frmCover.RefreshTIU;
end;

procedure TdmData.dsWorkflowDataChange(Sender: TObject; Field: TField);
begin
  if Assigned(frmFlow) then
    frmFlow.setWorkflowStatus;
end;

function TdmData.getGeneralInfoReport: string;
var
  //  i: Integer;
  s: string;
  //  sName: String;
begin
  Result := '';
  s := '';
  (*
    for i := 0 to cdsInfo.FieldDefs.Count - 1 do
      begin
        try
          sName := cdsInfo.FieldDefs[i].Name;
          s := s + cdsInfo.FieldDefs[i].DisplayName + Char(VK_TAB)+': ' +
            cdsInfo.FieldByName(sName).AsString + _CR_;
        except
        end;
      end;
  *)
  s := s + 'Current Treatment Date: '
    + cdsInfo.FieldByName(fnI_StudyDate).AsString + #13;
  s := s + 'Diagnosis:              '
    + cdsInfo.FieldByName(fnI_Diagnosis).AsString + #13;
  s := s + 'Diagnosis Date:         '
    + cdsInfo.FieldByName(fnI_DiagnosisDate).AsString + #13;
  s := s + 'Initial Therapy Date:   '
    + cdsInfo.FieldByName(fnI_TherapyDate).AsString + #13;
  s := s + 'Modalities:             '
    + cdsInfo.FieldByName(fnI_Modality).AsString + #13;
  s := s + 'Code Status:            '
    + cdsInfo.FieldByName(fnI_CodeStatus).AsString + #13;
  s := s + 'Attending Nephrologist: '
    + cdsInfo.FieldByName(fnI_Nephrologist).AsString + #13;

  s := s + #13 + 'Visit Schedule' + #13;
  if cdsInfo.FieldByName(fnI_Mon).AsBoolean then
    s := s + '      Monday' + #13;
  if cdsInfo.FieldByName(fnI_Tue).AsBoolean then
    s := s + '      Tuesday' + #13;
  if cdsInfo.FieldByName(fnI_Wed).AsBoolean then
    s := s + '      Wednesday' + #13;
  if cdsInfo.FieldByName(fnI_Thu).AsBoolean then
    s := s + '      Thursday' + #13;
  if cdsInfo.FieldByName(fnI_Fri).AsBoolean then
    s := s + '      Friday' + #13;
  if cdsInfo.FieldByName(fnI_Sat).AsBoolean then
    s := s + '      Saturday' + #13;
  if cdsInfo.FieldByName(fnI_Sun).AsBoolean then
    s := s + '      Sunday' + #13;

  s := s + #13 + 'Transplant status' + #13;
  if not cdsSummary.FieldByName(fnS_TrnCandidate).AsBoolean then
    s := s + '      Non Candidate'
  else
    begin
      s := s + '      Candidate' + #13;
      if cdsSummary.FieldByName(fnS_TrnWIP).AsBoolean then
        s := s + '                Work in progress' + #13;
      if cdsSummary.FieldByName(fnS_TrnReferred).AsBoolean then
        s := s + '                Referred to Transplant Center' + #13;
    end;

  Result := s;
end;

procedure TdmData.SaveErrorsLog;
begin
{$IFDEF SAVE_ERRORS_LOG}
  cdsEvents.MergeChangeLog;
  cdsEvents.First;

  while not cdsEvents.Eof do
    if cdsEvents.FieldByName(fnEvents_Type).AsString <> rtError then
      cdsEvents.Delete
    else
      cdsEvents.Next;
  cdsEvents.MergeChangeLog;
  Engine.fRenalController.SetSystemOption(onErrorLog,cdsEvents.XMLData);
{$ENDIF}
end;

procedure TdmData.dsInfoDataChange(Sender: TObject; Field: TField);
begin
  if not Assigned(frmMain) then Exit;
  if not Assigned(Field) then Exit;
  if Field.FieldName = fnI_StudyDate then
     frmMain.RefreshStudyStatus;
end;

function TdmData.UpdateMedlistIndex(aDescending: Boolean): Boolean;
const
  INDX_NAME = 'MedList_Index';

begin
  Result := False;
  with cdsMedList do
    begin
      try
        if not Active then
          try
            Open;
          except
            on e: Exception do
              Exit;
          end;

        try
          if (IndexName = INDX_NAME) then
            DeleteIndex(INDX_NAME);
        except
          on e: Exception do
            Exit;
        end;

        if Preferences.bValue(onReverseFlowsheetOrder) then
          AddIndex(INDX_NAME, fnML_StudyID + ';'+fnML_Date + ';' + fnML_Time,
            [ixDescending])
        else
          AddIndex(INDX_NAME, fnML_StudyID + ';'+fnML_Date + ';' + fnML_Time, []);
        IndexDefs.Update;
        IndexName := INDX_NAME;

      except
        on e: Exception do
          ShowMessage('Medlist Index update error:' + #13#10 + e.Message);
      end;
    end;
end;


procedure TdmData.dsMedListDataChange(Sender: TObject; Field: TField);
begin
  if Assigned(frmFlow) then
    frmFlow.SetActionStatus;
end;

end.

