unit fMDK_BugReport;
{
================================================================================
*
*       Package:      MDK - Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 12/06/07 9:36a $
*       Site Name:    Hines OIFO
*       Date Created: 7/29/05
*       Developers:   andriyevskiy                  
*       Description:  Contains the Application Report form
*       Notes:
*
================================================================================
*       $Archive: /Clinical Procedures v1.0 (CP Legacy)/CP Version 1.0 - Patch 19/HEMODIALYSIS_1_0_6_320/APP-HEMODIALYSIS/Forms/fMDK_BugReport.pas $
*
* $History: fMDK_BugReport.pas $
 * 
 * *****************  Version 1  *****************
 * User: Vhaishandria Date: 1/13/09    Time: 1:15p
 * Created in $/Clinical Procedures v1.0 (CP Legacy)/CP Version 1.0 - Patch 19/HEMODIALYSIS_1_0_6_320/APP-HEMODIALYSIS/Forms
 * Patch 19 source code
 * 
 * *****************  Version 1  *****************
 * User: Vhaishandria Date: 4/24/08    Time: 4:23p
 * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/HEMODIALYSIS_1_0_6_319_#18/APP-HEMODIALYSIS/Forms
 * 
 * *****************  Version 1  *****************
 * User: Vhaishandria Date: 4/02/08    Time: 9:24a
 * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI Current/HEMODIALYSIS_1_0_6_319/APP-HEMODIALYSIS/Forms
}
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, Buttons, ActnList, StdActns
  , Printers, Mask
  , uMDK_options
  , uMDK_Common
  ;

type
  TfrmBugReport = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    reReport: TRichEdit;
    SaveDialog1: TSaveDialog;
    ActionList1: TActionList;
    FileSaveAs1: TFileSaveAs;
    Panel3: TPanel;
    acPrint: TAction;
    PrintDialog1: TPrintDialog;
    PrinterSetupDialog1: TPrinterSetupDialog;
    WindowClose1: TWindowClose;
    acClose: TAction;
    FontDialog1: TFontDialog;
    FontEdit1: TFontEdit;
    acSendMail: TAction;
    spbSend: TBitBtn;
    SpeedButton8: TSpeedButton;
    spbLandscape: TSpeedButton;
    Label2: TLabel;
    acSaveToDB: TAction;
    sbDBSave: TSpeedButton;
    sbDBLoad: TSpeedButton;
    acLoadFromDB: TAction;
    bbFont: TBitBtn;
    BitBtn2: TBitBtn;
    BitBtn3: TBitBtn;
    acSaveAs: TAction;
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FormResize(Sender: TObject);
    procedure acPrintExecute(Sender: TObject);
    procedure WindowClose1Execute(Sender: TObject);
    procedure acCloseExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FontEdit1Accept(Sender: TObject);
    procedure FontEdit1BeforeExecute(Sender: TObject);
    procedure acSaveToDBExecute(Sender: TObject);
    procedure acLoadFromDBExecute(Sender: TObject);
    procedure acSaveAsExecute(Sender: TObject);
  private
    { Private declarations }
    Ignore: Boolean;
    fLastName: string;
    procedure setLastName(aName: string);
  public
    { Public declarations }
    OptionType: string;
    MailSubject,
      AddressTo,
      AddressFrom,
      SMTPHost: string;
    procedure Report(aStringList: TStringList; aLineLen: Integer);
    property LastName: string read fLastName write setLastName;
  end;

var
  frmBugReport: TfrmBugReport;

function NewBugReport(aHost, aToAddress, aFromAddress, aSubject: string):
  TfrmBugReport;
function NewReport(aLastName: string; aHost, aToAddress, aFromAddress, aSubject:
  string; aType: string = otSystem): TfrmBugReport;

implementation

uses
  uMDK_Engine
  ;
  
{$R *.dfm}

function NewBugReport(aHost, aToAddress, aFromAddress, aSubject: string):
  TfrmBugReport;
begin
  Result := TfrmBugReport.Create(Application);
  if Result <> nil then
    try
      Result.SMTPHost := aHost;
      Result.AddressTo := aToAddress;
      Result.AddressFrom := aFromAddress;
      Result.MailSubject := aSubject;
    except
    end;
end;

function NewReport(aLastName: string; aHost, aToAddress, aFromAddress, aSubject:
  string; aType: string = otSystem): TfrmBugReport;
begin
  Result := TfrmBugReport.Create(Application);
  if Result <> nil then
    try
      Result.SMTPHost := aHost;
      Result.AddressTo := aToAddress;
      Result.AddressFrom := aFromAddress;
      Result.MailSubject := aSubject;
      Result.BorderStyle := bsNone;
      Result.Panel1.Visible := False;
      Result.reReport.Color := clWindow;
      Result.OptionType := aType;
    except
    end;
end;

procedure TfrmBugReport.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = char(VK_ESCAPE) then
    Close;
end;

procedure TfrmBugReport.FormResize(Sender: TObject);
begin
  reReport.Invalidate;
end;

procedure TfrmBugReport.acPrintExecute(Sender: TObject);
var
  rr: TRect;
  iPage,
    i, j: Integer;

  procedure DrawFrame(aR: TRect);
  begin
    Printer.Canvas.Brush.Color := clBlack;
    Printer.Canvas.FrameRect(aR);
    Printer.Canvas.Brush.Style := bsClear;
  end;

  procedure ClosePage;
  begin
    Printer.Canvas.TextOut(Printer.PageWidth - 400, Printer.PageHeight - 100,
      IntToStr(iPage));
    inc(iPage);
  end;

begin
  if PrintDialog1.Execute then
    with Printer do
      begin
        Printer.Canvas.Font.Assign(FontEdit1.Dialog.Font);
        if spbLandscape.Down then
          Orientation := poLandscape
        else
          Orientation := poPortrait;

        rr := Rect(580, 180, (Pagewidth - 180), (PageHeight - 180));

        BeginDoc;
        Canvas.Brush.Style := bsClear;
        j := 0;
        iPage := 1;
        for i := 0 to ReReport.Lines.Count do
          begin
            if ((j + 1) * Canvas.TextHeight(reReport.Lines.Strings[i])) >
              (Pageheight - 400) then
              begin
                DrawFrame(rr);
                ClosePage;
                Newpage;
                j := 0;
              end;
            Canvas.TextOut(600, 200 + (j *
              Canvas.TextHeight(reReport.Lines.Strings[i])),
              reReport.Lines.Strings[i]);
            inc(j);
          end;
        DrawFrame(rr);
        ClosePage;
        EndDoc;
      end;
end;

procedure TfrmBugReport.WindowClose1Execute(Sender: TObject);
begin
  Close;
end;

procedure TfrmBugReport.acCloseExecute(Sender: TObject);
begin
  Close;
end;

procedure TfrmBugReport.FormCreate(Sender: TObject);
begin
  ignore := False;
  ignore := True;
  rereport.Font.Size := FontDialog1.Font.Size;
  ignore := False;

{$IFDEF AANTEST}
  sbDBSave.Visible := True;
  sbDBLoad.Visible := True;
{$ELSE}
  spbSend.Visible := False;
  sbDBSave.Visible := False;
  sbDBLoad.Visible := False;

{$ENDIF}
end;

procedure TfrmBugReport.FontEdit1Accept(Sender: TObject);
begin
  reReport.Font.Assign(FontEdit1.Dialog.Font);
  Printer.Canvas.Font.Assign(FontEdit1.Dialog.Font);
end;

procedure TfrmBugReport.Report(aStringList: TStringList; aLineLen: Integer);
var
  i: Integer;
  aSL: TStringList;

  function Replace13(aValue, aMargin: string): string;
  var
    ii: Integer;
    sList: TStringList;
  begin
    sList := TStringList.Create;
    sList.Text := aValue;
    for ii := 0 to sList.Count - 1 do
      sList[ii] := amargin + sList[ii];
    Result := sList.Text;
    sList.Free;
  end;

  procedure FormatAdd(aLine, aMargin: string; aLimit: Integer);
  var
    j, k: Integer;
    ss: string;
  begin
    j := 1;
    repeat
      k := aLimit - Length(amargin);
      ss := copy(aLine, j, k);
      if Length(ss) >= k then
        while (copy(ss, k, 1) <> ' ') and (k > 1) do
          begin
            ss := copy(ss, 1, k - 1);
            k := k - 1;
          end;
      aSL.Text := aSL.Text + Replace13(ss, amargin);
      j := j + k;
    until j >= Length(aLine);
  end;

begin
  aSL := TStringList.Create;
  for i := 0 to aStringList.Count - 1 do
    FormatAdd(aStringList[i], '  ', aLineLen);
  reReport.Lines.Text := aSL.Text;

  aSL.Free;

end;

procedure TfrmBugReport.FontEdit1BeforeExecute(Sender: TObject);
begin
  FontDialog1.Font.Name := reReport.Font.Name;
end;

procedure TfrmBugReport.acSaveToDBExecute(Sender: TObject);
begin
  if OptionType = otSystem then
    Engine.setSystemOptionByName(LastName, ReReport.Lines.Text)
  else
    Engine.setUserOptionByName(LastName, ReReport.Lines.Text)
end;

procedure TfrmBugReport.acLoadFromDBExecute(Sender: TObject);
var
  s: string;
begin
  if OptionType = otSystem then
    s := Engine.getSystemOptionByName(LastName)
  else
    s := Engine.getUserOptionByName(LastName);
  if s <> yoUnKnown then
    ReReport.Lines.Text := s
  else
    ReReport.Lines.Text := '';
end;

procedure TfrmBugReport.setLastName(aName: string);
begin
  fLastName := aName;
  acLoadFromDBExecute(nil);
end;

procedure TfrmBugReport.acSaveAsExecute(Sender: TObject);
begin
  if SaveDialog1.Execute then
    reReport.Lines.SaveToFile(SaveDialog1.FileName);
end;

(*
procedure TfrmBugReport.acSendMailExecute(Sender: TObject);
begin

  SendMail(
    Engine.getSMTPHost,
    Engine.getFromAddress,
    Engine.getToAddress,
    MailSubject,
    reReport.Text,
    ''
    );
end;
*)
end.

