unit fMDK_Comments;
{
================================================================================
*
*       Package:      MDK - Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 2/22/08 3:41p $
*       Site Name:    Hines OIFO
*       Date Created: 12/15/06
*       Developers:   andriyevskiy                  
*       Description:  this unit contains the Hemodialysis application
*                     Comments form
*       Notes:
*
================================================================================
*       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI with Comments Update/APP-HEMODIALYSIS/Forms/fMDK_Comments.pas $
*
*       $ History: fMDK_Comments.pas $
}
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, ComCtrls;

type
  TfrmComment = class(TForm)
    Panel2: TPanel;
    pnl: TPanel;
    btnOK: TBitBtn;
    BitBtn3: TBitBtn;
    Panel3: TPanel;
    pnlTiuText: TPanel;
    grbComments: TGroupBox;
    Panel4: TPanel;
    pnlSignature: TPanel;
    Label1: TLabel;
    edtSignature: TEdit;
    Panel1: TPanel;
    mmNote: TMemo;
    pnlDetails: TPanel;
    Label3: TLabel;
    Label4: TLabel;
    Edit1: TEdit;
    Label5: TLabel;
    Edit2: TEdit;
    Label7: TLabel;
    Edit3: TEdit;
    Label8: TLabel;
    Edit4: TEdit;
    Label9: TLabel;
    Edit5: TEdit;
    Label10: TLabel;
    Edit6: TEdit;
    Label11: TLabel;
    Edit7: TEdit;
    Label12: TLabel;
    Edit8: TEdit;
    pnlCommentBy: TPanel;
    Label13: TLabel;
    Label14: TLabel;
    edtAuthor: TEdit;
    edtCommentOn: TEdit;
    BitBtn1: TBitBtn;
    ckbWW: TCheckBox;
    lbl: TLabel;
    lblSize: TLabel;
    pnlTIUTemplate: TPanel;
    grbTemplates: TGroupBox;
    lbTemplates: TListBox;
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure btnOKClick(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure mmNoteChange(Sender: TObject);
    procedure ckbWWClick(Sender: TObject);
    procedure lbTemplatesClick(Sender: TObject);
  private
    { Private declarations }
    iCount: Integer;
  public
    { Public declarations }
  end;

var
  frmComment: TfrmComment;

function AddComment(var aText: string): TModalResult;
function EditComment(var aText: string; Add: Boolean = False): TModalResult;
function Submit(aText: string; var eSig: string): TModalResult;
function confirmComment(var aText, anAuthor, aDate: string; aReadOnly: Boolean =
  False): TModalResult;

implementation

uses uMDK_Engine, uMDK_Common, uMDK_Options, fMDK_SelectProvider,
  uMDK_Notes, uMDK_ReportEngine, uMDK_Utils;

{$R *.dfm}

function EditComment(var aText: string; Add: Boolean = False): TModalResult;
var
  dlg: TfrmComment;
begin
  Application.CreateForm(TfrmComment, dlg);

  with dlg do
    begin
      edtCommentOn.Text := FormatDateTime('mm/dd/yyyy hh:nn:ss', Now);
      edtAuthor.Text := Engine.CurrentOperatorID;
      pnlSignature.Visible := Preferences.bValue(onRequestSignatureForComments);
      try
        dlg.ActiveControl := dlg.mmNote;
      except
      end;
      if Add then
        mmNote.Text := ''
      else
        mmNote.Text := aText;
      mmNote.ReadOnly := False;
    end;

  dlg.ckbWWClick(nil);
  dlg.pnlTIUTemplate.Visible := Preferences.bValue(onShowTIUNoteTemplates);
  Result := dlg.ShowModal;
  if Result <> mrCancel then
    aText := dlg.mmNote.Text;

  FreeAndNil(dlg);
end;

function AddComment(var aText: string): TModalResult;
begin
  Result := EditComment(aText, True);
end;

function ConfirmComment(var aText, anAuthor, aDate: string; aReadOnly: Boolean =
  False): TModalResult;
var
  dlg: TfrmComment;
begin
  Application.CreateForm(TfrmComment, dlg);

  with dlg do
    begin
      mmNote.Text := aText;
      edtCommentOn.Text := FormatDateTime('mm/dd/yyyy hh:nn:ss', Now);
      edtCommentOn.ReadOnly := False;

      edtAuthor.Text := Engine.CurrentOperatorID;
      //      edtAuthor.ReadOnly := False;
      edtAuthor.ReadOnly := True;

      pnlSignature.Visible := Preferences.bValue(onRequestSignatureForComments);

      mmNote.ReadOnly := aReadOnly;

      ActiveControl := mmNote;

      dlg.ckbWWClick(nil);
      Result := dlg.ShowModal;
      anAuthor := edtAuthor.Text;
      aDate := edtCommentOn.Text;
      aText := mmNote.Text;
    end;

  FreeAndNil(dlg);
end;

function Submit(aText: string; var eSig: string): TModalResult;
var
  dlg: TfrmComment;
begin
  Application.CreateForm(TfrmComment, dlg);

  with dlg do
    begin
      iCount := 0;
      borderStyle := bsSizeable;
      width := 640;
      BorderIcons := [biSystemMenu, biMaximize];
      pnlCommentBy.Visible := False;
      pnlDetails.Visible := False;
      Caption := 'Study Report Submitted by ' + Engine.CurrentOperatorID;
      pnlSignature.Visible := True;
      mmNote.Text := aText;
      mmNote.ReadOnly := True;
      mmNote.Color := edtSignature.Color;
      ActiveControl := edtSignature;
      btnOK.Caption := 'Submit';
      grbComments.Caption := 'Study Summary Report';
      pnlTIUTemplate.Visible := False;
      ckbWWClick(nil);
    end;

  Result := dlg.ShowModal;
  eSig := dlg.edtSignature.Text;
  FreeAndNil(dlg);
end;

////////////////////////////////////////////////////////////////////////////////

procedure TfrmComment.FormCreate(Sender: TObject);
var
  sl: TSTringList;
  i: Integer;
begin
  pnlTIUTemplate.Visible := Preferences.bValue(onShowTIUNoteTemplates);
  if pnlTIUTemplate.Visible then
    begin
      sl := TStringList.Create;
      sl.Text := Engine.getSystemOptionByName(onNoteTemplates);
      for i := 0 to sl.Count - 1 do
        sl[i] := sl.Names[i];
      lbTemplates.Items.Text := sl.Text;
      sl.Free;
    end;
  mmNote.Text := '';
  edtSignature.Text := '';
end;

procedure TfrmComment.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if key = VK_Escape then
    ModalResult := mrCancel;
end;

procedure TfrmComment.btnOKClick(Sender: TObject);
var
  iCount: Integer;
  CanClose: Boolean;
  sMSG: string;
begin
  sMSG := '';
  iCount := Engine.InvalidESigCount;
  if pnlSignature.Visible then
    begin
      CanClose := Engine.ValidateSignature(edtSignature.Text);
      if not CanClose then
        begin
        sMSG := sMSG + 'Invalid signature.' + #13#10;
        inc(iCount);
          if iCount>2 then
            begin
              sMSG := 'Third attempt to provide value failed'+#13#10;
              MessageDlg('Input data contains errors:' + #13#10#13#10 + sMSG +
                'Try again later...', mtWarning, [mbOK], 0);
              ModalResult := mrCancel;
              Engine.InvalidESigCount := iCount;
              Exit;
            end;
        end;
      ActiveControl := edtSignature;
    end
  else
    CanClose := True;

  if btnOK.Caption <> 'Submit' then
    begin
      if not IsValidDateString(edtCommentOn.Text) then
        begin
          sMSG := sMSG + 'Invalid data value <' + edtCommentOn.Text + '>' +
            #13#10;
          CanClose := False;
        end;

      if edtAuthor.Text = '' then
        begin
          CanClose := False;
          sMSG := sMSG + 'Author name is not entered' + #13#10;
        end;
    end;

  if not CanClose then
    MessageDlg('Input data contains errors:' + #13#10#13#10 + sMSG +
      'Provide valid value(s) and try again...', mtWarning, [mbOK], 0)
  else
    ModalResult := mrOK;

  Engine.InvalidESigCount := iCount;
end;

procedure TfrmComment.BitBtn1Click(Sender: TObject);
begin
  edtAuthor.Text := SelectProvider('', Engine.fRenalController);
end;

procedure TfrmComment.mmNoteChange(Sender: TObject);
begin
  lblSize.Caption := IntToStr(Length(mmNote.Text));
  if (Length(mmNote.Text) > MAX_NOTE_SIZE) and (btnOK.Caption <> 'Submit') then
    lblSize.Font.Color := clRed
  else
    lblSize.Font.Color := clWindowText;
end;

procedure TfrmComment.ckbWWClick(Sender: TObject);
begin
  mmNote.WordWrap := ckbWW.Checked;
  if mmNote.WordWrap then
    mmNote.ScrollBars := ssVertical
  else
    mmNote.ScrollBars := ssBoth;
end;

procedure TfrmComment.lbTemplatesClick(Sender: TObject);
var
  sTemplateName: string;
begin
  if Sender = lbTemplates then
    try
      if lbTemplates.ItemIndex < 0 then
        mmNote.Text := ''
      else
        begin
          sTemplateName :=
            Engine.getSystemOptionByName(lbTemplates.Items[lbTemplates.ItemIndex]);
          mmNote.Text := getReportValue(sTemplateName);
          ActiveControl := mmNote;
        end;
    except
      mmNote.Text := '';
    end;
end;

end.
