unit fMDK_DictionaryEdit;
{
================================================================================
*
*       Package:      MDK - Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 4/22/08 11:00a $
*       Site Name:    Hines OIFO
*       Date Created: 7/29/05
*       Developers:   andriyevskiy                  
*       Description:  this unit contains the Hemodialysis application
*                     client data set record editor
*       Notes:
*
================================================================================
*       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI 1.0/APP-HEMODIALYSIS/Forms/fMDK_DictionaryEdit.pas $
*
*       $ History: fMDK_DictionaryEdit.pas $
}
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, Grids, ValEdit, ExtCtrls, Mask, DBCtrls
  , DB, DBClient, ActnList,
  uMDK_Vitals;

type
  TfrmMDK_DictionaryEdit = class(TForm)
    pnlEdit: TPanel;
    Panel2: TPanel;
    bbOK: TBitBtn;
    bbCancel: TBitBtn;
    Panel3: TPanel;
    vle: TValueListEditor;
    Panel4: TPanel;
    BitBtn3: TBitBtn;
    BitBtn4: TBitBtn;
    ActionList1: TActionList;
    acEditDomain: TAction;
    acSaveToDB: TAction;
    procedure vleDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure acEditDomainExecute(Sender: TObject);
    procedure acSaveToDBExecute(Sender: TObject);
    procedure vleSetEditText(Sender: TObject; ACol, ARow: Integer;
      const Value: string);
    procedure vleEnter(Sender: TObject);
    procedure vleGetEditText(Sender: TObject; ACol, ARow: Integer;
      var Value: string);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure vleMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure bbCancelClick(Sender: TObject);
  private
    { Private declarations }
    OriginalLabel:String;
    procedure setDomainEditStatus(aRow, aCol: Integer);
    procedure Prepare(aTable, aField: string; aVitalType: TVitalType =
      vtUnknown);
    procedure SetUp(aTable, aField: string; aStrings: TStrings);

    function getDisplayFormat: string;
    function getEditMask: string;

    function getMaxValue: Double;
    function getMinValue: Double;
  public
    { Public declarations }
  end;

const
  DOMAIN_PICK_LIST =
    'Anticoagulants' + #13#10
    + 'Attending Nephrologists' + #13#10
    + 'Code Statuses' + #13#10
    + 'Dialyzer List' + #13#10
    + 'Education Codes' + #13#10
    + 'ESRD Diagnoses' + #13#10
    //    +'LAB Codes'+#13#10
    + 'Medical Facilities' + #13#10
    + 'Medications' + #13#10
    + 'Medication Routes' + #13#10
    + 'Medication Units' + #13#10
    + 'Medication Statuses' + #13#10
    + 'Medication Reasons' + #13#10
    + 'Modalities' + #13#10
    + 'Surgeons' + #13#10
    + 'TIU Note Titles' + #13#10
    + 'Transportation Methods' + #13#10
    ;

  TRUE_FALSE = 'True'+#13#10+'False'+#13#10;

function EditRecord(aTable, aField: string; aVitalType: TVitalType = vtUnknown):
  Boolean;
procedure UpdateDBEditField(aField: TDBedit; aVitalType: TVitalType =
  vtUnknown);
procedure UpdateColumnField(aField: TField; dsName, dsFName: string
  ; aVitalType: TVitalType = vtUnknown);

var
  frmMDK_DictionaryEdit: TfrmMDK_DictionaryEdit;

implementation

uses dMDK_Data, uMDK_Engine, uMDK_Options, fMDK_Options, uMDK_Dictionary,
  uMDK_CDSCommon, uMDK_Common;

{$R *.dfm}

function EditRecord(aTable, aField: string; aVitalType: TVitalType = vtUnknown):
  Boolean;
var
  sOld: string;
begin
  Result := False;

  if not Assigned(frmMDK_DictionaryEdit) then
    frmMDK_DictionaryEdit := TfrmMDK_DictionaryEdit.Create(Application);
  with frmMDK_DictionaryEdit do
    begin
      Prepare(aTable, aField, aVitalType);
      if vle.RowCount < 2 then
        Exit;
      Refresh;
      sOld := vle.Strings.Text;
      if (ShowModal <> mrCancel) and (sOld <> vle.Strings.Text) then
        begin
          SetUp(aTable, aField, vle.Strings);
          Result := True;
        end;

    end;
  FreeAndNil(frmMDK_DictionaryEdit);  
end;

procedure setDictionaryRecord(aField: TField);
var
  bbIgnore: Boolean;
  s: string;
  i: Integer;
begin
  if not Assigned(aField) then
    Exit;
  bbIgnore := dmData.bIgnore;
  dmData.bIgnore := true;

  with frmMDK_DictionaryEdit do
    try
      case aField.DataType of
        ftDateTime:
          begin
            TDateTimeField(aField).DisplayFormat := getDisplayFormat;
            TDateTimeField(aField).EditMask := getEditMask;
          end;
        ftInteger:
          begin
            TintegerField(aField).DisplayFormat := getDisplayFormat;
            TIntegerField(aField).EditMask := getEditMask;
            TIntegerField(aField).MaxValue := round(getMaxValue);
            TIntegerField(aField).MinValue := round(getMinValue);
          end;
        ftFloat:
          begin
            TFloatField(aField).DisplayFormat := getDisplayFormat;
            TFloatField(aField).EditMask := getEditMask;
            TFloatField(aField).MaxValue := getMaxValue;
            TFloatField(aField).MinValue := getMinValue;
          end;
        ftString:
          begin
            i := TStringField(aField).DisplayWidth;
            s := vle.Values[fnDict_Width];
            if s <> '' then
              try
                TStringField(aField).DisplayWidth := StrToInt(s);
                TStringField(aField).Size := i;
              except
                begin
                  TStringField(aField).DisplayWidth := i;
                  TStringField(aField).Size := i;
                end;
              end;
          end;
      end;
    except
    end;

  dmData.bIgnore := bbIgnore;
end;

procedure UpdateDBEditField(aField: TDBedit; aVitalType: TVitalType =
  vtUnknown);
begin
  if not devOptions.bValue(onAllowDictionaryEdit) then
    Exit;

  if Engine.CurrentOperatorMode <> rnAdmin then
    Exit;

  if EditRecord(aField.DataSource.DataSet.Name, aField.Field.FieldName,
    aVitalType) then
    setDictionaryRecord(aField.Field);
end;

procedure UpdateColumnField(aField: TField; dsName, dsFName: string; aVitalType:
  TVitalType = vtUnknown);
begin
  if Engine.CurrentOperatorMode <> rnAdmin then
    Exit;

  if not dmData.cdsDict.Locate(fnDict_Table + ';' + fnDict_Column,
    VarArrayOf([dsName, dsFName]), []) then
    begin
      if MessageDlg('Dictionary does not contain information about' + #13#10 +
        'Table <' + dsName + '> Column <' + dsFName + '>' + #13#10#13#10 +
        'Add new description?',
        mtConfirmation, [mbYes, mbNo], 0) = mrYes then
        begin
          addFieldToDictionary(dsName, aField);
          Engine.SaveDictionaryToDB; // vhaishandria 060608
        end
      else
        Exit;
    end;

  if EditRecord(dsName, dsFName, aVitalType) then
    setDictionaryRecord(aField);
end;

////////////////////////////////////////////////////////////////////////////////
// populates VLE with dictionary data
////////////////////////////////////////////////////////////////////////////////

procedure TfrmMDK_DictionaryEdit.Prepare(aTable, aField: string
  ; aVitalType: TVitalType = vtUnknown);
var
  aDS: TClientDataSet;
  i, j: Integer;
begin
  vle.Strings.Clear;
  aDS := dmData.cdsDict;

  if not aDS.Locate(fnDict_Table + ';' + fnDict_Column, VarArrayOf([aTable,
    aField]), []) then
    Exit
  else
    begin
      j := 0;
      for i := 0 to aDS.FieldCount - 1 do
        begin
{$IFNDEF AANTEST}
          if aDS.Fields.Fields[i].FieldName = fnDict_ID then
            Continue;
          //          if aDS.Fields.Fields[i].FieldName = fnDict_Tag then Continue;
          if aDS.Fields.Fields[i].FieldName = fnDict_iTag then
            Continue;
          if aDS.Fields.Fields[i].FieldName = fnDict_Table then
            Continue;
          if aDS.Fields.Fields[i].FieldName = fnDict_Column then
            Continue;
          if aDS.Fields.Fields[i].FieldName = fnDict_Type then
            Continue;
          if aDS.Fields.Fields[i].FieldName = fnDict_Length then
            Continue;
          //          if aDS.Fields.Fields[i].FieldName = fnDict_Label then Continue;
          if aDS.Fields.Fields[i].FieldName = fnDict_Format then Continue; // vhaishandria 2007-06-29 hide fiels
          if aDS.Fields.Fields[i].FieldName = fnDict_editMask then Continue;  // vhaishandria 2007-06-29 hide field
          if aDS.Fields.Fields[i].FieldName = fnDict_Width then
            Continue;
          if aDS.Fields.Fields[i].FieldName = fnDict_Units then
            Continue;
          //          if aDS.Fields.Fields[i].FieldName = fnDict_Required then Continue;
//          if aDS.Fields.Fields[i].FieldName = fnDict_Overwrite then
//            Continue;
          if aDS.Fields.Fields[i].FieldName = fnDict_Section then
            Continue;
          //          if aDS.Fields.Fields[i].FieldName = fnDict_Domain then Continue;
          if aDS.Fields.Fields[i].FieldName = fnDict_Group then
            Continue;
          //          if aDS.Fields.Fields[i].FieldName = fnDict_Min then Continue;
          //          if aDS.Fields.Fields[i].FieldName = fnDict_Max then Continue;
          if aDS.Fields.Fields[i].FieldName = fnDict_Default then
            Continue;
          //          if aDS.Fields.Fields[i].FieldName = fnDict_Comments then Continue;
{$ENDIF}
          vle.Strings.Add(aDS.Fields.Fields[i].FieldName + '=' +
            aDS.Fields.Fields[i].AsString);

          inc(j);
        end;
      i := j * (vle.DefaultRowHeight + vle.GridLineWidth) + vle.GridLineWidth;
      Height := i + 50;

      vle.ItemProps[fnDict_Tag].ReadOnly := True; // vhaishandria 2007-06-29
                                                  // disable edit of the tag
// vhaishandria 2007-08-23 - to simplify documentation
//      vle.ItemProps[fnDict_Domain].EditStyle := esPickList;
//      vle.ItemProps[fnDict_Domain].PickList.Text := DOMAIN_PICK_LIST;
      vle.ItemProps[fnDict_Domain].ReadOnly := True;

      vle.ItemProps[fnDict_Required].EditStyle := esPickList;
      vle.ItemProps[fnDict_Required].PickList.Text := TRUE_FALSE;
      vle.ItemProps[fnDict_Required].ReadOnly := True;

      vle.ItemProps[fnDict_Overwrite].EditStyle := esPickList;
      vle.ItemProps[fnDict_Overwrite].PickList.Text := TRUE_FALSE;

      // vhaishandria 2007-08-10
      if aVitalType <> vtUnknown then // Only Vitals are Readonly
        begin
          vle.Values[fnDict_MaxValue] :=
            FloatToStr(GMVVitalHiRange[aVitalType]);
          vle.Values[fnDict_MinValue] :=
            FloatToStr(GMVVitalLoRange[aVitalType]);

//          vle.ItemProps[fnDict_MinValue].ReadOnly := True;
//          vle.ItemProps[fnDict_MaxValue].ReadOnly := True;
        end
      else
        begin
          if vle.Values[fnDict_MaxValue] <> '' then
            vle.Values[fnDict_MaxValue] := FloatToStr(getMaxValue);
          if vle.Values[fnDict_MinValue] <> '' then
            vle.Values[fnDict_MinValue] := FloatToStr(getMinValue);
        end;
      vle.ItemProps[fnDict_MinValue].ReadOnly := True;
      vle.ItemProps[fnDict_MaxValue].ReadOnly := True;

      OriginalLabel := aDS.FieldByName(fnDict_Label).AsString;
    end;

end;

procedure TfrmMDK_DictionaryEdit.SetUp(aTable, aField: string; aStrings:
  TStrings);
var
  aDS: TClientDataSet;
  i: integer;
  aName: string;

  procedure UpdateField(aName: string);
  var
    s: string;
  begin
{$IFNDEF AANTEST}
    if (aName = fnDict_Table) or // 'TableName';
    (aName = fnDict_Column) or // 'ColumnName';
    (aName = fnDict_Type) {// 'DataType';} then
      Exit;
{$ENDIF}
    try
      aDS.Edit;
      s := vle.Values[aName];
      aDS.FieldByName(aName).AsString := s;
      aDS.Post;
    except
    end;
  end;

begin
  aDS := dmData.cdsDict;
  if aDS.Locate(fnDict_Table + ';' + fnDict_Column, VarArrayOf([aTable,
    aField]),[]) then
    begin
      for i := 0 to vle.RowCount - 1 do
        begin
          aName := vle.Keys[i];
          if aDS.FieldDefs.Find(aName) <> nil then
            UpdateField(vle.Keys[i]);
        end;
    end;
  Engine.SaveDictionaryToDB; //  save dictionary in DB
end;

function TfrmMDK_DictionaryEdit.getDisplayFormat: string;
begin
  Result := vle.Values[fnDict_Format];
end;

function TfrmMDK_DictionaryEdit.getEditMask: string;
begin
  Result := vle.Values[fnDict_EditMask];
end;

function TfrmMDK_DictionaryEdit.getMinValue: Double;
begin
  Result := strToFloat(vle.Values[fnDict_MinValue]);
end;

function TfrmMDK_DictionaryEdit.getMaxValue: Double;
begin
  Result := strToFloat(vle.Values[fnDict_MaxValue]);
end;

procedure TfrmMDK_DictionaryEdit.vleDrawCell(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
var
  sName: string;
begin
  with vle.Canvas do
    begin
      if aCol <> 0 then
        begin
          Brush.Color := clWindow;
          sName := vle.Cells[0, aRow];
          if vle.ItemProps[sName].ReadOnly = True then
            begin
              Font.Color := clNavy;
              if vle.ItemProps[sName].EditStyle <> esPickList then
                Brush.Color := Preferences.iValue(onReadOnlyFieldColor);
            end
          else
            Font.Color := clWindowText;
          Font.Style := [];
        end
      else
        begin
          Brush.Color := clBtnFace;
          Font.Style := [fsBold];
        end;
      FillRect(Rect);
      TextOut(Rect.Left + 2, Rect.Top + 2, vle.Cells[aCol, aRow]);
    end;
end;

procedure TfrmMDK_DictionaryEdit.acEditDomainExecute(Sender: TObject);
var
  s: string;
begin
  s := vle.Values[fnDict_Domain];
  EditDomain(s);
end;

procedure TfrmMDK_DictionaryEdit.acSaveToDBExecute(Sender: TObject);
begin
  Engine.SaveDictionaryToDB;
end;

procedure TfrmMDK_DictionaryEdit.vleSetEditText(Sender: TObject; ACol,
  ARow: Integer; const Value: string);
begin
  acEditDomain.Enabled := (vle.Cells[0, aRow] = fnDict_Domain) and
    (trim(value) <> '');
end;

procedure TfrmMDK_DictionaryEdit.setDomainEditStatus(aRow, aCol: Integer);
begin
  acEditDomain.Enabled :=
    (vle.Cells[0, aRow] = fnDict_Domain) and
    (trim(vle.Cells[1, aRow]) <> '');
end;

procedure TfrmMDK_DictionaryEdit.vleEnter(Sender: TObject);
begin
  setDomainEditStatus(vle.Row, vle.Col);
end;

procedure TfrmMDK_DictionaryEdit.vleGetEditText(Sender: TObject; ACol,
  ARow: Integer; var Value: string);
begin
  setDomainEditStatus(aRow, aCol);
end;

procedure TfrmMDK_DictionaryEdit.FormKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if Key = VK_ESCAPE then
    begin
      if (vle.Values[fnDict_Label] = '') then
      (vle.Values[fnDict_Label] := ' ');

      ModalResult := mrCancel;
    end;
end;

procedure TfrmMDK_DictionaryEdit.vleMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (ssCTRL in Shift) and (ssALT in Shift) and (Button = mbRight) and
    (Engine.CurrentOperatorMode = rnADMIN) then
    begin
{$IFDEF DICTIONARY_RANGE}
      vle.ItemProps[fnDict_MinValue].ReadOnly := False;
      vle.ItemProps[fnDict_MaxValue].ReadOnly := False;
{$ENDIF}
    end;
end;

procedure TfrmMDK_DictionaryEdit.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if Sender = bbCancel then
    Exit;
  if (vle.Values[fnDict_Label] = '') then
    begin
      MessageDLG(fnDict_Label+' field cannot be blank.'+#13+
        'The original label will be restored.',mtWarning,[mbOK],0);
      vle.Values[fnDict_Label] := OriginalLabel;
      CanClose := False;
    end;
end;

procedure TfrmMDK_DictionaryEdit.bbCancelClick(Sender: TObject);
begin
  if (vle.Values[fnDict_Label] = '') then
  (vle.Values[fnDict_Label] := ' ');
  ModalResult := mrCancel;
end;

end.
