unit fMDK_FlowSheetEdit;
{
================================================================================
*
*       Package:      MDK - Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 12/03/07 1:06p $
*       Site Name:    Hines OIFO
*       Date Created: 7/29/05
*       Developers:   andriyevskiy                  
*       Description:  Contains the Application Flowsheet Editor form
*       Notes:
*
================================================================================
*       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI 1.0/APP-HEMODIALYSIS/Forms/fMDK_FlowSheetEdit.pas $
*
*       $ History: fMDK_FlowSheetEdit.pas $
 *
}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ActnList, StdCtrls, Grids, ValEdit
  , DBClient, DB, Buttons, ExtCtrls
  ;

type
  TfrmFlowSheetEdit = class(TForm)
    ActionList1: TActionList;
    acSave: TAction;
    Panel1: TPanel;
    Panel2: TPanel;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    BitBtn3: TBitBtn;
    GroupBox1: TGroupBox;
    vleFlow: TValueListEditor;
    procedure vleFlowEditButtonClick(Sender: TObject);
    procedure vleFlowDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure FormActivate(Sender: TObject);
    procedure vleFlowGetEditMask(Sender: TObject; ACol, ARow: Integer;
      var Value: string);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure BitBtn2Click(Sender: TObject);
  private
    { Private declarations }
    fIgnore: Boolean;
    procedure PrepareList(SL: TStringList; anInsert: Boolean = False);
    procedure PrepareListByCDS(aCDS: TClientDataSet; anInsert: Boolean = False);
    function ValidRecord: Boolean;
  public
    { Public declarations }
  end;

var
  frmFlowSheetEdit: TfrmFlowSheetEdit;

procedure EditFSRecord(aCDS: TClientDataSet; aPatient, aStudy, aSource, aStatus:
  string);
procedure AddFSRecord(aCDS: TClientDataSet; aPatient, aStudy, aSource, aStatus:
  string);

implementation

uses uMDK_Common, uROR_CommonUtils, uMDK_CDSCommon
  , uMDK_Engine
  , uMDK_Options, uMDK_Flowsheet, uMDK_Dictionary, uMDK_Vitals;

{$R *.dfm}

function Now: TdateTime;
begin
  try
    Result := Engine.getCurrentServerDateTime;
  except
    Result := Now;
  end;
end;

procedure SetUpCDSBySL(aCDS: TClientDataSet; SL: TStrings;
  aPatient, aStudy, aSource, aStatus: string; anInsert: Boolean = False);
var
  sLabel, sValue: string;
  i, jPos: Integer;
begin
  if anInsert then
    aCDS.Insert
  else
    aCDS.Edit;

  aCDS.FieldByName(fnFS_PatientID).AsString := aPatient;
  aCDS.FieldByName(fnFS_SessionID).AsString := aStudy;
  aCDS.FieldByName(fnFS_Source).AsString := aSource;
  aCDS.FieldByName(fnFS_Status).AsString := aStatus;
{$IFDEF IGNORE_DUPLICATES}
  aCDS.FieldByName(fnFS_Duplicate).AsString := 'False';
{$ENDIF}
  for i := 0 to aCDS.Fields.Count - 1 do
    begin
      if not aCDS.Fields[i].Visible then
        continue;
      try
        sLabel := aCDS.Fields[i].DisplayLabel;
        jPos := SL.IndexOfName(sLabel);
        if jPos >= 0 then
          begin
            sValue := SL.Names[jPos];
            aCDS.FieldByName(aCDS.FieldDefs[i].Name).AsString :=
              SL.Values[sValue];
          end;
      except
      end;
    end;
  aCDS.Post;
end;

procedure EditFSRecord(aCDS: TClientDataSet; aPatient, aStudy, aSource, aStatus:
  string);
begin
  if not Assigned(frmFlowSheetEdit) then
    frmFlowSheetEdit := TfrmFlowSheetEdit.Create(Application);
  with frmFlowSheetEdit do
    begin
      fIgnore := False;
      PrepareListByCDS(aCDS);
      if ShowModal <> mrCancel then
        SetUpCDSBySL(aCDS, vleFlow.Strings, aPatient, aStudy, aSource, aStatus,
          False);
    end;
end;

procedure AddFSRecord(aCDS: TClientDataSet; aPatient, aStudy, aSource, aStatus:
  string);
begin
  if not Assigned(frmFlowSheetEdit) then
    frmFlowSheetEdit := TfrmFlowSheetEdit.Create(Application);

  with frmFlowSheetEdit do
    begin
      fIgnore := False;
      PrepareListByCDS(aCDS, True);

      if ShowModal <> mrCancel then
        SetUpCDSBySL(aCDS, vleFlow.Strings, aPatient, aStudy, aSource, aStatus,
          True);
    end;
end;

////////////////////////////////////////////////////////////////////////////////

procedure TfrmFlowSheetEdit.PrepareList(SL: TStringList; anInsert: Boolean =
  False);
var
  i: Integer;
  dt: TDateTime;
begin
  vleFlow.Strings.Assign(SL);
  dt := Engine.getCurrentServerDateTime;
  if not anInsert then
    Exit;
  for i := 0 to vleFlow.Strings.Count - 1 do
    begin
      if vleFlow.Strings.Names[i] = 'Date' then
        vleFlow.Strings[i] := 'Date=' + FormatDateTime(gMDK_DateFmt, dt);
      if vleFlow.Strings.Names[i] = 'Time' then
        vleFlow.Strings[i] := 'Time=' + FormatDateTime(gMDK_TimeFmt, dt);
    end;
end;

procedure TfrmFlowSheetEdit.PrepareListByCDS(aCDS: TClientDataSet; anInsert:
  Boolean = False);
var
  iStart,
    i: Integer;
  sl: TStringList;
begin
  SL := TStringList.Create;
  iStart := 0;

  for i := iStart to aCDS.Fields.Count - 1 do // patch, sorry
    begin
{$IFDEF FLOWEVENTS}
      if aCDS.Fields[i].FieldName = fnFS_Event then
        continue;
      if aCDS.Fields[i].FieldName = fnFS_Comments then
        continue;
{$ENDIF}
{$IFDEF IGNORE_DUPLICATES}
      if aCDS.Fields[i].FieldName = fnFS_Duplicate then
        continue;
{$ENDIF}
      //      if not aCDS.Fields[i].Visible then continue;
            // Date and Time should be added always regardless of visibility
      if aCDS.Fields[i].DisplayLabel = 'Time' then
        begin
          if anInsert then
            SL.Add('Time=' + FormatDateTime(gMDK_TimeFmt, Now))
          else
            SL.Add('Time=' + FormatDateTime(gMDK_TimeFmt,
              aCDS.Fields[i].AsDateTime));
        end
      else if aCDS.Fields[i].DisplayLabel = 'Date' then
        begin
          if anInsert then
            SL.Add('Date=' + FormatDateTime(gMDK_DateFmt, Now))
          else
            SL.Add('Date=' + FormatDateTime(gMDK_DateFmt,
              aCDS.Fields[i].AsDateTime));
        end
      else if aCDS.Fields[i].FieldName = fnFS_SessionID then
        continue
      else if aCDS.Fields[i].FieldName = fnFS_PatientID then
        continue
      else if aCDS.Fields[i].FieldName = fnFS_Source then
        continue
      else if not aCDS.Fields[i].Visible then
        continue
      else
        begin
          if anInsert then
            SL.Add(aCDS.Fields[i].DisplayLabel + '=')
          else
            SL.Add(aCDS.Fields[i].DisplayLabel + '='
              + aCDS.Fields[i].AsString);
        end;
    end;
  PrepareList(SL);
  SL.Free;
end;

procedure TfrmFlowSheetEdit.vleFlowEditButtonClick(Sender: TObject);
begin
  ShowMessage('Custom edit window for the parameter:' + #13#13 +
    '                    ' + vleFlow.Strings[vleFlow.Row - 1]);
  //Header adjustment
end;

procedure TfrmFlowSheetEdit.vleFlowDrawCell(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
begin
  with vleFlow.Canvas do
    begin
      if (aRow = 0) then
        begin
          Brush.Color := clBtnFace;
          Font.Style := [fsBold];
        end
      else if aCol <> 0 then
        begin
          Brush.Color := clWindow;
          Font.Color := clWindowText;
          Font.Style := [];
        end
      else
        begin
          Brush.Color := clBtnFace;
          //            Brush.Color := clWindow;
          //            Font.Style := [fsBold];
        end;
      FillRect(Rect);
      TextOut(Rect.Left + 2, Rect.Top + 2, vleFlow.Cells[aCol, aRow]);
    end;
end;

procedure TfrmFlowSheetEdit.FormActivate(Sender: TObject);
begin
  ActiveControl := vleFlow;
end;

procedure TfrmFlowSheetEdit.vleFlowGetEditMask(Sender: TObject; ACol,
  ARow: Integer; var Value: string);
begin
  if vleFlow.Cells[0, aRow] = 'Time' then
    Value := devOptions.sValue(donFlowsheetTimeInputMask)
  else if vleFlow.Cells[0, aRow] = 'Date' then
    Value := devOptions.sValue(donFlowsheetDateInputMask)
  else
    Value := getDictionaryEditMaskByDisplayLabel('cdsFlowSheet',
      vleFlow.Cells[0, aRow]);
end;

procedure TfrmFlowSheetEdit.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if not fIgnore then
    CanClose := ValidRecord;
end;

function TfrmFlowSheetEdit.ValidRecord: Boolean;
var
  t: TDateTime;
  sMessage,
    sName,
    sValue: string;
  i: Integer;

  function ValidDate(aValue: string): string;
  begin
    Result := '';
    try
      t := strToDate(aValue);
    except
      on E: EConvertError do
        Result := 'Wrong Date format: <' + aValue + '> (should be ' +
          gMDK_DateFMT + ')' + #13#10;
    end;
  end;

  function ValidTime(aValue: string): string;
  begin
    Result := '';
    try
      t := strToTime(aValue);
    except
      on E: EConvertError do
        Result := 'Wrong time format: <' + aValue + '> ( should be ' +
          gMDK_TimeFMT + ')' + #13#10;
    end;
  end;

  function ValidBPressure(aValue: string): string;
  var
    _s: string;
    fLow, fHigh: Double;
  begin
    Result := '';
    if aValue = '' then
      begin
        _s := getDictionaryFieldByDisplayLabel('cdsFlowSheet', 'B/P',
          fnDict_Required);
        if ({(_s = '') or }(Uppercase(_s) = 'TRUE')) and (trim(aValue) = '')
          then
          Result := 'Entry value for required field B/P is not provided' + #13
      end
    else if not ValidBP(aValue) then
      begin
        fLow := GMVVitalLoRange[vtBP];
        fHigh := GMVVitalHiRange[vtBP];
        Result := 'Invalid format for B/P (' + aValue + ')' + #13 +
          'The entry should be in the following format:' + #13 +
          'nnn/nnn  or nnn/nnn/nnn' + #13 +
          'with values from  ' + FloatToStr(fLow) +
          ' to ' + FloatToStr(fHigh) + #13;
      end;
  end;

  function ValidField(aName, aValue: string): string;
  var
//    _Value: Double;
    _Value: Extended; // b. 312 #15
    sMin,
      sMax: string;
  begin
    Result := '';
    if (trim(aValue) = '') then
      begin
        sMin := getDictionaryFieldByDisplayLabel('cdsFlowSheet', aName,
          fnDict_Required);
        if ({(sMin = '') or }(Uppercase(sMin) = 'TRUE')) then
          Result := 'Entry value for required field ' + aName +
            ' is not provided' + #13
      end
    else
      begin
        sMin := getDictionaryFieldByDisplayLabel('cdsFlowSheet', aName,
          fnDict_MinValue);
        sMax := getDictionaryFieldByDisplayLabel('cdsFlowSheet', aName,
          fnDict_MaxValue);
        try
          _Value := strToFloat(aValue);
          if (sMin <> '') and (_Value < StrToFloat(sMin)) then
            Result := 'Entry Value for ' + aName + ' field (' + aValue +
              ') is below the valid minimum of ' + sMin + #13;
//          if (sMax <> '') and (_Value > StrToFloat(sMax)) then
          if (sMax <> '') and ( StrToFloat(sMax) < _Value) then
            Result := 'Value for ' + aName + ' field (' + aValue +
              ') exceeds the maximum of ' + sMax + #13;
        except
          on E: EConvertError do
            Result := 'Invalid integer of ' + aName + ' field (' + aValue + ')'
              +
              #13;
        end;
      end;
  end;

begin
  sMessage := '';
  for i := 0 to vleFlow.RowCount - 2 do
    begin
      sName := vleFlow.Strings.Names[i];
      sValue := vleFlow.Strings.Values[sName];

      if sName = 'Date' then
        sMessage := sMessage + ValidDate(sValue)
      else if sName = 'Time' then
        sMessage := sMessage + ValidTime(sValue)
      else if (pos('B/P', sName) > 0) then
        sMessage := sMessage + ValidBPressure(sValue)
      else
        sMessage := sMessage + ValidField(sName, sValue);
    end;

  Result := False;
  if sMessage <> '' then
    MessageDlg('Sorry, one or more of the input fields contains errors:' +
      #13#10#13#10 + sMessage +
      #13#10#13#10 + 'Please correct the value.', mtError, [mbOK], 0)
  else
    Result := True;
end;

procedure TfrmFlowSheetEdit.BitBtn2Click(Sender: TObject);
begin
  fIgnore := True;
  Close;
end;

end.
