unit fMDK_GeneralInfoDLG;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, DBCtrls, Mask, ActnList;

type
  TfrmGeneralInfo = class(TForm)
    gbObservations: TGroupBox;
    Panel69: TPanel;
    Panel1: TPanel;
    OkBtn: TBitBtn;
    CancelBtn: TBitBtn;
    BitBtn1: TBitBtn;
    pnlTreatmentInfo: TPanel;
    pnlInfo: TPanel;
    Label18: TLabel;
    Label33: TLabel;
    Label34: TLabel;
    Label35: TLabel;
    Label36: TLabel;
    Label38: TLabel;
    Label40: TLabel;
    dbeNephrologist: TDBEdit;
    dbcCodeStatus: TDBComboBox;
    dbcModality: TDBComboBox;
    dbeInitialTherapy: TDBEdit;
    dbeDiagnosisDate: TDBEdit;
    dbcESRD: TDBComboBox;
    dbeCTDate: TDBEdit;
    btnSelectProvider: TBitBtn;
    bbDiagnosisDate: TBitBtn;
    bbInitialTherapy: TBitBtn;
    Panel2: TPanel;
    bbTreatmentDate: TBitBtn;
    pnlSchedule: TPanel;
    Bevel11: TBevel;
    Label17: TLabel;
    dbcbMon: TDBCheckBox;
    dbcbTue: TDBCheckBox;
    dbcbWed: TDBCheckBox;
    dbcbTh: TDBCheckBox;
    dbcbFri: TDBCheckBox;
    dbcbSat: TDBCheckBox;
    dbcbSun: TDBCheckBox;
    pnlTransplantStatus: TPanel;
    Bevel1: TBevel;
    Label5: TLabel;
    dbcWIP: TDBCheckBox;
    dbcReferred: TDBCheckBox;
    rbNonCandidate: TRadioButton;
    rbCandidate: TRadioButton;
    ActionList1: TActionList;
    acTherapyDate: TAction;
    acDiagnosisDate: TAction;
    acTreatmentDate: TAction;
    Panel3: TPanel;
    acAttendingNephrologist: TAction;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure acTherapyDateExecute(Sender: TObject);
    procedure acDiagnosisDateExecute(Sender: TObject);
    procedure acTreatmentDateExecute(Sender: TObject);
    procedure dbeDiagnosisDateChange(Sender: TObject);
    procedure rbCandidateClick(Sender: TObject);
    procedure acAttendingNephrologistExecute(Sender: TObject);
  private
    Originals: TStringList;
    fIgnore: Boolean;
    { Private declarations }
    procedure SetUp;
    procedure SaveOriginals;
    procedure RestoreOriginals;
    procedure ReloadData;
  public
    { Public declarations }
  end;

var
  frmGeneralInfo: TfrmGeneralInfo;

function UpdateGeneralInfoData: Integer;

implementation

uses uMDK_Summary, uMDK_Info, dMDK_Data, fMDK_DateTimeSelector, DB,
  uMDK_Engine, uMDK_Utils, uMDK_Common, fMDK_SelectProvider;

{$R *.dfm}

function UpdateGeneralInfoData: Integer;
begin
  if not Assigned(frmGeneralInfo) then
    Application.CreateForm(TfrmGeneralInfo, frmGeneralInfo);
  frmGeneralInfo.setUp;
  frmGeneralInfo.SaveOriginals;
  result := frmGeneralInfo.ShowModal;
  if Result = mrCancel then
    frmGeneralInfo.RestoreOriginals
  else
    try
      if dmData.cdsSummary.State = dsEdit then
        dmData.cdsSummary.Post;
      if dmData.cdsInfo.State = dsEdit then
        dmData.cdsInfo.Post;
    except
    end;
  freeAndNil(frmGeneralInfo);
end;

////////////////////////////////////////////////////////////////////////////////

procedure TfrmGeneralInfo.FormCreate(Sender: TObject);
begin
  inherited;
  dbeCTDate.DataField := fnI_StudyDate;

  dbcESRD.DataField := fnI_Diagnosis;
  dbeDiagnosisDate.DataField := fnI_DiagnosisDate;
  dbeInitialTherapy.DataField := fnI_TherapyDate;
  dbcModality.DataField := fnI_Modality;
  dbcCodeStatus.DataField := fnI_CodeStatus;
  dbeNephrologist.DataField := fnI_Nephrologist;

  dbcbMon.DataField := fnI_Mon;
  dbcbTue.DataField := fnI_Tue;
  dbcbWed.DataField := fnI_Wed;
  dbcbTh.DataField := fnI_Thu;
  dbcbFri.DataField := fnI_Fri;
  dbcbSat.DataField := fnI_Sat;
  dbcbSun.DataField := fnI_sun;

  // Transplant Status /////////////////////////////////////////////////////////
  dbcWIP.DataField := fnS_TrnWIP;
  dbcReferred.DataField := fnS_TrnReferred;

  ReloadData;

  Originals := TStringList.Create;
end;

procedure TfrmGeneralInfo.FormDestroy(Sender: TObject);
begin
  FreeAndNil(Originals);
  inherited;
end;

procedure TfrmGeneralInfo.SetUp;

  function getValues(anOptionName: string): string;
  var
    sl: TStringList;
    i: integer;
  begin
    sl := nil;
    try
      Result := '';
      sl := TStringList.Create;
      sl.Text := Engine.getSystemOptionByName(anOptionName);
      for i := 0 to sl.Count - 1 do
        Result := result + piece(sl[i], '=', 2) + #13#10;
    finally
      sl.Free;
    end;
  end;

  procedure UpdateField(aField: TDBComboBox; sText: string);
  var
    sOldValue: string;
    i: Integer;
  begin
    try
      sOldValue := dmData.cdsInfo.FieldByName(aField.DataField).AsString;
      //aField.Text;
      aField.Items.Text := sText;
      if sOldValue <> '' then
        try
          i := aField.Items.IndexOf(sOldValue);
          if i > -1 then
            aField.ItemIndex := i;
        except
          on E: Exception do
            ShowMessage(E.Message);
        end;

    except
    end;
  end;

begin
  if not dmData.cdsInfo.Active then
    dmData.cdsInfo.Active := true;

  fIgnore := True;
  updateField(dbcESRD, getValues(onESRD_Diagnosys));
  updateField(dbcModality, getValues(onModalities));
  updateField(dbcCodeStatus, getValues(onCodeStatuses));
  fIgnore := False;

end;

procedure TfrmGeneralInfo.acTherapyDateExecute(Sender: TObject);
var
  sDate: string;
begin
  sDate := dbeInitialTherapy.Text;
  if getDateTime('Select Initial Therapy Date', 'mm/dd/yyyy', sDate) = mrOK then
    try
      //      fIgnore := True;
      if dmData.cdsInfo.State <> dsEdit then
        dmData.cdsInfo.Edit;
      dbeInitialTherapy.Text := sDate;
      dmData.cdsInfo.Post;
      //      fIgnore := False;
    except
      on E: Exception do
        ShowMessage(e.Message);
    end;
end;

procedure TfrmGeneralInfo.acDiagnosisDateExecute(Sender: TObject);
var
  sDate: string;
begin
  sDate := dbeDiagnosisDate.Text;
  if getDateTime('Select Diagnosis Date', 'mm/dd/yyyy', sDate) = mrOK then
    try
      //      fIgnore := True;
      if dmData.cdsInfo.State <> dsEdit then
        dmData.cdsInfo.Edit;
      dbeDiagnosisDate.Text := sDate;
      dmData.cdsInfo.Post;
      //      fIgnore := False;
    except
      on E: Exception do
        ShowMessage(e.Message);
    end;
end;

procedure TfrmGeneralInfo.acTreatmentDateExecute(Sender: TObject);
var
  sDate: string;
begin
  sDate := dbeCTDate.Text;
  if getDateTime('Select Treatment Date', 'mm/dd/yyyy', sDate) = mrOK then
    try
      if dmData.cdsInfo.State <> dsEdit then
        dmData.cdsInfo.Edit;
      dbeCTDate.Text := sDate;
      dmData.cdsInfo.Post;
    except
      on E: Exception do
        ShowMessage(e.Message);
    end;
end;

procedure TfrmGeneralInfo.SaveOriginals;

  procedure SaveItem(aName: string);
  begin
    try
      Originals.Add(aName + '=' +
        dmData.cdsInfo.FieldByName(aName).AsString);
    except
    end;
  end;

begin
  Originals.Clear;

  SaveItem(fnI_StudyDate);
  SaveItem(fnI_Diagnosis);
  SaveItem(fnI_DiagnosisDate);
  SaveItem(fnI_TherapyDate);
  SaveItem(fnI_Modality);
  SaveItem(fnI_CodeStatus);
  SaveItem(fnI_Nephrologist);

  SaveItem(fnI_Mon);
  SaveItem(fnI_Tue);
  SaveItem(fnI_Wed);
  SaveItem(fnI_Thu);
  SaveItem(fnI_Fri);
  SaveItem(fnI_Sat);
  SaveItem(fnI_Sun);

  SaveItem(fnS_TrnWIP);
  SaveItem(fnS_TrnReferred);
end;

procedure TfrmGeneralInfo.RestoreOriginals;
var
  sValue: string;

  procedure RestoreItem(aName: string);
  begin
    try
      sValue := Originals.Values[aName];
      dmData.cdsInfo.FieldByName(aName).AsString := sValue;
    except
    end;
  end;
begin
  with dmData.cdsSummary do
    begin
      if State <> dsEdit then
        Edit;
      RestoreItem(fnI_StudyDate);
      RestoreItem(fnI_Diagnosis);
      RestoreItem(fnI_DiagnosisDate);
      RestoreItem(fnI_TherapyDate);
      RestoreItem(fnI_Modality);
      RestoreItem(fnI_CodeStatus);
      RestoreItem(fnI_Nephrologist);

      RestoreItem(fnI_Mon);
      RestoreItem(fnI_Tue);
      RestoreItem(fnI_Wed);
      RestoreItem(fnI_Thu);
      RestoreItem(fnI_Fri);
      RestoreItem(fnI_Sat);
      RestoreItem(fnI_Sun);

      RestoreItem(fnS_TrnWIP);
      RestoreItem(fnS_TrnReferred);
      Post;
    end;

end;

procedure TfrmGeneralInfo.dbeDiagnosisDateChange(Sender: TObject);
begin
  if TDBEdit(Sender).Field.AsString = '12/30/1899' then
    TDBEdit(Sender).Text := '';
end;

procedure TfrmGeneralInfo.rbCandidateClick(Sender: TObject);
begin
  if fIgnore then
    Exit;
  try
    dmData.cdsSummary.Edit;
    dmData.cdsSummary.FieldByName(fnS_TrnCandidate).AsBoolean :=
      rbCandidate.Checked;
    if not rbCandidate.Checked then
      begin
        dmData.cdsSummary.FieldByName(fnS_TrnWIP).AsBoolean := False;
        dmData.cdsSummary.FieldByName(fnS_TrnReferred).AsBoolean := False;
      end;
    dmData.cdsSummary.Post;
  except
  end;
  dbcWIP.Enabled := rbCandidate.Checked;
  dbcReferred.Enabled := rbCandidate.Checked;
end;

procedure TfrmGeneralInfo.ReloadData;
begin
  try
    rbCandidate.Checked :=
      dmData.cdsSummary.FieldByName(fnS_TrnCandidate).AsBoolean;
    rbNonCandidate.Checked := not
      dmData.cdsSummary.FieldByName(fnS_TrnCandidate).AsBoolean;
  except
  end;
end;

procedure TfrmGeneralInfo.acAttendingNephrologistExecute(Sender: TObject);
var
  s: string;
begin
  s := SelectProvider('', Engine.fRenalController, 5);
  s := piece(s, '^', 1);
  if s <> '' then
    begin
      try
        if dmData.cdsInfo.State <> dsEdit then
          dmData.cdsInfo.Edit;
        dbeNephrologist.Text := s;
        dmData.cdsInfo.Post;
      except
        on E: Exception do
          ShowMessage(E.Message);
      end;
    end;
end;

end.
