unit fMDK_HospitalSelector;
{
================================================================================
*
*       Package:      MDK - Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 9/28/07 10:37a $
*       Site Name:    Hines OIFO
*       Date Created: 7/29/05
*       Developers:   andriyevskiy                  
*       Description:  This unit contains the Hemodialysis application
*                     Hospital selection dialog
*       Notes:
*
================================================================================
*       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/APP-HEMODIALYSIS/Forms/fMDK_HospitalSelector.pas $
*
*       $ History: fMDK_HospitalSelector.pas $
}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, ExtCtrls
  , StdCtrls, ComCtrls;

type
  TfrmMDK_HospitalSelector = class(TForm)
    gbHospital: TGroupBox;
    Panel1: TPanel;
    tmSearch: TTimer;
    lblLeft: TLabel;
    lvH: TListView;
    btnOK: TButton;
    Button2: TButton;
    cmbTarget: TComboBox;
    Label1: TLabel;
    procedure tmSearchTimer(Sender: TObject);
    procedure lvHChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure cmbTargetChange(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FormActivate(Sender: TObject);
    procedure cmbTargetKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure cmbTargetEnter(Sender: TObject);
    procedure cmbTargetExit(Sender: TObject);
    procedure lvHExit(Sender: TObject);
    procedure lvHEnter(Sender: TObject);
    procedure lvHKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure lvHDblClick(Sender: TObject);
  private
    { Private declarations }
    iCycle: Integer;
    fID,
      fName: string;
    procedure Search;
  public
    { Public declarations }
    property HospitalID: string read fID;
    property HospitalName: string read fName;
  end;

var
  frmMDK_HospitalSelector: TfrmMDK_HospitalSelector;

function SelectLocation(aDFN: string; var anID: string; var aName: string):
  Boolean;

implementation

uses uMDK_Engine, uROR_CommonUtils;

{$R *.DFM}
const
  clTabIn = clWindow;
  clTabOut = clInfoBk;

function SelectLocation(aDFN: string; var anID: string; var aName: string):
  Boolean;
begin
  Result := False;
  if not Assigned(frmMDK_HospitalSelector) then
    Application.CreateForm(TfrmMDK_HospitalSelector, frmMDK_HospitalSelector);
  if frmMDK_HospitalSelector = nil then
    exit;
{$IFDEF AANTEST}
  frmMDK_HospitalSelector.lblLeft.Visible := True;
{$ELSE}
  frmMDK_HospitalSelector.lblLeft.Visible := False;
{$ENDIF}
  if frmMDK_HospitalSelector.ShowModal = mrOK then
    begin
      frmMDK_HospitalSelector.tmSearch.Enabled := False;
      aName := frmMDK_HospitalSelector.HospitalName;
      anID := frmMDK_HospitalSelector.HospitalID;
      Result := True;
    end;
end;

procedure TfrmMDK_HospitalSelector.tmSearchTimer(Sender: TObject);
begin
  if iCycle > 1 then
    begin
      iCycle := iCycle - 1;
      lblLeft.Caption := IntToStr(iCycle);
    end
  else
    Search;
end;

procedure TfrmMDK_HospitalSelector.Search;
var
  tmpList: TStringList;
  i: integer;
  LI: TlistItem;
begin
  if (cmbTarget.Items.IndexOf(cmbTarget.Text) = -1) and (cmbTarget.Text <> '')
    then
    cmbTarget.Items.Add(cmbTarget.Text);
  tmSearch.Enabled := False;
  lblLeft.Caption := '';
  btnOK.Enabled := False;

  try
    tmpList := Engine.fRenalController.getLookupEntries('44', cmbTarget.Text);
    if tmpList.Count < 1 then
      begin
        MessageDlg('No entries found matching ''' + cmbTarget.Text + '''.',
          mtInformation, [mbok], 0);
        cmbTarget.SetFocus;
        cmbTarget.SelectAll;

        FreeAndNil(tmpList);
        Exit;
      end;

    if Piece(tmpList[0], '^', 1) = '-1' then
      begin
        MessageDlg(Piece(tmpList[0], '^', 2), mtError, [mbok], 0);
        cmbTarget.SelectAll;
        cmbTarget.SetFocus;
      end
    else if Piece(tmpList[0], '^', 1) = '0' then
      begin
        MessageDlg('No entries found matching ''' + cmbTarget.Text + '''.',
          mtInformation, [mbok], 0);
        cmbTarget.SetFocus;
        cmbTarget.SelectAll;
      end
    else
      begin
        tmpList.Delete(0); {Contains the header count record}
        lvH.Items.Clear;
        for i := 0 to tmpList.Count - 1 do
          begin
            li := lvH.Items.Add;
            li.Caption := Piece(tmpList[i], '^', 2);
            li.SubItems.Add(Piece(Piece(tmpList[i], '^', 1), ';', 2));
          end;
        lvH.SetFocus;
        lvH.ItemFocused := lvH.Items[0];
        lvH.ItemIndex := 0;
      end;
  finally
    FreeAndNil(tmpList);
  end;
end;

procedure TfrmMDK_HospitalSelector.lvHChange(Sender: TObject;
  Item: TListItem; Change: TItemChange);
begin
  if not assigned(lvH.Selected) then
    Exit;
  try
    fID := lvH.Selected.SubItems[0];
    fName := lvH.Selected.Caption;
    btnOK.Enabled := True;
    lblLeft.Caption := fName + '(' + fID + ')';
  except
  end;
end;

procedure TfrmMDK_HospitalSelector.cmbTargetChange(Sender: TObject);
begin
  if cmbTarget.Text = '' then
    Exit;
  tmSearch.Enabled := True;
  iCycle := 3;
  lblLeft.Caption := IntToStr(iCycle);
  tmSearch.Interval := 1000;
end;

procedure TfrmMDK_HospitalSelector.FormKeyPress(Sender: TObject;
  var Key: Char);
begin
  if Key = char(VK_ESCAPE) then
    ModalResult := mrCancel;
end;

procedure TfrmMDK_HospitalSelector.FormActivate(Sender: TObject);
begin
  try
    cmbTarget.SetFocus;
  except
  end;
end;

procedure TfrmMDK_HospitalSelector.cmbTargetKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if Key = VK_Return then
    Search;
end;

procedure TfrmMDK_HospitalSelector.cmbTargetEnter(Sender: TObject);
begin
  cmbTarget.Color := clTabIn;
end;

procedure TfrmMDK_HospitalSelector.cmbTargetExit(Sender: TObject);
begin
  cmbTarget.Color := clTabOut;
end;

procedure TfrmMDK_HospitalSelector.lvHExit(Sender: TObject);
begin
  lvH.Color := clTabOut;
end;

procedure TfrmMDK_HospitalSelector.lvHEnter(Sender: TObject);
begin
  lvH.Color := clTabIn;
end;

procedure TfrmMDK_HospitalSelector.lvHKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_RETURN:
      if (lvH.ItemIndex > -1) then
        begin
          Key := 0;
          fID := lvH.Selected.SubItems[0];
          fName := lvH.Selected.Caption;
          ModalResult := mrOK;
        end;
  end;
end;

procedure TfrmMDK_HospitalSelector.lvHDblClick(Sender: TObject);
begin
  fID := lvH.Selected.SubItems[0];
  fName := lvH.Selected.Caption;
  ModalResult := mrOK;
end;

end.
