unit fMDK_Medication;
{
================================================================================
*
*       Package:      MDK - Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 4/23/08 12:26p $
*       Site Name:    Hines OIFO
*       Date Created: 7/29/05
*       Developers:   andriyevskiy                  
*       Description:  this unit contains the Hemodialysis application
*                     Medication record dialog
*       Notes:
*
================================================================================
*       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI 1.0/APP-HEMODIALYSIS/Forms/fMDK_Medication.pas $
*
*       $ History: fMDK_Medication.pas $
 *
}
interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, DBCtrls, Mask
  , DBClient, ComCtrls
  ;

type
  TdlgMEdit = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    grbMedication: TGroupBox;
    Label9: TLabel;
    edDate: TEdit;
    Label1: TLabel;
    edTime: TEdit;
    Label2: TLabel;
    cmbMedications: TComboBox;
    Label3: TLabel;
    edDose: TEdit;
    Label4: TLabel;
    cmbUnits: TComboBox;
    Label5: TLabel;
    cmbRouts: TComboBox;
    Label6: TLabel;
    edGivenBy: TEdit;
    Label8: TLabel;
    edReason: TEdit;
    Label7: TLabel;
    mmOutcome: TMemo;
    grbStatus: TGroupBox;
    Label10: TLabel;
    cmbStatusName: TComboBox;
    Label11: TLabel;
    edStatusDate: TEdit;
    Label12: TLabel;
    edStatusBy: TEdit;
    pnlReplacementDetails: TPanel;
    pnlReplacement: TPanel;
    GroupBox1: TGroupBox;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    Label18: TLabel;
    Label19: TLabel;
    Label20: TLabel;
    Label21: TLabel;
    edReasonR: TEdit;
    edGivenByR: TEdit;
    cmbRoutsR: TComboBox;
    cmbUnitsR: TComboBox;
    edDoseR: TEdit;
    cmbMedicationsR: TComboBox;
    edTimeR: TEdit;
    edDateR: TEdit;
    Panel5: TPanel;
    OKbtn: TBitBtn;
    CancelBtn: TBitBtn;
    grbStatusR: TGroupBox;
    Label22: TLabel;
    Label23: TLabel;
    Label24: TLabel;
    cmbStatusNameR: TComboBox;
    edStatusDateR: TEdit;
    edStatusByR: TEdit;
    pnlSource: TPanel;
    Splitter1: TSplitter;
    edUpdatedBy: TEdit;
    ckbWrap: TCheckBox;
    mmOutcomeR: TRichEdit;
    lblCharCount: TLabel;
    Label25: TLabel;
    BitBtn3: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure dbmOutcomesChange(Sender: TObject);
    procedure dbeGivenByChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
    procedure ckbWrapClick(Sender: TObject);
    procedure mmOutcomeRKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure mmOutcomeRChange(Sender: TObject);
  private
    fLocked: Integer;
    fMode: string;
    fIgnore: Boolean;
    fStatusList: TStringList;
    { Private declarations }

    function getValidStatus: string;
    procedure setValidStatus(aValue: string);

    procedure ActivateDLG;
    procedure setComboByName(aCombo: TComboBox; aName: string);
  public
    { Public declarations }
    property ValidStatus: string read getValidStatus write setValidStatus;

    procedure ClearStringValues;
    procedure setStringValues;
    procedure setStatusValues;
    procedure updateValues;

    procedure getStringValues;
    function verifyStringValues: string;
    procedure setStatus(aReadOnly: Boolean);
  end;

var
  dlgMEdit: TdlgMEdit;

  //procedure EditMedication;
procedure EditMedicationStatus;
function AddMedication: string;

implementation

uses dMDK_Data, uMDK_MedList, uMDK_Engine, uMDK_CDSCommon, Dialogs,
  uMDK_Common, DB, fMDK_Flowsheet
  , uMDK_Options, uMDK_Flowsheet, uMDK_ReportEngine, uMDK_Utils,
  uMDK_Dictionary;

{$R *.dfm}
procedure EditMedicationStatus;
var
  bStatus: Boolean;
  dt: TDateTime;
begin
  if not Assigned(dlgMEdit) then
    Application.CreateForm(TdlgMEdit, dlgMedit);

  with dlgMEdit do
    begin
      ActivateDLG;
      Caption := 'Edit Medication';
      OkBtn.Caption := 'Update';
      fMode := 'UPDATE';

      getStringValues;

      //      grbMedication.Enabled := False;
      edStatusBy.Enabled := False;
      edStatusDate.Enabled := False;
      SetStatus(True);
      mmOutcome.Enabled := true;
      mmOutCome.ReadOnly := true;
      mmOutcome.Font.Color := clGrayText;

      //      grbStatus.Visible := true;
      //      grbStatusR.Visible := true;
      grbStatus.Visible := False;
      grbStatusR.Visible := False;

      pnlReplacementDetails.Visible := True;
      Width := 800;
      pnlReplacementDetails.Width := Width div 2;
      pnlSOurce.Color := clSilver;

      dt := Engine.getCurrentServerDateTime;
      edStatusDateR.Text := FormatDateTime('mm/dd/yyyy hh:nn', dt);
      edStatusByR.Text := Engine.CurrentOperatorID;
      edUpdatedBy.Text := Engine.CurrentOperatorID;
      edStatusByR.Enabled := False;
      edStatusDateR.Enabled := False;

      setComboByName(cmbStatusNameR, 'Valid');

      mmOutcomeR.Text := mmOutcome.Text + #13#10 +
        ' [' + edStatusDateR.Text + '  ' + edUpdatedBy.Text + '] ';

      fLocked := Length(mmOutcomeR.Text);

      ActiveControl := edDateR;

      if ShowModal = mrOK then
        try
          bStatus := frmFlow.tmrRefresh.Enabled;
          frmFlow.tmrRefresh.Enabled := False;

          updateValues;

          frmFlow.tmrRefresh.Enabled := bStatus;
        except
        end;
    end;
end;

function AddMedication: string;
var
  bStatus: Boolean;
  dt: TDateTime;

begin
  Result := 'OK';
  if not Assigned(dlgMEdit) then
    Application.CreateForm(TdlgMEdit, dlgMedit);

  with dlgMEdit do
    begin
      ActivateDLG;
      dt := Engine.getCurrentServerDateTime;
      Caption := 'Add Medication';
      OkBtn.Caption := 'Add';
      Width := 400;
      pnlReplacementDetails.Visible := False;
      pnlSOurce.Color := clBtnFace;
      fMode := 'ADD';

      ClearStringValues;
      edDate.Text := FormatDateTime('mm/dd/yyyy', dt);
      edTime.Text := FormatDateTime('hh:nn', dt);
      edGivenBy.Text := Engine.CurrentOperatorID;

      mmOutCome.ReadOnly := False;
      mmOutcome.Font.Color := clWindowText;
      grbMedication.Enabled := True;
      SetStatus(fALSE);
      edStatusBy.Enabled := True;
      edStatusDate.Enabled := True;
      ActiveControl := cmbMedications;

      grbStatus.Visible := False;

      if ShowModal = mrCancel then
        Result := 'CANCEL'
      else
        try
          bStatus := frmFlow.tmrRefresh.Enabled;
          frmFlow.tmrRefresh.Enabled := False;
          with dmData.cdsMedList do
            begin
              insert;

//              FieldByName(fnML_Status).AsString := fsValid; // 2007-08-01
              FieldByName(fnML_Status).AsString := 'NEW';
              FieldByName(fnML_StatusDate).AsDateTime := Now;
              FieldByName(fnML_StatusBy).AsString := Engine.CurrentOperatorID;

              setStringValues;
              Post;
            end;
          frmFlow.tmrRefresh.Enabled := bStatus;
        except
        end;
    end;
end;

////////////////////////////////////////////////////////////////////////////////

procedure TdlgMEdit.FormCreate(Sender: TObject);
begin
  fIgnore := False;
  fStatusList := TStringList.Create;
end;

procedure TdlgMEdit.setStatus(aReadOnly: Boolean);
var
  i: Integer;
begin
  for i := 0 to grbMedication.ControlCount - 1 do
    begin
      if grbMedication.Controls[i] is TEdit then
        begin
          //          TEdit(grbMedication.Controls[i]).Ctl3D := not aReadOnly;
          TEdit(grbMedication.Controls[i]).Enabled := not aReadOnly;
        end
      else if grbMedication.Controls[i] is TComboBox then
        begin
          //          TComboBox(grbMedication.Controls[i]).Ctl3D := not aReadOnly;
          TComboBox(grbMedication.Controls[i]).Enabled := not aReadOnly;
        end
      else if grbMedication.Controls[i] is TMemo then
        begin
          //          TMemo(grbMedication.Controls[i]).Ctl3D := not aReadOnly;
          TMemo(grbMedication.Controls[i]).Enabled := not aReadOnly;
        end
    end;
end;

procedure TdlgMEdit.ActivateDLG;
var
  i: Integer;
begin
  fStatusList.Text := Engine.getSystemOptionByName(onMedicationStatuses);
  cmbStatusName.Items.Clear;
  cmbStatusNameR.Items.Clear;
  for i := 0 to fStatusList.Count - 1 do
    begin
      cmbStatusName.Items.Add(fStatusList.Values[fStatusList.Names[i]]);
      cmbStatusNameR.Items.Add(fStatusList.Values[fStatusList.Names[i]]);
    end;

  AssignList(cmbMedications.Items, Engine.getSystemOptionByName(onMedications));
  AssignList(cmbRouts.Items, Engine.getSystemOptionByName(onMedicationRoutes));
  AssignList(cmbUnits.Items, Engine.getSystemOptionByName(onMedicationUnits));

  AssignList(cmbMedicationsR.Items,
    Engine.getSystemOptionByName(onMedications));
  AssignList(cmbRoutsR.Items, Engine.getSystemOptionByName(onMedicationRoutes));
  AssignList(cmbUnitsR.Items, Engine.getSystemOptionByName(onMedicationUnits));

end;

procedure TdlgMEdit.setValidStatus(aValue: string);
var
  s: string;
  i: Integer;
begin
  cmbStatusName.ItemIndex := -1;
  s := fStatusList.Values[aValue];
  for i := 0 to cmbStatusName.Items.Count - 1 do
    begin
      if cmbStatusName.Items[i] <> s then
        continue;
      cmbStatusName.ItemIndex := i;
      break;
    end;
end;

function TdlgMEdit.getValidStatus: string;
var
  i: Integer;
  s: string;
begin
  Result := '';
  for i := 0 to fStatusList.Count - 1 do
    begin
      s := fStatusList.Names[i];
      if fStatusList.Values[s] <> cmbStatusName.Text then
        continue;
      Result := fStatusList.Names[i];
      break;
    end;
end;

procedure TdlgMEdit.ClearStringValues;
begin
  with dmData.cdsMedList do
    begin
      edStatusBy.Text := '';
      edStatusDate.Text := '';
      cmbStatusName.ItemIndex := -1;

      edDate.Text := '';
      edTime.Text := '';
      edDose.Text := '';
      edReason.Text := '';
      edGivenBy.Text := '';
      try
        cmbMedications.Text := '';
      except
      end;
      cmbMedications.ItemIndex := -1;
      cmbRouts.ItemIndex := -1;
      cmbUnits.ItemIndex := -1;
      mmOutcome.Text := '';

      edStatusByR.Text := '';
      edStatusDateR.Text := '';
      cmbStatusNameR.ItemIndex := -1;

      edDateR.Text := '';
      edTimeR.Text := '';
      edDoseR.Text := '';
      edReasonR.Text := '';
      edGivenByR.Text := '';
      cmbMedicationsR.ItemIndex := -1;
      cmbRoutsR.ItemIndex := -1;
      cmbUnitsR.ItemIndex := -1;
      mmOutcomeR.Text := '';

    end;
end;

procedure TdlgMEdit.setComboByName(aCombo: TComboBox; aName: string);
var
  i: integer;
begin
  aCombo.ItemIndex := -1;
  for i := 0 to aCombo.Items.Count - 1 do
    begin
      if aCombo.Items[i] <> aName then
        continue;
      aCombo.ItemIndex := i;
      break;
    end;
  if i = aCombo.Items.Count then
    begin
      if aCombo.Style = csDropDown then
        aCombo.Text := aName;
    end;
end;

procedure TdlgMEdit.getStringValues;
var
  dValue: TDateTime;
  sValue: String;
begin
  with dmData.cdsMedList do
    begin
      edStatusBy.Text := FieldByName(fnML_StatusBy).AsString;
      edStatusDate.Text := FieldByName(fnML_StatusDate).AsString;

      ValidStatus := FieldByName(fnML_Status).AsString;

      edDate.Text := FieldByName(fnML_Date).AsString;
      edTime.Text := FieldByName(fnML_Time).AsString;

      setComboByName(cmbMedications, FieldByName(fnML_Medication).AsString);

      edDose.Text := FieldByName(fnML_Dose).AsString;
      edReason.Text := FieldByName(fnML_Reason).AsString;
      edGivenBy.Text := FieldByName(fnML_GivenBy).AsString;

      setComboByName(cmbRouts, FieldByName(fnML_Rout).AsString);
      setComboByName(cmbUnits, FieldByName(fnML_Units).AsString);

      mmOutcome.Text := FieldByName(fnML_Outcome).AsString;

      edDateR.Text := FieldByName(fnML_Date).AsString;
      dValue := FieldByName(fnML_Time).AsDateTime;
      dValue := dValue + 1/24/60/60;
      sValue := DateTimeToStr(dValue);
      edTimeR.Text := sValue;

      setComboByName(cmbMedicationsR, FieldByName(fnML_Medication).AsString);

      edDoseR.Text := FieldByName(fnML_Dose).AsString;
      edReasonR.Text := FieldByName(fnML_Reason).AsString;
      edGivenByR.Text := FieldByName(fnML_GivenBy).AsString;

      setComboByName(cmbRoutsR, FieldByName(fnML_Rout).AsString);
      setComboByName(cmbUnitsR, FieldByName(fnML_Units).AsString);

    end;
end;

procedure TdlgMEdit.setStatusValues;
begin
  with dmData.cdsMedList do
    begin
      FieldByName(fnML_StatusDate).AsDateTime :=
        Engine.getCurrentServerDateTime;
      //StrToDateTime(edStatusDate.Text);
      FieldByName(fnML_StatusBy).AsString := edStatusBy.Text;
      FieldByName(fnML_Status).AsString := ValidStatus;
    end;
end;

procedure TdlgMEdit.updateValues;
var
  sInitialTime,
  sOutcome,
    sID: string;

  function FieldsChanged: Boolean;
  begin
    Result := (edDate.Text <> edDateR.Text)
      or (edTime.Text <> edTimeR.Text)
      or (cmbMedications.Text <> cmbMedicationsR.Text)
      or (edDose.Text <> edDoseR.Text)
      or (cmbUnits.Text <> cmbUnitsR.Text)
      or (cmbRouts.Text <> cmbRoutsR.Text)
      or (edGivenBy.Text <> edGivenByR.Text)
      or (edReason.Text <> edreasonR.Text)
      ;
  end;

  procedure updateOutcome;
  begin
    if length(mmOutcomeR.Text) = fLocked then
      Exit;
    with dmData.cdsMedList do
      try
        if State <> dsEdit then
          Edit;
        FieldByName(fnML_Outcome).AsString := mmOutcomeR.Text;
      except
        on E: Exception do
          ShowMessage(E.Message);
      end;
  end;

  procedure setReplacementValues;
  begin
    with dmData.cdsMedList do
      begin
        FieldByName(fnML_Date).AsDateTime := StrToDateTime(edDateR.Text);
        FieldByName(fnML_Time).AsDateTime := StrToDateTime(edTimeR.Text);
        FieldByName(fnML_Medication).AsString := cmbMedicationsR.Text;
        FieldByName(fnML_Rout).AsString := cmbRoutsR.Text;
        FieldByName(fnML_Dose).AsString := edDoseR.Text;
        FieldByName(fnML_Reason).AsString := edReasonR.Text;
        FieldByName(fnML_GivenBy).AsString := edGivenByR.Text;
        FieldByName(fnML_Units).AsString := cmbUnitsR.Text;
        //        FieldByName(fnML_Outcome).AsString := mmOutcomeR.Text;
      end;
  end;

begin
  // was it only OUTCOME that had been changed?
  if (trim(mmOutcomeR.Text) <> '') and not FieldsChanged then
    begin
      // yes
      //    - update OUTCOME field only
      updateOutcome;
      dmData.cdsMedList.Post;
    end
  else
    begin
      // no
      with dmData.cdsMedList do
        begin
          disablecontrols;
          //    - change status of the current record to REPLACED

          sOutcome := FieldByName(fnML_Outcome).AsString;

          if State <> dsEdit then
            Edit;
          sID := FieldByName(fnML_ID).AsString;
          if sID = '' then
            begin
              sID := dmData.NextID;
              FieldByName(fnML_ID).AsString := sID;
            end;

          FieldByName(fnML_StatusDate).AsDateTime :=
            Engine.getCurrentServerDateTime;
          FieldByName(fnML_StatusBy).AsString := edStatusByR.Text;
          FieldByName(fnML_Status).AsString := 'REPLACED';
          sInitialTime := FieldByName(fnML_Time).AsString;
          Post;

          //    - create new record
          try
            Insert;
            //    - link to  original record
            FieldByName(fnML_ParentID).AsString := sID;
            //    - set status to VALID
            FieldByName(fnML_StatusDate).AsDateTime :=
              Engine.getCurrentServerDateTime;
            FieldByName(fnML_StatusBy).AsString := edStatusByR.Text;
//            FieldByName(fnML_Status).AsString := 'VALID';
            FieldByName(fnML_Status).AsString := 'NEW';
            //    - populate fields
            setReplacementValues;
            updateOutcome;
            //    - save data
            Post;
          except
            on E: Exception do
              ShowMessage(E.Message);
          end;
          EnableControls;
        end;
    end;
end;

procedure TdlgMEdit.setStringValues;
begin
  with dmData.cdsMedList do
    begin
      FieldByName(fnML_Date).AsDateTime := StrToDateTime(edDate.Text);
      FieldByName(fnML_Time).AsDateTime := StrToDateTime(edTime.Text);
      FieldByName(fnML_Medication).AsString := cmbMedications.Text;
      FieldByName(fnML_Rout).AsString := cmbRouts.Text;
      FieldByName(fnML_Dose).AsString := edDose.Text;
      FieldByName(fnML_Reason).AsString := edReason.Text;
      FieldByName(fnML_GivenBy).AsString := edGivenBy.Text;
      FieldByName(fnML_Units).AsString := cmbUnits.Text;
      FieldByName(fnML_Outcome).AsString := mmOutcome.Text;
    end;
end;

function TdlgMEdit.verifyStringValues: string;
var
  s: string;
  ac: TWinControl;

  function TestLength(aString,aMessage:String;aControl:TWinControl;aLimit:Integer):String;
  begin
    if Length(aString) > aLimit then
      begin
        Result := aMessage + ' value is longer then '+IntToStr(aLimit) +' characters'+ #13#10;
        if not Assigned(AC) then
          AC := aControl;
      end
    else
      Result := '';
  end;

  function TestFloat(aString,aMessage:String;aControl:TWinControl):String;
  begin
    Result := '';
    try
      strToFloat(aString);
    except
      begin
        Result := aMessage + ' "' + aString + '"' + #13#10;
        if not Assigned(ac) then
          AC := aControl;
      end;
    end;
  end;

  function TestDateTime(aString,aMessage:String;aControl:TWinControl):String;
  begin
    Result := '';
    if not IsValidDateString(aString) then
      begin
        if not Assigned(ac) then
          AC := aControl;
        Result := aMessage + ' "'+aString +'"'+ #13#10;
      end;
  end;


  function TestBlank(aString,aMessage:String;aControl:TWinControl;
    aColumn:String=''):String;
  var
    iRequired: Integer;
  begin
    Result := '';
    iRequired := 1;
    if aColumn <> '' then
      iRequired := getDictionaryRequired('cdsMedList',aColumn);
    if iRequired < 0 then Exit;

    if trim(aString) = '' then
      begin
        Result := aMessage + #13#10;
        if not Assigned(AC) then
          AC := aControl;
      end
    else
      Result := '';
  end;

begin
  s := '';
  ac := nil;
  if fMode = 'ADD' then
    begin
      s := s + testDateTime(edDate.Text,'Invalid Date value of ',edDate);
      s := s + testDateTime(edTime.Text,'Invalid Time value of ',edtime);
      s := s + testFloat(edDose.Text,'Invalid Dose value of ',edDose);

      s := s + testBlank(cmbMedications.Text,'Field "Medication" should be populated',cmbMedications{,fnML_Medication});
      s := s + testBlank(cmbUnits.Text,'Field "Units" should be populated',cmbUnits{,fnML_Units});
      s := s + testBlank(cmbRouts.Text,'Field "Route" should be populated',cmbRouts{,fnML_Rout});
      s := s + testBlank(edGivenBy.Text,'Field "Given By" should be populated',edGivenBy{, fnML_GivenBy});
      s := s + testBlank(edReason.Text,'Field "Reason" should be populated',edReason, fnML_Reason);

      s := s + testLength(mmOutcome.Text,'Outcome',mmOutCome,200);
    end
  else
    begin
      s := s + testDateTime(edDateR.Text,'Invalid Date value of ',edDateR);
      s := s + testDateTime(edTimeR.Text,'Invalid Time value of ',edtimeR);
      s := s + testFloat(edDoseR.Text,'Invalid Dose value of ',edDoseR);

      s := s + testBlank(cmbMedicationsR.Text,'Field "Medication" should be populated',cmbMedicationsR{, fnML_Medication});
      s := s + testBlank(cmbUnitsR.Text,'Field "Units" should be populated',cmbUnitsR{, fnML_Units});
      s := s + testBlank(cmbRoutsR.Text,'Field "Route" should be populated',cmbRoutsR{, fnML_Rout});
      s := s + testBlank(edGivenByR.Text,'Field "Given By" should be populated',edGivenByR{, fnML_GivenBy});
      s := s + testBlank(edReasonR.Text,'Field "Reason" should be populated',edReasonR, fnML_Reason);

      s := s + testLength(mmOutcomeR.Text,'Outcome',mmOutComeR,200);
    end;

  Result := s;

  if Assigned(ac) then
    try
      ActiveControl := ac;
    except
    end;
end;

procedure TdlgMEdit.dbmOutcomesChange(Sender: TObject);
begin
  if fIgnore then
    Exit;
  fIgnore := true;
  UpdateMemo(Sender, Engine.Signature);
  fIgnore := false;
end;

procedure TdlgMEdit.dbeGivenByChange(Sender: TObject);
begin
  if fIgnore then
    Exit;
  fIgnore := true;
  UpdateEdit(Sender, Engine.CurrentOperatorID);
  fIgnore := false;
end;

procedure TdlgMEdit.FormClose(Sender: TObject; var Action: TCloseAction);
var
  sMsg: string;

begin
  if (ModalResult = mrOK) then
    begin
      sMsg := verifyStringValues;
      if sMsg <> '' then
        begin
          MessageDlg('Update data and try again' + #13#10#13#10 + sMsg,
            mtWarning, [mbOK], 0);
          Action := caNone;
          Exit;
        end
    end;
end;

procedure TdlgMEdit.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_ESCAPE then
    ModalResult := mrCancel;
end;

procedure TdlgMEdit.FormDestroy(Sender: TObject);
begin
  fStatusList.Free;
  inherited;
end;

procedure TdlgMEdit.ckbWrapClick(Sender: TObject);
begin
  mmOutcome.WordWrap := ckbWrap.Checked;
  mmOutcomeR.WordWrap := ckbWrap.Checked;
  if ckbWrap.Checked then
    begin
      mmOutcome.ScrollBars := ssVertical;
      mmOutcomeR.ScrollBars := ssVertical;
    end
  else
    begin
      mmOutcome.ScrollBars := ssBoth;
      mmOutcomeR.ScrollBars := ssBoth;
    end;
end;

procedure TdlgMEdit.mmOutcomeRKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  mmOutcomeR.ReadOnly := mmOutcomeR.SelStart < fLocked;
  if (mmOutcomeR.SelStart = fLocked) and (Key = VK_BACK) then
    Key := 0
end;

procedure TdlgMEdit.mmOutcomeRChange(Sender: TObject);
begin
  lblCharCount.Caption := IntToStr(Length(mmOutcomeR.Text));
  if Length(mmOutcomeR.Text) > 1000 then
    lblCharCount.Font.Color := clRed
  else
    lblCharCount.Font.Color := clWindowText;
end;

end.
