unit fMDK_Options;
{
================================================================================
*
*       Package:      MDK - Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 4/23/08 3:29p $
*       Site Name:    Hines OIFO
*       Date Created: 7/29/05
*       Developers:   andriyevskiy                  
*       Description:  this unit contains the Hemodialysis application
*                     options definition form
*       Notes:
*
================================================================================
*       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI 1.0/APP-HEMODIALYSIS/Forms/fMDK_Options.pas $
*
* $ History: fMDK_Options.pas $
 *
}
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ActnList, Grids, ValEdit, Buttons, ExtCtrls
  , uMDK_Options, ImgList, StdCtrls, ComCtrls, AppEvnts
  , uMDK_Common
  ;

type
  TfrmMDK_Options = class(TForm)
    pnlOptionTools: TPanel;
    pnlOptionActions: TPanel;
    Panel1: TPanel;
    vleOptions: TValueListEditor;
    ActionList1: TActionList;
    acOptionAdd: TAction;
    acOptionDelete: TAction;
    acOptionLoad: TAction;
    acOptionSave: TAction;
    acSaveToDB: TAction;
    ImageList1: TImageList;
    sbSaveToFile: TBitBtn;
    sbLoadFromFile: TBitBtn;
    sbDelete: TBitBtn;
    sbAdd: TBitBtn;
    sbSaveToDB: TBitBtn;
    ColorDialog1: TColorDialog;
    acSetDefaults: TAction;
    acAssign: TAction;
    pnlOptionsAdmin: TPanel;
    Panel5: TPanel;
    bbtnAdd: TBitBtn;
    BitBtn2: TBitBtn;
    BitBtn3: TBitBtn;
    BitBtn4: TBitBtn;
    BitBtn5: TBitBtn;
    pnlUserPreferencesAdmin: TPanel;
    pnlUserAdminButtons: TPanel;
    BitBtn8: TBitBtn;
    BitBtn9: TBitBtn;
    BitBtn10: TBitBtn;
    BitBtn11: TBitBtn;
    pnlSystemPreferencesAdmin: TPanel;
    Panel11: TPanel;
    BitBtn15: TBitBtn;
    BitBtn16: TBitBtn;
    BitBtn17: TBitBtn;
    BitBtn18: TBitBtn;
    BitBtn6: TBitBtn;
    pnlOptionsUser: TPanel;
    Panel8: TPanel;
    BitBtn14: TBitBtn;
    BitBtn19: TBitBtn;
    BitBtn20: TBitBtn;
    pnlUserPreferencesUser: TPanel;
    Panel12: TPanel;
    BitBtn21: TBitBtn;
    BitBtn23: TBitBtn;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    BitBtn24: TBitBtn;
    Label7: TLabel;
    bbDefaults: TBitBtn;
    acDefaults: TAction;
    btnDefaults: TBitBtn;
    acCleanup: TAction;
    pnlComments: TPanel;
    mmComments: TMemo;
    dlgOpen: TOpenDialog;
    dlgSave: TSaveDialog;
    procedure acOptionAddExecute(Sender: TObject);
    procedure acOptionLoadExecute(Sender: TObject);
    procedure acOptionSaveExecute(Sender: TObject);
    procedure acOptionDeleteExecute(Sender: TObject);
    procedure vleOptionsGetEditText(Sender: TObject; ACol, ARow: Integer;
      var Value: string);
    procedure acSaveToDBExecute(Sender: TObject);
    procedure vleOptionsExit(Sender: TObject);
    procedure vleOptionsDblClick(Sender: TObject);
    procedure vleOptionsMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure vleOptionsDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure acSetDefaultsExecute(Sender: TObject);
    procedure vleOptionsSetEditText(Sender: TObject; ACol, ARow: Integer;
      const Value: string);
    procedure acAssignExecute(Sender: TObject);
    procedure acDefaultsExecute(Sender: TObject);
    procedure acCleanupExecute(Sender: TObject);
    procedure vleOptionsSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure vleOptionsEditButtonClick(Sender: TObject);
    procedure vleOptionsValidate(Sender: TObject; ACol, ARow: Integer;
      const KeyName, KeyValue: string);
    procedure vleOptionsKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure vleOptionsEnter(Sender: TObject);
  private
    { Private declarations }
    fIgnore: Boolean;
    fUpdated: Boolean;
    fLastName: string;
    fOptionsType: string;
    fReadOnly: Boolean;
    fOwnerTreeNodes: TTreeNodes;
    fOwnerNode: TTreeNode;
    fOptionName: string;

    procedure setLastName(aName: string);
    procedure setOptionsType(aName: string);
    procedure SaveOptions;
    procedure LoadOptions;
    procedure setReadOnly(aRO: Boolean);
    procedure RefreshOwnerNode;
  public
    { Public declarations }
    property OwnerTreeNode: TTreeNodes read fOwnerTreeNodes write
      fOwnerTreeNodes;
    property OwnerNode: TTreeNode read fOwnerNode write fOwnerNode;
    property OptionsType: string read fOptionsType write setOptionsType;
    property LastName: string read fLastName write setLastName;
    property Ignore: Boolean read fIgnore write fIgnore;
    property ROnly: Boolean read fReadOnly write setReadOnly;
    property ValueUpdated: Boolean read fUpdated write fUpdated;

    procedure SaveOptionList;
    procedure setRole(aRole: string);
    procedure setValueList(aValue: string);
    function AllowChange(var Reason: string): Boolean;
    function checkValue(anOption: string; var Reason: string): Boolean;
    procedure setComments(aRow: Integer);
    function checkValues: string;
  end;

var
  frmMDK_Options: TfrmMDK_Options;

function NewOptionForm(aLastName: string; oType: string = otSystem):
  TfrmMDK_Options;
procedure EditDomain(aDomainName: string);

implementation

uses
  uMDK_Engine
  , fMDK_OptionName,
  fMDK_HospitalSelector, fMDK_SelectProvider, fMDK_AddOptions, fMDK_Select,
  uMDK_Utils, fMDK_ListDLG;

{$R *.dfm}

function NewOptionForm(aLastName: string; oType: string = otSystem):
  TfrmMDK_Options;
begin
  Application.CreateForm(TfrmMDK_Options, Result);
  Result.fOptionsType := oType;
  Result.LastName := aLastName;
  Result.Ignore := False;
end;

procedure EditDomain(aDomainName: string);
var
  sOld: string;
begin
  if aDomainName = '' then
    Exit;
  with NewOptionForm(aDomainName) do
    begin
      BorderStyle := bsSizeable;
      Caption := 'Edit Domain <' + aDomainName + '>';
      sOld := vleOptions.Strings.Text;
      KeyPreview := True;
      setRole(rnAdmin);
      ShowModal;
      if sOld <> vleOptions.Strings.Text then
        acSaveToDB.Execute;
      Free;
    end;
end;

(*============================================================================*)

procedure TfrmMDK_Options.acOptionAddExecute(Sender: TObject);
var
  bIgnore: Boolean;
  opt: TUserOptions;
  sName,
    sValue: string;
  i: Integer;
begin
  if LastName = onAdministrators then
    begin
      sName := SelectProvider('', Engine.fRenalController, 2);
      if sName = '' then
        Exit
      else
        begin
          sValue := piece(sName, '^', 1);
          sName := piece(sName, '^', 2);
        end;
    end
  else if pos(onTIUNoteTitles, LastName) <> 0 then
    begin
      sName := SelectProvider('', Engine.fRenalController, 4);
      if sName = '' then
        Exit
      else
        begin
          sValue := piece(sName, '^', 1);
          sName := piece(sName,'^',2);
// 2007-07-27 vhaishandria          sName := '';
        end;
    end
  else if pos('User Options', LastName) <> 0 then
    begin
      AddOptions;
      vleOptions.Strings.Assign(userOptions.Values);
      Exit;
    end
  else if (EditOption('?', '') <> mrCancel) then
    begin
      sName := dlgOption.edName.Text;
      sValue := dlgOption.edValue.Text;
    end
  else
    Exit;

  begin
    if sName = '' then
      sName := IntToStr(vleOptions.Strings.Count);

    opt := nil;
    if (LastName = onSysOptions) then
      opt := sysOptions;
    if (LastName = onUserOptions) then
      opt := userOptions;

    if ((opt <> nil) and (opt.sValue(sname) = yoUnKnown)) then
      begin
        opt.AddNameValue(sName, sValue);
        vleOptions.Strings.Add(sName + '=' + sValue);
        ValueUpdated := True;
      end;

    i := 0;
    while i < vleOptions.Strings.Count do
      begin
        try
          if pos(sName + '='+ sValue, vleOptions.Strings[i]) = 1 then
            begin
              MessageDLG('Record:'+#13+
              sName + ' = ' + sValue + #13 +
                'Already included in the list', mtInformation,[mbOK],0);
              break;
            end;
        except
        end;
        inc(i);
      end;
    if i > vleOptions.Strings.Count - 1 then
      begin
        vleOptions.Strings.Add(sName + '=' + sValue);
      end
    else
      begin
        bIgnore := fIgnore;
        fIgnore := True;
        try
          vleOptions.Row := i;
        except
        end;
        fIgnore := bIgnore;
      end;
  end;
  RefreshOwnerNode;
  acSaveToDBExecute(nil);
end;

procedure TfrmMDK_Options.acOptionLoadExecute(Sender: TObject);
var
  i: Integer;
begin
  if dlgOpen.Execute then
    begin
      vleOptions.Strings.LoadFromFile(dlgOpen.FileName);
      ValueUpdated := true;
      for i := 0 to vleOptions.Strings.Count - 1 do
        if vleOptions.Strings.Names[i] = '' then
          vleOptions.Strings[i] := Format('%4.4d =%s', [i + 1,
            vleOptions.Strings[i]]);
    end;
  RefreshOwnerNode;
end;

procedure TfrmMDK_Options.acOptionSaveExecute(Sender: TObject);
begin
  if dlgSave.Execute then
    vleOptions.Strings.SaveToFile(dlgSave.FileName);
end;

procedure TfrmMDK_Options.SaveOptions;
begin
  CheckValues;

  if not Assigned(Engine) then
    exit;
  if (OptionsType = otUser) then
    begin
      Engine.setUserOptionByName(LastName, vleOptions.Strings.Text);
      userOptions.Overwrite(vleOptions.Strings.Text); // vhaishandria 060629
      ValueUpdated := False;
    end
  else if (OptionsType = otSystem)
{$IFNDEF AANTEST} and (Engine.CurrentOperatorMode = rnADMIN){$ENDIF} then
    begin
      Engine.setSystemOptionByName(LastName, vleOptions.Strings.Text);

//      if (LastName = onAdministrators) and Assigned(Administrators) then
//        Administrators.Values.Text := vleOptions.Strings.Text;

      ValueUpdated := False;
    end;

  if ((LastName = onUserOptions) or (LastName = onSysOptions))
//{$IFNDEF AANTEST} and (Engine.CurrentOperatorMode = rnADMIN){$ENDIF}  // vhaishandria 2008-04-02
then
    begin
      if LastName = onUserOptions then
        userOptions.ValueList.Text := vleOptions.Strings.Text;
      if LastName = onSysOptions then
        sysOptions.ValueList.Text := vleOptions.Strings.Text;
      try
        Preferences.ValueList.Text := '';
        Preferences.Overwrite(sysOptions.ValueList.Text);
        Preferences.Overwrite(userOptions.ValueList.Text);
        Preferences.Overwrite(userSettings.ValueList.Text);
      except
      end;
    end;
end;

procedure TfrmMDK_Options.SaveOptionList;
begin
  if not ValueUpdated then
    exit;
  if (vleOptions.Strings.Count = 0) then
    Exit; // we are not saving blank lists
  if LastName = onPreferences then
    Exit; // vhaishandria 061011
  SaveOptions;
end;

procedure TfrmMDK_Options.setLastName(aName: string);
var
  _Ignore: Boolean;
  i: Integer;
  sName,
    s: string;

  function sortStringList(aString: string): string;
  var
    _sl: TStringList;
  begin
    Result := '';
    if aString = '' then
      Exit
    else
      begin
        _sl := TStringList.Create;
        _sl.Text := aString;
        _sl.Sort;
        Result := _sl.Text;
        _sl.Free;
      end;
  end;

begin
  pnlComments.Visible :=
    (aName = onPreferences) or
    (aName = onSysOptions) or
    (aName = onUserOptions)
    ;

  if not Assigned(Engine) then
    Exit;

  _Ignore := fIgnore;
  fIgnore := True;
  try
    if (fLastName <> aName) then
      SaveOptionList;

    vleOptions.Strings.Clear;
    fUpdated := False;
    fLastName := aName;

    if OptionsType = otSystem then
      s := Engine.getSystemOptionByName(fLastName)
    else
      s := Engine.getUserOptionByName(fLastName);

    if pos('-1', s) <> 1 then
      begin
        s := sortStringList(s);

        if aName = onSysOptions then
          begin
            sysOptions.Update(s);
            s := sysOptions.ValueList.Text;
          end
        else if aName = onUserOptions then
          begin
            userOptions.Update(s);
            s := userOptions.ValueList.Text;
          end;

        vleOptions.Strings.Text := sortStringList(s);

        while (vleOptions.Strings.Count > 0)
          and ((trim(vleOptions.Strings[0]) = '') or (trim(vleOptions.Strings[0])
          = '=')) do
          vleOptions.Strings.Delete(0);

        for i := 0 to vleOptions.Strings.Count - 1 do
          begin
            s := uppercase(trim(piece(vleOptions.Strings[i], '=', 2)));
            sName := uppercase(trim(piece(vleOptions.Strings[i], '=', 1)));
            if (s = 'TRUE') or (s = 'FALSE') then
              begin
                vleOptions.ItemProps[i].EditStyle := esPickList;
                vleOptions.ItemProps[i].PickList.CommaText := 'TRUE,FALSE';
              end
            else if pos('PAIN LEVEL', sName) > 0 then
              begin
                vleOptions.ItemProps[i].EditStyle := esPickList;
                vleOptions.ItemProps[i].PickList.CommaText :=
                  '0,1,2,3,4,5,6,7,8,9,99';
                vleOptions.ItemProps[i].EditMask := '!9';
              end;
            if (fLastName = onSysOptions) then
              begin
{$IFDEF STUDY_STATUS_LIST}
                if
//                  (Uppercase(onAllowUserResetStudyStatus)=sName) or
                  (Uppercase(onAllowUserResetActiveStudyList)=sName) or
                  (Uppercase(onAllowUserReleaseSelectedStudy)=sName) or       // vhaishandria 2007-10-04
                  (Uppercase(onAllowUserReleaseStudySelectedBefore)=sName) // vhaishandria 2007-10-04
                  then
                  begin
                    vleOptions.ItemProps[i].ReadOnly := True;
                    vleOptions.ItemProps[i].EditStyle := esPickList;
                    vleOptions.ItemProps[i].PickList.CommaText := '';
                  end;
{$ENDIF}
              end;

{$IFDEF ALLOWTRUE}
            if aName = onUserOptions then
              begin
                if pos('ALLOW USER', sName) = 1 then
                  begin
                    vleOptions.Values[sName] := 'TRUE';
                    vleOptions.ItemProps[i].ReadOnly := True;

                    vleOptions.ItemProps[i].EditStyle := esSimple;
                  end;
              end;
{$ENDIF}
          end;
      end;
  except
  end;

  if VleOptions.Strings.Count > 0 then
    try
      vleOptions.Row := 1;
    except
    end;

  fIgnore := _Ignore;
  RefreshOwnerNode;
end;

procedure TfrmMDK_Options.acOptionDeleteExecute(Sender: TObject);
var
  aN, aV: string;
begin
  if vleOptions.Strings.Count < 1 then
    Exit;

  aN := Piece(vleOptions.Strings[vleOptions.Row - 1], '=', 1);
  aV := Piece(vleOptions.Strings[vleOptions.Row - 1], '=', 2);
  if MessageDlg('You are about to delete ' + #13#10#13#10 +
    'Item:  "' + aN + '"' + #13#10 +
    'Value: "' + aV + '"' + #13#10#13#10 +
    'From the list of  "' + LastName + '"' + #13#10 +
    'Do you want to continue?',
    mtConfirmation, [mbYes, mbNo], 0) = mrYes then
    begin
      vleOptions.Strings.Delete(vleOptions.Row - 1);
      if OptionsType = otSystem then
        Engine.deleteSystemOption(aN)
      else
        Engine.deleteUserOption(aN);
      ValueUpdated := True;
    end;
  RefreshOwnerNode;
//  acSaveToDBExecute(nil);
end;

procedure TfrmMDK_Options.vleOptionsGetEditText(Sender: TObject; ACol,
  ARow: Integer; var Value: string);
begin
  if Ignore then
    Exit;
  ValueUpdated := true;
  RefreshOwnerNode;
end;

procedure TfrmMDK_Options.setOptionsType(aName: string);
begin
  if (aName <> otUser) and (aName <> otSystem) and (aName <> otDeveloper) then
    Exit;
  fOptionsType := aName;
end;

procedure TfrmMDK_Options.LoadOptions;
begin
  SetLastName(fLastName);
end;

procedure TfrmMDK_Options.acSaveToDBExecute(Sender: TObject);
var
  s: string;
begin
  if ((LastName = onUserOptions) or (LastName = onSysOptions)) and
    not AllowChange(s) then
    MessageDLG('Please, fix errors prior to saving values.' + #13#10#13#10 + s,
      mtWarning, [mbOk], 0)
  else
    SaveOptions;
end;

procedure TfrmMDK_Options.setReadOnly(aRO: Boolean);
begin
  try
    fReadOnly := aRO;

    if not aRO then
      vleOptions.Color := clWindow
    else
      vleOptions.Color := clInfoBk;

    if not aRO then
      vleOptions.Options := vleOptions.Options + [goEditing]
    else
      vleOptions.Options := vleOptions.Options - [goEditing]
  except
  end;
end;

procedure TfrmMDK_Options.setRole(aRole: string);
begin
  try
{$IFDEF AANTEST}
    ROnly := False;
    pnlOptionTools.Visible := True;
{$ELSE}
    ROnly := (aRole <> rnAdmin) and (LastName <> 'User Options');
    pnlOptionTools.Visible := false;
{$ENDIF}

    if LastName = 'User Options' then
      begin
        pnlOptionsAdmin.Visible := False;
        pnlOptionsUser.Visible := False;

        pnlSystemPreferencesAdmin.Visible := False;
        pnlUserPreferencesUser.Visible := aRole <> rnAdmin;
        pnlUserPreferencesAdmin.Visible := aRole = rnAdmin;
      end
    else if LastName = 'System Options' then
      begin
        pnlOptionsAdmin.Visible := False;
        pnlOptionsUser.Visible := aRole <> rnAdmin;

        pnlSystemPreferencesAdmin.Visible := aRole = rnAdmin;
        pnlUserPreferencesUser.Visible := False;
        pnlUserPreferencesAdmin.Visible := False;
      end
    else if LastName = 'Preferences' then
      begin
        ROnly := True;
        pnlOptionsAdmin.Visible := False;
        pnlOptionsUser.Visible := True;

        pnlSystemPreferencesAdmin.Visible := False;
        pnlUserPreferencesUser.Visible := False;
        pnlUserPreferencesAdmin.Visible := False;
      end
    else
      begin
        pnlOptionsAdmin.Visible := aRole = rnAdmin;
        pnlOptionsUser.Visible := aRole <> rnAdmin;

        pnlSystemPreferencesAdmin.Visible := False;
        pnlUserPreferencesUser.Visible := False;
        pnlUserPreferencesAdmin.Visible := False;

        btnDefaults.Visible := (aRole = rnAdmin) and
          (
          (LastName = onAntiCoagulants) or
          (LastName = onNephrologiests) or
          (LastName = onCodeStatuses) or
          (LastName = onDialyzerList) or
          (LastName = onEducationCodes) or
          (LastName = onESRD_Diagnosys) or
          (LastName = onMedicalFacilities) or
          (LastName = onMedicationRoutes) or
          (LastName = onMedicationUnits) or
          (LastName = onModalities) or
          (LastName = onSurgeons) or
          (LastName = onTransType) or
          (LastName = onTIUNoteTitles) or
          (LastName = onMedications)
          );
        Panel5.Visible := not
          ((LastName=onMedicationStatuses) or (LastName=onMedicationReasons));
        vleOptions.Visible := not
          ((LastName=onMedicationStatuses) or (LastName=onMedicationReasons));
      end;

  except
    on E: Exception do
      MessageDlg('Error setting role of <' + aRole + '>:'#13#10#13#10 +
        E.Message, mtWarning, [mbOK], 0);
  end;
end;

procedure TfrmMDK_Options.vleOptionsExit(Sender: TObject);
begin
  if fIgnore then Exit;
  SaveOptions;
end;

procedure TfrmMDK_Options.setValueList(aValue: string);
begin
  vleOptions.Strings.Text := aValue;
end;

procedure TfrmMDK_Options.RefreshOwnerNode;
var
  tn0: TTreeNode;
  i: integer;
  tn: TTreeNode;
  bEx: Boolean;
begin
  if fOwnerNode = nil then
    Exit;
  if fOwnerTreeNodes = nil then
    Exit;

  tn0 := fOwnerNode;
  bEx := tn0.Expanded;
  tn0.DeleteChildren;

  if vleOptions.Strings.Count = 0 then
    Exit;
  for i := 0 to vleOptions.Strings.Count - 1 do
    begin
      tn := fOwnerTreeNodes.AddChild(tn0, vleOptions.Strings.Names[i]);
      tn.ImageIndex := tn0.ImageIndex;
      tn.SelectedIndex := tn0.SelectedIndex;
    end;
  tn0.Expanded := bEx;
end;

procedure TfrmMDK_Options.vleOptionsDblClick(Sender: TObject);
var
  tn: TTreeNode;
  aValue: string;
begin
{$IFDEF STUDY_STATUS_LIST}
  // 2007-10-02 vhaishandria --------------------------------------------- begin
  if (fLastName = onSysOptions) and
    (
    (onAllowUserResetActiveStudyList=fOptionName)
    or (onAllowUserReleaseSelectedStudy=fOptionName)
    or (onAllowUserReleaseStudySelectedBefore=fOptionName)
    ) then
    begin
      MessageDLG('The value of "'+fOptionName+ '" can be set at the User Preference level only.',
        mtInformation,[mbOK],0);
      Exit;
    end;
  // 2007-10-02 vhaishandria ----------------------------------------------- end
{$ENDIF}

  if (fOptionName = '') and
    (vleOptions.Strings.Count > 1) and
    (vleOptions.Row > 0) then
    fOptionName := vleOptions.Keys[vleOptions.Row];

  //  if (pos('Allow USER',fOptionName) = 1) and (fOptionsType =otUser) then Exit;

  if pos('COLOR', Uppercase(fOptionName)) <> 0 then
    begin
      if fReadOnly then
        Exit;
      if ColorDialog1.Execute then
        vleOptions.Values[fOptionName] := IntToStr(ColorDialog1.Color);
      exit;
    end;
  aValue := vleOptions.Values[fOptionName];

  if aValue = 'TRUE' then
    begin
      if fReadOnly then
        Exit;
      vleOptions.Values[fOptionName] := 'FALSE';
      exit;
    end;
  if aValue = 'FALSE' then
    begin
      if fReadOnly then
        Exit;
      vleOptions.Values[fOptionName] := 'TRUE';
      exit;
    end;

  if fOwnerNode = nil then
    Exit;
  if fOwnerTreeNodes = nil then
    Exit;
  if fOptionName <> '' then
    begin
      tn := fOwnerNode.getFirstChild;
      repeat
        if tn = nil then
          Break;
        tn := tn.getNextSibling;
      until (tn = nil) or (tn.Text = fOptionName);
    end;
end;

procedure TfrmMDK_Options.vleOptionsMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Column, Row: Longint;
begin
  try
    vleOptions.MouseToCell(X, Y, Column, Row);
    fOptionName := vleOptions.Cells[0, Row];
  except
    fOptionName := '';
  end;
end;

procedure TfrmMDK_Options.vleOptionsDrawCell(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
var
  s: string;
  i: Integer;
begin
  with vleOptions.Canvas do
    begin
      Font.Color := clWindowText;
      //      if aCol <> 0 then
      //        Font.Color := clNavy;

      Font.Style := [];
      if aRow = 0 then
        Brush.Color := clBtnFace
      else
        begin
          if fReadOnly then
            begin
              Brush.Color := clInfoBk;
            end
          else
            begin
              if (aCol = 0) then
                Brush.Color := clWindow
              else
                begin
                  s := vleOptions.Cells[0, aRow];
                  if pos('Color', s) > 0 then
                    i := StrToIntDef(vleOptions.Cells[1, aRow], clWindow)
                  else if fReadOnly then
                    i := clInfoBk
                  else
                    i := clWindow;
                  //                  if pos('Allow USER',s) = 1 then
                  //                    Font.Style := [fsBold];

                  Brush.Color := i;
                end
            end;
        end;
      FillRect(Rect);
      TextOut(Rect.Left + 2, Rect.Top + 2, vleOptions.Cells[aCol, aRow]);
    end;
end;

procedure TfrmMDK_Options.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_ESCAPE then
    ModalResult := mrCancel;
end;

procedure TfrmMDK_Options.vleOptionsSetEditText(Sender: TObject; ACol,
  ARow: Integer; const Value: string);
begin
  ValueUpdated := True;
end;

procedure TfrmMDK_Options.acAssignExecute(Sender: TObject);
begin
  AddOptions(True, self);
  LoadOptions;
end;

procedure TfrmMDK_Options.acDefaultsExecute(Sender: TObject);
var
  b: Boolean;
  sMSG,
    s: string;
begin
  s := getDefaultResource(LastName);
  if s = '' then
    begin
      sMSG := 'The default list of values is blank' + #13#10 +
        'Do you want to clear the current list?';
      b := MessageDlg(sMsg, mtConfirmation, [mbYes, mbNo], 0) = mrYes;
    end
  else
    b := IsReplacementNeeded(s);

  if b then
    try
      vleOptions.Strings.Text := s;
    except
    end;
end;

procedure TfrmMDK_Options.acSetDefaultsExecute(Sender: TObject);
begin
  if MessageDlg('Do you want to replace the current list with the default one?',
    mtConfirmation, [mbYes, mbNo], 0) <> mrYes then
    exit;

  SetSystemDefaults;
  Preferences.Overwrite(sysOptions.ValueList.Text);
  Preferences.Overwrite(userOptions.ValueList.Text);
  vleOptions.Strings.Text := sysOptions.ValueList.Text;

  CheckSummaryReport; // vhaishandria 2008-04-01

end;

procedure TfrmMDK_Options.acCleanupExecute(Sender: TObject);
begin
  Engine.CleanUpOptions;
end;

procedure TfrmMDK_Options.vleOptionsSelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
begin
  setComments(aRow);
end;

procedure TfrmMDK_Options.vleOptionsEditButtonClick(Sender: TObject);
var
  s: string;
begin
  try
    s := Engine.getSystemOptionByName(onTIUNoteTitles);
    if s = '' then
      MessageDlg('To select a default TIU title' + #13 + #10 +
        'Create the titles list first.', mtWarning, [mbOK], 0)
    else
      begin
        s := SelectListItem(s);
        if s <> '' then
          vleOptions.Cells[1, vleOptions.Row] := s;
      end;
  except
  end;
end;

procedure TfrmMDK_Options.vleOptionsKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  sName,
    sValue, sDefault: string;
begin
  if (Key = VK_RETURN) and (ssCtrl in Shift) and (ssShift in Shift) then
    begin
      sName := vleOptions.Cells[0, vleOptions.Row];
      if (LastName <> onSystemPreferences) and (LastName <> onUserPreferences)
        and
        (LastName <> 'System Options') and (LastName <> 'User Options') then
        Exit;
      sDefault := getSystemDefault(sName);
      sValue := vleOptions.Values[sName];
      if (sDefault = '') then
        Exit;
      if sDefault <> sValue then
        begin
          if
            MessageDlg('Option selected: ' + sName + #13#10#13#10 +
            'Current Value: ' + sValue + #13#10 +
            'Default Value: ' + sDefault + #13#10#13#10 +
            'Do you want to replace the current value with the default one?',
            mtConfirmation, [mbNo, mbOK], 0) = mrOK then
            vleOptions.Values[sName] := sDefault;
        end
      else
        MessageDlg('Option selected: ' + sName + #13#10#13#10 +
          'Current Value: ' + sValue + #13#10 +
          'Default Value: ' + sDefault, mtInformation, [mbOK], 0);
    end;
end;

const
  ValidPainLevel = '0,1,2,3,4,5,6,7,8,9,99,';

function TfrmMDK_Options.checkValue(anOption: string; var Reason: string):
  Boolean;
var
  sl: TStringList;
  i: Integer;
  sV, sMin, sMax,
    upValue,
    upName: string;
begin
  sV := vleOptions.Values[anOption];
  upValue := Uppercase(sV);
  upName := Uppercase(anOption);

  if (pos(onSummaryReportName, anOption) <> 0) then
    begin
      upValue := Engine.getSystemOptionByName(sV);
      Result := UpValue<>'';
      Reason := 'Report name <' + sV + '> is not found';
(**)
      upValue := Engine.getSystemOptionByName(onReportList);
      sl := TStringList.Create;
      sl.Text := upValue;
      for i := 0 to sl.Count - 1 do
        begin
          if pos(sV, sl[i]) = 1 then
            break;
        end;
//      Result := i < sl.Count;
      if i=sl.Count then
          Engine.setSystemOptionByName(onReportList,sV+'='+sV+#13#10+upValue);
      sl.Free;
(**)
    end
  else
    begin
      sMin := PreferenceComments.sMinValue(anOption);
      sMax := PreferenceComments.sMaxValue(anOption);
      Result := CheckOption(upName, upValue, sMin, sMax, Reason);
    end;
end;

procedure TfrmMDK_Options.vleOptionsValidate(Sender: TObject; ACol,
  ARow: Integer; const KeyName, KeyValue: string);
var
  sReason: string;
begin
  if not CheckValue(KeyName, sReason) then
    begin
      vleOptions.Values[KeyName] := getSystemDefault(KeyName);
      raise Exception.Create(sReason + #13#10#13#10 +
        'The invalid value has been replaced' + #13#10 +
        'with the default of "' + getSystemDefault(KeyName) + '"');
    end
end;

function TfrmMDK_Options.AllowChange(var Reason: string): Boolean;
var
  i: integer;
  KeyValue,
    KeyName: string;
begin
  Result := True;
  //  if fReadOnly then Exit;
  for i := 1 to vleOptions.RowCount - 1 do
    begin
      KeyValue := vleOptions.Cells[1, i];
      KeyName := vleOptions.Cells[0, i];
      Result := CheckValue(KeyName, Reason);
      if not Result then
        break;
    end;
end;

procedure TfrmMDK_Options.vleOptionsEnter(Sender: TObject);
begin
  setComments(vleOptions.Row);
end;

procedure TfrmMDK_Options.setComments(aRow: Integer);
var
  s, sOutput,
    sName, sValue: string;
  i: integer;
begin
  if not pnlComments.Visible then
    Exit;

  //  if aRow < 0 then Exit;

  sName := vleOptions.Cells[0, aRow];
  sValue := PreferenceComments.stringValue(sName);

  if sValue = yoUnKnown then
    mmComments.Lines.Text := ''
  else
    begin
      sOutput := '';
      for i := 1 to Length(sValue) do
        begin
          s := copy(sValue, i, 1);
          if s = '^' then
            sOutput := sOutput + #13#10
          else
            sOutput := sOutput + s;
        end;
      sOutput := piece(sOutput, '<MIN>', 1);
      sOutput := piece(sOutput, '<MAX>', 1);
      mmComments.Lines.Text := sOutput;
    end;
end;

function TfrmMDK_Options.checkValues: string;
var
  sName
    , sDefault, sReason: string;
  i: integer;
begin
  Result := '';
  if ((LastName <> onUserOptions) and (LastName <> onSysOptions)) then
    Exit;
  for i := 0 to vleOptions.Strings.Count - 1 do
    begin
      sName := vleOptions.Keys[i];
      if not CheckValue(sName, sReason) then
        begin
          Result := Result + vleOptions.Values[sName];
          sDefault := getSystemDefault(sName);
          vleOptions.Values[sName] := sDefault;
          Engine.Report(rtWarning, sReason + #13#10 + 'Value set to "' + sDefault
            + '"', '', 0, '', LastName);
          Result := Result + '  set to default of "' + sDefault + '"' + #13#10
        end;
    end;
end;

end.
