unit fMDK_TIUNote;
{
================================================================================
*
*       Package:      MDK - Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 4/17/08 4:00p $
*       Site Name:    Hines OIFO
*       Date Created: 7/29/05
*       Developers:   andriyevskiy                  
*       Description:  this unit contains the Hemodialysis application
*                     TIU Note form
*       Notes:
*
================================================================================
*       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/APP-HEMODIALYSIS/Forms/fMDK_TIUNote.pas $
*
*       $ History: fMDK_TIUNote.pas $
}
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, ComCtrls;

type
  TfrmNote = class(TForm)
    Panel2: TPanel;
    pnlSignature: TPanel;
    bbtnSave: TBitBtn;
    BitBtn3: TBitBtn;
    Panel3: TPanel;
    pnlTiuText: TPanel;
    pnlTIUTemplate: TPanel;
    pnlTiuTitle: TPanel;
    GroupBox1: TGroupBox;
    cbTIUNoteTitles: TComboBox;
    GroupBox2: TGroupBox;
    lbTemplates: TListBox;
    GroupBox3: TGroupBox;
    Panel4: TPanel;
    Panel7: TPanel;
    Label6: TLabel;
    Label1: TLabel;
    edProvider: TEdit;
    edSignature: TEdit;
    btnFallsAssessment: TBitBtn;
    Panel1: TPanel;
    mmNote: TMemo;
    Panel5: TPanel;
    Panel6: TPanel;
    Panel8: TPanel;
    procedure FormCreate(Sender: TObject);
    procedure sbSaveClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure cbTemplatesChange(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure edSignatureKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure mmNoteChange(Sender: TObject);
    procedure edSignatureChange(Sender: TObject);
  private
    { Private declarations }
    iAttempt: Integer;
  public
    { Public declarations }
    StudyID,
      providerID: string;
    providerName: string;
    NoteID: string;
    NoteText: string;
  end;

var
  frmNote: TfrmNote;

function AddTIUNote(aStudyID, aUserName, aUserDUZ: string): TModalResult;

implementation

uses uMDK_Engine, uROR_CommonUtils, uMDK_Options, uMDK_ReportEngine,
  fMDK_FallAssessement, uMDK_CDSCommon, uMDK_Common;

{$R *.dfm}

function AddTIUNote(aStudyID, aUserName, aUserDUZ: string): TModalResult;
var
  dlg: TfrmNote;
begin
  Application.CreateForm(TfrmNote, dlg);

  dlg.StudyID := aStudyID;
  dlg.ProviderID := aUserDUZ;
  dlg.ProviderName := aUserName;
  dlg.edProvider.Text := aUserName;
  dlg.iAttempt := 0;

  try
    dlg.ActiveControl := dlg.mmNote;
  except
  end;
  Result := dlg.ShowModal;

  FreeAndNil(dlg);
end;

procedure TfrmNote.FormCreate(Sender: TObject);
var
  s: string;
  sl: TStringList;
  i: integer;
begin
  btnFallsAssessment.Visible := Preferences.bValue(onFallsAssessmentAsTIU);
  pnlTIUTemplate.Visible := Preferences.bValue(onShowTIUNoteTemplates);

  mmNote.Text := '';
  edSignature.Text := '';

  sl := TStringList.Create;
  sl.Text := Engine.getSystemOptionByName(onNoteTemplates);
  for i := 0 to sl.Count - 1 do
    sl[i] := sl.Names[i];
  lbTemplates.Items.Text := sl.Text;
  sl.Clear;
  sl.Text := Engine.getSystemOptionByName(onTIUNoteTitles);
  i := 0;
  while i < sl.Count do
    begin
      s := sl.Values[sl.Names[i]];
      if trim(s) <> '' then
        begin
          sl[i] := s;
          inc(i);
        end
      else
        sl.Delete(i);
    end;

  if sl.Count > 0 then
    cbTIUNoteTitles.Items.Text := sl.Text;
  (*
      s :=  Preferences.sValue(onDefaultTIUTitle);
      if Trim(s) <> '' then
        begin
          i := cbTIUNoteTitles.Items.IndexOf(s);
          if i >=0 then
            cbTIUNoteTitles.ItemIndex := i;
        end;
  *)
  sl.Free;
end;

procedure TfrmNote.sbSaveClick(Sender: TObject);
var
  aSL: TStringList;
  s: string;
  i: Integer;

    function ValidESig:Boolean;
    begin
      Result := Engine.isESigValid(edSignature.Text);
      if not Result then
        if (Engine.InvalidESigCount<=2) then
          MessageDlg('Signature is not valid.'+#13#10+'Try again.',mtWarning,[mbOK],0)
        else
          begin
            MessageDLG('e-Signature verification failed 3 times'+#13#10+
              'The application will be closed', mtError, [mbOK],0);
            ModalResult := mrCancel;
          end;
    end;

begin
  if trim(cbTIUNoteTitles.Text) = '' then
    begin
      i := MessageDLG('TIU Note title is not assigned' + #13 +
        'Do you want to save the note without title?',
        mtWarning, [mbYes, mbNo, mbCancel], 0);
      case i of
        mrNo: Exit;
        mrCancel:
          begin
            ModalResult := mrCancel;
            exit;
          end;
      end;
    end;

  if not ValidESig then
    exit;
(*
  if trim(edSignature.Text) = '' then
    begin
      i := MessageDLG('No e-Signature provided' + #13 +
        'Do you want to use blank signature?',
        mtWarning, [mbYes, mbNo, mbCancel], 0);
      case i of
        mrNo: Exit;
        mrCancel:
          begin
            ModalResult := mrCancel;
            exit;
          end;
      end;
    end;
*)
  aSL := TStringList.Create;
  aSL.Text := mmNote.Lines.Text;
  s := Engine.SubmitNote(ProviderID, EdSignature.Text, aSL, cbTIUNoteTitles.Text);
  aSL.Free;
  if s <> '' then
    begin
      if pos('-', s) <> 1 then
        begin
          NoteText := mmNote.Lines.Text;
          NoteID := s;
          ModalResult := mrOK;
        end
      else
        begin
          MessageDlg(Piece(s, '^', 2),mtWarning,[mbOK],0);
          if pos('incorrect El',s)<>0 then
            begin
              if iAttempt > 1 then
                ModalResult := mrCancel;
              edSignature.SelStart := 0;
              edSignature.SelLength := Length(edSignature.Text);
              ActiveControl := edSignature;
              inc(iAttempt);
            end;
        end;
    end
  else
    begin
      s := DateTimeToStr(Now);
      NoteText := mmNote.Lines.Text;
      NoteID := s;
    end;
end;

procedure TfrmNote.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if key = VK_Escape then
    ModalResult := mrCancel;
end;

procedure TfrmNote.cbTemplatesChange(Sender: TObject);
var
  sTemplateName: string;
begin
  if Sender = lbTemplates then
    try
      if lbTemplates.ItemIndex < 0 then
        mmNote.Text := ''
      else
        begin
          sTemplateName :=
            Engine.getSystemOptionByName(lbTemplates.Items[lbTemplates.ItemIndex]);
          mmNote.Text := getReportValue(sTemplateName);
        end;
    except
      mmNote.Text := '';
    end;
end;

procedure TfrmNote.SpeedButton1Click(Sender: TObject);
var
  iSelStart: Integer;
  sPre, sPost: string;
begin
  if not Assigned(FallAssessment) then
    Application.CreateForm(TFallAssessment, FallAssessment);

  FallAssessment.CleanUpForm;

  sPre := '';
  sPost := '';
  with mmNote do
  try
    if SelLength = 0 then
      begin
        sPre := copy(Text, 1, SelStart);
        sPost := copy(Text, SelStart+1, length(Text));
      end
    else
      begin
        sPre := copy(Text, 1, SelStart);
        sPost := copy(Text, SelStart+SelLength+1, length(Text));
      end;
  except
  end;

  try
    FallAssessment.edAge.Text := Engine.CurrentPatient.Age;
    iSelStart := mmNote.SelStart;
    if FallAssessment.ShowModal = mrOK then
      begin
        mmNote.Text := sPre + FallAssessment.Report.Text + sPost;
        mmNote.SelStart := iSelStart;
        mmNote.SelLength := Length(FallAssessment.Report.Text);
      end;
  except
  end;
end;

procedure TfrmNote.edSignatureKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_Return then
    sbSaveClick(nil);
end;

procedure TfrmNote.mmNoteChange(Sender: TObject);
begin
  UpdateMemo(Sender, Engine.Signature);
end;

procedure TfrmNote.edSignatureChange(Sender: TObject);
begin
  bbtnSave.Enabled := Length(edSignature.Text) >=
    devOptions.iValue(donMinESigLength);
end;

end.
