unit frMDK_Comments;
{
================================================================================
*
*       Package:      MDK - Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 12/04/07 9:13a $
*       Site Name:    Hines OIFO
*       Date Created: 3/28/07
*       Developers:   andriyevskiy                  
*       Description:  this unit contains the Hemodialysis application
*                     frMDK_Comments frame definition
*       Notes:
*
================================================================================
*       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/APP-HEMODIALYSIS/Objects/uMDK_Common.pas $
*
*       $ History: uMDK_Common.pas $
*
}
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, ActnList
  , DBClient, DB
  , StdCtrls, Grids, DBGrids
  , frMDK_StudyNoteSection
  , Buttons, ImgList, ComCtrls, ToolWin
  , dMDK_Data
  ;

type
  TFrameMDK_Comments = class(TFrame)
    pnlTools: TPanel;
    ActionList1: TActionList;
    acRefresh: TAction;
    acAdd: TAction;
    sbxCanvas: TScrollBox;
    acValid: TAction;
    acShowReport: TAction;
    Panel1: TPanel;
    acLock: TAction;
    acEdit: TAction;
    acDelete: TAction;
    acCleanUp: TAction;
    acUpdateItem: TAction;
    ImageList1: TImageList;
    pnlToolbar: TPanel;
    SpeedButton6: TSpeedButton;
    SpeedButton5: TSpeedButton;
    SpeedButton2: TSpeedButton;
    SpeedButton1: TSpeedButton;
    pnlButtons2DH: TPanel;
    Label2: TLabel;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    BitBtn3: TBitBtn;
    BitBtn4: TBitBtn;
    pnlButtons3DH: TPanel;
    BitBtn5: TBitBtn;
    BitBtn6: TBitBtn;
    BitBtn7: TBitBtn;
    BitBtn8: TBitBtn;
    Panel2: TPanel;
    spbNCnew: TSpeedButton;
    spbNCEdit: TSpeedButton;
    spbNCReport: TSpeedButton;
    spbNCDelete: TSpeedButton;
    procedure acRefreshExecute(Sender: TObject);
    procedure acAddExecute(Sender: TObject);
    procedure acValidExecute(Sender: TObject);
    procedure acShowReportExecute(Sender: TObject);
    procedure acLockExecute(Sender: TObject);
    procedure acDeleteExecute(Sender: TObject);
    procedure acEditExecute(Sender: TObject);
    procedure acCleanUpExecute(Sender: TObject);
    procedure acUpdateItemExecute(Sender: TObject);
    procedure Label2Click(Sender: TObject);
  private
    fItemTag: string;
    fGroupTag: string;
    fCDS: TClientDataSet;

    fReadOnly: Boolean;
    fGlobalCollapsed: Boolean;

    //    iLineLength: Integer;

    fNL_ID: string; //      'ID';
    fNL_Lock: string;
//    fNL_ParentID: string; //'ParentID';
    fNL_PatientID: string; //'PatientID';
    fNL_StudyID: string; // 'StudyID';
    fNL_ItemID: string; //  'ItemID';
    fNL_Tag: string; //     'Tag';
    fNL_DateTime: string; //'DateTime';
    fNL_AuthorID: string; //  'Author';
    fNL_AuthorName: string; //  'Author';
    fNL_Text: string; //    'NoteText';
    fNL_Status: string; //  'Status';
    fNL_StatusBy: string; //'StatusBy';
    fNL_StatusDate: string; //'StatusDate';

    fCurrentFrame: TObject;
    { Private declarations }
    procedure setCommentFields;
    function ProcessRecord(sTag: string; iTag: Integer): TFrameStudyNoteSection;
    function LocateNoteByID(anID: string): Boolean;

    procedure setReadOnly(aValue: Boolean);
  public
    ReportTitle: string;
    { Public declarations }
    property GroupTag: string read fGroupTag write fGroupTag;
    property ItemTag: string read fItemTag write fItemTag;
    property ReadOnly: Boolean read fReadOnly write setReadOnly;

    procedure FindNext(Up: Boolean; aSender: TObject);
    procedure FindCurrent(anID: string; aSender: TObject);
    procedure setUpCanvas(aCDS: TClientDataSet; aStringTag: string);

    procedure setUpGUI(aValue: string);

    procedure CleanSBX;
    procedure ResetSBX;
    function setupScrollbox: string;
    procedure releaseCurrentFrame;
    procedure markCurrentFrame;
    procedure setActionStatus;
    procedure moveComment(aFieldName: string; anID: string; aCDS: TClientDataSet
      = nil);
    function addNewComment(aText: string; anID: string = ''; aReadOnly: Boolean
      = False): string;
    procedure ActivateFrame;
  end;

implementation

uses
  uMDK_Notes, uMDK_Options, uMDK_Common,
  uMDK_CDSCommon, uMDK_Engine, fMDK_Comments
  , uMDK_Flowsheet,
  fMDK_Info;

{$R *.dfm}

procedure TFrameMDK_Comments.setUpCanvas(aCDS: TClientDataSet; aStringTag:
  string);
begin
  fCDS := aCDS;
  fGroupTag := aStringTag;
  pnlButtons3DH.Visible := devOptions.bValue(donWindowsXPTheme);
  pnlButtons2DH.Visible := not devOptions.bValue(donWindowsXPTheme);

//  spbNCNew.Caption := '';
//  spbNCEdit.Caption := '';
//  spbNCDelete.Caption := '';
//  spbNCReport.Caption := '';
end;

procedure TFrameMDK_Comments.setCommentFields;
var
  aCDS: TClientDataSet;

  function getField(aName, aDefault: string): string;
  begin
    try
      result := aCDS.FieldByName(aName).AsString;
    except
      Result := aDefault;
    end;
  end;

begin
  aCDS := fCDS;
  if not Assigned(aCDS) then
    Exit;

  try
    fNL_ID := getField(fnNL_ID, ''); //      'ID';
    fNL_Lock := getField(fnNL_Lock, ''); //      'ID';
//    fNL_ParentID := getField(fnNL_ParentID, ''); //'ParentID';
    fNL_PatientID := getField(fnNL_PatientID, ''); //'PatientID';
    fNL_StudyID := getField(fnNL_StudyID, ''); // 'StudyID';
    fNL_ItemID := getField(fnNL_ItemID, ''); //  'ItemID';
    fNL_Tag := getField(fnNL_Tag, ''); //     'Tag';
    fNL_DateTime := getField(fnNL_DateTime, ''); //'DateTime';
    fNL_AuthorID := getField(fnNL_AuthorID, ''); //  'Author';
    fNL_AuthorName := getField(fnNL_AuthorName, ''); //  'Author';
    fNL_Text := getField(fnNL_Text, ''); //    'NoteText';
    fNL_Status := getField(fnNL_Status, ''); //  'Status';
    fNL_StatusBy := getField(fnNL_StatusBy, ''); //'StatusBy';
    fNL_StatusDate := getField(fnNL_StatusDate, ''); //'StatusDate';
  except
  end;
end;

function TFrameMDK_Comments.ProcessRecord(sTag: string; iTag: Integer):
  TFrameStudyNoteSection;
var
  aSection: TFrameStudyNoteSection;
  sName: string;

begin
  setCommentFields;

  try
    sName := Engine.getUserNameByID(fNL_AuthorID);
    if sName = '' then
      sName := fNL_AuthorID;
  except
    sName := fNL_AuthorID;
  end;

  aSection := newStudyNoteSection(sbxCanvas,
    fNL_ID,
    fNL_LOCK,
    sTag,
    sName, //fNL_AuthorID, //Name,
    fNL_DateTime,
    fNL_Text, // Text of Comment
    'NOTE', // Type of Comment
    fNL_Status,
    fNL_StatusBy,
    fNL_StatusDate,
    200, // Max Height
    iTag, // Tab Order
    True // Collapsed
    );

  aSection.Parent := sbxCanvas;
  aSection.pnlTypeIcon.Visible := False;
  aSection.MoveUp;
  Result := aSection;

  acEdit.Enabled := aSection.LockStatus = fsUnlocked;
  acDelete.Enabled := aSection.LockStatus = fsUnlocked;
end;

procedure TFrameMDK_Comments.CleanSBX;
var
  i: Integer;
  aSection: TFrameStudyNoteSection;
begin
  i := 0;
  while i < sbxCanvas.ComponentCount do
    begin
      if sbxCanvas.Components[i] is TFrameStudyNoteSection then
        begin
          aSection := TFrameStudyNoteSection(sbxCanvas.Components[i]);
          sbxCanvas.RemoveComponent(aSection);
          if Assigned(aSection) then
            begin
              aSection.CleanUp;
              FreeAndNil(aSection);
            end;
        end
      else
        inc(i);
    end;
end;

procedure TFrameMDK_Comments.ResetSBX;
var
  //  sFilter,
  //    s: string;
  BM: TBookmark;
  aSection: TFrameStudyNoteSection;
  iTab: Integer;
begin
  if not Assigned(fCDS) then
    Exit;
  if not fCDS.Active then
    Exit;

  iTab := 0;
  BM := fCDS.GetBookmark;
  fCDS.DisableControls;
  //  while not fCDS.EOF do
  //  fCDS.First;
  fCDS.Last;
  while not fCDS.BOF do
    begin
      aSection := ProcessRecord(intToStr(iTab), iTab);
      if Assigned(aSection) then
        begin
          aSection.onFindCurrent := FindCurrent;
          inc(iTab);
        end;
      //      fCDS.Next;
      fCDS.Prior;
    end;

  fCDS.GotoBookmark(BM);
  fCDS.FreeBookmark(BM);
  fCDS.EnableControls;

end;

function TFrameMDK_Comments.setupScrollbox: string;
var
  aSection: TFrameStudyNoteSection;
  iTab: Integer;

  aCDS: TClientDataSet;
  aTag: string;
  aSBX: TScrollBox;
  aFindNext: TFindNext;

begin
  Result := '';

  aCDS := fCDS;
  if not Assigned(aCDS) or not aCDS.Active then
    Exit;

  aTag := GroupTag;
  aSBX := sbxCanvas;
  aFindNext := FindNext;

  if not aSBX.Visible then
    Exit;

  CleanSBX;
  fCurrentFrame := nil;
  iTab := 0;

  aCDS.DisableControls;
  aCDS.IndexFieldNames := fnNL_DateTime;
  aCDS.Last;
  while not aCDS.BOF do
    //      aCDS.First;
    //      while not aCDS.EOF do
    begin
      aSection := ProcessRecord(intToStr(iTab), iTab);
      if Assigned(aSection) then
        begin
          aSection.onFindCurrent := FindCurrent;
          inc(iTab);
          if not Assigned(fCurrentFrame) then
            fCurrentFrame := aSection;
        end;
      //          aCDS.Next;
      aCDS.Prior;
    end;
  aCDS.EnableControls;

end;

////////////////////////////////////////////////////////////////////////////////

procedure TframeMDK_Comments.FindNext(Up: Boolean; aSender: TObject);
var
  j,
    i: integer;
begin
  for i := 0 to sbxCanvas.ComponentCount - 1 do
    begin
      if sbxCanvas.Components[i] = aSender then
        begin
          if Up then
            begin
              if i = 0 then
                j := sbxCanvas.ComponentCount - 1
              else
                j := i - 1;
              TframeStudyNoteSection(sbxCanvas.Components[j]).FocusLast;
            end
          else
            begin
              if i = sbxCanvas.ComponentCount - 1 then
                j := 0
              else
                j := i + 1;

              TframeStudyNoteSection(sbxCanvas.Components[j]).FocusFirst;
            end;
          break;
        end;
    end;
end;

////////////////////////////////////////////////////////////////////////////////

procedure TFrameMDK_Comments.releaseCurrentFrame;
begin
  if Assigned(fCurrentFrame) then
    try
      TFrameStudyNoteSection(fCurrentFrame).lblDate.Font.Style :=
        TFrameStudyNoteSection(fCurrentFrame).lblDate.Font.Style - [fsBold];
      TFrameStudyNoteSection(fCurrentFrame).lblTime.Font.Style :=
        TFrameStudyNoteSection(fCurrentFrame).lblTime.Font.Style - [fsBold];
      TFrameStudyNoteSection(fCurrentFrame).lblby.Font.Style :=
        TFrameStudyNoteSection(fCurrentFrame).lblBy.Font.Style - [fsBold];

      TFrameStudyNoteSection(fCurrentFrame).SectionSelected := False;
    except
    end;
end;

procedure TFrameMDK_Comments.markCurrentFrame;
begin
  if Assigned(fCurrentFrame) then
    begin
      TFrameStudyNoteSection(fCurrentFrame).lblDate.Font.Style :=
        TFrameStudyNoteSection(fCurrentFrame).lblDate.Font.Style + [fsBold];
      TFrameStudyNoteSection(fCurrentFrame).lblTime.Font.Style :=
        TFrameStudyNoteSection(fCurrentFrame).lblTime.Font.Style + [fsBold];
      TFrameStudyNoteSection(fCurrentFrame).lblby.Font.Style :=
        TFrameStudyNoteSection(fCurrentFrame).lblBy.Font.Style + [fsBold];

      TFrameStudyNoteSection(fCurrentFrame).SectionSelected := True;
    end;
end;

////////////////////////////////////////////////////////////////////////////////

procedure TFrameMDK_Comments.FindCurrent(anID: string; aSender: TObject);
var
  sAuthor: string;
begin
  ReleaseCurrentFrame;

  if fCDS.Locate(fnNL_ID, VarArrayOf([anID]), []) then
    begin
      sAuthor := fCDS.FieldByName(fnNL_AuthorID).AsString;
      acValid.Enabled :=
        (sAuthor = Engine.CurrentOperatorID) or
        (Engine.CurrentOperatorMode = rnAdmin);
    end
  else
    acValid.Enabled := False;

  fCurrentFrame := aSender;
  if Assigned(fCurrentFrame) then
    begin
      acAdd.Enabled := not Engine.CurrentPatient.ReadOnly;
      acEdit.Enabled := TFrameStudyNoteSection(fCurrentFrame).LockStatus =
        fsUnlocked;
      acDelete.Enabled := TFrameStudyNoteSection(fCurrentFrame).LockStatus =
        fsUnlocked;
    end
  else
    acAdd.Enabled := not ReadOnly;

  MarkCurrentFrame;
end;

////////////////////////////////////////////////////////////////////////////////

function TFrameMDK_Comments.locateNoteByID(anID: string): Boolean;
begin
  Result := False;
  try
    Result := dmData.cdsNotes.Locate(fnNL_ID, VarArrayOf([anID]), []);
  except
    on E: Exception do
      MessageDLG('Locate Note By ID'+#13#10+E.Message, mtError, [mbOK], 0);
  end;
end;

procedure TFrameMDK_Comments.setActionStatus;
begin
  setCommentFields;

  acEdit.Enabled := fNL_Lock = fsUnLocked;
  acLock.Enabled := fNL_Lock = fsUnLocked;
  acDelete.Enabled := fNL_Lock = fsUnLocked;

  ReadOnly := fReadOnly;
end;

////////////////////////////////////////////////////////////////////////////////

procedure TFrameMDK_Comments.acValidExecute(Sender: TObject);
var
  sAuthor: string;
begin
  Exit;
  if Assigned(fCurrentFrame) then
    try
      sAuthor := TFrameStudyNoteSection(fCurrentFrame).Author;
      if (sAuthor = Engine.CurrentOperatorID)
        {//        or Preferences.bValue(onAllowUserInvalidateOtherUserComments)} then
        TFrameStudyNoteSection(fCurrentFrame).pnlCanvasDblClick(nil);
    except
    end;
end;

procedure TFrameMDK_Comments.acShowReportExecute(Sender: TObject);
begin
  if devOptions.bValue(donShowAllComments) then
    ShowInfo(getNotesReport(ReportTitle, fCDS))
  else if Assigned(fCurrentFrame) then
    begin
      setCommentFields;
      ShowInfo(fNL_Text);
    end;
end;

procedure TFrameMDK_Comments.acRefreshExecute(Sender: TObject);
var
  i: Integer;
  sID: string;
begin
  if not visible then // vhaishandria 070613 8:21
    Exit;
  sID := '';
  if Assigned(fCurrentFrame) then
    begin
      //      setCommentFields;
      try
        sID := TFrameStudyNoteSection(fCurrentFrame).ID;
      except
        sID := '';
      end;
    end;

  setupScrollBox;

  if sID <> '' then
    try
      for i := 0 to sbxCanvas.ComponentCount - 1 do
        begin
          if (sbxCanvas.Components[i] is TFrameStudyNoteSection) and
            (TFrameStudyNoteSection(sbxCanvas.Components[i]).ID = sID) then
            begin
              fCurrentFrame := TFrameStudyNoteSection(sbxCanvas.Components[i]);
              MarkCurrentFrame;
              if Assigned(fCurrentFrame) and
                TFrameStudyNoteSection(fCurrentFrame).CanFocus then
                try
                  TFrameStudyNoteSection(fCurrentFrame).SetFocus;
                except
                end;
              break;
            end;
        end;
    except
    end;
end;

procedure TFrameMDK_Comments.acLockExecute(Sender: TObject);
begin
  if not Assigned(fCurrentFrame) then
    Exit;
  setCommentFields;
  if LocateNoteByID(fNL_ID) then
    begin
      dmData.LockCurrentNote;
      TFrameStudyNoteSection(fCurrentFrame).LockStatus := fsLocked;
      setActionStatus;
    end;
end;

procedure TFrameMDK_Comments.acDeleteExecute(Sender: TObject);
begin
  if not Assigned(fCurrentFrame) then
    Exit;
  setCommentFields;
  if LocateNoteByID(fNL_ID) then
    begin
      if MessageDLG('Delete note: ' + #13#10#13#10 + fNL_Text, mtConfirmation,
        [mbYes, mbNo], 0) = mrYes then
        begin
          dmData.cdsNotes.Delete;
          acRefreshExecute(nil);
          SetActionStatus;
        end;
    end;
end;

procedure TFrameMDK_Comments.acEditExecute(Sender: TObject);
var
  s: string;
begin
  if not Assigned(fCurrentFrame) then
    Exit;
  setCommentFields;
  if LocateNoteByID(fNL_ID) then
    begin
      s := fNL_Text;
      if EditComment(s) = mrOK then
        try
          dmData.cdsNotes.Edit;
          dmData.cdsNotes.FieldByName(fnNL_Text).AsString := s;
          dmData.cdsNotes.Post;
          acRefresh.Execute;
          if assigned(fCurrentFrame) then
            TFrameStudyNoteSection(fCurrentFrame).Collapsed := false;

        except
        end;
    end;
end;

procedure TFrameMDK_Comments.acCleanUpExecute(Sender: TObject);
begin
  CleanSBX;
end;

procedure TFrameMDK_Comments.acUpdateItemExecute(Sender: TObject);
begin
  if not Assigned(fCurrentFrame) then
    Exit;
  TFrameStudyNoteSection(fCurrentFrame).updateFrame;
end;

procedure TFrameMDK_Comments.acAddExecute(Sender: TObject);
begin
  AddNewComment('');
end;

procedure TFrameMDK_Comments.moveComment(aFieldName: string; anID: string; aCDS:
  TClientDataSet = nil);
var
  s: string;
  CDS: TClientDataSet;
begin
  s := dmData.cdsSummary.FieldByName(aFieldName).AsString;
  if s = '' then
    Exit;

  if aCDS = nil then
    CDS := dmData.cdsSummary
  else
    CDS := aCDS;

  if MessageDLG('Field <' + aFieldName + '> contains data in old format.' +
    #13#10#13#10 +
    s + #13#10#13#10 + 'Do you want to convert this comment to a new format?',
    mtConfirmation, [mbYes, mbNo], 0) = mrYes then
    begin
      if addNewComment(s, anID, True) <> '' then
        try
          CDS.Edit;
          CDS.FieldByName(aFieldName).AsString := '';
          CDS.Post;
          acLock.Execute;
          markCurrentFrame;
          setActionStatus;
        except
        end;
    end;
end;

function TFrameMDK_Comments.addNewComment(aText: string; anID: string = '';
  aReadOnly: Boolean = False): string;
var
  iTag: Integer;
  sFilter,
    sDate,
    sAuthor,
    sID,
    sText: string;
  aSection: TFrameStudyNoteSection;
  bFiltered: Boolean;

begin
  sText := aText;
  Result := '';

  sFilter := Engine.NoteList_CDS.Filter;
  bFiltered := Engine.NoteList_CDS.Filtered;

  if ConfirmComment(sText, sAuthor, sDate, aReadOnly) = mrOK then
    begin
      Result := sText;

      if anID = '' then
        sID := fItemTag
      else
        sID := anID;

      if sAuthor = Engine.CurrentOperatorID then
        sAuthor := Engine.UserDUZ;

      addNote(Engine.NoteList_CDS,
        sText,
        sAuthor,
        GroupTag,
        sID,
        //sDate,
        fsUnlocked,
        StrToDateTime(sDate),
        Engine.getCurrentServerDateTime);

      iTag := sbxCanvas.ComponentCount;
      aSection := ProcessRecord(IntTostr(iTag), iTag);
      aSection.onFindCurrent := FindCurrent;
      aSection.LockStatus := fsUnlocked;

      fCurrentFrame := aSection;

      if bFiltered then
        Engine.NoteList_CDS.Filter := sFilter;

      acRefresh.Execute;
    end;
end;

procedure TFrameMDK_Comments.Label2Click(Sender: TObject);
var
  i: Integer;
begin
  fGlobalCollapsed := not fGlobalCollapsed;
  try
    for i := 0 to sbxCanvas.ComponentCount - 1 do
      if (sbxCanvas.Components[i] is TFrameStudyNoteSection) then
        TFrameStudyNoteSection(sbxCanvas.Components[i]).Collapsed :=
          fGlobalCollapsed;
  except
  end;
end;

procedure TFrameMDK_Comments.setUpGUI(aValue: string);
begin
  pnlButtons3DH.Visible := aValue = '3D';
  pnlButtons2DH.Visible := not (aValue = '3D');
end;

procedure TFrameMDK_Comments.setReadOnly(aValue: Boolean);
begin
  fReadOnly := aValue;
  if ReadOnly then
    begin
      acAdd.Enabled := false;
      acEdit.Enabled := false;
      acDelete.Enabled := false;
    end;
end;

procedure TFrameMDK_Comments.ActivateFrame;
begin
  acRefresh.Execute;

  if Assigned(fCurrentFrame) and (TObject(fCurrentFrame) is
    TFrameStudyNoteSection) then
    begin
      LocateNoteByID(TFrameStudyNoteSection(fCurrentFrame).ID);
      //      setActionStatus;
    end
  else
    begin
      acEdit.Enabled := False;
      acDelete.Enabled := False;
    end;
  acAdd.Enabled := not Engine.CurrentPatient.ReadOnly;
end;

end.

