unit uMDK_Gambro;
{
================================================================================
*
*       Package:      MDK - Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 9/28/07 10:38a $
*       Site Name:    Hines OIFO
*       Date Created: 7/29/05
*       Developers:   andriyevskiy                  
*       Description:  this unit contains the Hemodialysis application
*                     Gambro data field names
*       Notes:
*
================================================================================
*       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/APP-HEMODIALYSIS/Instruments/uMDK_Fresenius.pas $
*
*       $ History: uMDK_Gambro.pas $
}

interface
uses
  uMDK_InstrumentRecord
  , XMLIntf;

procedure UpdateGambroProblemList(aPatientID, aStudyID: string; anIR:
  TInstrumentRecord);
procedure UpdateGambroMedicationList(aPatientID, aStudyID: string; anIR:
  TInstrumentRecord);
procedure setGambroRecord(aNode: IXMLNode; const anIR: TInstrumentRecord);

implementation

uses uROR_CommonUtils, dMDK_Data, uMDK_Engine, SysUtils, uMDK_MedList,
  uMDK_Common, uMDK_Flowsheet;

function IsValidDateString(aString: string): Boolean;
begin
  Result := False;
  try
    StrToDateTime(aString);
    Result := True;
  except
  end;
end;

procedure UpdateGambroProblemList(aPatientID, aStudyID: string; anIR:
  TInstrumentRecord);
var
  dt: TDateTime;
  sDateTime,
    sEvent: string;
  i: Integer;
begin
  for i := 0 to anIR.ProblemList.Count - 1 do
    begin
      dmData.cdsFlowSheet.Insert;

      dmData.cdsFlowsheet.FieldByName(fnFS_PatientID).AsString := aPatientID;
      dmData.cdsFlowsheet.FieldByName(fnFS_SessionID).AsString := aStudyID;
      dmData.cdsFlowsheet.FieldByName(fnFS_Source).AsString := fsDevice;
      dmData.cdsFlowsheet.FieldByName(fnFS_Status).AsString := fsValid;

      sDateTime := piece(anIR.ProblemList[i], '  ');
      sEvent := piece(anIR.ProblemList[i], '  ', 2);

(*
      try
        dt := StrToDateTime(sDateTime);
      except
        on e: Exception do
          dt := anIR.FS_DateTime;
      end;
*)
      if IsValidDateString(sDateTime) then
        dt := StrToDateTime(sDateTime)
      else
        dt := anIR.FS_DateTime;
      try
        dmData.cdsFlowsheet.FieldByName(fnFS_Date).AsDateTime := Trunc(dt);
        dmData.cdsFlowsheet.FieldByName(fnFS_Time).AsDateTime := Frac(dt);
      except
      end;

{$IFDEF FLOWEVENTS}
      dmData.cdsFlowsheet.FieldByName(fnFS_Event).AsString := sEvent;
      //      dmData.cdsFlowsheet.FieldByName(fnFS_Comments).AsString := sDateTime;
{$ENDIF}

      if ((anIR.Vendor = 'GAMBRO_EXALIS') or (anIR.Vendor = 'FRESENIUS'))
        and (anIR.PatientSSN <> Engine.CurrentPatient.PatientSSN) then
        dmData.cdsFlowsheet.FieldByName(fnFS_Status).AsString :=
          fsInvalidSource;

      dmData.cdsFlowSheet.Post;
    end;
end;

procedure UpdateGambroMedicationList(aPatientID, aStudyID: string; anIR:
  TInstrumentRecord);
var
  i: Integer;
  s: string;
begin
  if (anIR.Vendor = 'GAMBRO_EXALIS') and (pos('EGS', anIR.Segment) = 1) then
    begin
      for i := 0 to anIR.MedList.Count - 1 do
        begin
          s := anIR.MedList[i];
          dmData.cdsMedList.Insert;
          dmData.cdsMedList.FieldByName(fnML_PatientID).AsString := aPatientID;
          dmData.cdsMedList.FieldByName(fnML_StudyID).AsString := aStudyID;
          dmData.cdsMedList.FieldByName(fnML_Date).AsDateTime :=
            anIR.FS_DateTime;
          dmData.cdsMedList.FieldByName(fnML_Time).AsDateTime :=
            Frac(anIR.FS_DateTime); // vhaishandria 060807
          dmData.cdsMedList.FieldByName(fnML_Medication).AsString := piece(s,
            '^', 1);
          dmData.cdsMedList.FieldByName(fnML_Dose).AsString := piece(s, '^', 2);
          dmData.cdsMedList.FieldByName(fnML_Units).AsString := piece(s, '^',
            3);
          dmData.cdsMedList.FieldByName(fnML_Rout).AsString := 'IV';
          dmData.cdsMedList.FieldByName(fnML_GivenBy).AsString := anIR.Vendor;
          dmData.cdsMedList.FieldByName(fnML_Reason).AsString := anIR.Vendor;
          dmData.cdsMedList.FieldByName(fnML_Source).AsString := 'GAMBRO';
          //          dmData.cdsMedList.Post;
        end;
    end;

end;

procedure setGambroRecord(aNode: IXMLNode; const anIR: TInstrumentRecord);
var
  sText,
    //  sKeyWord,
//  sValueName,
  sSegment,
    //  sScheme,
  sValue: string;
  iValueCode: Integer;

  // for DGS ignore values 0 and 0.000
  procedure GambroDGS; // startin time, ending time treatment time

    function StoI(aVal: string; aLen: Integer = 1): string;
    begin
      if pos('.', aVal) > 0 then
        Result := copy(aVal, 1, pos('.', aVal) - 1)
      else
        Result := aVal;
      if Result = '0' then
        Result := '';
    end;

  begin
    with anIR do
      begin
        if sValue = '' then
          Exit; // To avoid blank assignments
        case iValueCode of
          1022: TreatmentPrescribed := sValue;
          1048: TreatmentDate := sValue; //
          1049: TreatmentStarted := sValue; //

          1052: WeightBefore := sValue; // float  ignore 0.0
          1053: PreStandingSys := SToI(sValue);
          1054: PreStandingDia := SToI(sValue);
          1055: ; // Pre Lying Sys. Pressure
          1056: ; // Pre Lying Dia. Pressure
          1057: PreStandingPulse := SToI(sValue);
          1058: ; // Pre Lying Pulse
          1059: TreatmentFinished := sValue;
          1060: WeightAfter := sValue; // float
          1061: PostStandingSys := SToI(sValue);
          1062: PostStandingDia := SToI(sValue);
          1063: ; // Post Lying Sys.Pressure
          1064: ; // Post Lying Dia.Pressure
          1065: PostStandingPulse := SToI(sValue);
          1066: ; // Post Lying Pulse
          1187: PreSeatedSys := SToI(sValue);
          1188: PreSeatedDia := SToI(sValue);
          1189: PreSeatedPulse := SToI(sValue);
          1191: PostSeatedSys := SToI(sValue);
          1192: PostSeatedDia := SToI(sValue);
          1193: PostSeatedPulse := SToI(sValue);
        end;
      end;
  end;

  procedure GambroBGS; // flowsheet data
  begin
    with anIR do
      begin
        case iValueCode of
          2001: FS_DIAL := StoF(sValue); //  StrToFloat(sValue); // +
          2002: FS_COND := StoF(sValue); // StrToFloat(sValue); // +
          2009: FS_VP := StoF(sValue); // StrToFloat(sValue);   // --
          2015: TotalLP := sValue;
          2022: FS_AP := StoF(sValue); // StrToFloat(sValue);   // --
          2023: FS_Sys := Piece(sValue, '.', 1); //BP systolic  --
          2024: FS_Dias := Piece(sValue, '.', 1); //BP Diastolic  --
          2025: FS_HR := StoF(sValue); // StrToFloat(sValue); // heart rate  --
          // concentration of heparin is unknown it is not exported by the system for now
          2029: FS_HEP := StrToFloat(sValue) * 1000.0;
          // + ml - not units of the heparin
          2039: FS_Fluid := StoF(sValue, 2);
          // StrToFloat(sValue);// Fluid Off...- Totals Weight loss  ++
          2041: FS_uFR := StoF(sValue, 2); //
          2047: FS_DFR := StoF(sValue);
          // StrToFloat(sValue);//DFR -- Dialysis flow rate   --
          2054: FS_BIC := StoF(sValue);
          //  StrToFloat(sValue);//Bicarbonate conductivity  +
          2056: FS_TMP := StoF(sValue);
          // StrToFloat(sValue);//TMP -- TM Pressure          --
          2057: TotalKT := sValue;
          2068: FS_IONI := StoF(sValue, 2);
          // StrToFloat(sValue);//IONIC ... -- Ionic effective dialysance   ++ K
          2069: FS_BFR := StoF(sValue);
          // StrToFloat(sValue);//Blood Flow Rate      --
          2079:
            begin
              TotalKTV := sValue; // KT/V ++ ---
              FS_KTV := sToF(sValue);
            end;
        end;
      end;
  end;

  procedure GambroMGS; // summary fields  - similar to BGS
  begin
    with anIR do
      begin
        case iValueCode of
          2001: MeanDTemp := sValue;
          2002: MeanCond := sValue;
          2015: TotalLP := sValue;
          2039: TotalUF := sValue;
          2041: MeanUFR := sValue;
          2047: AverageDFR := sValue;
          2056: MeanTMP := sValue;
          2057: TotalKT := sValue;
          2069: AverageBFR := sValue;
          2079: TotalKTV := sValue;
          2089:
            try // Time remaining
              DiffusionTimeSec := Round(StrToFloat(sValue)); // 050510
            except
              Engine.Report(rtError, 'Error processing <' + sText + '> <' +
                sValue + '>');
            end;
        end;
      end;
  end;

  (*
  OBX	EGS...	ST	7101	ANALGESICS		200	mg
  OBX	EGS...	ST	7102	IRON		5	mg
  OBX	EGS...	ST	7103	STEROIDS		10	mg
  OBX	EGS...	ST	7104	ALBUMIN		5	g
  OBX	EGS...	ST	7105	ANTIANXIETY		2	mg
  OBX	EGS...	ST	7106	EPINEPHRINE		10	mcg
  OBX	EGS...	ST	7107	VASOPRESSOR		9	mcg
  OBX	EGS...	ST	7108	0.9% NS		100	ml
  OBX	EGS...	ST	7109	HYPERTONIC		19	ml
  OBX	EGS...	ST	7110	ANTIHYPER		395	mg
  OBX	EGS...	ST	7111	MANNITOL		450	mg
  OBX	EGS...	ST	7112	CALCITRIOL		19	mcg
  OBX	EGS...	ST	7113	EPO		10900	U.I
  OBX	EGS...	ST	7114	TRANSFUSION		9	n
  OBX	EGS...	ST	7115	DFO		950	mg
  OBX	EGS...	ST	7116	ZEMPLAR		49	mcg
  OBX	EGS...	ST	7117	VALIUM		9	mg

  OBX	EGS...	ST	7501	NAUSEA
  OBX	EGS...	ST	7502	BLOOD PRESSURE DROP
  OBX	EGS...	ST	7503	ANGINA
  OBX	EGS...	ST	7504	RINSEBACK PROB.
  OBX	EGS...	ST	7505	BLEEDING
  OBX	EGS...	ST	7506	VOMITING
  OBX	EGS...	ST	7507	CRAMPS
  OBX	EGS...	ST	7508	HEADACHE
  OBX	EGS...	ST	7509	COUNTER PRES. PROB.
  OBX	EGS...	ST	7510	ARTERIAL HYPERTENS.
  OBX	EGS...	ST	7511	F.A.V. PUNCTURE
  OBX	EGS...	ST	7512	CONNECTION INTOL.
  OBX	EGS...	ST	7513	HYPOTENSION
  OBX	EGS...	ST	7514	ARRHYTHMIA
  OBX	EGS...	ST	7515	DIAPHORETIC
  OBX	EGS...	ST	7516	DYSPNEA
  OBX	EGS...	ST	7517	PAIN
  OBX	EGS...	ST	7518	ABD PAIN
  OBX	EGS...	ST	7519	DIARRHEA
  OBX	EGS...	ST	7520	THIRST
  OBX	EGS...	ST	7520	THIRST
  OBX	EGS...	ST	7521	CHILLS
  OBX	EGS...	ST	7522	FEVER
  OBX	EGS...	ST	7523	HYPERTENSION
  OBX	EGS...	ST	7524	CLOTTING
  OBX	EGS...	ST	7525	BRADYCARDIA
  OBX	EGS...	ST	7526	TACHYCARDIA
  OBX	EGS...	ST	7527	MALAISE
  OBX	EGS...	ST	7528	ITCH
  OBX	EGS...	ST	7529	HYPERTENSION
  OBX	EGS...	ST	7530	BREAKAGE FILTER
  OBX	EGS...	ST	7531	HEMATOMA F.A.V.
  OBX	EGS...	ST	7532	REACT.ANAPHILACTIC

  OBX	EGS...	ST	7301	FOOD		10	g
  OBX	EGS...	ST	7302	DRINK		10	ml

  *)

  procedure GambroEGS; // medications, events, intake
  var
    mlName, mlUnits, mlValue: string;
  begin
    with anIR do
      begin
        case iValueCode of
          -1: ;
          7101..7117:
            begin
              mlName := piece(aNode.ChildNodes['CODE'].ChildNodes['VALUE'].Text,
                '^', 2);
              mlUnits := aNode.ChildNodes['VALUE'].Text;
              mlValue := aNode.ChildNodes['UNITS'].Text;
              MedList.Add(mlName + '^' + mlUnits + '^' + mlValue);
            end;
          7301..7302:
            ProblemList.Add(
              Observation_Date_time + '  ' +
              piece(aNode.ChildNodes['CODE'].ChildNodes['VALUE'].Text, '^', 2) +
              '  ' +
              aNode.ChildNodes['VALUE'].Text + '  ' +
              aNode.ChildNodes['UNITS'].Text);
          7501..7532:
            ProblemList.Add(
              Observation_Date_time + '  ' +
              piece(aNode.ChildNodes['CODE'].ChildNodes['VALUE'].Text, '^', 2));
        end;
      end;
  end;

  procedure GambroPGS; // prescription data - similar to DGS
  begin
    with anIR do
      begin
        case iValueCode of
          1001: InstrumentType := sValue;
          1002: InstrumentID := sValue;
          1003: EDW := sValue;
          // 1004: PRESCRIPTION DATE
          // 1008: PHYSICIAN
          1011: Dialyzer := sValue;
          1019: DType := sValue;
          1022: TxLength := sValue;
          1024: Temperature := sValue;
          1025: BFR := sValue;
          1037: Bolus := sValue;
          1038: Duration2 := sValue;
          // 1072: NURSE
          1079: Ultrafiltration := sValue;
          1086: Maintenance := sValue;
          1117: DialysateFlow := sValue;
        end;
      end;
  end;

begin
  sSegment := aNode.ChildNodes['SEQUENCE'].Text;
  anIR.Segment := sSegment;
  anIR.Vendor := 'GAMBRO_EXALIS';

  sText := aNode.ChildNodes['CODE'].ChildNodes['VALUE'].Text;
  iValueCode :=
    StrToIntDef(piece(aNode.ChildNodes['CODE'].ChildNodes['VALUE'].Text, '^',
    1),
    -1);
  sValue := aNode.ChildNodes['VALUE'].Text;

  if pos('BGS', sSegment) = 1 then
    GambroBGS
  else if pos('MGS', sSegment) = 1 then
    GambroMGS
  else if pos('EGS', sSegment) = 1 then
    GambroEGS
  else if pos('DGS', sSegment) = 1 then
    GambroDGS
  else if pos('PGS', sSegment) = 1 then
    GambroPGS
  else
    begin
      case iValueCode of
        -1: ;
        1000: anIR.VendorPtID := sValue;
      end;
    end;
end;

end.
