unit uMDK_Flowsheet;

interface

uses
  DB, DBClient;

const
  fnFS_ID = 'ID'; // vhaishandria 20070412
  fnFS_TYPE = 'TYPE_ID'; // vhaishandria 20070412

  fnFS_PatientID = 'PatientID';
  fnFS_SessionID = 'SessionID';
  fnFS_Source = 'Source';
  fnFS_Status = 'Status';

  fnFS_Date = 'DateOnly';
  fnFS_Time = 'TimeOnly';

  fnFS_BP = 'BP';
  fnFS_BPSys = 'Systolic';
  fnFS_BPDias = 'Diastolic';
  fnFS_MAP = 'MAP';
  fnFS_HR = 'HR';
  fnFS_BFR = 'BFR';
  fnFS_DFR = 'DFR';
  fnFS_AP = 'AP';
  fnFS_VP = 'VP';
  fnFS_TMP = 'TMP';
  fnFS_UFR = 'UFR';
  fnFS_Fluid = 'FLUID';
  fnFS_HEP = 'HEP';
  fnFS_COND = 'COND';
  fnFS_BIC = 'BIC';
  fnFS_DIAL = 'DIAL';
  fnFS_IONI = 'IONI';
  fnFS_KTV = 'KTV'; // vhaishandria 060808

  fnFS_Event = 'EVENT'; // vhaishandria 060725
  fnFS_Comments = 'COMMENTS'; // vhaishandria 060725

  fnFS_Description = 'DESCRIPTION'; // vhaishandria 20070320

  fnFS_Submitted = 'Submitted';
{$IFDEF IGNORE_DUPLICATES}
  fnFS_Duplicate = 'Duplicate';
{$ENDIF}
  //////////////////////////////////////////////////////////////////////////////
  fsDevice = 'D';
  fsManual = 'M';
  fsValid = 'VALID';
  fsInvalid = 'INVALID';
  fsInvalidSource = 'INVALID SOURCE';

  //////////////////////////////////////////////////////////////////////////////

procedure MarkDuplicates(aCDS: TClientDataSet;aFieldName:String);
procedure CreateFlowSheetDS(aCDS: TClientDataSet);
procedure setFlowSheetFields(aCDS: TClientDataSet);
procedure setFlowSheetSpecificFields(aCDS: TClientDataSet);

implementation
uses
  uMDK_CDSCommon
  , SysUtils
  ;

procedure setFlowSheetSpecificFields(aCDS: TClientDataSet);
begin
  try
    NewStringField(aCDS, fnFS_Source, 'Source', 6, 20);
    aCDS.FieldList.Fields[aCDS.FieldList.Count - 1].DisplayWidth := 1;

    NewStringField(aCDS, fnFS_BP, 'B/P', 15, 15); // sys 2023, dias 2024
    NewFloatField(aCDS, fnFS_BPSys, 'Systolic');
{$IFNDEF AANTEST}aCDS.FieldList.Fields[aCDS.FieldList.Count - 1].Visible :=
    False;
{$ENDIF}
    NewFloatField(aCDS, fnFS_BPDias, 'Diastolic');
{$IFNDEF AANTEST}aCDS.FieldList.Fields[aCDS.FieldList.Count - 1].Visible :=
    False;
{$ENDIF}

    NewFloatField(aCDS, fnFS_MAP, 'MAP');
    NewFloatField(aCDS, fnFS_HR, 'HR'); // heart rate 2025
    NewFloatField(aCDS, fnFS_BFR, 'BFR'); //
    NewFloatField(aCDS, fnFS_DFR, 'DFR'); //  dialysis flow rate
    NewFloatField(aCDS, fnFS_AP, 'AP'); // arterial pressure 2022
    NewFloatField(aCDS, fnFS_VP, 'VP'); // venous pressure 2009
    NewFloatField(aCDS, fnFS_TMP, 'TMP'); // tm pressure
    NewFloatField(aCDS, fnFS_UFR, 'UFR'); //
    NewFloatField(aCDS, fnFS_Fluid, 'Fluid Off');
    NewFloatField(aCDS, fnFS_HEP, 'HEP (CUM)'); // heparin infusion 2029

    NewFloatField(aCDS, fnFS_COND, 'COND'); //   conductivity 2002
    NewFloatField(aCDS, fnFS_BIC, 'BIC');
    NewFloatField(aCDS, fnFS_DIAL, 'DIAL TEMP'); //  temperature 2001
    NewFloatField(aCDS, fnFS_IONI, 'IONIC DIAL');

    NewFloatField(aCDS, fnFS_KTV, 'Kt/V'); // vhaishandria 060808
    //{$IFNDEF AANTEST}aCDS.FieldList.Fields[aCDS.FieldList.Count-1].Visible := False;{$ENDIF}

    NewBooleanField(aCDS, fnFS_Submitted, 'Submitted');
{$IFNDEF AANTEST}aCDS.FieldList.Fields[aCDS.FieldList.Count - 1].Visible :=
    False;
{$ENDIF}

{$IFDEF FLOWEVENTS}
    NewStringField(aCDS, fnFS_Event, fnFS_Event, 40, 40); // vhaishandria 060725
{$ENDIF}
{$IFNDEF AANTEST}aCDS.FieldList.Fields[aCDS.FieldList.Count - 1].Visible :=
    False;
{$ENDIF}
    NewStringField(aCDS, fnFS_Comments, fnFS_Comments, 80, 80);
    // vhaishandria 060725
{$IFNDEF AANTEST}aCDS.FieldList.Fields[aCDS.FieldList.Count - 1].Visible :=
    False;
{$ENDIF}

{$IFDEF IGNORE_DUPLICATES}
    NewBooleanField(aCDS, fnFS_Duplicate, fnFS_Duplicate);
{$IFNDEF AANTEST}aCDS.FieldList.Fields[aCDS.FieldList.Count - 1].Visible :=
    False;
{$ENDIF}
{$ENDIF}

  except
    raise Exception.Create('setFlowsheetSpecificFields Error');
  end;
end;

procedure setFlowSheetFields(aCDS: TClientDataSet);
begin
  try
    NewStringField(aCDS, fnFS_ID, fnFS_ID, LENGTH_ID, LENGTH_ID);
{$IFNDEF AANTEST}aCDS.FieldList.Fields[aCDS.FieldList.Count - 1].Visible :=
    False;
{$ENDIF}

    NewStringField(aCDS, fnFS_PatientID, '##', 4, 20);
{$IFNDEF AANTEST}aCDS.FieldList.Fields[aCDS.FieldList.Count - 1].Visible :=
    False;
{$ENDIF}
    NewStringField(aCDS, fnFS_SessionID, 'Session #', 4, 20);
{$IFNDEF AANTEST}aCDS.FieldList.Fields[aCDS.FieldList.Count - 1].Visible :=
    False;
{$ENDIF}

    NewDateTimeField(aCDS, fnFS_Date, 'Date', 8, gMDK_DateFMT);
{$IFNDEF AANTEST}aCDS.FieldList.Fields[aCDS.FieldList.Count - 1].Visible :=
    False;
{$ENDIF}
    NewDateTimeField(aCDS, fnFS_Time, 'Time', 8, gMDK_TimeFMT);

    NewStringField(aCDS, fnFS_Status, 'Status', 2, 20);
{$IFNDEF AANTEST}aCDS.FieldList.Fields[aCDS.FieldList.Count - 1].Visible :=
    False;
{$ENDIF}

    setFlowSheetSpecificFields(aCDS);
  except
    raise Exception.Create('setFlowsheetFields Error');
  end;
end;

procedure CreateFlowSheetDS(aCDS: TClientDataSet);
begin
  try
    setFlowsheetFields(aCDS);
    aCDS.CreateDataSet;
    aCDS.AddIndex(fnFS_Date + '_Index', fnFS_Date + ';' + fnFS_Time,
      [ixDescending]);
    aCDS.IndexFieldNames := fnFS_Date + ';' + fnFS_Time;
    aCDS.IndexDefs.Update;

{$IFDEF IGNORE_DUPLICATES}
    aCDS.Filter := fnFS_Duplicate + '<>''True''';
    aCDS.Filtered := True;
{$ENDIF}
  except
    raise Exception.Create('createFlowsheetCDS Error');
  end;
end;

procedure MarkDuplicates(aCDS: TClientDataSet;aFieldName:String);
var
  bFilter: Boolean;
  sFilter: String;

  BM: TBookmark;
  sValuePrev,sValueCurr: String;

  function getCurrentString:String;
  var
    i: Integer;
    sName: String;
  begin
    Result := '';
    for i := 0 to aCDS.Fields.Count - 1 do
      begin
          try
            sName := aCDS.Fields[i].FieldName;
            if sName <> aFieldName then
                Result := Result + aCDS.FieldByName(sName).AsString;
          except
          end;
      end;
  end;

  procedure MarkCurrentRecord;
  begin
    if aCDS.State <> dsEdit then
      aCDS.Edit;
    try
      aCDS.FieldByName(aFieldName).AsBoolean := True;
      aCDS.Post;
    except
    end;
  end;

begin
  sFilter := aCDS.Filter;
  bFIlter := aCDS.Filtered;

  sValuePrev := '';
  sValueCurr := '';
  aCDS.DisableControls;

  aCDS.Filtered := False;
  BM := aCDS.GetBookmark;

  aCDS.First;
  sValuePrev := getCurrentString;
  aCDS.Next;
  while not aCDS.Eof do
    begin
      sValueCurr := getCurrentString;
      if sValueCurr = sValuePrev then
        MarkCurrentRecord;
      sValuePrev := sValueCurr;
      aCDS.Next;
    end;

  aCDS.GotoBookmark(BM);
  aCDS.FreeBookmark(BM);

  aCDS.Filtered := bFilter;
  aCDS.EnableControls;
end;

end.
