unit uMDK_InstrumentRecord;
{
================================================================================
*
*       Package:      MDK - Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 9/28/07 10:38a $
*       Site Name:    Hines OIFO
*       Date Created: 7/29/05
*       Developers:   andriyevskiy                  
*       Description:  this unit contains the Hemodialysis application
*                     Instrument object
*       Notes:
*
================================================================================
*       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/APP-HEMODIALYSIS/Objects/uMDK_InstrumentRecord.pas $
*
*       $ History: uMDK_InstrumentRecord.pas $
}
interface
uses
  Classes,
  SysUtils,
  DB
  , Dialogs
  , XMLIntf
  , DBClient
  ;

type
  TInstrumentRecord = class(TObject)
  private
    fFS_PatientID: string;
    fFS_SessionID: string;

    fInstrumentType: string; // vhaishandria 060214
    fInstrumentID: string; // vhaishandria 060214  GAMBRO Only
    fStationID: string; // vhaishandria 060214

    fObservation_Date_Time: string;
    fTransmission_Date_Time: string;
    fMessageControlNumber: string;
    fFiller_Order_Number: string;

    fFS_Source: string;
    fFS_Status: string;

    fFS_DateTime: TDateTime;

    // timing //////////////////////////////////////////////////////////////////
    fTreatmentStarted: string;
    fTreatmentFinished: string;
    fTreatmentDate: string;

    fTreatmentPrescribed: string;
    fDiffusionTimeSec: Integer; //Gambro 2089

    // totals //////////////////////////////////////////////////////////////////
    fTotalUF: string;
    fTotalLP: string;
    fTotalKTV: string;
    fTotalKT: string;
    fMeanDTemp: string;
    fMeanCond: string;
    fMeanUFR: string;
    fMeanTMP: string;
    fAverageBFR: string;
    fAverageDFR: string;
    fTotalURR: string; // not supported by GAMBRO

    //pre and post vitals //////////////////////////////////////////////////////
    fPreTemperature,
      fPostTemperature,
      fWeightBefore,
      fWeightAfter,
      //Seated and Standing
    fPreSeatedPulse,
      fPreSeatedSys,
      fPreSeatedDia,
      fPreStandingPulse,
      fPreStandingSys,
      fPreStandingDia,

    fPostSeatedPulse,
      fPostSeatedSys,
      fPostSeatedDia,
      fPostStandingPulse,
      fPostStandingSys,
      fPostStandingDia: string;

    // flowsheet ///////////////////////////////////////////////////////////////

    fFS_Sys,
      fFS_Dias,
      fFS_BP: string;

    fFS_MAP: Real;
    fFS_HR: Real;
    fFS_BFR: Real;
    fFS_DFR: Real;
    fFS_AP: Real;
    fFS_VP: Real;
    fFS_TMP: Real;
    fFS_UFR: Real;
    fFS_Fluid: Real;
    fFS_HEP: Real;
    fFS_COND: Real;
    fFS_BIC: Real;
    fFS_DIAL: Real;
    fFS_IONI: Real;
    fFS_KTV: Real;

    // RxLab ///////////////////////////////////////////////////////////////////
    fDialyzer,
      fMaxReuseNbr,
      fTxGoalWeight,
      fTxLength,
      fUltrafiltration,
      fEDW,
      fBFR,
      fDialysateFlow,
      fTemperature,

    fK,
      fHC03,
      fNA,
      fCA,

    fType,
      fBolus,
      fMaintenance,
      //    fDuration1,
    fDuration2,
      fDuration: string;

    // Medications

    fMedType,
      fMedICD9Code,
      fMedICD9Description,
      fMedDose,
      fMedMeasurement,
      fMedRouteType,
      fMedPatientCareInitials: string;

    fNoteText: string; // Fresenius Notes
    fNoteDateTime: string; // Fresenius Notes
    fNoteBy: string; // Fresenius Notes

    function getTreatmentDuration: TDateTime;
  public
    bFreseniusSummary: Boolean;
    ContainsFlowsheetData: Boolean;

    LastRecord: Boolean;

    VendorPtID: string;

    PatientFirstName,
      PatientLastName,
      PatientSSN,
      PatientDOB,
      Segment,
      Vendor: string;

    ProblemList,
      MedList: TstringList;

    property InstrumentType: string read fInstrumentType write fInstrumentType;
    property InstrumentID: string read fInstrumentID write fInstrumentID;
    property StationID: string read fStationID write fStationID;

    property TreatmentDate: string read fTreatmentDate write fTreatmentDate;
    property TreatmentStarted: string read fTreatmentStarted write
      fTreatmentStarted;
    property TreatmentFinished: string read fTreatmentFinished write
      fTreatmentFinished;
    property TreatmentPrescribed: string read fTreatmentPrescribed write
      fTreatmentPrescribed;
    property DiffusionTimeSec: Integer read fDiffusionTimeSec write
      fDiffusionTimeSec;

    property TreatmentDuration: TDateTime read getTreatmentDuration;

    property TotalUF: string read fTotalUF write fTotalUF;
    property TotalLP: string read fTotalLP write fTotalLP;
    property TotalKTV: string read fTotalKTV write fTotalKT;
    property TotalKT: string read fTotalKT write fTotalKT;
    property MeanDTemp: string read fMeanDTemp write fMeanDTemp;
    property MeanCond: string read fMeanCond write fMeanCond;
    property MeanUFR: string read fMeanUFR write fMeanUFR;
    property MeanTMP: string read fMeanTMP write fMeanTMP;
    property AverageBFR: string read fAverageBFR write fAverageBFR;
    property AverageDFR: string read fAverageDFR write fAverageDFR;

    property Transmission_Date_time: string read fTransmission_Date_Time write
      fTransmission_Date_Time;
    property Observation_Date_Time: string read fObservation_Date_Time write
      fObservation_Date_Time;

    property FS_PatientID: string read fFS_PatientID write fFS_PatientID;
    property FS_SessionID: string read fFS_SessionID write fFS_SessionID;
    property FS_Source: string read fFS_Source write fFS_Source;
    property FS_Status: string read fFS_Status write fFS_Status;
    property FS_DateTime: TDateTime read fFS_DateTime write fFS_DateTime;

    property FS_BP: string read fFS_BP write fFS_BP;
    property FS_Sys: string read fFS_Sys write fFS_Sys;
    property FS_Dias: string read fFS_Dias write fFS_Dias;

    property FS_MAP: Real read fFS_MAP write fFS_MAP;
    property FS_HR: Real read fFS_HR write fFS_HR;
    property FS_BFR: Real read fFS_BFR write fFS_BFR;
    property FS_DFR: Real read fFS_DFR write fFS_DFR;
    property FS_AP: Real read fFS_AP write fFS_AP;
    property FS_VP: Real read fFS_VP write fFS_VP;
    property FS_TMP: Real read fFS_TMP write fFS_TMP;
    property FS_UFR: Real read fFS_UFR write fFS_UFR;
    property FS_Fluid: Real read fFS_Fluid write fFS_Fluid;
    property FS_HEP: Real read fFS_HEP write fFS_HEP;
    property FS_COND: Real read fFS_COND write fFS_COND;
    property FS_BIC: Real read fFS_BIC write fFS_BIC;
    property FS_DIAL: Real read fFS_DIAL write fFS_DIAL;
    property FS_IONI: Real read fFS_IONI write fFS_IONI;
    property FS_KTV: Real read fFS_KTV write fFS_KTV;

    property PreTemperature: string read fPreTemperature write fPreTemperature;
    property PostTemperature: string read fPostTemperature write
      fPostTemperature;

    property WeightBefore: string read fWeightBefore write fWeightBefore;

    property WeightAfter: string read fWeightAfter write fWeightAfter;
    property PreSeatedPulse: string read fPreSeatedPulse write fPreSeatedPulse;
    property PreSeatedSys: string read fPreSeatedSys write fPreSeatedSys;
    property PreSeatedDia: string read fPreSeatedDia write fPreSeatedDia;
    property PreStandingPulse: string read fPreStandingPulse write
      fPreStandingPulse;
    property PreStandingSys: string read fPreStandingSys write fPreStandingSys;
    property PreStandingDia: string read fPreStandingDia write fPreStandingDia;

    property PostSeatedPulse: string read fPostSeatedPulse write
      fPostSeatedPulse;
    property PostSeatedSys: string read fPostSeatedSys write fPostSeatedSys;
    property PostSeatedDia: string read fPostSeatedDia write fPostSeatedDia;
    property PostStandingPulse: string read fPostStandingPulse write
      fPostStandingPulse;
    property PostStandingSys: string read fPostStandingSys write
      fPostStandingSys;
    property PostStandingDia: string read fPostStandingDia write
      fPostStandingDia;

    property Dialyzer: string read fDialyzer write fDialyzer;
    property TxGoalWeight: string read fTxGoalWeight write fTXGoalWeight;
    property TxLength: string read fTxLength write fTxLength;
    property Ultrafiltration: string read fUltrafiltration write
      fUltrafiltration;
    property EDW: string read fEDW write fEDW;
    property BFR: string read fBFR write fBFR;
    property DialysateFlow: string read fDialysateFlow write fDialysateFlow;
    property Temperature: string read fTemperature write fTemperature;

    property MaxReuseNbr: string read fMaxReuseNbr write fMaxReuseNbr;
    property K: string read fK write fK;
    property HC03: string read fHC03 write fHC03;
    property NA: string read fNA write fNA;
    property CA: string read fCA write fCA;

    property DType: string read fType write fType;
    property Bolus: string read fBolus write fBolus;
    property Maintenance: string read fMaintenance write fMaintenance;
    property Duration2: string read fDuration2 write fDuration2;
    property Duration: string read fDuration write fDuration;

    property MedType: string read fMedType write fMedType;
    property MedICD9Code: string read fMedICD9Code write fMedICD9Code;
    property MedICD9Description: string read fMedICD9Description write
      fMedICD9Description;
    property MedDose: string read fMedDose write fMedDose;
    property MedMeasurement: string read fMedMeasurement write fMedMeasurement;
    property MedRouteType: string read fMedRouteType write fMedRouteType;
    property MedPatientCareInitials: string read fMedPatientCareInitials write
      fMedPatientCareInitials;

    property NoteText: string read fNoteText write fNoteText;
    // vhaishandria 2007-06-27 Fresenius notes are not coming through ----------
    property NoteDateTime: string read fNoteDateTime write fNoteDateTime;
    property NoteBy: string read fNoteBy write fNoteBy;

    constructor Create;
    destructor Destroy; override;

    function LocalStrToFloat(aValue: string): Real;
    function StoF(aVal: string; aLen: Integer = 1): Real;
    function StoI(aVal: string; aLen: Integer = 1): string;

    procedure setBBraunRecord(aNode: IXMLNode);
{$IFNDEF FLOWEVENTS}
    procedure setGambroRecord(aNode: IXMLNode);
    procedure setFreseniusRecord(aNode: IXMLNode; aProcType: string);
{$ENDIF}

    function SetUp(aValue: string): Boolean;
  end;

function getInstrumentRecord(aPatientID, aStudyID, aValue: string):
  TInstrumentRecord;

implementation

uses
  uMDK_CDSCommon
  , uMDK_Common
  , uXML
  , uROR_CommonUtils
  , uMDK_Engine
  , uMDK_Options
  , dMDK_Data
  , uMDK_Summary
  , uMDK_MedList
  , uMDK_Notes
  , uMDK_Fresenius
  , uMDK_Gambro
  , uMDK_Flowsheet
  ;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

procedure setStringValue(
  aName: string;
  aValue: string;
  sIgnore: string;
  CurrentTime,
  LastTimeObserved: TDateTime;
  OverwriteManualInput: Boolean = False
  );
var
  bb: Boolean;
  ss: string;

begin
  try
    ss := dmData.cdsSummary.FieldByName(aName).asString;
    // here we can get dictionary data o this field
    bb := OverwriteManualInput;

    if (trim(ss) = '') then // value is not assigned yet
      begin
        if (aValue <> sIgnore) then // vhaishandria 050919
          begin
            dmData.cdsSummary.FieldByName(aName).asString := aValue;
            // 2007-07-12 vhaishandria
            dmData.cdsSummary.FieldByName(fnS_LastTimeUpdated).AsDateTime :=
              CurrentTime;
          end;
      end
    else
      begin
        if not bb then // do not overwrite 'manual' input
          Exit;
        if (aValue <> sIgnore) and (CurrentTime >= LastTimeObserved) then
          begin
            dmData.cdsSummary.FieldByName(aName).asString := trim(aValue);
            // 2007-07-12 vhaishandria
            dmData.cdsSummary.FieldByName(fnS_LastTimeUpdated).AsDateTime :=
              CurrentTime;
          end;
      end;
  except
{$IFDEF AANTEST}
    on E: Exception do
      ShowMessage('Error assigning string value <' + aValue + '> to the field <'
        + aName + '>' +
        #13#10 + E.Message);
{$ENDIF}
  end
end;

procedure setSummaryStringValue(aName: string; aValue, sIgnore: string;
  CurrentTime, LastTimeObserved: TDateTime);
var
  ss: string;
begin
  ss := dmData.cdsSummary.FieldByName(aName).asString;
  if (trim(ss) = '') and
    (dmData.cdsSummary.FieldByName(aName).asString <> sIgnore) then
    begin
      try
        dmData.cdsSummary.FieldByName(aName).asString := aValue;
      except
      end;
      exit;
    end;

  if (aValue <> sIgnore) and (CurrentTime >= LastTimeObserved) then
    dmData.cdsSummary.FieldByName(aName).asString := trim(aValue);
end;

function getLastFlowsheetDateTime: TDateTime;
var
  aBM: TBookmark;
  tmpDT: TDateTime;
begin
  Result := 0;
  // Summary fields processing==================================================
  dmData.cdsFlowSheet.DisableControls;
  aBM := dmData.cdsFlowSheet.GetBookmark;
  dmData.cdsFlowSheet.First;

  // vhaishandria 061011
  while not dmData.cdsFlowSheet.EOF do // find the last time summary was updated
    begin
      tmpDT := Trunc(dmData.cdsFlowSheet.FieldByName(fnFS_Date).AsDateTime) +
        Frac(dmData.cdsFlowSheet.FieldByName(fnFS_Date).AsDateTime);
      if tmpDT > Result then
        Result := tmpDT;
      dmData.cdsFlowsheet.Next;
    end;
  dmData.cdsFlowSheet.GoToBookmark(aBM);
  dmData.cdsFlowSheet.FreeBookmark(aBM);
  dmData.cdsFlowSheet.EnableControls;
end;

procedure UpdateSummaryFields(anIR: TInstrumentRecord);
var
  LastTimeObserved,
    ObservationTime: TDateTime;

   // vhaishandria 2007-06-27 Fresenius notes are not coming through...
  procedure AddFreseniusNote;
  var
    aNoteDate,
    dd: Double;
    aNoteBy: String;
  begin
    try
      if anIR.fNoteText = '' then
        Exit;

      if anIR.fNoteBy = '' then
        aNoteBy := 'FRESENIUS'
      else
        aNoteBy := anIR.fNoteBy;

      if anIR.NoteDateTime <> '' then
        begin
          dd := FreseniusDateTimeStringToWindows(anIR.NoteDateTime);
          aNoteDate := dd;
//          ShowMessage(FormatDateTime('mm/dd/yyyy hh:nn:ss',dd));
        end
    else
//      aNoteDate := anIR.fObservation_Date_Time;
      aNoteDate := 0;

      addNote(Engine.NoteList_CDS,
        anIR.fNoteText,
        anIR.NoteBy,          //'FRESENIUS',
        pnFlow,               //'FLOWSHEET',
        anIR.NoteDateTime,    //'FLOWSHEET',

        //aNoteDate, //anIR.fObservation_Date_Time,

        fsLocked,
        aNoteDate,
        Engine.getCurrentServerDateTime);

    except
    end;
  end;

begin
  if not Assigned(anIR) then
    Exit;

  try
    LastTimeObserved := getLastFlowsheetDateTime;

    ObservationTime := strToDateTime(anIR.Observation_Date_time);

    dmData.cdsSummary.Edit;

    SetSummaryStringValue(fnS_TreatmentStarted, anIR.TreatmentStarted, '',
      ObservationTime, LastTimeObserved);
    SetSummaryStringValue(fnS_TreatmentFinished, anIR.TreatmentFinished, '',
      ObservationTime, LastTimeObserved);

    SetSummaryStringValue(fnS_TotalUF, anIR.TotalUF, '', ObservationTime,
      LastTimeObserved);
    SetSummaryStringValue(fnS_TotalLP, anIR.TotalLP, '', ObservationTime,
      LastTimeObserved);
    SetSummaryStringValue(fnS_TotalKT, anIR.TotalKT, '', ObservationTime,
      LastTimeObserved);
    SetSummaryStringValue(fnS_TotalKTV, anIR.TotalKTV, '', ObservationTime,
      LastTimeObserved);
    //    SetSummaryStringValue(fnS_TotalURR,anIR.TotalURR,'',ObservationTime,LastTimeObserved);
    SetSummaryStringValue(fnS_MeanUFR, anIR.MeanUFR, '', ObservationTime,
      LastTimeObserved);
    SetSummaryStringValue(fnS_MeanTMP, anIR.MeanTMP, '', ObservationTime,
      LastTimeObserved);
    SetSummaryStringValue(fnS_MeanTemp, anIR.MeanDTemp, '', ObservationTime,
      LastTimeObserved);
    SetSummaryStringValue(fnS_MeanConductivity, anIR.MeanCond, '',
      ObservationTime, LastTimeObserved);
    SetSummaryStringValue(fnS_AverageBFR, anIR.AverageBFR, '', ObservationTime,
      LastTimeObserved);
    SetSummaryStringValue(fnS_AverageDFR, anIR.AverageDFR, '', ObservationTime,
      LastTimeObserved);

    // Treatment duration ========================================================
    if (anIR.Vendor = 'GAMBRO_EXALIS') then
      begin
        if (anIR.DiffusionTimeSec <> -1) then
          begin
            dmData.cdsSummary.FieldByName(fnS_TreatmentDuration).AsDateTime :=
              dmData.cdsSummary.FieldByName(fnS_rxTxLength).AsDateTime -
              anIR.DiffusionTimeSec / (24 * 3600);
          end;
      end
    else if (anIR.Vendor = 'FRESENIUS') then
      begin
        dmData.cdsSummary.FieldByName(fnS_TreatmentDuration).AsDateTime :=
          anIR.DiffusionTimeSec / (24 * 60 * 60);
      end
    else
      SetSummaryStringValue(fnS_TreatmentDuration, FormatDateTime('hh:nn:ss',
        anIR.TreatmentDuration), '', ObservationTime, LastTimeObserved);

    // Last record flag processing ===============================================
    if anIR.LastRecord then
      dmData.cdsSummary.FieldByName(fnS_LastRecord).AsBoolean :=
        anIR.LastRecord;

    // Vendor Patient ID processing ==============================================
    if anIR.VendorPtID <> '' then
      dmData.cdsSummary.FieldByName(fnS_VendorPtID).AsString := anIR.VendorPtID;

    dmData.cdsSummary.Post;

  except
  end;

  if (anIR.Vendor = 'FRESENIUS') then
    AddFreseniusNote;

end;

procedure UpdateFlowsheetFields(aPatientID, aStudyID: string; anIR:
  TInstrumentRecord);

  procedure setValue(aName: string; aValue: Real);
  begin
    try
      dmData.cdsFlowsheet.FieldByName(aName).asFloat := aValue;
    except
      on E: Exception do
        Showmessage('Error assigning value to the field' + #13 +
          'Field <' + aName + '>' + #13 +
          'Error message: ' + #13 + E.Message);
    end;
  end;

begin
  if ((Uppercase(anIR.Vendor) = 'BRAUN')
    and anIR.ContainsFlowsheetData) // BBraun sends flowsheet data only
  or
    ((Uppercase(anIR.Vendor) = 'FRESENIUS')
    and anIR.ContainsFlowsheetData
    and (Frac(anIR.FS_DateTime) <> 0)) // to avoid blank records in the flowsheet
  or
    ((anIR.Vendor = 'GAMBRO_EXALIS')
    // GAMBRO BGS segment contains flowsheet data
    and (pos('BGS', anIR.Segment) = 1)) {// BGS segment contains flowsheet data}
      then
    begin // Flowsheet processing ======================================================
      dmData.cdsFlowSheet.Insert;

      dmData.cdsFlowsheet.FieldByName(fnFS_PatientID).AsString := aPatientID;
      dmData.cdsFlowsheet.FieldByName(fnFS_SessionID).AsString := aStudyID;
      dmData.cdsFlowsheet.FieldByName(fnFS_Source).AsString := fsDevice;
      dmData.cdsFlowsheet.FieldByName(fnFS_Status).AsString := fsValid;
      try
        dmData.cdsFlowsheet.FieldByName(fnFS_Date).AsDateTime :=
          Trunc(anIR.FS_DateTime);
        dmData.cdsFlowsheet.FieldByName(fnFS_Time).AsDateTime :=
          Frac(anIR.FS_DateTime);
      except
      end;

      dmData.cdsFlowsheet.FieldByName(fnFS_BP).AsString := anIR.FS_BP;

      if anIR.FS_Sys <> '' then
        setValue(fnFS_BPSys, StrToInt(anIR.FS_Sys));
      if anIR.FS_Dias <> '' then
        setValue(fnFS_BPDias, StrToInt(anIR.FS_Dias));

      setValue(fnFS_HR, anIR.FS_HR); //'HR';
      setValue(fnFS_AP, anIR.FS_AP); //'AP';
      setValue(fnFS_VP, anIR.FS_VP); //'VP';
      setValue(fnFS_HEP, anIR.FS_HEP); //'HEP';
      setValue(fnFS_COND, anIR.FS_COND); //'COND';
      setValue(fnFS_DIAL, anIR.FS_DIAL); //'DIAL';
      setValue(fnFS_MAP, anIR.FS_MAP); //'MAP';
      setValue(fnFS_BFR, anIR.FS_BFR); //'BFR';
      setValue(fnFS_DFR, anIR.FS_DFR); //'DFR';
      setValue(fnFS_TMP, anIR.FS_TMP); //'TMP';
      setValue(fnFS_UFR, anIR.FS_UFR); //'UFR';
      setValue(fnFS_Fluid, anIR.FS_Fluid); //'Fluid Off';
      setValue(fnFS_BIC, anIR.FS_BIC); //'BIC';
      setValue(fnFS_IONI, anIR.FS_IONI); //'IONIC';
      setValue(fnFS_KTV, anIR.FS_KTV); //'IONIC';

      if ((anIR.Vendor = 'GAMBRO_EXALIS') or (anIR.Vendor = 'FRESENIUS'))
        and (anIR.PatientSSN <> Engine.CurrentPatient.PatientSSN) then
        dmData.cdsFlowsheet.FieldByName(fnFS_Status).AsString :=
          fsInvalidSource;
      if (anIR.Vendor = 'BRAUN')
        and (anIR.PatientSSN <> Engine.CurrentPatient.PatientSSN) then
        dmData.cdsFlowsheet.FieldByName(fnFS_Status).AsString :=
          fsInvalidSource;

      dmData.cdsFlowSheet.Post;
    end;
end;

procedure UpdatePrescriptionInfo(anIR: TInstrumentRecord; OverwriteManualInput:
  Boolean);
var
  LastTimeObserved,
    ObservationTime: TDateTime;
  bUpdated: Boolean;

  function Overwrite(aName: string): Boolean;
  begin
    Result := Engine.DictionaryOvewriteManualInput(aName); // default is FALSE
    bUpdated := bUpdated or Result;
  end;

begin
  if // EXALIS PGS segment processing - prescription
  (anIR.Vendor = 'GAMBRO_EXALIS') and (pos('PGS', anIR.Segment) = 1)
    or
    (anIR.Vendor = 'FRESENIUS') then
    begin
      bUpdated := False;

      ObservationTime := strToDateTime(anIR.Observation_Date_time);
      LastTimeObserved :=
        dmData.cdsSummary.FieldByName(fnS_LastTimeUpdated).AsDateTime;

      dmData.cdsSummary.Edit;

      SetStringValue(fnS_rxDialyser, anIR.Dialyzer, '', ObservationTime,
        LastTimeObserved, Overwrite(fnS_rxDialyser));
      SetStringValue(fnS_rxTxLength, anIR.TxLength, '', ObservationTime,
        LastTimeObserved, Overwrite(fnS_rxTxLength));
      SetStringValue(fnS_rxReuseNum, anIR.MaxReuseNbr, '', ObservationTime,
        LastTimeObserved, Overwrite(fnS_rxReuseNum));
      SetStringValue(fnS_RxUltra, anIR.Ultrafiltration, '', ObservationTime,
        LastTimeObserved, Overwrite(fnS_rxUltra));
      SetStringValue(fns_RxEDW, anIR.EDW, '', ObservationTime, LastTimeObserved,
        Overwrite(fnS_rxEDW));
      SetStringValue(fnS_rxBFR, anIR.BFR, '', ObservationTime, LastTimeObserved,
        Overwrite(fnS_rxBFR));
      SetStringValue(fnS_rxDialysisFlow, anIR.DialysateFlow, '',
        ObservationTime, LastTimeObserved, Overwrite(fnS_rxDialysisFlow));
      SetStringValue(fnS_rxTemp, anIR.Temperature, '', ObservationTime,
        LastTimeObserved, Overwrite(fnS_rxTemp));

      SetStringValue(fnS_AC_Type, anIR.DType, '', ObservationTime,
        LastTimeObserved, Overwrite(fnS_AC_Type));
      SetStringValue(fnS_AC_Load, anIR.Bolus, '', ObservationTime,
        LastTimeObserved, Overwrite(fnS_AC_Load));
      SetStringValue(fnS_AC_Doses, anIR.Maintenance, '', ObservationTime,
        LastTimeObserved, Overwrite(fnS_AC_Doses));

      SetStringValue(fnS_DF_K, anIR.K, '', ObservationTime, LastTimeObserved,
        Overwrite(fnS_DF_K));
      SetStringValue(fnS_DF_HCO3, anIR.HC03, '', ObservationTime,
        LastTimeObserved, Overwrite(fnS_DF_HCO3));
      SetStringValue(fnS_DF_NA, anIR.NA, '', ObservationTime, LastTimeObserved,
        Overwrite(fnS_DF_NA));
      SetStringValue(fnS_DF_CA, anIR.CA, '', ObservationTime, LastTimeObserved,
        Overwrite(fnS_DF_CA));

      SetStringValue(fnS_AC_EndTime, anIR.Duration, '', ObservationTime,
        LastTimeObserved, OverwriteManualInput);

      SetSummaryStringValue(fnS_Machine, anIR.InstrumentID, '', ObservationTime,
        LastTimeObserved);
      SetSummaryStringValue(fnS_Station, anIR.StationID, '', ObservationTime,
        LastTimeObserved);

(* vhaishandria 2007-07-12 -----------------------------------------------------
      if (ObservationTime > LastTimeObserved) and bUpdated then
        dmData.cdsSummary.FieldByName(fnS_LastTimeUpdated).AsDateTime :=
          ObservationTime;
------------------------------------------------------------------------------*)

      dmData.cdsSummary.Post;
    end;
end;

procedure UpdatePrePostValues(anIR: TInstrumentRecord; OverwriteManualInput:
  Boolean);
var
  sIgnore: string;
  LastTimeObserved,
    ObservationTime: TDateTime;

begin
  if // EXALIS DGS segment processing: pre- and post- assessements
  (anIR.Vendor = 'GAMBRO_EXALIS') and (pos('DGS', anIR.Segment) = 1)
    or
    (anIR.Vendor = 'FRESENIUS') then
    begin
      if (anIR.Vendor = 'GAMBRO_EXALIS') then
        sIgnore := '0'
      else
        sIgnore := '';

      ObservationTime := strToDateTime(anIR.Observation_Date_time);
      // vhaishandria 061010
      LastTimeObserved :=
        dmData.cdsSummary.FieldByName(fnS_LastTimeUpdated).AsDateTime;

      (*
      In the beginning of a treatment CP-Hemo gets a message from EBI with the entire DGS.
      If fields change during treatment, CP-Hemo gets a message from EBI with only the changed fields.

      1) The standard action is to keep the latest VALID value for each field.

      So using Pre-Sitting Systolic as an example:
      If CP-Hemo gets repeated DGS Pre-Sitting Systolic measurements CP-Hemo keeps
      the newest valid value, which means 'ignore zeroes'.

      Example:

      1st DGS1187 Pre-Sitting Systolic = 125
      2nd DGS1187 Pre-Sitting Systolic = 130
      3rd DGS1187 Pre-Sitting Systolic =   0
      Keep '130'.

      (If you wish, validate the whole 'Sys/Dia,Pulse' as a unit. )

      ---------------------------------------------------------------------051109-----
        0 is a placeholder for 'NO DATA' in integer  fields
        0.00000000 - for float
        '' - for text
      --------------------------------------------------------------------------------

      2) For these two fields the opposite action is more appropriate -- keep the earliest VALID value.
      1st DGS1048 Date of Session = 20051121
      2nd DGS1048 Date of Session = 20051122
      Non-Standard, CP-Hemo keeps the earliest VALID value, '20051121'.

      1st DGS1049 Treatment Starting Time = 07:32
      2nd DGS1049 Treatment Starting Time = 09:44
      Non-Standard, CP-Hemo keeps the earliest VALID value, '07:32'.

      3) For MachineID (DGS1002) do as you prefer...keep the oldest or newest.

      Ideally it would be nice to document a changed MachineID
      as a line in the CP-Hemo Flowsheet Comments, but that is not a big
      concern for CP-Hemo version 1.0 (maybe next year).

      4) Theoretically, the entire DGS could retransmit with no or few changed values.
      CP-Hemo should just follow the above rules and keep going...no special issues.
      *)

      dmData.cdsSummary.Edit;

      // by default the data will not be updated if the field contains something
      // if OverwriteManualInput is set to TRUE, the data will be overwritten
      SetStringValue(fnS_PreTemp, anIR.PreTemperature, '', ObservationTime,
        LastTimeObserved, OverwriteManualInput);
      SetStringValue(fnS_PostTemp, anIR.PostTemperature, '', ObservationTime,
        LastTimeObserved, OverwriteManualInput);

      SetStringValue(fnS_PreWeight, anIR.WeightBefore, '', ObservationTime,
        LastTimeObserved, OverwriteManualInput);

      SetStringValue(fnS_PrePulseStanding, anIR.PreStandingPulse, '',
        ObservationTime, LastTimeObserved, OverwriteManualInput);
      SetStringValue(fnS_PrePulseSeated, anIR.PreSeatedPulse, '',
        ObservationTime, LastTimeObserved, OverwriteManualInput);
      SetStringValue(fnS_PreBPSysStanding, anIR.PreStandingSys, '',
        ObservationTime, LastTimeObserved, OverwriteManualInput);
      SetStringValue(fnS_PreBPDiasStanding, anIR.PreStandingDia, '',
        ObservationTime, LastTimeObserved, OverwriteManualInput);
      SetStringValue(fnS_PreBPSysSitting, anIR.PreSeatedSys, '',
        ObservationTime, LastTimeObserved, OverwriteManualInput);
      SetStringValue(fnS_PreBPDiasSitting, anIR.PreSeatedDia, '',
        ObservationTime, LastTimeObserved, OverwriteManualInput);

      SetStringValue(fnS_PostWeight, anIR.WeightAfter, '', ObservationTime,
        LastTimeObserved, OverwriteManualInput);

      // vhaishandria o61010 - '0' - NO DATA value for post field --
      SetStringValue(fnS_PostPulseStanding, anIR.PostStandingPulse, sIgnore,
        ObservationTime, LastTimeObserved, OverwriteManualInput);
      SetStringValue(fnS_PostPulseSeated, anIR.PostSeatedPulse, sIgnore,
        ObservationTime, LastTimeObserved, OverwriteManualInput);
      SetStringValue(fnS_PostBPSysStanding, anIR.PostStandingSys, sIgnore,
        ObservationTime, LastTimeObserved, OverwriteManualInput);
      SetStringValue(fnS_PostBPDiasStanding, anIR.PostStandingDia, sIgnore,
        ObservationTime, LastTimeObserved, OverwriteManualInput);
      SetStringValue(fnS_PostBPSysSitting, anIR.PostSeatedSys, sIgnore,
        ObservationTime, LastTimeObserved, OverwriteManualInput);
      SetStringValue(fnS_PostBPDiasSitting, anIR.PostSeatedDia, sIgnore,
        ObservationTime, LastTimeObserved, OverwriteManualInput);

      SetStringValue(fnS_RxGoalWeight, anIR.TxGoalWeight, sIgnore,
        ObservationTime, LastTimeObserved, OverwriteManualInput);

(* vhaishandria 2007-07-12 -----------------------------------------------------
      if ObservationTime > LastTimeObserved then
        dmData.cdsSummary.FieldByName(fnS_LastTimeUpdated).AsDateTime :=
          ObservationTime;
------------------------------------------------------------------------------*)
      dmData.cdsSummary.Post;
    end;
end;

function getInstrumentRecord(aPatientID, aStudyID, aValue: string):
  TInstrumentRecord;
var
  FSR: TInstrumentRecord;
  fOverwriteManualInput: Boolean;
begin
  // =============================================================================
  // XSL Transformation of input data
  //  aValue := ConvertDeviceData(fCurrentPatient.Machine,aValue);
  // =============================================================================

  Result := nil;
  FSR := TInstrumentRecord.Create;
  try
    if not FSR.SetUp(aValue) then
      // SetUp will populate fields of the InstrumentRecord
      begin
        FSR.Free;
        exit;
      end;
  except
    on E: Exception do
      begin
        FSR.Free;
        Exit;
      end;
  end;
  fOverwriteManualInput := Preferences.bValue(onOverwriteManualInput);

  UpdateSummaryFields(FSR); // same for all instruments

  UpdateFlowsheetFields(aPatientID, aStudyID, FSR);

  if FSR.Vendor = 'GAMBRO_EXALIS' then
    begin
      UpdateGambroMedicationList(aPatientID, aStudyID, FSR);
      UpdateGambroProblemList(aPatientID, aStudyID, FSR);
    end
  else if FSR.Vendor = 'FRESENIUS' then
    begin
      UpdateFreseniusMedicationList(aPatientID, aStudyID, FSR);
    end;

  UpdatePrescriptionInfo(FSR, fOverwriteManualInput);
  UpdatePrePostValues(FSR, fOverwriteManualInput);

  Result := FSR; // use Result data to update the GUI
end;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

constructor TInstrumentRecord.Create;
begin
  inherited;
  ContainsFlowsheetData := False;
  ProblemList := TStringList.Create;
  MedList := TStringList.Create;
end;

destructor TInstrumentRecord.Destroy;
begin
  ProblemList.Free;
  MedList.Free;
  inherited;
end;

function TInstrumentRecord.getTreatmentDuration: TDateTime;
var
  aDT: TDateTime;
  aDTTotal: TDateTime;
begin
  try
    aDTTotal := StrToDateTime(fTreatmentPrescribed);
  except
    on E: Exception do
      aDTTotal := 0;
  end;
  try
    aDT := fDiffusionTimeSec / (3600 * 24);
  except
    on E: Exception do
      aDT := 0;
  end;
  Result := aDTTotal - aDT;
end;

////////////////////////////////////////////////////////////////////////////////

function TInstrumentRecord.LocalStrToFloat(aValue: string): Real;
begin
  try
    Result := StrToFloat(aValue);
  except
    Result := -99999.99;
  end;
end;

function TInstrumentRecord.StoF(aVal: string; aLen: Integer = 1): Real;
begin
  Result := StrToFloat(copy(aVal, 1, pos('.', aVal) + aLen));
end;

function TInstrumentRecord.StoI(aVal: string; aLen: Integer = 1): string;
begin
  if pos('.', aVal) > 0 then
    Result := copy(aVal, 1, pos('.', aVal) - 1)
  else
    Result := aVal;
end;

procedure TInstrumentRecord.setBBraunRecord(aNode: IXMLNode);
var
  sKeyWord,
    sValue, sValueName: string;
  iValueCode: Integer;
begin
  //  1  4  7  10 13 16 19 22 25 28 31 34 37 40 43 46
  sKeyword := 'IL IM GN IN II HU HV IG HR HQ HT IJ IH IK HT GT';
  Vendor := 'BRAUN';

  sValueName := aNode.ChildNodes['CODE'].ChildNodes['VALUE'].Text;
  sValueName := Piece(sValueName, '^', 1); // IN^.... - Systolic IM^... - Dias
  sValue := aNode.ChildNodes['VALUE'].Text;

  ContainsFlowsheetData := True;
  iValueCode := pos(sValueName, sKeyword);
  case iValueCode of
    0: ContainsFlowsheetData := False;
    1: fFS_Sys := sValue;
    4: fFS_Dias := sValue;
    7: VendorPtId := sValue; //GN^Patient id
    10: fFS_HR := LocalStrToFloat(sValue); //IN^NIBP pulse rate
    13: fFS_DFR := LocalStrToFloat(sValue); //II^Act dial flow
    16: fFS_AP := LocalStrToFloat(sValue); //HU^Act arterial pressure
    19: fFS_VP := LocalStrToFloat(sValue); // HV^Act venous pressure
    22: fFS_TMP := LocalStrToFloat(sValue); // IG^Act TMP
    25: fFS_UFR := LocalStrToFloat(sValue); // HR^Act UF rate
    28: fFS_Fluid := LocalStrToFloat(sValue); // HQ^UF removed (net)
    31: fFS_HEP := LocalStrToFloat(sValue); // HT^Act. heparin volume
    34: fFS_COND := LocalStrToFloat(sValue); // IJ^Act. dial. final cond.
    37: fFS_DIAL := LocalStrToFloat(sValue); // IH\S\Act. dial. temperature
    40: fFS_BIC := LocalStrToFloat(sValue); //IK^Act dial bic cond
    43: fFS_HEP := LocalStrToFloat(sValue); //HT^Act heparin volume
    46: fFS_BFR := LocalStrToFloat(sValue); //GT^Act heparin volume
    //   fFS_MAP :Real; // MAP=Mean Arterial Pressure not supported by BBraun -- Not displayed
    //   fFS_BFR :Real; // BFR=Blood Flow Rate information not provided by BBraun -- GT
    //    fFS_IONI :Real; // IONIC DIAL=Ionic Dialysance not supported by BBraun -- Not Supported
  end;
end;

function TInstrumentRecord.SetUp(aValue: string): Boolean;
var
  b: Boolean;
  sProcType,
    sScheme: string;
  s: string;
  xDoc: IXMLDocument;
  xxNode,
    xNode: IXMLNode;
  i, j: Integer;

  function getTransmissionDate(xxNode: IXMLNode): string;
  begin
    Result :=
      xxNode.ChildNodes['YEAR'].Text + ' ' +
      xxNode.ChildNodes['MONTH'].Text + ' ' +
      xxNode.ChildNodes['DAY'].Text + ' ' +
      xxNode.ChildNodes['HOUR'].Text + ' ' +
      xxNode.ChildNodes['MINUTE'].Text + ' ' +
      xxNode.ChildNodes['SECOND'].Text;
  end;

  procedure setObservationDate(xxNode: IXMLNode);
  begin
    fObservation_Date_Time :=
      xxNode.ChildNodes['MONTH'].Text + '/' +
      xxNode.ChildNodes['DAY'].Text + '/' +
      xxNode.ChildNodes['YEAR'].Text + ' ' +
      xxNode.ChildNodes['HOUR'].Text + ':' +
      xxNode.ChildNodes['MINUTE'].Text + ':' +
      xxNode.ChildNodes['SECOND'].Text;
  end;

  procedure CleanUpTotals;
  begin
    fTreatmentStarted := '';
    fTreatmentFinished := '';
    fTreatmentPrescribed := '';
    fDiffusionTimeSec := 0;

    // totals
    fTotalUF := '';
    fTotalLP := '';
    fTotalKTV := '';
    fTotalKT := '';
    fMeanDTemp := '';
    fMeanCond := '';
    fMeanUFR := '';
    fMeanTMP := '';
    fAverageBFR := '';
    fAverageDFR := '';
    fTotalURR := ''; // not supported by GAMBRO
  end;

begin
  Result := False;
  LastRecord := False;
  VendorPtID := '';

  CleanUpTotals;

  try
    xDoc := XMLDocument(aValue);
  except
    on E: Exception do
      begin
{$IFDEF AANTEST}ShowMessage('Create XML from String Error:' + #13#10 +
          E.Message);
{$ENDIF}
        Engine.Report(rtError,
          'Create XML from String Error' + #13#10 + E.Message + #13#10#13#10 +
          aValue,
          '', 0, '', 'Create XML from String', 'Failed', TRUE);
        Exit;
      end;
  end;

  with xDoc.DocumentElement do
    begin
      for i := 0 to ChildNodes.Count - 1 do
        begin
          xNode := ChildNodes[i];
          if xNode.NodeName = 'MESSAGE_HEADER' then
            begin
              xxNode := xNode.ChildNodes['TRANSMISSION_DATE_TIME'];
              fTransmission_Date_Time := getTransmissionDate(xxNode);
              fMessageControlNumber :=
                xNode.ChildNodes['MESSAGE_CONTROL_ID'].Text;
              continue;
            end;

          if xNode.NodeName = 'PATIENT_INFORMATION' then
            begin
              PatientSSN := xNode.ChildNodes['PATIENT_ID'].Text;
              // vhaishandria 060321
              PatientLastName := xNode.ChildNodes['LAST_NAME'].Text;
              PatientFirstName := xNode.ChildNodes['FIRST_NAME'].Text;
              continue;
            end;

          if xNode.NodeName = 'RESULTS' then
            begin
              fFiller_Order_Number :=
                xNode.ChildNodes['FILLER_ORDER_NUMBER'].Text;
              xxNode := xNode.ChildNodes['DATE_OBSERVED'];
              setObservationDate(xxNode);
              fFS_DateTime := StrToDateTime(fObservation_Date_Time);

              b := False;
              for j := 0 to xNode.ChildNodes.Count - 1 do
                if xNode.ChildNodes[j].NodeName = 'OBSERVATION' then
                  begin
                    xxNode := xNode.ChildNodes[j];
                    sScheme :=
                      xxNode.ChildNodes['CODE'].ChildNodes['SCHEME'].Text;
                    if uppercase(sScheme) = 'BRAUN' then
                      setBBraunRecord(xxNode);
                    if uppercase(sScheme) = 'FRESENIUS' then
                      begin
                        sProcType := xNode.ChildNodes['PROCEDURE_TYPE'].Text;
{$IFNDEF FLOWEVENTS}
                        setFreseniusRecord(xxNode, sProcType);
{$ELSE}
                        setFreseniusRecord(xxNode, sProcType, Self);
{$ENDIF}
                      end;
{$IFNDEF FLOWEVENTS}
                    if Uppercase(sScheme) = 'GAMBRO_EXALIS' then
                      setGambroRecord(xxNode);
{$ELSE}
                    if Uppercase(sScheme) = 'GAMBRO_EXALIS' then
                      setGambroRecord(xxNode, Self);
{$ENDIF}
                    try
                      s := xNode.ChildNodes[j].ChildNodes['VALUE'].Text;
                      if s = STUDY_STATUS_DONE then
                        LastRecord := True;
                    except
                    end;

                    b := b or ContainsFlowsheetData;
                  end;
              ContainsFlowsheetData := b;
              fFS_BP := fFS_Sys + '/' + fFS_Dias;
              if fFS_BP = '/' then
                fFS_BP := '';
            end;

        end;
    end;
  Result := True;
end;

{$IFNDEF FLOWEVENTS}

procedure TInstrumentRecord.setFreseniusRecord(aNode: IXMLNode; aProcType:
  string);
var
  sText,
    sKeyWord,
    sValue, sValueName,
    sSegment,
    sScheme: string;
  iValueCode: Integer;
  dd: Double;

  procedure FreseniusMedication; // medications, events, intake
  var
    mlName, mlUnits, mlValue: string;
  begin
    case iValueCode of
      -1: ;
      7101..7117:
        begin
          mlName := piece(aNode.ChildNodes['CODE'].ChildNodes['VALUE'].Text,
            '^', 2);
          mlUnits := aNode.ChildNodes['VALUE'].Text;
          mlValue := aNode.ChildNodes['UNITS'].Text;
          MedList.Add(mlName + '^' + mlUnits + '^' + mlValue);
        end;
      7301..7302:
        ProblemList.Add(//              sScheme+' '+
          Observation_Date_time + '  ' +
          piece(aNode.ChildNodes['CODE'].ChildNodes['VALUE'].Text, '^', 2) + '  '
          +
          aNode.ChildNodes['VALUE'].Text + '  ' +
          aNode.ChildNodes['UNITS'].Text);
      7501..7532:
        ProblemList.Add(//              sScheme+' '+
          Observation_Date_time + '  ' +
          piece(aNode.ChildNodes['CODE'].ChildNodes['VALUE'].Text, '^', 2));
    end;
  end;

begin
  Vendor := 'FRESENIUS';

  sValueName := aNode.ChildNodes['CODE'].ChildNodes['VALUE'].Text;
  sValue := aNode.ChildNodes['VALUE'].Text;
  // not sure the next use of Integer is correct
  iValueCode :=
    StrToIntDef(piece(aNode.ChildNodes['CODE'].ChildNodes['VALUE'].Text, '^',
    1),
    -1);

  ContainsFlowsheetData := False;

  if (aProcType = kwFRES_PreviousPostTreatmentParameters) or
    //vhaishandria 060306
  (aProcType = kwFRES_PreviousPreTreatmentParameters) then //vhaishandria 060306
    Exit;

  ContainsFlowsheetData := True;

  if sValueName = kwFres_TotalUF then
    fFS_Fluid := LocalStrToFloat(sValue)
  else if (sValueName = kwFres_Map) and (sValue <> '') then
    fFS_MAP := LocalStrToFloat(sValue)
  else if (sValueName = kwFres_BPSystolic) and (sValue <> '') then
    begin
      fFS_Sys := piece(sValue, '^', 1);
      bFreseniusSummary := piece(sValue, '^', 2) <> '';
    end
  else if (sValueName = kwFres_BPDiastolic) and (sValue <> '') then
    fFS_Dias := piece(sValue, '^', 1)
  else if (sValueName = kwFres_Conductivity) and (sValue <> '') then
    fFS_COND := LocalStrToFloat(sValue)
  else if (sValueName = kwFres_Pulse) and (sValue <> '') then
    fFS_HR := LocalStrToFloat(sValue)
  else if (sValueName = kwFres_BloodFlowRate) and (sValue <> '') then
    fFS_BFR := LocalStrToFloat(sValue)
  else if (sValueName = kwFres_VenousPressure) and (sValue <> '') then
    fFS_VP := LocalStrToFloat(sValue)
  else if (sValueName = kwFres_ArterialPressure) and (sValue <> '') then
    fFS_AP := LocalStrToFloat(sValue)
  else if (sValueName = kwFres_UltraFiltrationRate) and (sValue <> '') then
    fFS_UFR := LocalStrToFloat(sValue)
  else if (sValueName = kwFres_Temperature) and (sValue <> '') then
    fFS_DIAL := LocalStrToFloat(sValue)
  else if (sValueName = kwFres_HeparinInfused) and (sValue <> '') then
    fFS_HEP := LocalStrToFloat(sValue)
  else if (sValueName = kwFRES_DialysateFlowRate) and (sValue <> '') then
    fFS_DFR := LocalStrToFloat(sValue)
  else if (sValueName = kwFRES_TransMembranePressure) and (sValue <> '') then
    fFS_TMP := LocalStrToFloat(sValue)
  else if (sValueName = kwFRES_EffectiveKTV) and (sValue <> '') then
    fTotalKTV := sValue

  else if (sValueName = kwFRES_TreatmentStarted) and (sValue <> '') then
    fTreatmentStarted := sValue
  else if (sValueName = kwFRES_TreatmentStoped) and (sValue <> '') then
    fTreatmentFinished := sValue

  else
    ContainsFlowsheetData := False;

  ////////////////////////////////////////////////////////////////////////////

  if (sValueName = kwFres_PreTemperature) and (sValue <> '') then
    fPreTemperature := sValue
  else if (sValueName = kwFres_PostTemperature) and (sValue <> '') then
    fPostTemperature := sValue

  else if (sValueName = kwFres_PreWeight) and (sValue <> '') then
    WeightBefore := sValue
  else if (sValueName = kwFres_PreBPSystolicStanding) and (sValue <> '') then
    PreStandingSys := sValue
  else if (sValueName = kwFres_PreBPDiastolicStanding) and (sValue <> '') then
    PreStandingDia := sValue
  else if (sValueName = kwFres_PrePulse) and (sValue <> '') then
    PreSeatedPulse := SToI(sValue)

  else if (sValueName = kwFres_PostWeight) and (sValue <> '') then
    WeightAfter := sValue
  else if (sValueName = kwFres_PostBPSystolicStanding) and (sValue <> '') then
    PostStandingSys := sValue
  else if (sValueName = kwFres_PostBPDiastolicStanding) and (sValue <> '') then
    PostStandingDia := sValue
  else if (sValueName = kwFres_PostPulse) and (sValue <> '') then
    PostSeatedPulse := SToI(sValue)

  else if (sValueName = kwFres_PreBPSystolicSitting) and (sValue <> '') then
    PreSeatedSys := sValue
  else if (sValueName = kwFres_PreBPDiastolicSitting) and (sValue <> '') then
    PreSeatedDia := sValue

  else if (sValueName = kwFres_PostBPSystolicSitting) and (sValue <> '') then
    PostSeatedSys := sValue
  else if (sValueName = kwFres_PostBPDiastolicSitting) and (sValue <> '') then
    PostSeatedDia := sValue

  else if (sValueName = kwFRES_TotalUltraFiltration) and (sValue <> '') then
    fTotalUF := sValue
  else if (sValueName = kwFRES_TotalBV) and (sValue <> '') then
    fTotalLP := sValue
  else if (sValueName = kwFRES_MeanBF) and (sValue <> '') then
    fAverageBFR := sValue
  else if (sValueName = kwFRES_MeanDF) and (sValue <> '') then
    fAverageDFR := sValue
  else if (sValueName = kwFRES_MeanUFR) and (sValue <> '') then
    fMeanUFR := sValue
  else if (sValueName = kwFRES_MeanTMP) and (sValue <> '') then
    fMeanTMP := sValue
  else if (sValueName = kwFRES_MeanTemp) and (sValue <> '') then
    fMeanDTemp := sValue

    // Prescription Info
  else if (sValueName = kwFRES_DialyzerType) and (sValue <> '') then
    fDialyzer := sValue
  else if (sValueName = kwFRES_DialyzerReuse) and (sValue <> '') then
    fMaxReuseNbr := sValue
  else if (sValueName = kwFRES_TreatmentLength) and (sValue <> '') then
    begin
      dd := StrToInt(sValue) / (24 * 60);
      fTxLength := FormatDateTime('hh:mm', dd);
    end
      // 060627 vhaishandria
  else if (sValueName = kwFRES_TreatmentDuration) and (sValue <> '') then
    begin
      dd := StrToInt(sValue) / (24 * 60);
      //        fDuration := FormatDateTime('hh:mm',dd);
      fDuration := FormatDateTime('hh:mm', dd); // vhaishandria 060824
    end

  else if (sValueName = kwFRES_DryWeight) and (sValue <> '') then
    fEDW := sValue
  else if (sValueName = kwFRES_PrBloodFlowRate) and (sValue <> '') then
    fBFR := sValue

  else if (sValueName = kwFRES_DialysateFlow) and (sValue <> '') then
    fDialysateFlow := sValue
  else if (sValueName = kwFRES_AnticoagulantType) and (sValue <> '') then
    fType := sValue
  else if (sValueName = kwFRES_AnticoagulantLoad) and (sValue <> '') then
    fBolus := sValue
  else if (sValueName = kwFRES_AnticoagulantUnitsPerHour) and (sValue <> '')
    then
    fMaintenance := sValue
  else if (sValueName = kwFRES_DialyzerReuse) and (sValue <> '') then
    fMaxReuseNbr := sValue

    //    else if (sValueName = kwFRES_DialysateFormula) and (sValue<>'') then fK := sValue
  else if (sValueName = kwFRES_ModelingNa) and (sValue <> '') then
    fNA := sValue

    //  kwFRES_SodiumProgram  = 'Sodium Program';
    //  kwFRES_SodiumProgramTime  = 'Sodium Program Time';
    //  kwFRES_SodiumBaseConductivity  = 'Sodium Base Conductivity';
    {
        fUltrafiltration,
        fTemperature,
        fDuration1,
        fDuration2,
        fDuration:String;
    }

  else if (sValueName = kwFRES_Note) and (sValue <> '') then
    fNoteText := sValue

  else if (sValueName = kwFRES_MedicationType) and (sValue <> '') then
    fMedType := sValue
  else if (sValueName = kwFRES_ICD9Description) and (sValue <> '') then
    fMedICD9Description := sValue
  else if (sValueName = kwFRES_ICD9Code) and (sValue <> '') then
    fMedICD9Code := sValue
  else if (sValueName = kwFRES_Dose) and (sValue <> '') then
    fMedDose := sValue
  else if (sValueName = kwFRES_MeasurementType) and (sValue <> '') then
    fMedMeasurement := sValue
  else if (sValueName = kwFRES_RouteType) and (sValue <> '') then
    fMedRouteType := sValue
  else if (sValueName = kwFRES_PatientCareInitials) and (sValue <> '') then
    fMedPatientCareInitials := sValue

    // 060627 vhaishandria
  else if (sValueName = kwFRES_InstrumentID) and (sValue <> '') then
    fInstrumentID := sValue
  else if (sValueName = kwFRES_StationID) and (sValue <> '') then
    fStationID := sValue

  else if (sValueName = kwFRES_GoalWeight)
    and (aProcType = kwFRES_CurrentPostTreatmentParameters)
    and (sValue <> '') then
    fTxGoalWeight := sValue
      ;
end;

procedure TInstrumentRecord.setGambroRecord(aNode: IXMLNode);
var
  sText,
    sKeyWord,
    sValueName,
    sSegment,
    sScheme,
    sValue: string;
  iValueCode: Integer;

  // for DGS ignore values 0 and 0.000
  procedure GambroDGS; // startin time, ending time treatment time

    function StoI(aVal: string; aLen: Integer = 1): string;
    begin
      if pos('.', aVal) > 0 then
        Result := copy(aVal, 1, pos('.', aVal) - 1)
      else
        Result := aVal;
      if Result = '0' then
        Result := '';
    end;

  begin
    if sValue = '' then
      Exit; // To avoid blank assignments
    case iValueCode of
      1022: fTreatmentPrescribed := sValue;
      1048: fTreatmentDate := sValue; //
      1049: fTreatmentStarted := sValue; //

      1052: WeightBefore := sValue; // float  ignore 0.0
      1053: PreStandingSys := SToI(sValue);
      1054: PreStandingDia := SToI(sValue);
      1055: ; // Pre Lying Sys. Pressure
      1056: ; // Pre Lying Dia. Pressure
      1057: PreStandingPulse := SToI(sValue);
      1058: ; // Pre Lying Pulse
      1059: fTreatmentFinished := sValue;
      1060: WeightAfter := sValue; // float
      1061: PostStandingSys := SToI(sValue);
      1062: PostStandingDia := SToI(sValue);
      1063: ; // Post Lying Sys.Pressure
      1064: ; // Post Lying Dia.Pressure
      1065: PostStandingPulse := SToI(sValue);
      1066: ; // Post Lying Pulse
      1187: PreSeatedSys := SToI(sValue);
      1188: PreSeatedDia := SToI(sValue);
      1189: PreSeatedPulse := SToI(sValue);
      1191: PostSeatedSys := SToI(sValue);
      1192: PostSeatedDia := SToI(sValue);
      1193: PostSeatedPulse := SToI(sValue);
    end;
  end;

  procedure GambroBGS; // flowsheet data
  begin
    case iValueCode of
      2001: fFS_DIAL := StoF(sValue); //  StrToFloat(sValue); // +
      2002: fFS_COND := StoF(sValue); // StrToFloat(sValue); // +
      2009: fFS_VP := StoF(sValue); // StrToFloat(sValue);   // --
      2015: fTotalLP := sValue;
      2022: fFS_AP := StoF(sValue); // StrToFloat(sValue);   // --
      2023: fFS_Sys := Piece(sValue, '.', 1); //BP systolic  --
      2024: fFS_Dias := Piece(sValue, '.', 1); //BP Diastolic  --
      2025: fFS_HR := StoF(sValue); // StrToFloat(sValue); // heart rate  --
      // concentration of heparin is unknown it is not exported by the system for now
      2029: fFS_HEP := StrToFloat(sValue) * 1000.0;
      // + ml - not units of the heparin
      2039: fFS_Fluid := StoF(sValue, 2);
      // StrToFloat(sValue);// Fluid Off...- Totals Weight loss  ++
      2041: fFS_uFR := StoF(sValue, 2); //
      2047: fFS_DFR := StoF(sValue);
      // StrToFloat(sValue);//DFR -- Dialysis flow rate   --
      2054: fFS_BIC := StoF(sValue);
      //  StrToFloat(sValue);//Bicarbonate conductivity  +
      2056: fFS_TMP := StoF(sValue);
      // StrToFloat(sValue);//TMP -- TM Pressure          --
      2057: fTotalKT := sValue;
      2068: fFS_IONI := StoF(sValue, 2);
      // StrToFloat(sValue);//IONIC ... -- Ionic effective dialysance   ++ K
      2069: fFS_BFR := StoF(sValue);
      // StrToFloat(sValue);//Blood Flow Rate      --
      2079: fTotalKTV := sValue; // KT/V ++ ---
    end;
  end;

  procedure GambroMGS; // summary fields  - similar to BGS
  begin
    case iValueCode of
      2001: fMeanDTemp := sValue;
      2002: fMeanCond := sValue;
      2015: fTotalLP := sValue;
      2039: fTotalUF := sValue;
      2041: fMeanUFR := sValue;
      2047: fAverageDFR := sValue;
      2056: fMeanTMP := sValue;
      2057: fTotalKT := sValue;
      2069: fAverageBFR := sValue;
      2079: fTotalKTV := sValue;
      2089:
        try // Time remaining
          fDiffusionTimeSec := Round(StrToFloat(sValue)); // 050510
        except
          Engine.Report(rtError, 'Error processing <' + sText + '> <' + sValue +
            '>');
        end;
    end;
  end;

  procedure GambroEGS; // medications, events, intake
  var
    mlName, mlUnits, mlValue: string;
  begin
    case iValueCode of
      -1: ;
      7101..7117:
        begin
          mlName := piece(aNode.ChildNodes['CODE'].ChildNodes['VALUE'].Text,
            '^', 2);
          mlUnits := aNode.ChildNodes['VALUE'].Text;
          mlValue := aNode.ChildNodes['UNITS'].Text;
          MedList.Add(mlName + '^' + mlUnits + '^' + mlValue);
        end;
      7301..7302:
        ProblemList.Add(
          Observation_Date_time + '  ' +
          piece(aNode.ChildNodes['CODE'].ChildNodes['VALUE'].Text, '^', 2) + '  '
          +
          aNode.ChildNodes['VALUE'].Text + '  ' +
          aNode.ChildNodes['UNITS'].Text);
      7501..7532:
        ProblemList.Add(
          Observation_Date_time + '  ' +
          piece(aNode.ChildNodes['CODE'].ChildNodes['VALUE'].Text, '^', 2));
    end;
  end;

  procedure GambroPGS; // prescription data - similar to DGS
  begin
    case iValueCode of
      1001: fInstrumentType := sValue;
      1002: fInstrumentID := sValue;
      1003: fEDW := sValue;
      // 1004: PRESCRIPTION DATE
      // 1008: PHYSICIAN
      1011: fDialyzer := sValue;
      1019: fType := sValue;
      1022: fTxLength := sValue;
      1024: fTemperature := sValue;
      1025: fBFR := sValue;
      1037: fBolus := sValue;
      1038: fDuration2 := sValue;
      // 1072: NURSE
      1079: fUltrafiltration := sValue;
      1086: fMaintenance := sValue;
      1117: fDialysateFlow := sValue;
    end;
  end;

begin
  sSegment := aNode.ChildNodes['SEQUENCE'].Text;
  Segment := sSegment;
  Vendor := 'GAMBRO_EXALIS';

  sText := aNode.ChildNodes['CODE'].ChildNodes['VALUE'].Text;
  iValueCode :=
    StrToIntDef(piece(aNode.ChildNodes['CODE'].ChildNodes['VALUE'].Text, '^',
    1),
    -1);
  sValue := aNode.ChildNodes['VALUE'].Text;

  if pos('BGS', Segment) = 1 then
    GambroBGS
  else if pos('MGS', Segment) = 1 then
    GambroMGS
  else if pos('EGS', Segment) = 1 then
    GambroEGS
  else if pos('DGS', Segment) = 1 then
    GambroDGS
  else if pos('PGS', Segment) = 1 then
    GambroPGS
  else
    begin
      case iValueCode of
        -1: ;
        1000: VendorPtID := sValue;
      end;
    end;
end;
{$ENDIF}

end.
