unit uMDK_LocalDB;

interface
uses
  Classes
  , SysUtils
  ;
const
  lmActiveStudies = 'Active Studies';

  lmPatientProceduresInfo = 'Patient Procedures Info';
  lmPatientDiagnosisInfo = 'Patient Diagnosis Info';
  lmPatientInfo = 'Patient Info Component';
  lmPatientServiceInfo = 'Patient service Info';
  lmPatientStudyList = 'Patient Study List';

  lmInstrumentResults = 'Instrument Results';

  lmStudyComponent = 'Study Components';
  lmStudyBilling = 'Study Billing';
  lmStudyEncounter = 'Study Encounter';
  lmStudyLocation = 'Study Location';
  lmStudyReport = 'Study Summary Report';
  lmStudyResultList = 'Study Result List';
  lmStudyStatus = 'Study Status';

  lmStudyStatuses = 'Statuses';

  lmAdministrators = 'Administrators';
  lmFacilities = 'Facilities';
  lmLexicon = 'Lexicon';
  lmOptions = 'Options';
  lmPCE = 'PCE';
  lmProviders = 'Providers';
  lmTIU = 'TIU';
  lmTIUList = 'TIU List';
  lmTIUTitles = 'TIU Titles';
  lmVitalsQualifiersAll = 'Vitals All Qualifiers';
  lmVitalsQualifiers = 'Vitals Qualifiers';
  lmVitalsTypes = 'Vitals Tyes';
  lmUserNamesList = 'User Names List';
  lmSensitivItem = 'Sensitive Patients';
  lmSensitivAccess = 'Sensitive Access Info';
  lmSensitivBulletin = 'Sensitive Bulletin Info';

function setItemLocal(aFileName, anItemName, aValue: string): string;
function getItemLocal(aFileName, anItemName: string): string;
function getLocalList(aOptionName: string; var aValue: string): Boolean;
function getLocal(aOptionName: string; var aValue: string): Boolean;
function setLocal(aOptionName: string; aValue: string): Boolean;
function deleteLocal(aOptionName: string): Boolean;
function LocalName(aTag: string; Params: array of const): string;

function setLocalLock(aStudy,aValue: string): Boolean;
function getLocalLock(aStudy: string; var aValue:String): Boolean;

function getLocalStatus(anID: String; var aStatus,aStatusBy,aStatusAt,aWS:String):Boolean;
function setLocalStatus(anID,aStatus,aStatusBy,aStatusAt,aWS:String):Boolean;

procedure setLocalMap;

var
  LocalMap: TStringList;
  SaveRPCResults: Boolean;
  IgnoreNotFoundFIles: Boolean;
  LOCAL_DIR: String = 'CP_HEMO_LOCAL\';

implementation
uses
  Dialogs
  , Controls
  , uMDK_Utils;

////////////////////////////////////////////////////////////////////////////////

function CheckFileName(aName: string): string;
var
  sName: string;
begin
  sName := StringReplace(aName, ',\', '\', []);
  if pos(',', sName) = length(sName) then
    Result := copy(sName, 1, Length(sName) - 1)
  else
    Result := sName;
end;
////////////////////////////////////////////////////////////////////////////////

procedure setLocalMap;
begin
  if not Assigned(LocalMap) then
    LocalMap := TStringList.Create
  else
    LocalMap.Clear;

  if copy(LOCAL_DIR,Length(LOCAL_DIR),1) <> '\' then
    LOCAL_DIR := LOCAL_DIR+'\';

  with LocalMap do
    begin
      add(lmActiveStudies + '=' + LOCAL_DIR + 'STUDIES\ACTIVESTUDIES.XML');

      add(lmPatientStudyList + '=' + LOCAL_DIR + 'PATIENTS\%s\StudyList.XML');
      add(lmPatientInfo + '=' + LOCAL_DIR + 'PATIENTS\%s\%s');
      add(lmPatientProceduresInfo + '=' + LOCAL_DIR +
        'PATIENTS\%s\Clinic_%s\Procedures.TXT');
      add(lmPatientDiagnosisInfo + '=' + LOCAL_DIR +
        'PATIENTS\%s\Clinic_%s\Diagnosis.TXT');
      add(lmPatientServiceInfo + '=' + LOCAL_DIR +
        'PATIENTS\%s\Clinic_%s\ServiceInfo.TXT');

      add(lmStudyComponent + '=' + LOCAL_DIR + 'STUDIES\%s\%s.XML');
      add(lmStudyBilling + '=' + LOCAL_DIR + 'STUDIES\%s\BILLING.RPC');
      add(lmStudyEncounter + '=' + LOCAL_DIR + 'STUDIES\%s\Encounter.TXT');
      add(lmStudyLocation + '=' + LOCAL_DIR + 'STUDIES\%s\Location.XML');
      add(lmStudyReport + '=' + LOCAL_DIR + 'STUDIES\%s\SummaryReport.TXT');
      add(lmStudyResultList + '=' + LOCAL_DIR + 'STUDIES\%s\ResultList.XML');
      add(lmStudyStatus + '=' + LOCAL_DIR + 'STUDIES\%s\Status.XML');

      add(lmStudyStatuses + '=' + LOCAL_DIR + 'STUDIES\STATUSES.TXT');

      add(lmInstrumentResults + '=' + LOCAL_DIR + 'RESULTS\%s.XML');

      add(lmAdministrators + '=' + LOCAL_DIR + 'USERS\ADMINISTRATORS.XML');
      add(lmFacilities + '=' + LOCAL_DIR + 'USERS\FACILITIES.TXT');
      add(lmLexicon + '=' + LOCAL_DIR + 'LEXICON\%s.TXT');
      add(lmOptions + '=' + LOCAL_DIR + 'USERS\%s\%s');
      add(lmPCE + '=' + LOCAL_DIR + 'PCE\%s');
      add(lmProviders + '=' + LOCAL_DIR + 'USERS\PROVIDERS.TXT');
      add(lmSensitivItem + '=' + LOCAL_DIR + 'PATIENTS\SENSITIVE_ITEMS.TXT');
      add(lmSensitivAccess + '=' + LOCAL_DIR + 'PATIENTS\SENSITIVE_ACCESS.TXT');
      add(lmSensitivBulletin + '=' + LOCAL_DIR +
        'PATIENTS\SENSITIVE_BULLETIN.TXT');
      add(lmTIU + '=' + LOCAL_DIR + 'TIU\%s\%s');
      add(lmTIUList + '=' + LOCAL_DIR + 'TIU\%d\');
      add(lmTIUTitles + '=' + LOCAL_DIR + 'TIU\TITLES.TXT');
      add(lmVitalsQualifiers + '=' + LOCAL_DIR +
        'VITALS\QUALIFIERS_BY_TYPE\%s.TXT');
      add(lmVitalsTypes + '=' + LOCAL_DIR + 'VITALS\Types.TXT');
      add(lmVitalsQualifiersAll + '=' + LOCAL_DIR + 'VITALS\Qualifiers.TXT');
      add(lmUserNamesList + '=' + LOCAL_DIR + 'USERS\USERS.TXT');
    end;
end;

function getItemLocal(aFileName, anItemName: string): string;
var
  SL: TStringList;
  sName: string;
begin
  SL := TStringList.Create;
  sName := CheckFileName(aFileName);
  if FileExists(sName) then
    begin
      SL.LoadFromFile(sName);
      if SL.IndexOfName(anItemName) >= 0 then
        Result := SL.Values[anItemName]
      else
        Result := '';
    end
  else
    Result := '';
  SL.Free;
end;

function setItemLocal(aFileName, anItemName, aValue: string): string;
var
  i: Integer;
  s: string;
  SL: TStringList;
  sName: string;
begin
  s := anItemName;
  i := pos(#13#10, s);
  if i > 0 then
    s := copy(s, 1, i - 1);
  SL := TStringList.Create;
  sName := CheckFileName(aFileName);
  if FileExists(sName) then
    begin
      SL.LoadFromFile(sName);
      if SL.IndexOfName(s) >= 0 then
        SL.Values[s] := aValue
      else
        SL.Add(s + '=' + aValue);
    end
  else
    SL.Add(s + '=' + aValue);
  SL.SaveToFile(sName);
  SL.Free;
end;

function LocalName(aTag: string; Params: array of const): string;
var
  s: string;

begin
  if LocalMap.IndexOfName(aTag) >= 0 then
    begin
      s := LocalMap.Values[aTag];
      Result := Format(s, Params);
    end
  else
    Result := aTag;
end;

function getLocalList(aOptionName: string; var aValue: string): Boolean;
var
  s: string;
  sr: TSearchRec;
  FileAttrs: Integer;
  sName: string;

begin
  FileAttrs := faAnyFile;
  s := '';
  sName := CheckFileName(aOptionName);
  if FindFirst(sName + '*.*', FileAttrs, sr) = 0 then
    begin
      repeat
        if sr.Name = '.' then
          continue;
        if sr.Name = '..' then
          continue;
        s := s + sr.Name + #13;
      until FindNext(sr) <> 0;
      FindClose(sr);
    end;
  aValue := s;
  Result := True;
end;

function getLocal(aOptionName: string; var aValue: string): Boolean;
var
  SL: TStringList;
  sName: string;
begin
  aValue := '';
  Result := True;
  sName := CheckFileName(aOptionName);
  if FileExists(sName) then
    begin
      SL := TStringList.Create;
      SL.LoadFromFile(sName);
      aValue := SL.Text;
      SL.Free;
    end
  else if SaveRPCResults and not IgnoreNotFoundFiles then
    ShowMessage('File <' + sName + '> not found');
end;

function setLocal(aOptionName: string; aValue: string): Boolean;
var
  sName: string;
  SL: TStringList;
  sDir: string;
begin
  Result := True;
  sName := CheckFileName(aOptionName);
  sDir := ExtractFilePath(sName);
  if not DirectoryExists(sDir) then
    if MessageDLG('Directory <' + sDir + '> is not found' + #13 +
      'Create directory?', mtConfirmation, [mbOK, mbNo], 0) <> mrOK then
      Exit
    else
      CreateDir(sDir);

  SL := TStringList.Create;
  SL.Text := aValue;
  try
    SL.SaveToFile(sName);
  except
    on E: Exception do
      if SaveRPCResults then
        ShowMessage(E.Message);
  end;
  SL.Free;
end;

function deleteLocal(aOptionName: string): Boolean;
begin
  if FileExists(aOptionName) then
    Result := DeleteFile(aOptionName)
  else
    begin
      Result := False;
      ShowMessage('File <' + aOptionName + '> is not found');
    end;
end;

function setLocalLock(aStudy,aValue: string): Boolean;
var
  sl: TStringList;
  sName:String;
  s: String;
  i: integer;
begin
  Result := False;
  sName := LocalMap.Values[lmStudyStatuses];
  if not FileExists(sName) then exit;
  sl := TStringList.Create;
  sl.LoadFromFile(sName);
  for i := 0 to sl.Count - 1 do
    begin
      if piece(sl[i],'^',1)<> aStudy then continue;
      s := piece(sl[i],'^',1,5);
      sl[i] := s + '^' + aValue;
      Result := TRUE;
      sl.SaveToFile(sName);
      break;
    end;
  sl.Free;
end;

function getLocalLock(aStudy: string; var aValue:String): Boolean;
var
  sl: TStringList;
  i: integer;
  sName:String;
begin
  aValue := 'Undefined';
  Result := False;
  sName := LocalMap.Values[lmStudyStatuses];
  if not FileExists(sName) then exit;
  sl := TStringList.Create;
  sl.LoadFromFile(sName);
  for i := 0 to sl.Count - 1 do
    begin
      if piece(sl[i],'^',1)<> aStudy then continue;
      aValue := piece(sl[i],'^',6);
      Result := TRUE;
      break;
    end;
  sl.Free;
end;

function getLocalStatus(anID: String; var aStatus,aStatusBy,aStatusAt,aWS:String):Boolean;
var
  sl: TStringList;
  sName:String;
  i: integer;
begin
  Result := False;
  sName := LocalMap.Values[lmStudyStatuses];
  if not FileExists(sName) then exit;
  sl := TStringList.Create;
  sl.LoadFromFile(sName);
  for i := 0 to sl.Count - 1 do
    begin
      if piece(sl[i],'^',1)<> anID then continue;
      aStatus := piece(sl[i],'^',2);
      aStatusBy := piece(sl[i],'^',3);
      aStatusAt := piece(sl[i],'^',4);
      aWS := piece(sl[i],'^',5);
      Result := True;
      break;
    end;
  sl.Free;
end;

function setLocalStatus(anID,aStatus,aStatusBy,aStatusAt,aWS:String):Boolean;
var
  sl: TStringList;
  sName:String;
  s: String;
  i: integer;
begin
  Result := False;
  sName := LocalMap.Values[lmStudyStatuses];
  if not FileExists(sName) then exit;
  sl := TStringList.Create;
  sl.LoadFromFile(sName);
  for i := 0 to sl.Count - 1 do
    begin
      if piece(sl[i],'^',1)<> anID then continue;
      s := piece(sl[i],'^',6);
      sl[i] := anID + '^' + aStatus + '^' + aStatusBy + '^' +aStatusAt + '^'+
        aWS + '^' + s;
      Result := TRUE;
      sl.SaveToFile(sName);
      break;
    end;
  sl.Free;
end;

initialization
  SaveRPCResults := False;
  IgnoreNotFoundFiles := TRUE;
  LocalMap := TStringList.Create;
  setLocalMap;

finalization
  FreeandNil(LocalMap);

end.

