unit uMDK_Notes;
{
================================================================================
*
*       Package:      MDK - Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 12/03/07 5:40p $
*       Site Name:    Hines OIFO
*       Date Created: 7/29/05
*       Developers:   andriyevskiy                  
*       Description:  this unit contains the Hemodialysis application
*                     Notes CDS definition
*       Notes:
*
================================================================================
*       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/APP-HEMODIALYSIS/Objects/uMDK_Notes.pas $
*
*       $ History: uMDK_Notes.pas $
}
interface

uses DB
  , DBClient
  , ComCtrls
  , Forms
  ;

const
  fnNL_ID = 'ID';
  fnNL_Lock = 'Lock';
  fnNL_ParentID = 'ParentID';
  fnNL_PatientID = 'PatientID';
  fnNL_StudyID = 'StudyID';
  fnNL_ItemID = 'ItemID';
  fnNL_Tag = 'Tag';
  fnNL_DateTime = 'DateTime';
  fnNL_AuthorID = 'Author';
  fnNL_AuthorName = 'AuthorName';
  fnNL_Text = 'NoteText';
  fnNL_Status = 'Status';
  fnNL_StatusBy = 'StatusBy';
  fnNL_StatusDate = 'StatusDate';

  fsLocked = 'L';
  fsUnLocked = 'U';

  MAX_NOTE_SIZE = 1000;

procedure SetNoteListFields(aCDS: TClientDataSet);
procedure setNoteListSpecificFields(aCDS: TClientDataSet);
procedure CreateNoteListCDS(aCDS: TClientDataSet);
function addNote(aCDS: TClientDataSet;
  Text,
  AuthorID,
  Tag,
  ItemID,
  Status: string;
  NoteDate,
  StatusDate: TDateTime): string;

function getNotesReport(aTitle: string; aCDS: TClientDataSet; IncludeInvalid:
  Boolean = True): string;
function getNotesContextFilter(aTab: Integer): string;

implementation
uses
  uMDK_CDSCommon
  , SysUtils
  , Graphics
  , uMDK_Options, uMDK_Common, Classes, uMDK_Engine, uMDK_Flowsheet;

function addNote(aCDS: TClientDataSet;
  Text,
  AuthorID,
  Tag,
  ItemID,
  Status: string;
  NoteDate,
  StatusDate: TDateTime): string;
//var
//  dt: TDateTime;
begin
//  try
//    dt := StrToDateTime(NoteDate);
//  except
//    dt := StatusDate;
//  end;

  try
    aCDS.Insert;

    aCDS.FieldByName(fnNL_DateTime).AsDateTime := NoteDate; //dt;
    //anIR.fObservation_Date_Time;

    aCDS.FieldByName(fnNL_AuthorID).AsString := AuthorID; //'FRESENIUS';
    aCDS.FieldByName(fnNL_Tag).AsString := Tag; //'FRESENIUS';
    aCDS.FieldByName(fnNL_ItemID).AsString := ItemID; //'FRESENIUS';
    aCDS.FieldByName(fnNL_Text).AsString := Text; //anIR.fNoteText;
    aCDS.FieldByName(fnNL_Lock).AsString := Status; //fsLocked;

    aCDS.FieldByName(fnNL_Status).AsString := fsValid; //fsLocked;
    aCDS.FieldByName(fnNL_StatusBy).AsString := AuthorID; //
    aCDS.FieldByName(fnNL_StatusDate).AsDateTime := StatusDate;

    aCDS.post;
    Result := '';
  except
    on E: Exception do
      Result := 'addNot Error:+#13#10' + E.Message;
  end;
end;

function getNotesReport(aTitle: string; aCDS: TClientDataSet; IncludeInvalid:
  Boolean = True): string;
var
  SL: TStringList;
  ss, s, sStatus: string;
  BM: TBookmark;
  i, iLL: Integer;

const
  LeftMargin = '    ';

  function getAuthorName: string;
  var
    sName: string;
  begin
    sName := aCDS.FieldByName(fnNL_AuthorID).AsString;
    //check if it is DFN
    if pos(copy(sName, 1, 1), '0123456789') <> 0 then
      begin
        sName :=
          Engine.getUserNameByID(aCDS.FieldByName(fnNL_AuthorID).AsString);
        if trim(sName) = '' then
          sName := ' ' + aCDS.FieldByName(fnNL_AuthorID).AsString;
      end;
    Result := sName;
  end;

  procedure processLine(aLine: string);
  var
    ii: integer;
    sCurrent,
      sUsed,
      sLeft: string;
  begin
    sLeft := leftMargin + aLine;
    repeat
      if length(sLeft) > iLL then
        begin
          sCurrent := copy(sLeft, 1, ill);
          ii := iLL;
          while (copy(sCurrent, ii, 1) <> ' ') and (ii > 1) do
            dec(ii);
          if ii = 1 then
            ii := ill;
          //          s := s + copy(sLeft, 1, ill) + _CR_;
          //          sLeft := leftMargin + copy(sLeft, ill + 1, Length(sLeft));
          sUsed := copy(sLeft, 1, ii);
          if trim(sUsed) = '' then
            begin
              s := s + sCurrent+ _CR_;
              sLeft := leftMargin + copy(sLeft, ill + 1, Length(sLeft));
            end
          else
            begin
              s := s + copy(sLeft, 1, ii) + _CR_;
              sLeft := leftMargin + copy(sLeft, ii + 1, Length(sLeft));
            end;

//          s := s + copy(sLeft, 1, ii) + _CR_;
//          sLeft := leftMargin + copy(sLeft, ii + 1, Length(sLeft));
        end
      else
        begin
          s := s + sLeft + _CR_;
          sLeft := '';
        end;
    until sLeft = '';
  end;

begin
  Result := '';
  if not aCDS.Active then
    Exit;

  iLL := devOptions.iValue(donReportLineLength);
  s := '';
  BM := aCDS.GetBookmark;
  aCDS.First;
  if not aCDS.Eof then
    s := aTitle + _CR_ + _CR_;

  while not aCDS.Eof do
    begin
      sStatus := aCDS.FieldByName(fnNL_Status).AsString;
      if (sStatus <> fsInvalid) or
        IncludeInvalid
        {//        Preferences.bValue(onIncludeInvalidatedCommentsInReport)}then
        begin
          if sStatus = fsInvalid then
            begin
              s := s +
                '-------- The following note was invalidated -------------------------' + _CR_
                +
                '-------- by ' + aCDS.FieldByName(fnNL_StatusBy).AsString +
                ' on ' + aCDS.FieldByName(fnNL_StatusDate).AsString + _CR_
                ;
            end;

          s := s + aCDS.FieldByName(fnNL_DateTime).AsString;
          s := s + '  by ' + getAuthorName + _CR_;
          ss := aCDS.FieldByName(fnNL_Text).AsString;
          if trim(ss) = '' then
            ss := Preferences.sValue(onBlanksPlaceholder)
          else
            begin
              SL := TStringList.Create;
              SL.Text := ss;
              for i := 0 to SL.Count - 1 do
                ProcessLine(SL[i]);
            end;

          if sStatus = fsInvalid then
            s := s +
              '-------- End of the invalidated text --------------------------------' +
              _CR_;
          s := s + _CR_;
        end;
      aCDS.Next;
    end;
  aCDS.GoToBookmark(BM);
  aCDS.FreeBookmark(BM);
  if s = '' then
    s := 'No comments found' + _CR_;
  Result := s;
end;

function getNotesContextFilter(aTab: Integer): string;
begin
  case aTab of
    0: Result := '';
    1: Result := fnNL_Tag + '=''' + pnRxLab + '''';
    2: Result := fnNL_Tag + '=''' + pnPre + '''';
    3: Result := fnNL_Tag + '=''' + pnAccess + '''';
    4: Result := fnNL_Tag + '=''' + pnFlow + '''' + ' or ' + fnNL_Tag +
      '= ''frmFlow''';
    5: Result := fnNL_Tag + '=''' + pnPost + '''';
    6: Result := fnNL_Tag + '=''' + pnSummary + '''';
  else
    Result := '';
  end;
end;

procedure setNoteListSpecificFields(aCDS: TClientDataSet);
var
  aF: TField;
begin
  try
    NewStringField(aCDS, fnNL_Lock, fnNL_Lock, 2, 2);
    aF := NewStringField(aCDS, fnNL_ItemID, fnNL_ItemID, 4, 20);
{$IFNDEF AANTEST}aF.Visible := False;
{$ENDIF}
    aF := NewStringField(aCDS, fnNL_Tag, fnNL_Tag, 4, 20);
{$IFNDEF AANTEST}aF.Visible := False;
{$ENDIF}
    NewStringField(aCDS, fnNL_AuthorID, fnNL_AuthorID, 20, 20);
    NewStringField(aCDS, fnNL_Text, fnNL_text, MAX_NOTE_SIZE, MAX_NOTE_SIZE);
  except
  end;
end;

procedure setNoteListFields(aCDS: TClientDataSet);
var
  aF: TField;
begin
  try
    aF := NewStringField(aCDS, fnNL_ID, fnNL_ID, 4, 20);
{$IFNDEF AANTEST}aF.Visible := False;
{$ENDIF}
    aF := NewStringField(aCDS, fnNL_PatientID, fnNL_PatientID, 4, 20);
{$IFNDEF AANTEST}aF.Visible := False;
{$ENDIF}
    aF := NewStringField(aCDS, fnNL_StudyID, fnNL_StudyID, 4, 20);
{$IFNDEF AANTEST}aF.Visible := False;
{$ENDIF}

    aF := NewStringField(aCDS, fnNL_Status, fnNL_Status, 10, 20);
{$IFNDEF AANTEST}aF.Visible := False;
{$ENDIF}
    aF := NewStringField(aCDS, fnNL_StatusBy, fnNL_StatusBy, 10, 20);
{$IFNDEF AANTEST}aF.Visible := False;
{$ENDIF}
    aF := NewDateTimeField(aCDS, fnNL_StatusDate, fnNL_StatusDate, 20,
      gMDK_DateTimeFMT);
{$IFNDEF AANTEST}aF.Visible := False;
{$ENDIF}

    NewDateTimeField(aCDS, fnNL_DateTime, fnNL_DateTime, 20, gMDK_DateTimeFMT);
    setNoteListSpecificFields(aCDS);
  except
  end;
end;

procedure CreateNoteListCDS(aCDS: TClientDataSet);
begin
  try
    setNoteListFields(aCDS);
    aCDS.CreateDataSet;
    aCDS.IndexFieldNames := fnNL_DateTime;
  except
    raise Exception.Create('createNoteListCDS Error');
  end;
end;

end.
