unit uMDK_Patients;
{
================================================================================
*
*       Package:      MDK - Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 4/22/08 2:48p $
*       Site Name:    Hines OIFO
*       Date Created: 7/29/05
*       Developers:   andriyevskiy                  
*       Description:  this unit contains the Hemodialysis application
*                     Patient object
*       Notes:
*
================================================================================
*       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/APP-HEMODIALYSIS/Objects/uMDK_Patients.pas $
*
*       $ History: uMDK_Patients.pas $
}
interface
uses
  Controls,
  Classes
  , XMLIntf
  ;

type
  TMDKPatient = class(TObject)
  private
    function getAge: string;
  public
    Name: string;
    DOB: TDate;
    Sex: string;
    SSN: string;

    property Age: string read getAge;
    constructor Create(AName, ASSN, ASex: string; ADOB: TDate);
  end;

  TMDKRegisteredPatient = class(TMDKPatient)
  private
    function getDescription: string;
    function getIntID: Int64;
    function getIStudy: Integer;

    function getStudyInfo: string;
    function getStudyInfoLine: string;
    function getPatientInfo: string;
    function getPatientInfoLine: string;

    function getPatientSSN: string;
  public
    ID: string;
    Station: string;
    Machine: string;
    SessionID: string;
    VendorID: string;
    StudyStatus: string;
    StatusBy: string;
    StatusDate: TDateTime;
    Workstation: string;
    StudyCheckin: string;
    LockStatus: string;
    LockedBy: string;
    LastModified: TDateTime;
    Comments: string;
    Source: string;

    ReadOnly: Boolean;

    xmlStudyInfo,
      xmlStudySummary,
      xmlStudyFlowsheet,
      xmlStudyMedList,
      xmlStudyNotes,
      xmlStudyEvents,

    xmlStudyVAP,
      xmlStudyVAPH,
      xmlStudyVAPIH: string;

    constructor Create(AName, ASSN, ASex: string; ADOB: TDate; aStation,
      aMachine, anID, aSessionID, aVendorID, aLocationID: string);
    property Description: string read getDescription;
    property IntID: Int64 read getIntID;
    property iStudy: Integer read getIStudy;

    property StudyInfo: string read getStudyInfo;
    property StudyInfoLine: string read getStudyInfoLine;
    property PatientInfo: string read getPatientInfo;
    property PatientInfoLine: string read getPatientInfoLine;

    property PatientSSN: string read getPatientSSN;
  end;

procedure RefreshPatientList(aList:TStringList);
procedure EmptyPatientList(aList:TStringList);
function NewRegisteredPatientList(aValue: string): TStringList;

var
  MDKPatientList: TStringList;

const
  kw_PtName = 'PatientName';
  kw_PtAge = 'PatientAge';
  kw_PtSex = 'PatientSex';
  kw_PtSSN = 'PatientSSN';
  kw_PtDOB = 'PatientDOB';

  kw_PtID = 'PatientID';
  kw_StationID = 'StationID';
  kw_StudyID = 'StudyID';

const
  snInUse = 'IN USE';
  snEnabled = 'ENABLED';
  snDisabled = 'DISABLED';
  snUnknown = 'unknown';

implementation

uses
  SysUtils
  , uROR_CommonUtils
  , uXML, uMDK_Common
  , uMDK_CDSCommon
  ;

// =========================================================================

function NewRegisteredPatientList(aValue: string): TStringList;
var
  i: Integer;
  sl: TStringList;
  xDOC: IXMLDocument;
  xNode: IXMLNode;
  s,
    aLockStatus, aLockedBy,
    aName, aSSN, aSex: string;
  aDOB: TDate;
  aStatus,aStatusDT,aWS,aUserID,
  aStudyCheckIn,
    aStation, aMachine, anID, aSessionID, aVendorID, aLocation: string;
  anRP: TMDKRegisteredPatient;
const
  sUnknown = 'unknown';
  // Could be placed in the Option List;
  sVendorName = 'INSTRUMENT_VENDOR';
  sStationID = 'STATION_SERIAL_NUMBER';

  sRecord = 'RECORD';
  sPatientName = 'PATIENT_NAME';
  sPatientSSN = 'PATIENT_SSN';
  sPatientSex = 'PATIENT_SEX';
  sPatientID = 'PATIENT_ID';
  sPatientDOB = 'PATIENT_DOB';
  sStudyID = 'STUDY_ID';
  sVendorID = 'VENDOR_ID';
  sLocation = 'STUDY_LOCATION';
  sStudyCheckIn = 'STUDY_CHECKIN';

  sStudyLockStatus = 'STUDY_LOCK_STATUS';
  sStudyLockedBy = 'STUDY_LOCKED_BY';

  sUserID = 'USER_ID';
  sWS = 'WORKSTATION_ID';
  sDateTime = 'DATETIME_ACCESSED';
  sUseStatus = 'STATUS';

  function getValue(aNode: IXMLNode; aValueName, aDefault: string): string;
  begin
    try
      Result := aNode.ChildValues[aValueName];
    except
      Result := aDefault;
    end;
  end;

begin
  Result := TStringList.Create;
  Result.Sorted := True; //vhaishandria 050316
  if aValue = '' then
    Exit;
  try
    sl := TStringList.Create;
    sl.Text := aValue;
    try
      xDOC := XMLDocument(sl.Text);
      with xDoc.DocumentElement do
        begin
          for i := 0 to ChildNodes.Count - 1 do
            begin
              xNode := ChildNodes[i];
              if xNode.NodeName <> sRecord then
                continue;
              aName := getValue(xNode, sPatientName, sUnknown);
              if pos(', ', aName) = 0 then
                aName := piece(aname, ',', 1) + ', ' + piece(aName, ',', 2);
              aSSN := getValue(xNode, spatientSSN, sUnknown);
              if pos('-', aSSN) = 0 then
                aSSN := FormatSSN(aSSN);
              aSex := getValue(xNode, sPatientSex, sUnknown);
              anID := getValue(xNode, sPatientID, sUnknown);

              aSessionID := getValue(xNode, sStudyID, sUnknown);
              try
                s := getValue(xNode, sPatientDOB, sUnknown);
                if s <> sUnKnown then
                  aDOB := FMDateTimeToWindowsDateTime(StrToInt(s))
                else
                  aDOB := 0;
              except
                aDOB := 0;
              end;

              aMachine := getValue(xNode, sVendorName, sUnknown);
              aStation := getValue(xNode, sStationID, sUnknown);
              aVendorID := getValue(xNode, sVendorID, sUnknown);
              aLocation := getValue(xNode, sLocation, sUnknown);
              aStudyCheckIn := getValue(xNode, sStudyCheckIn, sUnknown);

              aLockStatus := getValue(xNode, sStudyLockStatus, '');
              aLockedBy := getValue(xNode, sStudyLockedBy, '');

              aUserID := getValue(xNode,sUserID,'');
              aStatus := getValue(xNode,sUseStatus,'---');
              aStatusDT := getValue(xNode,sDateTime,'');
              aWS := getValue(xNode,sWS,'');

              if aName <> '' then
                begin
                  anRP := TMDKRegisteredPatient.Create(aName, aSSN, aSex, aDOB,
                    aStation, aMachine, anID, aSessionID, aVendorID, aLocation);
                  try
                    anRP.StudyCheckin := FormatDateTime(gMDK_DateTimeFMT,
                      FMDateTimeToWindowsDateTime(StrToFloat(aStudyCheckIn)));
                  except
                    anRP.StudyCheckin := '';
                  end;
                  anRP.LockStatus := aLockStatus;
                  anRP.LockedBy := aLockedBy;
                  anRP.StudyStatus := aStatus;
                  anRP.StatusBy := aUserID;
                  anRP.Workstation := aWS;

                  if aStatusDT <>'' then
                    try
                      anRP.StatusDate := FMDateTimeToWindowsDateTime(
                        StrToFloat(aStatusDT));
                    except
                    end;

                  Result.AddObject(aSessionID, anRP);
                end;
            end;
        end;
    except
    end;
    sl.Free;
  except
  end;
end;

procedure EmptyPatientList(aList:TStringList);
var
  aRP: TMDKRegisteredPatient;
begin
  if Assigned(aList) then
    begin
      while aList.Count > 0 do
        begin
          try
            aRP := TMDKRegisteredPatient(aList.Objects[0]);
            FreeAndNil(aRP);
            aList.Delete(0);
          except
          end;
        end;
    end;
end;

procedure RefreshPatientList(aList:TStringList);
begin
  if Assigned(aList) then
    begin
      EmptyPatientList(aList);
      FreeAndNil(aList);
    end;
end;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
{ TMDKPatient }

constructor TMDKPatient.Create(AName, ASSN, ASex: string; ADOB: TDate);
begin
  inherited Create; //(aName+#13#10+aSSN+#13#10+aSex+#13#10);
  Name := AName;
  SSN := ASSN;
  Sex := ASex;
  DOB := ADOB;
end;

function TMDKPatient.getAge: string;
begin
  result := FormatDateTime('yy', Now - DOB);
end;

{ TMDKRegisteredPatient}

constructor TMDKRegisteredPatient.Create(AName, ASSN, ASex: string; ADOB: TDate;
  aStation, aMachine, anID, aSessionID, aVendorID, aLocationID: string);
begin
  inherited Create(aName, aSSN, aSex, aDOB);
  ID := anID;
  Station := aStation;
  Machine := aMachine;
  SessionID := aSessionID;
  VendorID := aVendorID;
  ReadOnly := False;
end;

function TMDKRegisteredPatient.getDescription: string;
var
  s: string;
begin
  if pos('-', SSN) = 0 then
    s := FormatSSN(SSN)
  else
    s := SSN;
  Result := Format('%s   %s  (%s)  %s' + #13 + 'Station: %s ', [s,
    FormatDateTime('MM/dd/yyyy', DOB), Age, Sex, Station]);
end;

function TMDKRegisteredPatient.getIntID: Int64;
var
  i64: Int64;
begin
  try
    i64 := StrToInt64(ID);
  except
    i64 := 0;
  end;
  Result := i64;
end;

function TMDKRegisteredPatient.getIStudy: Integer;
begin
  try
    Result := StrToIntDef(SessionID, -1);
  except
    Result := 0;
  end;
end;

function TMDKRegisteredPatient.getStudyInfo: string;
begin
  Result := 'Patient <' + ID + '> Study <' + SessionID + '>';
end;

function TMDKRegisteredPatient.getPatientInfo: string;
begin
  Result := 'Patient <' + ID + '>';
end;

function TMDKRegisteredPatient.getStudyInfoLine: string;
begin
  Result := StudyInfo + #1310;
end;

function TMDKRegisteredPatient.getPatientInfoLine: string;
begin
  Result := PatientInfo + #1310;
end;

function TMDKRegisteredPatient.getPatientSSN: string;
var
  i: Integer;
  s: string;
begin
  s := SSN;
  i := 1;
  while i < length(s) do
    if copy(s, i, 1) = '-' then
      s := copy(s, 1, i - 1) + copy(s, i + 1, Length(s))
    else
      inc(i);

  Result := s;
end;

end.
