unit fMDK_VAAdd;
{
================================================================================
*
*       Package:      MDK - Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 4/01/08 8:47a $
*       Site Name:    Hines OIFO
*       Date Created: 7/29/05
*       Developers:   andriyevskiy                  
*       Description:  this unit contains the Hemodialysis application
*                     new Vascular Access record form
*       Notes:
*
================================================================================
*       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/APP-HEMODIALYSIS/VascularAccess/fMDK_VAAdd.pas $
*
*       $ History: fMDK_VAAdd.pas $
}
interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, ComCtrls, Dialogs
  , uMDK_VASite
  ;
type
  TdlgAPAdd = class(TForm)
    Panel2: TPanel;
    Panel4: TPanel;
    Panel1: TPanel;
    pnlAdd: TPanel;
    Panel5: TPanel;
    Label4: TLabel;
    lblOperator: TLabel;
    Label7: TLabel;
    cmbType: TComboBox;
    Label6: TLabel;
    cmbLocation: TComboBox;
    Label2: TLabel;
    Label3: TLabel;
    cmbSurgeon: TComboBox;
    Label1: TLabel;
    cmbFacility: TComboBox;
    dtpOpened: TDateTimePicker;
    OkBtn: TBitBtn;
    CancelBtn: TBitBtn;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    Bevel8: TBevel;
    BitBtn5: TBitBtn;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure BitBtn1Click(Sender: TObject);
    procedure BitBtn2Click(Sender: TObject);
    procedure btnUnlockClick(Sender: TObject);
    procedure bbUnlockSurgeonClick(Sender: TObject);
    procedure bbUnlockFacilityClick(Sender: TObject);
    procedure cmbSurgeonChange(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  dlgAPAdd: TdlgAPAdd;

function NewAccessSite(aPatient, anOperator: string): TVASite;

implementation
uses
  uMDK_Engine, uMDK_Options, fMDK_SelectProvider, uROR_CommonUtils, uMDK_Common;

{$R *.dfm}

function NewAccessSite(aPatient, anOperator: string): TVASite;
var
  ap: TVASite;
  aTime: TdateTime;

begin
  if not Assigned(dlgAPAdd) then
  begin
    Application.CreateForm(TdlgAPAdd, dlgAPAdd);
    aTime := Engine.getCurrentServerDateTime;
    dlgAPAdd.dtpOpened.MaxDate := aTime + 0.1;
    dlgAPAdd.dtpOpened.Date := aTime;
    dlgAPAdd.dtpOpened.Format := 'MM/dd/yy';
  end;
  with dlgAPAdd do
  begin
    AssignList(dlgAPAdd.cmbSurgeon.Items,
      Engine.getSystemOptionByName(onSurgeons));
    dlgAPAdd.cmbSurgeon.Items.Insert(0, '');
    dlgAPAdd.cmbSurgeon.Items.Insert(1, '-- Other --');
    AssignList(dlgAPAdd.cmbFacility.Items,
      Engine.getSystemOptionByName(onMedicalFacilities));
    dlgAPAdd.cmbFacility.Items.Insert(0, '');
    dlgAPAdd.cmbFacility.Items.Insert(1, '-- Other --');

    ActiveControl := cmbType;
    OKBtn.Enabled := True;
    CancelBtn.Enabled := True;
    pnlAdd.Enabled := True;

    cmbType.Style := csDropDownList;
    cmbLocation.Style := csDropDownList;
    dlgAPAdd.lblOperator.Caption := anOperator;

    dtpOpened.Refresh;

    cmbType.Text := '';
    cmbLocation.Text := '';
    cmbSurgeon.Text := '';
    cmbFacility.Text := '';

    try
      cmbType.ItemIndex := -1;
      cmbLocation.ItemIndex := -1;
    except
    end;

    if dlgAPAdd.ShowModal = mrOK then
    begin
      ap := TVASite.Create;

      ap.ID := FormatDateTime('yymmddhhMMsszzz', now);
      ap.Status := asPlaced;
      ap.AccessType := SiteTypeByName(cmbType.Text);
      ap.Surgeon := cmbSurgeon.Text;
      ap.Location := cmbLocation.Text;
      ap.Facility := cmbFacility.Text;
      ap.PlacedDate := dtpOpened.DateTime;
      ap.RegisteredBy := anOperator;
      ap.PatientID := aPatient;
      Result := ap;
    end
    else
      Result := nil;
  end;

  FreeAndNil(dlgAPAdd);
end;

procedure TdlgAPAdd.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var
  s: string;
begin
  if ModalResult = mrCancel then
  begin
  end
  else if (cmbType.Text = '')
    or (cmbLocation.Text = '')
    or (cmbSurgeon.Text = '')
    or (cmbFacility.Text = '') then
  begin
    if cmbType.Text = '' then
      s := s + char(vk_tab) + 'Type' + #13#10;
    if cmbLocation.Text = '' then
      s := s + char(vk_tab) + 'Location' + #13#10;
    if cmbsurgeon.Text = '' then
      s := s + char(vk_tab) + 'Surgeon' + #13#10;
    if cmbFacility.Text = '' then
      s := s + char(vk_tab) + 'Facility' + #13#10;
    MessageDLG('Input is not complete.' + #13#10 +
      'Please, specify' + #13#10 + s + #13#10 + 'of the access site', mtError,
      [mbOK], 0);
    CanClose := False;
  end;
end;

procedure TdlgAPAdd.BitBtn1Click(Sender: TObject);
var
  s: string;
  i: integer;
begin
  s := SelectProvider('', Engine.fRenalController);
  s := piece(s, '^', 1);
  if s <> '' then
  begin
    i := cmbSurgeon.Items.IndexOf(s);
    if i < 0 then
    begin
      cmbSurgeon.Items.Insert(0, s);
      i := 0;
    end;
    cmbSurgeon.ItemIndex := i;
  end
  else
    cmbSurgeon.ItemIndex := 0;
end;

procedure TdlgAPAdd.BitBtn2Click(Sender: TObject);
var
  s: string;
  i: integer;
begin
  s := SelectProvider('', Engine.fRenalController, 1);
  s := piece(s, '^', 1);
  if s <> '' then
  begin
    i := cmbFacility.Items.IndexOf(s);
    if i < 0 then
    begin
      cmbFacility.Items.Insert(0, s);
      i := 0;
    end;
    cmbFacility.ItemIndex := i;
  end
  else
    cmbFacility.ItemIndex := 0;
end;

procedure TdlgAPAdd.btnUnlockClick(Sender: TObject);
begin
  cmbSurgeon.Style := csDropDown;
  cmbFacility.Style := csDropDown;
end;

procedure TdlgAPAdd.bbUnlockSurgeonClick(Sender: TObject);
begin
  cmbSurgeon.Style := csDropDown;
end;

procedure TdlgAPAdd.bbUnlockFacilityClick(Sender: TObject);
begin
  cmbFacility.Style := csDropDown;
end;

procedure TdlgAPAdd.cmbSurgeonChange(Sender: TObject);
var
  s: string;
begin
  if TComboBox(Sender).Text = '-- Other --' then
  begin
    if Sender = cmbSurgeon then
      s := InputBox('Surgeon Name Input Box', 'Surgeon Name', '');
    if Sender = cmbFacility then
      s := InputBox('Medical Facility Name Input Box',
        'Medical Facility Name',
        '');
    if s <> '' then
    begin
      TComboBox(Sender).Items.Insert(1, s);
      TComboBox(Sender).ItemIndex := 1;
    end
    else
      TComboBox(Sender).ItemIndex := 0;
  end;
end;

end.

