unit uMDK_VADataSet;
{
================================================================================
*
*       Package:      MDK - Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 9/28/07 10:39a $
*       Site Name:    Hines OIFO
*       Date Created: 7/29/05
*       Developers:   andriyevskiy                  
*       Description:  this unit contains the Hemodialysis application
*                     Vascular access routines
*       Notes:
*
================================================================================
*       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/APP-HEMODIALYSIS/VascularAccess/uMDK_VADataSet.pas $
*
*       $ History: uMDK_VADataSet.pas $
}
interface
uses
  SysUtils
  , DBClient, DB
  , uMDK_VASite
  , Variants
  ;

procedure CreateVASiteDS(cdsAP: TClientDataSet);
procedure CreateVAInfectionDS(cdsVAInfection: TClientDataSet; aMaster:
  TDataSource);
procedure CreateVAHistoryDS(cdsAPHistory: TClientDataSet; aMaster: TDataSource);

function AddSiteRecord(aDS: TClientDataSet; aSite: TVASite): Boolean;
function AddVAHistoryRecord(aDS: TClientDataSet; aSite: TVASite; aSession:
  string): Boolean;
function AddVAInfectionHistoryRecord(aDS: TClientDataSet; aSite: TVASite;
  aSession: string): Boolean;

function getVASiteRecord(aDS: TClientDataSet): TVASite;
function getVAHRecord(aDS: TClientDataSet): TVASIteHistoryRecord;
function getVAIHRecord(aDS: TClientDataSet): TVASiteInfectionRecord;

function setVASiteRecord(aDS: TClientDataSet; aSite: TVASite): string;
function setVAHRecord(aDS: TClientDataSet; aHistoryRecord:
  TVASIteHistoryRecord): string;
function setVAIHRecord(aDS: TClientDataSet; anInfectionRecord:
  TVASiteInfectionRecord): string;

const
  fnSite_ID = 'ID';
  fnSite_Patient = 'Patient';
  fnSite_Status = 'Status';
  fnSite_AType = 'Kind';
  fnSite_Location = 'Location';
  fnSite_PlacedDate = 'Placed';
  fnSite_PlacedBy = 'Surgeon';
  fnSite_PlacedAt = 'Facility';
  fnSite_PlacementID = 'PlacementID';
  fnSite_RegisteredBy = 'Registered';
  fnSite_RemovedDate = 'Removed';
  fnSite_RemovedBy = 'RSurgeon';
  fnSite_RemovedAt = 'RFacility';
  fnSite_RRegisteredBy = 'UnRegistered';
  fnSite_RReason = 'RReason';
  fnSite_RemovalID = 'RemovalID';

  ////////////////////////////////////////////////////////////////////////////////

  fnAPH_ID = 'ID';
  fnAPH_StudyID = 'StudyID';
  fnAPH_Selected = 'Selected';
  fnAPH_Assessed = 'Assessed';
  fnAPH_DateTime = 'DateTime';
  fnAPH_RegisteredBy = 'Registered';

  fnAPH_Thrill = 'Thrill';
  fnAPH_Bruit = 'Bruit';
  fnAPH_Mature = 'Mature';
  fnAPH_Infiltrated = 'Infiltrated';

  fnAPH_Comments = 'Comments';

  //  fnAPH_ABF = 'APBloodFlow';
  //  fnAPH_VP0 = 'VP0';
  //  fnAPH_VP200 = 'VP200';

  fnAPH_TTreatment = 'TTreatment';

  // All
  fAPH_Condition = 'Condition';

  // Catheters
  fAPH_BloodCultures = 'BloodCultures';
  fAPH_DressingChange = 'DressingChange';
  fAPH_Sutures = 'Sutures';

  fAPH_PortVolume = 'PortVolume';
  fAPH_PortTx = 'PortTx';
  fAPH_Arterial = 'Arterial';
  fAPH_Venous = 'Venous';
  fAPH_HeparinAdded = 'HeparinAdded';

  // Fistula
  fAHP_ThrillInfo = 'ThrillInfo';
  fAHP_BruitInfo = 'BruitInfo';

  ////////////////////////////////////////////////////////////////////////////////

  fnAPIH_ID = 'ID';
  fnAPIH_StudyID = 'StudyID'; // vhaishandria 050328
  fnAPIH_InfectionDateTime = 'InfectionDateTime';
  fnAPIH_Infection = 'Infection';
  fnAPIH_RegisteredBy = 'RegisteredBy';
  fnAPIH_RegisteredDateTime = 'RegisteredDateTime';

  ////////////////////////////////////////////////////////////////////////////////

    //----------------------------------------------------------------------------
    //  Error Messages
    //----------------------------------------------------------------------------

  etVASiteNew = 'VASiteNew';
  etVASiteNewFailed = 'VASiteNewFailed';
  etVASiteSelected = 'VASiteSelected';
  etVASiteSelectedFailed = 'VASiteSelectionFailed';
  etVASiteUnSelected = 'VASiteUnSelected';
  etVASiteUnSelectedFailed = 'VASiteUnSelectionFailed';
  etVASiteRemoved = 'VASIteRemoved';
  etVASiteRemovedFailed = 'VASIteRemovalFailure';
  etVASiteUpdated = 'VASiteUPdated';
  etVASiteUpdateFailed = 'VASiteUpdate Failed';

  etVAHistoryAdded = 'VAHistoryAdded';
  etVAInfectionHistoryAdded = 'VAInfectionHistoryAdded';
  etVAHistoryUpdated = 'VAHistoryUpdated';
  etVAHistoryUpdateFailed = 'VAHistoryUpdateFailed';

  etVAInfectionHistoryUpdated = 'Infection History Updated';
  etVAInfectionHistoryUpdateFailed = 'Infection History Update Failed';

implementation

uses uMDK_CDSCommon, uMDK_Engine, uMDK_Info;

procedure CreateVASiteDS(cdsAP: TClientDataSet);
begin
  try
    NewStringField(cdsAP, fnSite_ID, 'Site #', 4, 20);
    NewStringField(cdsAP, fnSite_Patient, 'Patient #', 4, 20);

    NewStringField(cdsAP, fnSite_Status, 'Status', 9, 20);
    NewStringField(cdsAP, fnSite_AType, 'Type', 9, 20);
    NewStringField(cdsAP, fnSite_Location, 'Location', 15, 40);

    NewDateTimeField(cdsAP, fnSite_PlacedDate, 'Placed Date', 8);
    NewStringField(cdsAP, fnSite_PlacedBy, 'Place By', 9, 40);
    NewStringField(cdsAP, fnSite_PlacedAt, 'Facility', 15, 40);
    NewStringField(cdsAP, fnSite_PlacementID, 'Procedure #', 15, 40);
{$IFNDEF AANTEST}
    cdsAP.FieldList.Fields[cdsAP.FieldCount - 1].Visible := False;
{$ENDIF}
    NewStringField(cdsAP, fnSite_RegisteredBy, 'Registered By', 15, 40);
{$IFNDEF AANTEST}
    cdsAP.FieldList.Fields[cdsAP.FieldCount - 1].Visible := False;
{$ENDIF}

    NewDateTimeField(cdsAP, fnSite_RemovedDate, 'Removed', 8);
    NewStringField(cdsAP, fnSite_RemovedBy, 'RemovedBy', 9, 40);
    NewStringField(cdsAP, fnSite_RReason, 'Reason', 15, 40);
    NewStringField(cdsAP, fnSite_RemovedAt, 'Removal Facility', 15, 40);
    NewStringField(cdsAP, fnSite_RemovalID, 'Procedure #', 15, 40);
{$IFNDEF AANTEST}
    cdsAP.FieldList.Fields[cdsAP.FieldCount - 1].Visible := False;
{$ENDIF}
    NewStringField(cdsAP, fnSite_RRegisteredBy, 'RRegistered By', 15, 40);
{$IFNDEF AANTEST}
    cdsAP.FieldList.Fields[cdsAP.FieldCount - 1].Visible := False;
{$ENDIF}
    cdsAP.CreateDataSet;
  except
  end;
end;

procedure CreateVAHistoryDS(cdsAPHistory: TClientDataSet; aMaster: TDataSource);
begin
  try
    NewStringField(cdsAPHistory, fnAPH_ID, '#', 4, 20);

    NewStringField(cdsAPHistory, fnAPH_StudyID, 'Session', 4, 40);
    NewStringField(cdsAPHistory, fnAPH_RegisteredBy, 'RegisteredBy', 15, 40);
    NewDateTimeField(cdsAPHistory, fnAPH_DateTime, 'Date', 8);
    NewBooleanField(cdsAPHistory, fnAPH_Selected, 'Selected', 4);
    NewBooleanField(cdsAPHistory, fnAPH_Assessed, 'Assessed', 4);

    NewBooleanField(cdsAPHistory, fnAPH_Thrill, 'Thrill', 4);
    NewBooleanField(cdsAPHistory, fnAPH_Bruit, 'Bruit', 4);
    NewBooleanField(cdsAPHistory, fnAPH_Mature, 'Mature', 4);
    NewBooleanField(cdsAPHistory, fnAPH_Infiltrated, 'Infiltrated', 4);

    NewBooleanField(cdsAPHistory, fnAPH_TTreatment, 'TTreatment', 4);
    //    NewStringField(cdsAPHistory,fnAPH_ABF,'BloodFlow',15,10);
    //    NewStringField(cdsAPHistory,fnAPH_VP0,'VP0',15,10);
    //    NewStringField(cdsAPHistory,fnAPH_VP200,'VP200',15,10);
    NewStringField(cdsAPHistory, fnAPH_Comments, 'Comments', 400, 400);

    ////////////////////////////////////////////////////////////////////////////
    NewStringField(cdsAPHistory, fAPH_Condition, fAPH_Condition, 15, 10);
    // Catheters
    NewStringField(cdsAPHistory, fAPH_BloodCultures, fAPH_BloodCultures, 3, 3);
    NewStringField(cdsAPHistory, fAPH_DressingChange, fAPH_DressingChange, 3,
      3);
    NewStringField(cdsAPHistory, fAPH_Sutures, fAPH_Sutures, 20, 10);

    NewStringField(cdsAPHistory, fAPH_PortVolume, fAPH_PortVolume, 10, 10);
    NewStringField(cdsAPHistory, fAPH_PortTx, fAPH_PortTx, 10, 10);
    NewStringField(cdsAPHistory, fAPH_Arterial, fAPH_Arterial, 10, 10);
    NewStringField(cdsAPHistory, fAPH_Venous, fAPH_Venous, 10, 10);
    NewStringField(cdsAPHistory, fAPH_HeparinAdded, fAPH_HeparinAdded, 10, 10);

    // Fistula
    NewStringField(cdsAPHistory, fAHP_ThrillInfo, fAHP_ThrillInfo, 10, 10);
    NewStringField(cdsAPHistory, fAHP_BruitInfo, fAHP_BruitInfo, 20, 20);

// v. 1.0 build 312 (#18) - sort order changed for history records ------- start
//    cdsAPHistory.IndexFieldNames := fnAPH_ID;
    cdsAPHistory.IndexFieldNames := fnAPH_ID+';'+fnAPH_StudyID;
// v. 1.0 build 312 (#18) - sort order changed for history records  -------- end
    cdsAPHistory.MasterFields := fnAPH_ID;
    cdsAPHistory.MasterSource := aMaster;
    cdsAPHistory.CreateDataSet;

    //    cdsAPHistory.AddIndex(fnAPH_DateTime+'_Index',fnAPH_DateTime,[ixDescending]);
    //    cdsAPHistory.IndexFieldNames := fnAPH_DateTime;
    //    cdsAPHistory.IndexDefs.Update;

  except
  end;
end;

procedure CreateVAInfectionDS(cdsVAInfection: TClientDataSet; aMaster:
  TDataSource);
begin
  try
    NewStringField(cdsVAInfection, fnAPIH_ID, '#', 4, 20);
{$IFNDEF AANTEST}
    cdsVAInfection.FieldList.Fields[cdsVAInfection.FieldList.Count - 1].Visible
      := False;
{$ENDIF}
    NewStringField(cdsVAInfection, fnAPIH_StudyID, 'StudyID', 4, 40);
{$IFNDEF AANTEST}
    cdsVAInfection.FieldList.Fields[cdsVAInfection.FieldList.Count - 1].Visible
      := False;
{$ENDIF}
    NewDateTimeField(cdsVAInfection, fnAPIH_InfectionDateTime, 'Date', 8);
    NewStringField(cdsVAInfection, fnAPIH_Infection, 'Infection', 15, 200);
    NewStringField(cdsVAInfection, fnAPIH_RegisteredBy, 'Registered By', 15,
      40);
    NewDateTimeField(cdsVAInfection, fnAPIH_RegisteredDateTime,
      'Date Registered', 8);

    cdsVAInfection.IndexFieldNames := fnAPIH_ID;
    cdsVAInfection.MasterFields := fnSite_ID;
    cdsVAInfection.MasterSource := aMaster;

    cdsVAInfection.CreateDataSet;
  except
  end;
end;

////////////////////////////////////////////////////////////////////////////////

function AddSiteRecord(aDS: TClientDataSet; aSite: TVASite): Boolean;
begin
  Result := False;
  if not Assigned(aDS) then
    Exit;
  try
    aDS.Insert;
    aDS.FieldByName(fnSite_ID).AsString := aSite.ID;
    aDS.FieldByName(fnSite_PlacedDate).AsDateTime := aSite.PlacedDate;
    aDS.FieldByName(fnSite_Location).AsString := aSite.Location;
    aDS.FieldByName(fnSite_PlacedAt).AsString := aSite.Facility;
    aDS.FieldByName(fnSite_PlacedBy).AsString := aSite.Surgeon;
    aDS.FieldByName(fnSite_RegisteredBy).AsString := aSite.RegisteredBy;
    aDS.FieldByName(fnSite_AType).AsString :=
      SiteTypeNameByType(aSite.AccessType);

    aDS.FieldByName(fnSite_Patient).AsString := aSite.PatientID;
    aDS.FieldByName(fnSite_Status).AsString := SiteStatusNameByStatus(asPlaced);
    // PlacementID -- ?
    aDS.Post;
    Result := True;
  except
  end;
end;

function AddVAHistoryRecord(aDS: TClientDataSet; aSite: TVASite; aSession:
  string): Boolean;
begin
  Result := True;
  if not Assigned(aDS) then
    Exit;
  if not aDS.Active then
    aDS.Active := true;
  try
    aDS.Insert;
    aDS.FieldByName(fnAPH_ID).AsString := aSite.ID;

    aDS.FieldByName(fnAPH_StudyID).AsString := aSession;

    aDS.FieldByName(fnAPH_DateTime).AsDateTime := aSite.AssessData.APH_DateTime;
    aDS.FieldByName(fnAPH_Selected).AsBoolean := aSite.AssessData.APH_Selected;
    aDS.FieldByName(fnAPH_Thrill).AsBoolean := aSite.AssessData.APH_Thrill;
    aDS.FieldByName(fnAPH_Bruit).AsBoolean := aSite.AssessData.APH_Bruit;
    aDS.FieldByName(fnAPH_Mature).AsBoolean := aSite.AssessData.APH_Mature;
    aDS.FieldByName(fnAPH_Infiltrated).AsBoolean :=
      aSite.AssessData.APH_Infiltrated;
    aDS.FieldByName(fnAPH_TTreatment).AsBoolean :=
      aSite.AssessData.APH_TTreatment;

    //    aDS.FieldByName(fnAPH_ABF).AsString := aSite.AssessData.APH_ABF;
    //    aDS.FieldByName(fnAPH_VP0).AsString := aSite.AssessData.APH_VP0;
    //    aDS.FieldByName(fnAPH_VP200).AsString := aSite.AssessData.APH_VP200;

    aDS.FieldByName(fnAPH_Comments).AsString := aSite.AssessData.APH_Comments;
    aDS.Post;
    Result := True;
  except
    Result := False;
  end;
end;

function AddVAInfectionHistoryRecord(aDS: TClientDataSet; aSite: TVASite;
  aSession: string): Boolean;
begin
  Result := True;
  if not Assigned(aDS) then
    Exit;
  if not aDS.Active then
    try
      aDS.Open;
      aDS.Insert;

      aDS.FieldByName(fnAPH_ID).AsString := aSite.ID;
      aDS.FieldByName(fnAPH_StudyID).AsString := aSession;
      aDS.FieldByName(fnAPIH_InfectionDateTime).AsDateTime :=
        aSite.InfectionData.APIH_InfectionDateTime;
      aDS.FieldByName(fnAPIH_Infection).AsString :=
        aSite.InfectionData.APIH_Infection; //'Infection';
      aDS.FieldByName(fnAPIH_RegisteredBy).AsString :=
        aSite.InfectionData.APIH_RegisteredBy; //'RegisteredBy';
      aDS.FieldByName(fnAPIH_RegisteredDateTime).AsDateTime := Now;

      aDS.Post;
      Result := True;
    except
      Result := False;
    end;
end;

////////////////////////////////////////////////////////////////////////////////

function getVASiteRecord(aDS: TClientDataSet): TVASite;
var
  aVAS: TVASite;

begin
  try
    Result := nil;
    if not Assigned(aDS) then
      Exit;
    if aDS.FieldByName(fnSite_ID).AsString = '' then
      Exit;

    aVAS := TVASite.Create;
    with aVAS do
      begin
        ID := aDS.FieldByName(fnSite_ID).AsString;
        Status := SiteStatusByName(aDS.FieldByName(fnSite_Status).AsString);
        AccessType := SiteTypeByName(aDS.FieldByName(fnSite_AType).AsString);

        Location := aDS.FieldByName(fnSite_Location).AsString;
        PlacedDate := aDS.FieldByName(fnSIte_PlacedDate).AsDateTime;
        Surgeon := aDS.FieldByName(fnSite_PlacedBy).AsString;
        Facility := aDS.FieldByName(fnSite_PlacedAt).AsString;
        RegisteredBy := aDS.FieldByName(fnSite_RegisteredBy).AsString;
        //    fPlacementID: String;

        if aDS.FieldByName(fnSite_RemovedDate).AsFloat > 1 then
          RemovalDate := aDS.FieldByName(fnSite_RemovedDate).AsDateTime;

        RemovalSurgeon := aDS.FieldByName(fnSite_RemovedBy).AsString;
        RemovalFacility := aDS.FieldByName(fnSite_RemovedAt).AsString;
        RemovalReason := aDS.FieldByName(fnSite_RReason).AsString;
        //        RemovalID:= aDS.FieldByName(fnSite_RemovalID).AsString;
        RemovalRegisteredBy := aDS.FieldByName(fnSite_RRegisteredBy).AsString;
{$IFDEF AANTEST}
        if (RemovalSurgeon <> '') or (RemovalFacility <> '')
          or (RemovalReason <> '') or (RemovalDate > 1) then
          Status := asRemoved;
{$ENDIF}
        {
             Selected: Boolean;//  Will be updated later
             fAssessed: Boolean;
             fLastInfected: TDateTime;
         }
        Result := aVAS;
      end;
  except
    if Assigned(aVAS) then
      FreeAndNil(aVAS);
    Result := nil;
  end;
end;

function getVAHRecord(aDS: TClientDataSet): TVASIteHistoryRecord;
var
  aVAHR: TVASiteHistoryRecord;
begin
  Result := nil;
  if not Assigned(aDS) then
    Exit;

  aVAHR := TVASiteHistoryRecord.Create;
  with aVAHR do
    begin
      APH_ID := aDS.FieldByName(fnAPH_ID).AsString; //'ID';
      APH_SessionID := aDS.FieldByName(fnAPH_StudyID).AsString; //'SessionID';
      if aDS.FieldByName(fnAPH_DateTime).AsDateTime < 1 then
        APH_DateTime := 0
      else
        APH_DateTime := aDS.FieldByName(fnAPH_DateTime).AsDateTime; //'DateTime';

      APH_Selected := aDS.FieldByName(fnAPH_Selected).AsBoolean; //'Selected';
      APH_Assessed := aDS.FieldByName(fnAPH_Assessed).AsBoolean; //'Selected';
      APH_RegisteredBy := aDS.FieldByName(fnAPH_RegisteredBy).AsString;
      //'Registered';

      APH_Thrill := aDS.FieldByName(fnAPH_Thrill).AsBoolean; //'Thrill';
      APH_Bruit := aDS.FieldByName(fnAPH_Bruit).AsBoolean; //'Bruit';
      APH_Mature := aDS.FieldByName(fnAPH_Mature).AsBoolean; //'Mature';
      APH_Infiltrated := aDS.FieldByName(fnAPH_Infiltrated).AsBoolean;
      //'Infiltrated';
      APH_Comments := aDS.FieldByName(fnAPH_Comments).AsString; //'Comments';

      APH_TTreatment := aDS.FieldByName(fnAPH_TTreatment).AsBoolean; //'Thrill';
      //        APH_ABF := aDS.FieldByName(fnAPH_ABF).AsString;
      //        APH_VP0 := aDS.FieldByName(fnAPH_VP0).AsString;
      //        APH_VP200 := aDS.FieldByName(fnAPH_VP200).AsString;
    end;
  Result := aVAHR;
end;

function getVAIHRecord(aDS: TClientDataSet): TVASiteInfectionRecord;
var
  aVAIHR: TVASiteInfectionRecord;
begin
  Result := nil;
  if not Assigned(aDS) then
    Exit;
  aVAIHR := TVASiteInfectionRecord.Create;
  with aVAIHR do
    begin
      APIH_ID := aDS.FieldByName(fnAPIH_ID).AsString; // 'ID';
      APIH_StudyID := aDS.FieldByName(fnAPIH_StudyID).AsString; // 'StudyID';
      APIH_InfectionDateTime :=
        aDS.FieldByName(fnAPIH_InfectionDateTime).AsDateTime;
      // 'InfectionDateTime';
      APIH_Infection := aDS.FieldByName(fnAPIH_Infection).AsString;
      // 'Infection';
      APIH_RegisteredBy := aDS.FieldByName(fnAPIH_RegisteredBy).AsString;
      // 'RegisteredBy';
      APIH_RegisteredDateTime :=
        aDS.FieldByName(fnAPIH_RegisteredDateTime).AsDateTime;
      // 'RegisteredDateTime';
    end;
  Result := aVAIHR;
end;

////////////////////////////////////////////////////////////////////////////////

function setVASiteRecord(aDS: TClientDataSet; aSite: TVASite): string;

begin
  Result := etVASiteUpdateFailed;
  try
    try // find the access point
      if not aDS.Locate(fnSite_ID, VarArrayOf([aSite.ID]), [loPartialKey]) then
        begin
          aDS.Insert; // add new one if not found
          aDS.Edit;
          aDS.FieldByName(fnSite_ID).AsString := aSite.ID;
          aDS.Post;
        end;
    except
      aDS.Insert;
      aDS.Edit;
      aDS.FieldByName(fnSite_ID).AsString := aSite.ID;
      aDS.Post;
    end;
    aDS.Edit;

    aDS.FieldByName(fnSite_Patient).AsString := aSite.PatientID;
    aDS.FieldByName(fnSite_Status).AsString :=
      SiteStatusNameByStatus(aSite.Status);

    aDS.FieldByName(fnSite_PlacedDate).AsDateTime := aSite.PlacedDate;
    aDS.FieldByName(fnSite_Location).AsString := aSite.Location;
    aDS.FieldByName(fnSite_PlacedAt).AsString := aSite.Facility;
    aDS.FieldByName(fnSite_PlacedBy).AsString := aSite.Surgeon;
    aDS.FieldByName(fnSite_RegisteredBy).AsString := aSite.RegisteredBy;
    aDS.FieldByName(fnSite_AType).AsString :=
      SiteTypeNameByType(aSite.AccessType);

    aDS.FieldByName(fnSite_RemovedDate).AsDateTime := aSite.RemovalDate;
    aDS.FieldByName(fnSite_RemovedBy).AsString := aSite.RemovalSurgeon;
    aDS.FieldByName(fnSite_RemovedAt).AsString := aSite.RemovalFacility;
    aDS.FieldByName(fnSite_RReason).AsString := aSite.RemovalReason;
    aDS.FieldByName(fnSite_RRegisteredBy).AsString := aSite.RemovalRegisteredBy;

    //    aDS.FieldByName(fnSite_Patient).AsString := aSite.PatientID;
    //    aDS.FieldByName(fnSite_Status).AsString := SiteStatusNameByStatus(asPlaced);
        // PlacementID -- ?
    aDS.Post;

    Result := etVASiteUpdated;
  except
  end;
end;

function setVAHRecord(aDS: TClientDataSet; aHistoryRecord:
  TVASIteHistoryRecord): string;

  procedure FindRecord;
  begin
    aDS.First;
    try
      if not aDS.Locate(fnSite_ID + ';' + fnAPH_StudyID,
        VarArrayOf([aHistoryRecord.APH_ID, aHistoryRecord.APH_SessionID]),
        [loPartialKey]) then
        begin
          aDS.Insert;

          aDS.Edit;
          aDS.FieldByName(fnSite_ID).AsString := aHistoryRecord.APH_ID;
          aDS.FieldByName(fnAPH_StudyID).AsString :=
            aHistoryRecord.APH_SessionID;
          aDS.Post;
        end
      else
        aDS.Edit;
    except
      aDS.Insert;
      aDS.Edit;
      aDS.FieldByName(fnSite_ID).AsString := aHistoryRecord.APH_ID;
      aDS.FieldByName(fnAPH_StudyID).AsString := aHistoryRecord.APH_SessionID;
      //      aDS.FieldByName(fnAPH_DateTime).AsDateTime :=
      //        Engine.Info_CDS.FieldByName(fnI_StudyDate).AsDateTime;
      aDS.Post;
    end;
  end;

begin
  Result := etVAHistoryUpdateFailed;
  // try to find add if not found;
  try
    FindRecord;
    aDS.Edit;
    aDS.FieldByName(fnAPH_Selected).AsBoolean := aHistoryRecord.APH_Selected;
    aDS.FieldByName(fnAPH_Assessed).AsBoolean := aHistoryRecord.APH_Assessed;
    aDS.FieldByName(fnAPH_TTreatment).AsBoolean :=
      aHistoryRecord.APH_TTreatment;
    aDS.FieldByName(fnAPH_Thrill).AsBoolean := aHistoryRecord.APH_Thrill;
    aDS.FieldByName(fnAPH_Bruit).AsBoolean := aHistoryRecord.APH_Bruit;
    aDS.FieldByName(fnAPH_Mature).AsBoolean := aHistoryRecord.APH_Mature;
    aDS.FieldByName(fnAPH_Infiltrated).AsBoolean :=
      aHistoryRecord.APH_Infiltrated;
    aDS.FieldByName(fnAPH_RegisteredBy).AsString :=
      aHistoryRecord.APH_RegisteredBy;
    if aHistoryRecord.APH_DateTime <> 0 then
      aDS.FieldByName(fnAPH_DateTime).AsDateTime := aHistoryRecord.APH_DateTime
        // vhaishandria 20070613. build 312 #8
      // The next code added to avoid saving blank values as data for the
      // treatment date in case no one review the status of the access point
    else
      aDS.FieldByName(fnAPH_DateTime).AsDateTime :=
        Engine.Info_CDS.FieldByName(fnI_StudyDate).AsDateTime;

    //    aDS.FieldByName(fnAPH_ABF).AsString:= aHistoryRecord.APH_ABF;
    //    aDS.FieldByName(fnAPH_VP0).AsString:= aHistoryRecord.APH_VP0;
    //    aDS.FieldByName(fnAPH_VP200).AsString:= aHistoryRecord.APH_VP200;

    aDS.FieldByName(fnAPH_Comments).AsString := aHistoryRecord.APH_Comments;

    aDS.Post;
    Result := etVAHistoryUpdated;
  except
  end;
end;

function setVAIHRecord(aDS: TClientDataSet; anInfectionRecord:
  TVASiteInfectionRecord): string;

  procedure FindRecord;
  begin
    aDS.First;
    try
      if not aDS.Locate(fnAPIH_ID {+';'+fnAPIH_StudyID},
        VarArrayOf([anInfectionRecord.APIH_ID]), [loPartialKey]) then
        begin
          aDS.Insert;
          aDS.Edit;
          aDS.FieldByName(fnAPIH_ID).AsString := anInfectionRecord.APIH_ID;
          aDS.FieldByName(fnAPIH_StudyID).AsString :=
            anInfectionRecord.APIH_StudyID;
          aDS.Post;
        end
      else
        aDS.Edit;
    except
      aDS.Insert;
      aDS.Edit;
      aDS.FieldByName(fnAPIH_ID).AsString := anInfectionRecord.APIH_ID;
      aDS.FieldByName(fnAPIH_StudyID).AsString :=
        anInfectionRecord.APIH_StudyID;
      aDS.Post;
    end;
  end;

begin
  Result := etVAInfectionHistoryUpdateFailed;
  // try to find add if not found;
  try
    FindRecord;
    if anInfectionRecord.APIH_Infection = '' then
      begin
        aDS.Delete;
      end
    else
      begin
        aDS.Edit;
        aDS.FieldByName(fnAPIH_Infection).AsString :=
          anInfectionRecord.APIH_Infection;
        aDS.FieldByName(fnAPIH_InfectionDateTime).AsDateTime :=
          anInfectionRecord.APIH_InfectionDateTime;
        aDS.FieldByName(fnAPIH_RegisteredBy).AsString :=
          anInfectionRecord.APIH_RegisteredBy;
        aDS.FieldByName(fnAPIH_RegisteredDateTime).AsDateTime :=
          anInfectionRecord.APIH_RegisteredDateTime;
        aDS.FieldByName(fnAPIH_StudyID).AsString :=
          anInfectionRecord.APIH_StudyID;
        aDS.FieldByName(fnAPIH_Infection).AsString :=
          anInfectionRecord.APIH_Infection;
        aDS.Post;
      end;
    Result := etVAInfectionHistoryUpdated;
  except
  end;
end;

end.
