unit uMDK_VASite;
{
================================================================================
*
*       Package:      MDK - Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 9/28/07 10:39a $
*       Site Name:    Hines OIFO
*       Date Created: 7/29/05
*       Developers:   andriyevskiy                  
*       Description:  this unit contains the Hemodialysis application
*                     Vascular Access Site object
*       Notes:
*
================================================================================
*       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/APP-HEMODIALYSIS/VascularAccess/uMDK_VASite.pas $
*
*       $ History: uMDK_VASite.pas $
}

interface
uses
  SysUtils
  , Classes
  , uMDK_Common
  , Windows
  , uMDK_CDSCommon
  ;

type
  TAccessStatus = (asPlaced, asClotted, asUnClotted, asRemoved, asUnKnown);
  TVAType = (atFistula, atGraft, atTmpCatheter, atTunneled, atUnKnown);

  TVASiteInfectionRecord = class(TObject)
  private
    fAPIH_ID: string; // 'ID';
    fAPIH_StudyID: string; // 'ID';
    fAPIH_InfectionDateTime: TDateTime; // 'InfectionDateTime';
    fAPIH_Infection: string; // 'Infection';
    fAPIH_RegisteredBy: string; // 'RegisteredBy';
    fAPIH_RegisteredDateTime: TDateTime; // 'RegisteredDateTime';
    function getDescription: string;
  public
    property APIH_ID: string read fAPIH_ID write fAPIH_ID; //'ID';
    property APIH_StudyID: string read fAPIH_StudyID write fAPIH_StudyID; //'ID';
    property APIH_InfectionDateTime: TDateTime read fAPIH_InfectionDateTime write
      fAPIH_InfectionDateTime; //'InfectionDateTime';
    property APIH_Infection: string read fAPIH_Infection write fAPIH_Infection;
    //'Infection';
    property APIH_RegisteredBy: string read fAPIH_RegisteredBy write
      fAPIH_RegisteredBy; //'RegisteredBy';
    property APIH_RegisteredDateTime: TDateTime read fAPIH_RegisteredDateTime
      write fAPIH_RegisteredDateTime; //'RegisteredDateTime';
    property Description: string read getDescription;
    function getDescriptionLine: string;
    function getTitleLine: string;
  end;

  TVASiteHistoryRecord = class(TObject)
  private
    fAPH_ID: string; //'ID';
    fAPH_SessionID: string; //'SessionID';
    fAPH_Selected: Boolean; //'Selected';
    fAPH_Assessed: Boolean; //'Selected';
    fAPH_DateTime: TDateTime; //'DateTime';
    fAPH_RegisteredBy: string; //'Registered';

    fAPH_Thrill: Boolean; //'Thrill';
    fAPH_Bruit: Boolean; //'Bruit';
    fAPH_Mature: Boolean; //'Mature';
    fAPH_Infiltrated: Boolean; //'Infiltrated';
    fAPH_Comments: string; //'Comments';

    fAPH_TTreatment: Boolean; //'Thrill';
    fAPH_VP0: string;
    fAPH_VP200: string;
    fAPH_ABF: string;

    ////////////////////////////////////////////////////////////////////////////
    fAPH_Condition: string;

    // Catheters
    fAPH_BloodCultures: string;
    fAPH_DressingChange: string;
    fAPH_Sutures: string;

    fAPH_PortVolume: string;
    fAPH_PortTx: string;
    fAPH_Arterial: string;
    fAPH_Venous: string;
    fAPH_HeparinAdded: string;

    // Fistula
    fAHP_ThrillInfo: string;
    fAHP_BruitInfo: string;

    function getDescription: string;
  public
    property APH_ID: string read fAPH_ID write fAPH_ID; //'ID';
    property APH_SessionID: string read fAPH_SessionID write fAPH_SessionID;
    //'SessionID';
    property APH_Selected: Boolean read fAPH_Selected write fAPH_Selected;
    //'Selected';
    property APH_Assessed: Boolean read fAPH_Assessed write fAPH_Assessed;
    //'Selected';
    property APH_DateTime: TDateTime read fAPH_DateTime write fAPH_DateTime;
    //'DateTime';
    property APH_RegisteredBy: string read fAPH_RegisteredBy write
      fAPH_RegisteredBy; //'Registered';

    property APH_Thrill: Boolean read fAPH_Thrill write fAPH_Thrill; //'Thrill';
    property APH_Bruit: Boolean read fAPH_Bruit write fAPH_Bruit; //'Bruit';
    property APH_Mature: Boolean read fAPH_Mature write fAPH_Mature; //'Mature';
    property APH_Infiltrated: Boolean read fAPH_Infiltrated write
      fAPH_Infiltrated; //'Infiltrated';
    property APH_TTreatment: Boolean read fAPH_TTreatment write fAPH_TTreatment;
    //'Thrill';
    property APH_Comments: string read fAPH_Comments write fAPH_Comments;
    //'Comments';

    property Description: string read getDescription;
    function getDescriptionLine(ShowComments: Boolean = True): string;
    function getTreatmentLine(ShowComments: Boolean = True): string;
    function getTitleLine: string;
    function getStatus: string;

    property APH_ABF: string read fAPH_ABF write fAPH_ABF;
    property APH_VP0: string read fAPH_VP0 write fAPH_VP0;
    property APH_VP200: string read fAPH_VP200 write fAPH_VP200;

    // All
    property APH_Condition: string read fAPH_Condition write fAPH_Condition;

    // Catheters
    property APH_BloodCultures: string read fAPH_BloodCultures write
      fAPH_BloodCultures;
    property APH_DressingChange: string read fAPH_DressingChange write
      fAPH_DressingChange;
    property APH_Sutures: string read fAPH_Sutures write fAPH_Sutures;

    property APH_PortVolume: string read fAPH_PortVolume write fAPH_PortVolume;
    property APH_PortTx: string read fAPH_PortTx write fAPH_PortTx;
    property APH_Arterial: string read fAPH_Arterial write fAPH_Arterial;
    property APH_Venous: string read fAPH_Venous write fAPH_Venous;
    property APH_HeparinAdded: string read fAPH_HeparinAdded write
      fAPH_HeparinAdded;

    // Fistula
    property APH_ThrillInfo: string read fAHP_ThrillInfo write fAHP_ThrillInfo;
    property APH_BruitInfo: string read fAHP_BruitInfo write fAHP_BruitInfo;

  end;

  //////////////////////////////////////////////////////////////////////////////

  TVASite = class(TObject)
  private
    fStatus: TAccessStatus;

    fType: TVAtype;

    fLocation: string;
    fPlacedDate: TDateTime;
    fSurgeon: string;
    fFacility: string;
    fRegisteredBy: string;

    //    fPlacementID: String;

    fRemovalDate: TDateTime;
    fRemovalSurgeon: string;
    fRemovalFacility: string;
    //    fRemovalID: String;
    fRemovalRegistered: string;
    fRemovalReason: string;

    fSelected: Boolean;
    fAssessed: Boolean;

    //    fAssessHistory: TStringList;
    //    fInfectionHistory: TStringList;

    procedure setSelected(aValue: Boolean);
    procedure setAssessed(aValue: Boolean);
    function getSelected: Boolean;

  public
    ID: string;
    PatientID: string;

    AssessData: TVASiteHistoryRecord;
    InfectionData: TVASiteInfectionRecord;

    constructor Create;
    destructor Destroy; override;
    function Description: string;

    property AccessType: TVAType read fType write ftype;
    property Status: TAccessStatus read fStatus write fStatus;

    property Selected: Boolean read getSelected write setSelected;
    property Assessed: Boolean read fAssessed write setAssessed;

    property Location: string read fLocation write fLocation;
    property PlacedDate: TDateTime read fPlacedDate write fPlacedDate;
    property Surgeon: string read fSurgeon write fSurgeon;
    property Facility: string read fFacility write fFacility;
    property RegisteredBy: string read fRegisteredBy write fRegisteredBy;

    property RemovalDate: TDateTime read fRemovalDate write fRemovalDate;
    property RemovalSurgeon: string read fRemovalSurgeon write fRemovalSurgeon;
    property RemovalFacility: string read fRemovalFacility write
      fRemovalFacility;
    property RemovalRegisteredBy: string read fRemovalRegistered write
      fRemovalRegistered;
    property RemovalReason: string read fRemovalReason write fRemovalReason;

    //    property LastInfected: TDateTime read fLastInfected write fLastInfected;
    //    property AssessHistory: TStringList read fAssessHistory write fAssessHistory;
    //    property InfectionHistory: TStringList read fInfectionHistory write fInfectionHistory;
  end;

function NewVASite(aPatient: string;
  aStatus: TAccessStatus;
  aType: TVAType;
  aDate: TDateTime;
  aLocation,
  aDoc,
  aFacility,
  anOperator: string): TVASite;

const
  apImageSelected = 2;
  apImageAssessed = 7;
  apImageOpened = 8;
  apImageClosed = 14;

const
  apDateTimeFormat = gMDK_DateFMT;
  sTab = char(VK_TAB);

function SiteStatusByName(aName: string): TAccessStatus;
function SiteTypeByName(aName: string): TVAType;

function SiteStatusNameByStatus(aStatus: TAccessStatus): string;
function SiteTypeNameByType(aType: TVAType): string;

implementation

uses
  dMDK_Data;

function NewVASite(aPatient: string;
  aStatus: TAccessStatus;
  aType: TVAType;
  aDate: TDateTime;
  aLocation,
  aDoc,
  aFacility,
  anOperator: string): TVASite;
var
  ap: TVASite;
begin
  ap := TVASite.Create;
  ap.Status := aStatus;
  ap.AccessType := aType;
  ap.PlacedDate := aDate;
  ap.Location := aLocation;
  ap.Surgeon := aDoc;
  ap.Facility := aFacility;
  ap.PatientID := aPatient;
  Result := ap;
end;

function SiteStatusByName(aName: string): TAccessStatus;
begin
  if UpperCase(aname) = 'PLACED' then
    Result := asPlaced
  else if UpperCase(aname) = 'CLOTTED' then
    Result := asClotted
  else if UpperCase(aname) = 'UNCLOTTED' then
    Result := asUnClotted
  else if UpperCase(aname) = 'REMOVED' then
    Result := asRemoved
  else
    Result := asUnKnown;
end;

function SiteTypeByName(aName: string): TVAType;
begin
  if UpperCase(aName) = 'FISTULA' then
    Result := atFistula
  else if UpperCase(aName) = 'GRAFT' then
    Result := atGraft
  else if (UpperCase(aName) = 'TEMP CATHETER') then
    Result := atTmpCatheter
  else if (UpperCase(aName) = 'TMP CATHETER') then
    Result := atTmpCatheter
  else if (UpperCase(aName) = 'TEMPORARY CATHETER') or (UpperCase(aName) =
    'TEMPORARY') then
    Result := atTmpCatheter
  else if (UpperCase(aName) = 'TUNNELED CATHETER') or (UpperCase(aName) =
    'TUNNELED') then
    Result := atTunneled
  else
    Result := atUnKnown;
end;

function SiteStatusNameByStatus(aStatus: TAccessStatus): string;
begin
  case aStatus of
    asPlaced: Result := 'Placed';
    asClotted: Result := 'Clotted';
    asUnClotted: Result := 'UnClotted';
    asRemoved: Result := 'Removed';
  else
    Result := 'UNKNOWN';
  end;
end;

function SiteTypeNameByType(aType: TVAType): string;
begin
  case aType of
    atFistula: Result := 'Fistula';
    atGraft: Result := 'Graft';
    atTmpCatheter: Result := 'Temp Catheter';
    atTunneled: Result := 'Tunneled';
  else
    Result := 'UnKnown';
  end;
end;

////////////////////////////////////////////////////////////////////////////////

constructor TVASite.Create;
begin
  inherited;
  AssessData := TVASiteHistoryRecord.Create;
  InfectionData := TVASiteInfectionRecord.Create;

  //  fAssessHistory := TStringList.Create;
  //  fInfectionHistory := TStringList.Create;
end;

destructor TVASite.Destroy;
{
var
  ar: TVAAssessRecord;
  ir: TVAInfectionRecord;
}
begin
  FreeAndNil(AssessData);
  FreeAndNil(InfectionData);
  (*
    while fAssessHistory.Count > 0 do
      begin
        ar := TVAAssessRecord(fAssessHistory.Objects[0]);
        try
          FreeAndNil(ar);
        except
        end;
        fAssessHistory.Delete(0);
      end;
    FreeAndNil(fAssessHistory);

    while fInfectionHistory.Count > 0 do
      begin
        ir := TVAInfectionRecord(fInfectionHistory.Objects[0]);
        try
          FreeAndNil(ir);
        except
        end;
        fInfectionHistory.Delete(0);
      end;
    FreeAndNil(fInfectionHistory);
  *)
  inherited;
end;

function TVASite.Description: string;
var
  s: string;
begin
  s := SiteTypenameByType(fType);
  //  Result := Format('%s -- %s -- %s',[s,Location,FormatDateTime(apDateTimeFormat,PlacedDate)]);
  case AccessType of
    //    atFistula,atGraft: Result := AssessData.Description;
    atFistula, atGraft: Result := AssessData.getTitleLine + #13#10 +
      AssessData.getDescriptionLine;
  else
    Result := AssessData.GetStatus;
  end;
end;

procedure TVASite.setAssessed(aValue: Boolean);
begin
  fAssessed := aValue;
  AssessData.fAPH_Assessed := aValue;
end;

procedure TVASite.setSelected(aValue: Boolean);
begin
  fSelected := aValue;
end;

function TVASite.getSelected: Boolean;
begin
  result := fSelected;
end;

////////////////////////////////////////////////////////////////////////////////

function TVASiteInfectionRecord.getDescription: string;
var
  s: string;
begin
  result := '';
  s :=
    'Date:' + char(VK_TAB) + FormatDateTime(gMDK_DateFMT,
    fAPIH_InfectionDateTime) + #13#10 +
    'Infection:' + char(VK_TAB) + fAPIH_Infection + #13#10 +
    'Registered by:' + char(VK_TAB) + fAPIH_RegisteredBy + #13#10;
  result := s;
end;

function TVASiteInfectionRecord.getDescriptionLine: string;
var
  s: string;
begin
  result := '';
  s :=
    FormatDateTime(gMDK_DateFMT, fAPIH_InfectionDateTime) + '  ' +
    fAPIH_Infection + '  ' +
    fAPIH_RegisteredBy + #13#10;
  result := s;
end;

function TVASiteInfectionRecord.getTitleLine: string;
begin
  Result := 'Date    Infection    Registered By';
end;

////////////////////////////////////////////////////////////////////////////////

function TVASiteHistoryRecord.getTreatmentLine(ShowComments: Boolean = True):
  string;
var
  s: string;
begin
  Result := FormatDateTime(gMDK_DateFMT, fAPH_DateTime);
  Result := Result + '  (' + APH_SessionID + ') ';
  if APH_TTreatment then
    Result := Result + sTab + sBB + 'Treatment Done' + sBE
  else
    Result := Result + sTab + sBB + 'No Treatment Done' + sBE;

  if ShowComments and (fAPH_Comments <> '') then
    try
      s := #13#10 + 'Comments: ' + sBB + fAPH_Comments + sBE + #13#10;
    except
      s := '';
    end;
  Result := Result + s;
end;

function TVASiteHistoryRecord.getDescription: string;
var
  s: string;
begin
  result := '';
  Result := Result + getStatus;
  s := s + 'Thrill:' + sTab + sTab;
  if fAPH_Thrill then
    s := s + 'Yes' + #13#10
  else
    s := s + 'No' + #13#10;
  s := s + 'Bruit:' + sTab + sTab;
  if fAPH_Bruit then
    s := s + 'Yes' + #13#10
  else
    s := s + 'No' + #13#10;
  s := s + 'Mature:' + sTab + sTab;
  if fAPH_Mature then
    s := s + 'Yes' + #13#10
  else
    s := s + 'No' + #13#10;
  s := s + 'Infiltrated:' + sTab + sTab;
  if fAPH_Infiltrated then
    s := s + 'Yes' + #13#10
  else
    s := s + 'No' + #13#10;
  if fAPH_Comments <> '' then
    s := s + 'Comments: ' + fAPH_Comments + #13#10;

  result := s;
end;

function TVASiteHistoryRecord.getStatus: string;
var
  s: string;
begin
  result := '';
  s := #13#10;
  s := s + 'Assessed:' + sTab;
  if fAPH_Assessed then
    s := s + 'Yes' + #13#10
  else
    s := s + 'No' + #13#10;
  s := s + 'Selected:' + sTab + sTab;
  if fAPH_Selected then
    s := s + 'Yes' + #13#10
  else
    s := s + 'No' + #13#10;
  result := s;
end;

function TVASiteHistoryRecord.getTitleLine: string;
begin
  Result :=
    'Date ' + sTab + sTab + sTab +
    'Selected' + sTab +
    'Thrill  ' + sTab +
    'Bruit   ' + sTab +
    'Mature  ' + sTab +
    'Infiltrated '
    ;
end;

function TVASiteHistoryRecord.getDescriptionLine(ShowComments: Boolean = True):
  string;
var
  s: string;

  function IsTrue(aValue: Boolean): string;
  begin
    if aValue then
      Result := 'Yes'
    else
      Result := 'No';
  end;

begin
  result := '';
  s := FormatDateTime(gMDK_DateFMT, fAPH_DateTime) + '  ';

  if trim(APH_SessionID) = '' then
    s := s + '(blank)'
  else
    s := s + '(' + APH_SessionID + ')  ';

  s := s + sTab;
  s := s + sBB;

  if fAPH_Assessed then
    begin
      s := s + IsTrue(fAPH_Selected) + sTab;
      s := s + IsTrue(fAPH_Thrill) + sTab;
      s := s + IsTrue(fAPH_Bruit) + sTab;
      s := s + IsTrue(fAPH_Mature) + sTab;
      s := s + IsTrue(fAPH_Infiltrated) + sTab;
    end
  else
    s := s + 'Site was not assessed';

  s := s + sBE;

  if ShowComments and (fAPH_Comments <> '') then
    s := s + #13#10#13#10 + 'Comments: ' + sBB + fAPH_Comments + sBE + #13#10
  else
    s := s + #13#10;

  result := s;
end;

end.
