unit fROR_PCall;
{
================================================================================
*
*       Package:        ROR - Clinical Case Registries
*       Date Created:   $Revision: 1 $  $Modtime: 9/28/07 10:43a $
*       Site:           Hines OIFO
*       Developers:     Gavrilov                  
*
*       Description:    RPC call and RPC Error Window
*
*       Notes:
*
================================================================================
*       $Archive: /Clinical Procedures v1.0 (CP Legacy)/CP Version 1.0 - Patch 19/HEMODIALYSIS_1_0_6_320/ROR/fROR_PCall.pas $
*
* $History: fROR_PCall.pas $
 * 
 * *****************  Version 1  *****************
 * User: Vhaishandria Date: 1/13/09    Time: 1:16p
 * Created in $/Clinical Procedures v1.0 (CP Legacy)/CP Version 1.0 - Patch 19/HEMODIALYSIS_1_0_6_320/ROR
 * Patch 19 source code
 * 
 * *****************  Version 1  *****************
 * User: Vhaishandria Date: 4/24/08    Time: 4:24p
 * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/HEMODIALYSIS_1_0_6_319_#18/ROR
 * 
 * *****************  Version 1  *****************
 * User: Vhaishandria Date: 4/02/08    Time: 9:25a
 * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI Current/HEMODIALYSIS_1_0_6_319/ROR
 * 
 * *****************  Version 1  *****************
 * User: Vhaishandria Date: 5/22/07    Time: 9:13a
 * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI 1.0/ROR
 * 
 * *****************  Version 3  *****************
 * User: Vhaishandria Date: 3/29/07    Time: 4:20p
 * Updated in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/ROR
 * 
 * *****************  Version 2  *****************
 * User: Vhaishandria Date: 3/26/07    Time: 3:32p
 * Updated in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/ROR
 *
 * *****************  Version 1  *****************
 * User: Vhaishandria Date: 3/26/07    Time: 1:05p
 * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/ROR
 *
 * *****************  Version 1  *****************
 * User: Vhaishandria Date: 3/26/07    Time: 12:30p
 * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/APP_HEMODIALYSIS/CP MODERNIZATION/ROR
 *
 * *****************  Version 1  *****************
 * User: Vhaishandria Date: 1/25/07    Time: 9:41a
 * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/ROR
 *
 * *****************  Version 1  *****************
 * User: Vhaishandria Date: 1/22/07    Time: 1:29p
 * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI 1.0.6.308/CP MODERNIZATION/ROR
 * GUI 1.0.6.308 RC January 2007
 *
 * *****************  Version 1  *****************
 * User: Vhaishandria Date: 4/28/06    Time: 5:12p
 * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/ROR
 *
 * *****************  Version 16  *****************
 * User: Vhaishgavris Date: 1/18/05    Time: 3:40p
 * Updated in $/CCR v1.0/Current/Components
 *
 * *****************  Version 15  *****************
 * User: Vhaishgavris Date: 1/10/05    Time: 3:49p
 * Updated in $/CCR v1.0/Current/Components
 *
 * *****************  Version 13  *****************
 * User: Vhaishgavris Date: 10/15/04   Time: 10:15a
 * Updated in $/CCR v1.0/Current
 *
 * *****************  Version 12  *****************
 * User: Vhaishgavris Date: 10/15/04   Time: 10:11a
 * Updated in $/CCR v1.0/Current/Components
 *
*
================================================================================
}
interface

uses
  SysUtils, Classes, Forms, Dialogs, TRPCB, StdCtrls, Controls, CCOWRPCBroker;

type
  TRPCMode = set of (

    rpcSilent, // Do not show any error messages
    // (only return the error codes)

    rpcNoResChk // Do not check the Results array for the errors
    // returned by the remote procedure. This flag must be
    // used for the remote procedures that do not conform
    // to the error reporting format supported by the
    // CheckRPCError function.

    );

  //------------------------------- TRPCErrorForm ------------------------------

  TRPCErrorForm = class(TForm)
    Msg: TMemo;
    btnOk: TButton;
    procedure btnOkClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

  //------------------------------- IRPCInterface ------------------------------

  IRPCInterface = interface(IInterface)
    ['{FAD698BB-63E9-408C-BFF7-8E4A8451B63B}']
    function CallProc(const RemoteProcedure: string;
      const Parameters: array of string; MultList: TStringList = nil;
      const RPCMode: TRPCMode = []; RetList: TStrings = nil): Boolean;
    function CheckError(const RPCName: string; RetList: TStrings = nil;
      const RPCMode: TRPCMode = []): Integer;
  end;

  //-------------------------------- TCCRLogMode -------------------------------

  TCCRLogMode = class(TPersistent)
  private
    fEnabled: Boolean;
    fLimitParams: Word;
    fLimitResults: Word;
    fParameters: Boolean;
    fResults: Boolean;

  public
    constructor Create;

    procedure Assign(aSource: TPersistent); override;

  published
    property Enabled: Boolean read fEnabled write fEnabled;
    property LimitParams: Word read fLimitParams write fLimitParams default 0;
    property LimitResults: Word read fLimitResults write fLimitResults default
      0;
    property Parameters: Boolean read fParameters write fParameters default
      True;
    property Results: Boolean read fResults write fResults default True;

  end;

  //------------------------------- TCCRRPCBroker ------------------------------

  TCCRRPCBroker = class(TCCOWRPCBroker, IRPCInterface)
  private
    fLog: TCCRLogMode;
    fLogData: TStringList;

    procedure setLog(const aValue: TCCRLogMode);

  public
    constructor Create(anOwner: TComponent); override;
    destructor Destroy; override;

    procedure AddLogString(const aValue: string); overload;
    procedure AddLogString(const aFormat: string; const ArgList: array of const);
      overload;
    function CallProc(const aRemoteProcedure: string;
      const Parameters: array of string; MultList: TStringList = nil;
      const RPCMode: TRPCMode = []; RetList: TStrings = nil): Boolean; virtual;
    function CheckError(const RPCName: string; RetList: TStrings = nil;
      const RPCMode: TRPCMode = []): Integer;

    property LogData: TStringList read fLogData;

  published
    property Log: TCCRLogMode read fLog write setLog;

  end;

function CallRemoteProc(Broker: TRPCBroker; RemoteProcedure: string;
  Parameters: array of string; MultList: TStringList = nil;
  RPCMode: TRPCMode = []; RetList: TStrings = nil): Boolean;

function CheckRPCError(RPCName: string; Results: TStrings;
  RPCMode: TRPCMode = []): Integer;

implementation
{$R *.DFM}

uses
  Math
  , uROR_CommonUtils // replaces uROR_Utilities
  ;

function CallRemoteProc(Broker: TRPCBroker; RemoteProcedure: string;
  Parameters: array of string; MultList: TStringList = nil;
  RPCMode: TRPCMode = []; RetList: TStrings = nil): Boolean;
var
  i, j: Integer;
begin
  Broker.RemoteProcedure := RemoteProcedure;
  i := 0;
  while i <= High(Parameters) do
    begin
      if (Copy(Parameters[i], 1, 1) = '@') and (Parameters[i] <> '@') then
        begin
          Broker.Param[i].Value := Copy(Parameters[i], 2,
            Length(Parameters[i]));
          Broker.Param[i].PType := Reference;
        end
      else
        begin
          Broker.Param[i].Value := Parameters[i];
          Broker.Param[i].PType := Literal;
        end;
      Inc(i);
    end;

  if MultList <> nil then
    if MultList.Count > 0 then
      begin
        for j := 1 to MultList.Count do
          Broker.Param[i].Mult[IntToStr(j)] := MultList[j - 1];
        Broker.Param[i].PType := List;
      end;

  try
    Result := True;
    if RetList <> nil then
      begin
        RetList.Clear;
        Broker.lstCall(RetList);
      end
    else
      begin
        Broker.Call;
        RetList := Broker.Results;
      end;

    if not (rpcNoResChk in RPCMode) then
      if CheckRPCError(RemoteProcedure, RetList, RPCMode) <> 0 then
        Result := False;

  except
    on e: EBrokerError do
      begin
        if not (rpcSilent in RPCMode) then
          MessageDialog('RPC Error',
            'Error encountered retrieving VistA data.' + #13 +
            'Server: ' + Broker.Server + #13 +
            'Listener port: ' + IntToStr(Broker.ListenerPort) + #13 +
            'Remote procedure: ' + Broker.RemoteProcedure + #13 +
            'Error is: ' + #13 +
            e.Message, mtError, [mbOK], mrOK, e.HelpContext);
        with Broker do
          begin
            Results.Clear;
            Results.Add('-1000^1');
            Results.Add('-1000^' + e.Message);
          end;
        Result := False;
      end;
  else
    raise;
  end;
end;

function CheckRPCError(RPCName: string; Results: TStrings;
  RPCMode: TRPCMode = []): Integer;
var
  i, n: Integer;
  buf, rc: string;
  form: TRPCErrorForm;
begin
  if Results.Count = 0 then
    begin
      Result := mrOK;
      buf := 'The ''' + RPCName + ''' remote procedure returned nothing!';
      if not (rpcSilent in RPCMode) then
        MessageDialog('RPC Error', buf, mtError, [mbOK], mrOK, 0);
      Results.Add('-1001^1');
      Results.Add('-1001^' + buf);
      Exit;
    end;

  Result := 0;
  rc := Piece(Results[0], '^');

  if StrToIntDef(rc, 0) < 0 then
    begin
      Result := mrOK;
      if not (rpcSilent in RPCMode) then
        begin
          form := TRPCErrorForm.Create(Application);
          n := StrToIntDef(Piece(Results[0], '^', 2), 0);
          with form.Msg.Lines do
            begin
              buf := 'The error code ''' + rc + ''' was returned by the '''
                + RPCName + ''' remote procedure!';
              if n > 0 then
                begin
                  buf := buf + ' The problem had been caused by the following ';
                  if n > 1 then
                    buf := buf + 'errors (in reverse chronological order):'
                  else
                    buf := buf + 'error: ';
                  Add(buf);
                  for i := 1 to n do
                    begin
                      buf := Results[i];
                      Add('');
                      Add(' ' + Piece(buf, '^', 2));
                      Add(' Error Code: ' + Piece(buf, '^', 1) + ';' + #9 +
                        'Place: ' + StringReplace(Piece(buf, '^', 3), '~', '^',
                          []));
                    end;
                end
              else
                Add(buf);
            end;
          Result := form.ShowModal;
          form.Free;
        end;
    end;
end;

///////////////////////////////// TCCRLogMode \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

constructor TCCRLogMode.Create;
begin
  inherited;
  fEnabled := False;
  fLimitParams := 0;
  fLimitResults := 0;
  fParameters := True;
  fResults := True;
end;

procedure TCCRLogMode.Assign(aSource: TPersistent);
begin
  if aSource is TCCRLogMode then
    with TCCRLogMode(aSource) do
      begin
        Self.fEnabled := fEnabled;
        Self.fLimitParams := fLimitParams;
        Self.fLimitResults := fLimitResults;
        Self.fParameters := fParameters;
        Self.fResults := fResults;
      end
  else
    inherited;
end;

//////////////////////////////// TCCRRPCBroker \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

constructor TCCRRPCBroker.Create(anOwner: TComponent);
begin
  inherited;
  fLog := TCCRLogMode.Create;
  fLogData := TStringList.Create;
end;

destructor TCCRRPCBroker.Destroy;
begin
  FreeAndNil(fLogData);
  FreeAndNil(fLog);
  inherited;
end;

procedure TCCRRPCBroker.AddLogString(const aValue: string);
begin
  LogData.Add(aValue);
end;

procedure TCCRRPCBroker.AddLogString(const aFormat: string; const ArgList: array
  of const);
begin
  AddLogString(Format(aFormat, ArgList));
end;

function TCCRRPCBroker.CallProc(const aRemoteProcedure: string;
  const Parameters: array of string; MultList: TStringList = nil;
  const RPCMode: TRPCMode = []; RetList: TStrings = nil): Boolean;
var
  i, j, n: Integer;
begin
  RemoteProcedure := aRemoteProcedure;
  n := High(Parameters);

  if Log.Enabled then
    begin
      AddLogString('');
      AddLogString('RPC: %s', [aRemoteProcedure]);
      if Log.Parameters then
        for j := 0 to n do
          AddLogString('     P[%02d]: ''%s''', [j, Parameters[j]]);
    end;

  i := 0;
  while i <= n do
    begin
      if (Copy(Parameters[i], 1, 1) = '@') and (Parameters[i] <> '@') then
        begin
          Param[i].Value := Copy(Parameters[i], 2, Length(Parameters[i]));
          Param[i].PType := Reference;
        end
      else
        begin
          Param[i].Value := Parameters[i];
          Param[i].PType := Literal;
        end;
      Inc(i);
    end;

  if MultList <> nil then
    if MultList.Count > 0 then
      begin
        if Log.Enabled and Log.Parameters then
          begin
            if Log.LimitParams > 0 then
              n := Min(MultList.Count, Log.LimitParams) - 1
            else
              n := MultList.Count - 1;
            for j := 0 to n do
              AddLogString('     M[%02d]: ''%s''', [j, MultList[j]]);
            if n < (MultList.Count - 1) then
              AddLogString('     ...');
          end;
        for j := 1 to MultList.Count do
          Param[i].Mult[IntToStr(j)] := MultList[j - 1];
        Param[i].PType := List;
      end;

  try
    Result := True;
    if RetList <> nil then
      begin
        RetList.Clear;
        lstCall(RetList);
      end
    else
      begin
        Call;
        RetList := Results;
      end;

    if not (rpcNoResChk in RPCMode) then
      if CheckRPCError(aRemoteProcedure, RetList, RPCMode) <> 0 then
        Result := False;

  except
    on e: EBrokerError do
      begin
        if not (rpcSilent in RPCMode) then
          MessageDialog('RPC Error',
            'Error encountered retrieving VistA data.' + #13 +
            'Server: ' + Server + #13 +
            'Listener port: ' + IntToStr(ListenerPort) + #13 +
            'Remote procedure: ' + aRemoteProcedure + #13 +
            'Error is: ' + #13 +
            e.Message, mtError, [mbOK], mrOK, e.HelpContext);
        Results.Clear;
        Results.Add('-1000^1');
        Results.Add('-1000^' + e.Message);
        Result := False;
      end;
  else
    raise;
  end;

  if Log.Enabled and Log.Results then
    begin
      if Log.LimitResults > 0 then
        n := Min(RetList.Count, Log.LimitResults) - 1
      else
        n := RetList.Count - 1;
      for j := 0 to n do
        AddLogString('     R[%02d]: ''%s''', [j, RetList[j]]);
      if n < (RetList.Count - 1) then
        AddLogString('     ...');
    end;
end;

function TCCRRPCBroker.CheckError(const RPCName: string; RetList: TStrings;
  const RPCMode: TRPCMode): Integer;
var
  i, n: Integer;
  buf, rc: string;
  form: TRPCErrorForm;
begin
  if not Assigned(RetList) then
    RetList := Results;

  if RetList.Count = 0 then
    begin
      Result := mrOK;
      buf := 'The ''' + RPCName + ''' remote procedure returned nothing!';
      if not (rpcSilent in RPCMode) then
        MessageDialog('RPC Error', buf, mtError, [mbOK], mrOK, 0);
      RetList.Add('-1001^1');
      RetList.Add('-1001^' + buf);
      Exit;
    end;

  Result := 0;
  rc := Piece(RetList[0], '^');

  if StrToIntDef(rc, 0) < 0 then
    begin
      Result := mrOK;
      if not (rpcSilent in RPCMode) then
        begin
          form := TRPCErrorForm.Create(Application);
          try
            n := StrToIntDef(Piece(RetList[0], '^', 2), 0);
            with form.Msg.Lines do
              begin
                buf := 'The error code ''' + rc + ''' was returned by the '''
                  + RPCName + ''' remote procedure!';
                if n > 0 then
                  begin
                    buf := buf +
                      ' The problem had been caused by the following ';
                    if n > 1 then
                      buf := buf + 'errors (in reverse chronological order):'
                    else
                      buf := buf + 'error: ';
                    Add(buf);
                    for i := 1 to n do
                      begin
                        buf := RetList[i];
                        Add('');
                        Add(' ' + Piece(buf, '^', 2));
                        Add(' Error Code: ' + Piece(buf, '^', 1) + ';' + #9 +
                          'Place: ' + StringReplace(Piece(buf, '^', 3), '~', '^',
                            []));
                      end;
                  end
                else
                  Add(buf);
              end;
            Result := form.ShowModal;
          finally
            form.Free;
          end;
        end;
    end;
end;

procedure TCCRRPCBroker.setLog(const aValue: TCCRLogMode);
begin
  fLog.Assign(aValue);
end;

//////////////////////////////// TRPCErrorForm \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

procedure TRPCErrorForm.btnOkClick(Sender: TObject);
begin
  ModalResult := mrOk;
end;

end.
