unit uROR_RPCBroker;
{
================================================================================
*
*       Package:        ROR - Clinical Case Registries
*       Date Created:   $Revision: 1 $  $Modtime: 7/29/08 9:37a $
*       Site:           Hines OIFO
*       Developers:     Gavrilov                  
*
*       Description:    RPC call and RPC Error Window
*
*       Notes:
*
================================================================================
*       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/ROR/uROR_RPCBroker.pas $
*
*       $ History: uROR_RPCBroker.pas $
*
================================================================================
}
interface

uses
  SysUtils, Classes, Controls
  , uMDK_Utils // replaces the uROR_Utilities
  , TRPCB
  , CCOWRPCBroker
  , RpcConf1
{$IFDEF RORCCOW}
  , VERGENCECONTEXTORLib_TLB
{$ENDIF}
  , Dialogs, Forms;

type
  TRPCBrokerParams = record
    Server: string; //BROKERSERVER;
    ListenerPort: Integer; //9200;
    ClearResults: Boolean; //True;
    ClearParameters: Boolean; //True;
    AccessVerifyCodes: string; //
    DebugMode: Boolean;
  end;

var
  CheckStartupParams: Boolean;
  RPCBroker: TCCOWRPCBroker;
  ContextName: string = 'MD HEMODIALYSIS USER';

function SelectBroker(Context: string{$IFDEF RORCCOW}; aContextor:
  TContextorControl{$ENDIF}): TCCOWRPCBroker;

function GMVConnect: Boolean;
procedure GMVDisconnect;

implementation
{$IFDEF DEMO}
  uses
    fMDK_Wait,
    uMDK_LocalDB,
    fMDK_LocalLogin;
{$ENDIF}

var
  BrokerParams: TRPCBrokerParams;

function GMVConnect: Boolean;
{$IFDEF DEMO}
var
  sLocal: String;
{$ENDIF}
begin
  Result := False;
  if Assigned(RPCBroker) then
    GMVDisconnect;

  cmdLineSwitch(['/CONTEXTNAME'], ContextName); // vhaishandria 050908
  if ContextName = '' then
    ContextName := 'MD HEMODIALYSIS USER';

{$IFDEF DEMO}
  sLocal := '';
  if cmdLineSwitch(['/DEMO'],sLocal)
  then
    begin
      RPCBroker := nil;
      if not assigned(dlgLocalLogin) then
        Application.CreateForm(TdlgLocalLogin, dlgLocalLogin);
      if not Assigned(dlgLocalLogin) or (dlgLocalLogin.ShowModal <> mrOK) then
        Exit;
    end
  else
{$ENDIF}
    begin
{$IFDEF RORCCOW}
      CCRContextor.Enabled := True;
      RPCBroker := SelectBroker(ContextName,CCRContextor.Contextor);
{$ELSE}
      RPCBroker := SelectBroker(ContextName);
{$ENDIF}

      if not Assigned(RPCBroker) then
        Exit;
    end;
{$IFDEF DEMO}
  if sLocal <> '' then
    LOCAL_DIR := sLocal;
  setLocalMap;
{$ENDIF}
  Result := True;
end;

procedure GMVDisconnect;
begin
  if Assigned(RPCBroker) then
    begin
      { Disconnect and destroy the RPC Broker }
      RPCBroker.Connected := False;
      FreeAndNil(RPCBroker);
    end;
end;

function GetBrokerParameters(var ParamRecord: TRPCBrokerParams): boolean;
var
  UseServerList: Boolean;
  sPass,
    SLServer, SLPort: string;

  procedure SetByParams;
  var
    i: Integer;
  begin
    for i := 1 to ParamCount do
      begin
{$IFDEF AVTEST}
        if InString(ParamStr(i), ['/pass', '-pass'], False) then
          sPass := Piece(ParamStr(i), '=', 2);
        if InString(ParamStr(i), ['/av', '-av'], False) then
          sPass := Piece(ParamStr(i), '=', 2);
{$ENDIF}
        if InString(ParamStr(i), ['/s=','-s=','s='], False) or
          InString(ParamStr(i), ['/server=', '-server='], False) then
          begin
            ParamRecord.Server := Piece(ParamStr(i), '=', 2);
            UseServerList := False;
          end;

        if InString(ParamStr(i), ['/p=','-p=','p='], False) or
          InString(ParamStr(i), ['/port=', '-port='], False) then
          begin
            ParamRecord.ListenerPort := StrToIntDef(Piece(ParamStr(i), '=', 2),
              9200);
            UseServerList := False;
          end;

        if InString(ParamStr(i), ['/debug', '-debug'], False) then
          ParamRecord.DebugMode := True;
      end;
{$IFDEF AVTEST}
    if sPass <> '' then
      begin
        UseServerList := False;
        ParamRecord.AccessVerifyCodes := sPass
      end;
{$ENDIF}
  end;

begin
  ParamRecord.Server := 'BROKERSERVER';
  ParamRecord.ListenerPort := 9200;
  ParamRecord.ClearParameters := True;
  ParamRecord.ClearResults := True;

  sPass := '';
  UseServerList := True;
  if CheckStartupParams then
    SetByParams;

  if UseServerList then
    begin
      if GetServerInfo(SLServer, SLPort) <> 1 then
        begin
          MessageDlg('Sign-On Cancelled while setting connection parameters',
            mtInformation, [mbok], 0);
          Result := False;
          Exit;
        end
      else
        begin
          ParamRecord.Server := SLServer;
          ParamRecord.ListenerPort := StrToIntDef(SLPort, 9200);
          Application.ProcessMessages; {Refresh screen prior to connecting}
        end;
    end;
  Result := True;
end;

function SetBrokerParameters(ParamRecord: TRPCBrokerParams; var RPCB:
  TCCOWRPCBroker; var ErrorString: string): boolean;
begin
  RPCB.Server := ParamRecord.Server;
  RPCB.ListenerPort := ParamRecord.ListenerPort;
  RPCB.ClearParameters := ParamRecord.ClearParameters;
  RPCB.ClearResults := ParamRecord.ClearResults;
  RPCB.AccessVerifyCodes := ParamRecord.AccessVerifyCodes;
  RPCB.DebugMode := ParamRecord.DebugMode;

  ErrorString := '';

  if RPCB.Socket > 0 then // ???
    begin
      Result := True;
      exit;
    end;

  try
    RPCB.Connected := True;
    Application.ProcessMessages;
    Result := True;
  except
    on E: EBrokerError do
      begin
        ErrorString := E.Message;
        Result := False;
        Exit;
      end;
  else
    raise;
  end;

end;

function SelectBroker(Context: string{$IFDEF RORCCOW}; aContextor:
  TContextorControl{$ENDIF}): TCCOWRPCBroker;
var
  s: string;
  RPCB: TCCOWRPCBroker;
  NewAttempt: Boolean;
  AttemptCount: Integer;

const
  AttemptLimit = 3;

  procedure ErrorReport;
  begin
    MessageDlg('Error Encountered' + #13 + #13 +
      'User Sign-on is not complete.' + #13 +
      'Attempted connection using the following:' + #13 +
      'VistA Server: ' + RPCB.Server + #13 +
      'Listener Port: ' + IntToStr(RPCB.ListenerPort) + #13 +
      'Error Message: ' + #13 + s,
      mtError,
      [mbok],
      0);
  end;

begin
  if not GetBrokerParameters(BrokerParams) then // process parameter string
    begin
      Result := nil;
      Exit;
    end;
  {
  for i := 1 to ParamCount do
    if InString(ParamStr(i), ['/nonsharedbroker', '-nonsharedbroker'], False) then
      break;
  }

  AttemptCount := 0;
  repeat
    NewAttempt := False;
    {
    if i <= ParamCount then // force use of nonshared broker
      begin
        RPCB := TCCOWRPCBroker.Create(Application);
        if not SetBrokerParameters(BrokerParams,RPCB,s) then
          begin
            ErrorReport;
            FreeAndNil(RPCB);
          end;
      end
    else //try shared broker first
      begin
          RPCB := TSharedRPCBroker.Create(Application);
          TSharedRPCBroker(RPCB).AllowShared := True;
          if not SetBrokerParameters(BrokerParams,RPCB,s) then
            if (pos('Class not registered',s)<>0) then // shared broker is not available
              begin
               FreeAndNil(RPCB);
               RPCB := TRPCBroker.Create(Application); // try nonshared broker
               if not SetBrokerParameters(BrokerParams,RPCB,s) then
                 begin
                   ErrorReport;
                   FreeAndNil(RPCB);
                 end;
              end
            else // unknown error type - just report and stop
              begin
                ErrorReport;
                FreeAndNil(RPCB);
              end;
      end;
      }
    try
      RPCB := TCCOWRPCBroker.Create(Application);
    except
      on E: Exception do
        ShowMessage('TCCOWRPCBroker creation error' + #13 + E.Message);
    end;
    try
{$IFDEF RORCCOW}
      RPCB.Contextor := aContextor;
{$ELSE}
      RPCB.Contextor := nil;
{$ENDIF}
    except
      on E: Exception do
        ShowMessage('Error assigning Contextro to TCCOWRPCBroker' + #13 +
          E.Message);
    end;

    if not SetBrokerParameters(BrokerParams, RPCB, s) then
      begin
        ErrorReport;
        FreeAndNil(RPCB);
      end;

    if RPCB <> nil then
      try
        if not RPCB.CreateContext(Context) then
          FreeAndNil(RPCB);
      except
        on E: Exception do
          begin
            FreeAndNil(RPCB);
            Inc(AttemptCount);
            if AttemptCount > AttemptLimit then
              MessageDlg(
                'You exceeded the limit of connection attempts.' +
                #13 + #13 + 'Try again later.',
                mtError, [mbok], 0)
            else
              NewAttempt := MessageDlg(
                'User Sign-on is not complete.' + #13 + #13 +
                'Error Message: ' + #13 + E.Message + #13 + #13 +
                'Do you want to repeat the attempt?',
                mtError, [mbok, mbCancel], 0) = mrOK;
          end;
      end

  until (RPCB <> nil) or (AttemptCount > AttemptLimit) or not NewAttempt;

  Result := RPCB;
end;

begin
  CheckStartupParams := True;
end.

