unit fMDK_Wait;
{
================================================================================
*
*       Package:      MDK - Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 9/28/07 10:44a $
*       Site Name:    Hines OIFO
*       Date Created: 7/29/05
*       Developers:   andriyevskiy                  
*       Description:  this unit contains the Hemodialysis application
*                     Wait form
*       Notes:
*
================================================================================
*       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/UTILS/fMDK_Wait.pas $
*
*       $ History: fMDK_Wait.pas $
}
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, ExtCtrls;

type
  TfrmWait = class(TForm)
    pnlAnimate: TPanel;
    Animate: TAnimate;
    Panel1: TPanel;
    Panel2: TPanel;
    Button1: TButton;
    Memo1: TRichEdit;
    Bevel1: TBevel;
    procedure Button1Click(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    fParentControl: TWinControl;
    function CurrText: TTextAttributes;
    procedure PaintError(iLen: Integer; aTarget, aLine: string);
  public
    { Public declarations }
    Stop: Boolean;
    procedure AddLogLine(aLine: string);
    procedure AppendLogLine(aLine: string);
    procedure WakeUp(
      aParentControl: TWinControl;
      aCaption: string;
      anAVI: TCommonAVI = aviCopyFiles);
    procedure LayDown;
  end;

var
  frmWait: TfrmWait;

const
  wstOK = 'OK';
  wstAbort = 'ABORT';

implementation

uses fMDK_Main;

{$R *.dfm}

function TfrmWait.CurrText: TTextAttributes;
begin
  if Memo1.SelLength > 0 then
    Result := Memo1.SelAttributes
  else
    Result := Memo1.DefAttributes;
end;

procedure TfrmWait.PaintError(iLen: Integer; aTarget, aLine: string);
var
  iPos: Integer;
begin
  iPos := pos(aTarget, aLine);
  if iPos > 0 then
    begin
      with Memo1 do
        begin
          SelStart := iLen + iPos - 1 - 2;
          SelLength := Length(aTarget);
          CurrText.Color := clRed;
          SelStart := 0;
        end;
      Stop := True;
    end;
end;

procedure TfrmWait.AppendLogLine(aLine: string);
var
  iLen: Integer;
begin
  iLen := Length(Memo1.Lines.Text);
  Memo1.Lines[Memo1.Lines.Count - 1] := Memo1.Lines[Memo1.Lines.Count - 1] +
    aLine;
  PaintError(iLen, 'Failed', aLine);
  Application.ProcessMessages;
end;

procedure TfrmWait.AddLogLine(aLine: string);
var
  iLen: Integer;
begin
  iLen := Length(Memo1.Lines.Text);
  Memo1.Lines.Add(aLine);
  PaintError(iLen, 'Failed', aLine);
  Application.ProcessMessages;
end;

procedure TfrmWait.Button1Click(Sender: TObject);
begin
  Hide;
end;

procedure TfrmWait.FormActivate(Sender: TObject);
begin
  Stop := False;
end;

procedure TfrmWait.WakeUp(
  aParentControl: TWinControl;
  aCaption: string;
  anAVI: TCommonAVI);
begin
  try
    fParentControl := aParentControl;
    Caption := aCaption;
    if Animate.CommonAVI <> anAVI then
      Animate.CommonAVI := anAVI;

    pnlAnimate.Visible := True;
    Show;
    Animate.Open := true;
    Animate.Active := True;
    Memo1.Clear;
    Stop := False;
    Panel2.Visible := False;
    if Assigned(fParentControl) then
      fParentControl.Enabled := False;
  except
  end;
end;

procedure TfrmWait.LayDown;
begin
  Animate.Active := False;
  pnlAnimate.Visible := False;
  hide;
  if Assigned(fParentControl) then
    fParentControl.Enabled := True;
  if Stop then
    Panel2.Visible := True;
end;

procedure TfrmWait.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if (Key = char(VK_ESCAPE)) and (Panel2.Visible) then
    Hide;
end;

end.
